@@ -119,7 +119,6 @@ static bool FindNearestReachableTerrainType(int movemask, int resmask, int range
 	terrainTraversal.SetSize(Map.Info.MapWidth, Map.Info.MapHeight);
 	terrainTraversal.Init();
 
-	Assert(Map.Field(startPos)->CheckMask(resmask));
 	terrainTraversal.PushPos(startPos);
 
 	NearReachableTerrainFinder nearReachableTerrainFinder(player, range, movemask, resmask, terrainPos);
@@ -135,13 +134,28 @@ static bool FindNearestReachableTerrainType(int movemask, int resmask, int range
 	COrder_Resource *order = new COrder_Resource(harvester);
 	Vec2i ressourceLoc;
 
+	int targetFlag;
+	CMapField *mf = Map.Field(pos);
+	if (mf->Cost4OnMap()) {
+		order->CurrentResource = Cost4;
+		targetFlag = MapFieldCost4;
+	} else if (mf->Cost5OnMap()) {
+		order->CurrentResource = Cost5;
+		targetFlag = MapFieldCost5;
+	} else if (mf->Cost6OnMap()) {
+		order->CurrentResource = Cost6;
+		targetFlag = MapFieldCost6;
+	} else {
+		order->CurrentResource = WoodCost;
+		targetFlag = MapFieldForest;
+	}
+
 	//  Find the closest piece of wood next to a tile where the unit can move
-	if (!FindNearestReachableTerrainType(harvester.Type->MovementMask, MapFieldForest, 20, *harvester.Player, pos, &ressourceLoc)) {
+	if (!FindNearestReachableTerrainType(harvester.Type->MovementMask, targetFlag, 20, *harvester.Player, pos, &ressourceLoc)) {
 		DebugPrint("FIXME: Give up???\n");
 		ressourceLoc = pos;
 	}
 	order->goalPos = ressourceLoc;
-	order->CurrentResource = WoodCost; // Hard-coded resource.
 	return order;
 }
 
@@ -213,6 +227,8 @@ COrder_Resource::~COrder_Resource()
 		worker->DeAssignWorkerFromMine(*mine);
 	}
 
+	Depot = NULL;
+
 	CUnit *goal = this->GetGoal();
 	if (goal) {
 		// If mining decrease the active count on the resource.
@@ -345,6 +361,11 @@ COrder_Resource::~COrder_Resource()
 	}
 	if (this->IsGatheringStarted()  && unit.ResourcesHeld > 0) {
 		// escape to Depot with what you have
+		const ResourceInfo &resinfo = *unit.Type->ResInfo[this->CurrentResource];
+		if (resinfo.TerrainHarvester && unit.ResourcesHeld < resinfo.ResourceCapacity) {
+			// We don't have anything yet.
+			return false;
+		}
 		this->DoneHarvesting = true;
 		return true;
 	}
@@ -720,11 +741,18 @@ int COrder_Resource::GatherResource(CUnit &unit)
 		}
 
 		if (resinfo.TerrainHarvester) {
+			CMapField *mf = Map.Field(this->goalPos);
+			addload = std::min((int)mf->Value, addload);
 			unit.ResourcesHeld += addload;
-
-			if (addload && unit.ResourcesHeld == resinfo.ResourceCapacity) {
-				Map.ClearWoodTile(this->goalPos);
+			mf->Value -= addload;
+			if (mf->Value == 0) {
+				Map.ClearTile(this->goalPos);
+				this->DoneHarvesting = true;
+			} else if (unit.ResourcesHeld == resinfo.ResourceCapacity) {
+				// Mark as complete.
+				this->DoneHarvesting = true;
 			}
+			return 0;
 		} else {
 			if (resinfo.HarvestFromOutside) {
 				source = this->GetGoal();
@@ -777,14 +805,6 @@ int COrder_Resource::GatherResource(CUnit &unit)
 				source = NULL;
 				return 0;
 			}
-		}
-		if (resinfo.TerrainHarvester) {
-			if (unit.ResourcesHeld == resinfo.ResourceCapacity) {
-				// Mark as complete.
-				this->DoneHarvesting = true;
-			}
-			return 0;
-		} else {
 			if (resinfo.HarvestFromOutside) {
 				if ((unit.ResourcesHeld == resinfo.ResourceCapacity) || (source == NULL)) {
 					// Mark as complete.
@@ -895,10 +915,19 @@ int COrder_Resource::StopGathering(CUnit &unit)
 
 	// Find and send to resource deposit.
 	CUnit *depot = FindDeposit(unit, 1000, unit.CurrentResource);
+	// There's a bug in the traversal that leads to workers "sometimes" not finding their way to the old depot.
+	// timfel: of course, maybe it's actually nice that workers drop out towards their last depot...
+	if (!depot && (!(resinfo.HarvestFromOutside || resinfo.TerrainHarvester)) && Depot && Depot->IsAlive()) {
+		CUnit *depot = FindDeposit(unit, 1000, unit.CurrentResource);
+		Assert(unit.Container);
+		DropOutNearest(unit, Depot->tilePos + Depot->Type->GetHalfTileSize(), source);
+	}
+	Depot = depot;
 	if (!depot || !unit.ResourcesHeld || this->Finished) {
 		if (!(resinfo.HarvestFromOutside || resinfo.TerrainHarvester)) {
-			Assert(unit.Container);
-			DropOutOnSide(unit, LookingW, source);
+			if (unit.Container) {
+				DropOutOnSide(unit, LookingW, source);
+			}
 		}
 		CUnit *mine = this->Resource.Mine;
 
@@ -908,13 +937,15 @@ int COrder_Resource::StopGathering(CUnit &unit)
 		}
 
 		DebugPrint("%d: Worker %d report: Can't find a resource [%d] deposit.\n"
-				   _C_ unit.Player->Index _C_ UnitNumber(unit) _C_ unit.CurrentResource);
+				_C_ unit.Player->Index _C_ UnitNumber(unit) _C_ unit.CurrentResource);
 		this->Finished = true;
 		return 0;
 	} else {
 		if (!(resinfo.HarvestFromOutside || resinfo.TerrainHarvester)) {
-			Assert(unit.Container);
-			DropOutNearest(unit, depot->tilePos + depot->Type->GetHalfTileSize(), source);
+			if (unit.Container) {
+				// may have dropped out above
+				DropOutNearest(unit, depot->tilePos + depot->Type->GetHalfTileSize(), source);
+			}
 		}
 		UnitGotoGoal(unit, depot, SUB_MOVE_TO_DEPOT);
 	}
@@ -1052,11 +1083,13 @@ bool COrder_Resource::WaitInDepot(CUnit &unit)
 			return false;
 		}
 	} else {
+		/// FIXME: Make it customizable
 		const unsigned int tooManyWorkers = 15;
+
 		CUnit *mine = this->Resource.Mine;
 		const int range = 15;
-		CUnit *newdepot = NULL;
-		CUnit *goal = NULL;
+		CUnit *newdepot = nullptr;
+		CUnit *goal = nullptr;
 		const bool longWay = unit.pathFinderData->output.Cycles > 500;
 
 		if (unit.Player->AiEnabled && AiPlayer && AiPlayer->BuildDepots) {
@@ -1073,8 +1106,20 @@ bool COrder_Resource::WaitInDepot(CUnit &unit)
 
 		// If goal is not NULL, then we got it in AiGetSuitableDepot
 		if (!goal) {
-			goal = UnitFindResource(unit, newdepot ? *newdepot : (mine ? *mine : unit), mine ? range : 1000,
-									this->CurrentResource, unit.Player->AiEnabled, newdepot ? newdepot : depot);
+			if (mine != nullptr && mine->IsAlive()) {
+			goal = mine;
+			} else {
+				const CUnit *start_unit = nullptr;
+				if (newdepot != nullptr) {
+					start_unit = newdepot;
+				} else if (depot != nullptr) {
+					start_unit = depot;
+				}
+				goal = UnitFindResource(unit, (start_unit ? *start_unit : unit), 1000, this->CurrentResource, 
+										unit.Player->AiEnabled, (newdepot ? newdepot : depot));
+			}
+
+									
 		}
 
 		if (goal) {
@@ -1355,4 +1400,19 @@ void COrder_Resource::Execute(CUnit &unit)
 	}
 }
 
+/**
+**  Get goal position
+*/
+/* virtual */ const Vec2i COrder_Resource::GetGoalPos() const
+{
+	const Vec2i invalidPos(-1, -1);
+	if (goalPos != invalidPos) {
+		return goalPos;
+	}
+	if (this->HasGoal()) {
+		return this->GetGoal()->tilePos;
+	}
+	return invalidPos;
+}
+
 //@}
