@@ -37,6 +37,7 @@
 
 #include "actions.h"
 #include "action/action_built.h"
+#include "action/action_build.h"
 #include "action/action_research.h"
 #include "action/action_train.h"
 #include "action/action_upgradeto.h"
@@ -69,6 +70,8 @@ static void ReleaseOrders(CUnit &unit)
 	for (size_t i = 0; i != unit.Orders.size(); ++i) {
 		if (unit.Orders[i]->Action == UnitActionBuilt) {
 			(dynamic_cast<COrder_Built *>(unit.Orders[i]))->Cancel(unit);
+		} if (unit.Orders[i]->Action == UnitActionBuild) {
+			(dynamic_cast<COrder_Build *>(unit.Orders[i]))->Cancel(unit);
 		} else if (unit.Orders[i]->Action == UnitActionResearch) {
 			(dynamic_cast<COrder_Research *>(unit.Orders[i]))->Cancel(unit);
 		} else if (unit.Orders[i]->Action == UnitActionTrain) {
@@ -403,6 +406,19 @@ void CommandPatrolUnit(CUnit &unit, const Vec2i &pos, int flush)
 	if (IsUnitValidForNetwork(unit) == false) {
 		return ;
 	}
+
+	const Vec2i invalidPos(-1, -1);
+
+	Vec2i startPos = unit.tilePos;
+	COrderPtr *prevOrder = &unit.Orders.back();
+
+	if(*prevOrder != NULL) {
+		Vec2i prevGoalPos = (*prevOrder)->GetGoalPos();
+		if(prevGoalPos != invalidPos) {
+			startPos = prevGoalPos;
+		}
+	}
+
 	COrderPtr *order;
 
 	if (!unit.CanMove()) {
@@ -414,7 +430,7 @@ void CommandPatrolUnit(CUnit &unit, const Vec2i &pos, int flush)
 			return;
 		}
 	}
-	*order = COrder::NewActionPatrol(unit.tilePos, pos);
+	*order = COrder::NewActionPatrol(startPos, pos);
 
 	ClearSavedAction(unit);
 }
@@ -499,6 +515,33 @@ void CommandBuildBuilding(CUnit &unit, const Vec2i &pos, CUnitType &what, int fl
 	ClearSavedAction(unit);
 }
 
+/**
+**  Send a unit exploring
+**
+**  @param unit   pointer to unit.
+**  @param flush  if true, flush command queue.
+*/
+void CommandExplore(CUnit &unit, int flush)
+{
+	if (IsUnitValidForNetwork(unit) == false) {
+		return ;
+	}
+	COrderPtr *order;
+
+	if (!unit.CanMove()) {
+		ClearNewAction(unit);
+		order = &unit.NewOrder;
+	} else {
+		order = GetNextOrder(unit, flush);
+		if (order == NULL) {
+			return;
+		}
+	}
+	*order = COrder::NewActionExplore(unit);
+
+	ClearSavedAction(unit);
+}
+
 /**
 **  Cancel the building construction, or kill a unit.
 **
@@ -701,7 +744,7 @@ void CommandCancelTraining(CUnit &unit, int slot, const CUnitType *type)
 **  @param type   upgrade to type
 **  @param flush  if true, flush command queue.
 */
-void CommandUpgradeTo(CUnit &unit, CUnitType &type, int flush)
+void CommandUpgradeTo(CUnit &unit, CUnitType &type, int flush, bool instant)
 {
 	if (IsUnitValidForNetwork(unit) == false) {
 		return ;
@@ -717,7 +760,7 @@ void CommandUpgradeTo(CUnit &unit, CUnitType &type, int flush)
 	if (order == NULL) {
 		return;
 	}
-	*order = COrder::NewActionUpgradeTo(unit, type);
+	*order = COrder::NewActionUpgradeTo(unit, type, instant);
 	ClearSavedAction(unit);
 }
 
@@ -869,56 +912,52 @@ void CommandDiplomacy(int player, int state, int opponent)
 /**
 **  Shared vision changed.
 **
-**  @param player    Player which changes his state.
-**  @param state     New shared vision state.
-**  @param opponent  Opponent.
+**  @param playerIndex   Player which changes his state.
+**  @param state         New shared vision state.
+**  @param opponentIndex Opponent.
 */
-void CommandSharedVision(int player, bool state, int opponent)
+void CommandSharedVision(int playerIndex, bool state, int opponentIndex)
 {
-	// Do a real hardcore seen recount. First we unmark EVERYTHING.
-	for (CUnitManager::Iterator it = UnitManager.begin(); it != UnitManager.end(); ++it) {
-		CUnit &unit = **it;
-		if (!unit.Destroyed) {
-			MapUnmarkUnitSight(unit);
+	CPlayer *player = &Players[playerIndex];
+	CPlayer *opponent = &Players[opponentIndex];
+
+	if (state == player->HasSharedVisionWith(*opponent) || opponent->Type == PlayerTypes::PlayerNobody 
+														|| player->Type == PlayerTypes::PlayerNobody) {
+		return;
+	}
+
+	// Do a real hardcore seen recount. First we unmark sight for all units of the player.
+	for (CUnit *const unit : player->GetUnits()) {
+		if (!unit->Destroyed) {
+			MapUnmarkUnitSight(*unit);
 		}
 	}
-
-	// Compute Before and after.
-	const int before = Players[player].IsBothSharedVision(Players[opponent]);
+	
 	if (state == false) {
-		Players[player].UnshareVisionWith(Players[opponent]);
-	} else {
-		Players[player].ShareVisionWith(Players[opponent]);
-	}
-	const int after = Players[player].IsBothSharedVision(Players[opponent]);
+		player->UnshareVisionWith(*opponent);
 
-	if (before && !after) {
-		// Don't share vision anymore. Give each other explored terrain for good-bye.
-
-		for (int i = 0; i != Map.Info.MapWidth * Map.Info.MapHeight; ++i) {
+		// Don't share vision anymore. Give explored terrain for good-bye.
+		const size_t fieldsNum = Map.Info.MapWidth * Map.Info.MapHeight;
+		for (size_t i = 0; i != fieldsNum; ++i) {
 			CMapField &mf = *Map.Field(i);
 			CMapFieldPlayerInfo &mfp = mf.playerInfo;
 
-			if (mfp.Visible[player] && !mfp.Visible[opponent]) {
-				mfp.Visible[opponent] = 1;
-				if (opponent == ThisPlayer->Index) {
-					Map.MarkSeenTile(mf);
-				}
-			}
-			if (mfp.Visible[opponent] && !mfp.Visible[player]) {
-				mfp.Visible[player] = 1;
-				if (player == ThisPlayer->Index) {
+			if (mfp.Visible[playerIndex] && !mfp.Visible[opponentIndex]) {
+				mfp.Visible[opponentIndex] = 1;
+				/// TODO: change ThisPlayer to currently rendered player/players #RenderTargets
+				if (opponent == ThisPlayer) {
 					Map.MarkSeenTile(mf);
 				}
 			}
 		}
+	} else {
+		player->ShareVisionWith(*opponent);
 	}
 
-	// Do a real hardcore seen recount. Now we remark EVERYTHING
-	for (CUnitManager::Iterator it = UnitManager.begin(); it != UnitManager.end(); ++it) {
-		CUnit &unit = **it;
-		if (!unit.Destroyed) {
-			MapMarkUnitSight(unit);
+	// Do a real hardcore seen recount. Now we remark sight for all units of the player
+	for (CUnit *const unit : player->GetUnits()) {
+		if (!unit->Destroyed) {
+			MapMarkUnitSight(*unit);
 		}
 	}
 }
@@ -932,7 +971,7 @@ void CommandQuit(int player)
 {
 	// Set player to neutral, remove allied/enemy/shared vision status
 	// If the player doesn't have any units then this is pointless?
-	Players[player].Type = PlayerNeutral;
+	Players[player].Type = PlayerTypes::PlayerNeutral;
 	for (int i = 0; i < NumPlayers; ++i) {
 		if (i != player && Players[i].Team != Players[player].Team) {
 			Players[i].SetDiplomacyNeutralWith(Players[player]);
