@@ -57,6 +57,64 @@
 #define AIATTACK_BUILDING 2
 #define AIATTACK_AGRESSIVE 3
 
+class EnemyUnitFinder
+{
+public:
+	EnemyUnitFinder(const CUnit &unit, CUnit **result_unit, int find_type) :
+	//Wyrmgus end
+		unit(unit),
+		movemask(unit.Type->MovementMask & ~(MapFieldLandUnit | MapFieldAirUnit | MapFieldSeaUnit)),
+		attackrange(unit.Stats->Variables[ATTACKRANGE_INDEX].Max),
+		find_type(find_type),
+		result_unit(result_unit)
+	{
+		*result_unit = NULL;
+	}
+	VisitResult Visit(TerrainTraversal &terrainTraversal, const Vec2i &pos, const Vec2i &from);
+private:
+	const CUnit &unit;
+	unsigned int movemask;
+	const int attackrange;
+	const int find_type;
+	CUnit **result_unit;
+};
+
+VisitResult EnemyUnitFinder::Visit(TerrainTraversal &terrainTraversal, const Vec2i &pos, const Vec2i &from)
+{
+	if (!CanMoveToMask(pos, movemask)) { // unreachable
+		return VisitResult_DeadEnd;
+	}
+
+	std::vector<CUnit *> table;
+	Vec2i minpos = pos - Vec2i(attackrange, attackrange);
+	Vec2i maxpos = pos + Vec2i(unit.Type->TileWidth - 1 + attackrange, unit.Type->TileHeight - 1 + attackrange);
+	Select(minpos, maxpos, table, HasNotSamePlayerAs(Players[PlayerNumNeutral]));
+	for (size_t i = 0; i != table.size(); ++i) {
+		CUnit *dest = table[i];
+		const CUnitType &dtype = *dest->Type;
+
+		if (
+			!unit.IsEnemy(*dest) // a friend or neutral
+			|| !CanTarget(*unit.Type, dtype)
+		) {
+			continue;
+		}
+
+		// Don't attack invulnerable units
+		if (dtype.BoolFlag[INDESTRUCTIBLE_INDEX].value || dest->Variable[UNHOLYARMOR_INDEX].Value) {
+			continue;
+		}
+
+		if ((find_type != AIATTACK_BUILDING || dtype.BoolFlag[BUILDING_INDEX].value) && (find_type != AIATTACK_AGRESSIVE || dest->IsAgressive())) {
+			*result_unit = dest;
+			return VisitResult_Finished;
+		} else if (*result_unit == NULL) { // if trying to search for buildings or aggressive units specifically, still put the first found unit (even if it doesn't fit those parameters) as the result unit, so that it can be returned if no unit with the specified parameters is found
+			*result_unit = dest;
+		}
+	}
+	return VisitResult_Ok;
+}
+
 template <const int FIND_TYPE>
 class AiForceEnemyFinder
 {
@@ -84,21 +142,35 @@ public:
 		}
 		if (FIND_TYPE == AIATTACK_RANGE) {
 			*enemy = AttackUnitsInReactRange(*unit);
-		} else if (FIND_TYPE == AIATTACK_ALLMAP) {
-			*enemy = AttackUnitsInDistance(*unit, MaxMapWidth);
-		} else if (FIND_TYPE == AIATTACK_BUILDING) {
-			*enemy = AttackUnitsInDistance(*unit, MaxMapWidth, IsBuildingType());
-			Assert(!*enemy);
-			if (*enemy == NULL || !(*enemy)->Type->Building) {
-				*enemy = AttackUnitsInDistance(*unit, MaxMapWidth);
-			}
-		} else if (FIND_TYPE == AIATTACK_AGRESSIVE) {
-			*enemy = AttackUnitsInDistance(*unit, MaxMapWidth, IsAggresiveUnit());
-			Assert(!*enemy || (*enemy)->IsAgressive());
-			if (*enemy == NULL) {
-				*enemy = AttackUnitsInDistance(*unit, MaxMapWidth);
-			}
+		} else {
+			// Terrain traversal by Andrettin
+			TerrainTraversal terrainTraversal;
+			terrainTraversal.SetSize(Map.Info.MapWidth, Map.Info.MapHeight);
+			terrainTraversal.Init();
+			terrainTraversal.PushUnitPosAndNeighboor(*unit);
+			CUnit *result_unit = NULL;
+			EnemyUnitFinder enemyUnitFinder(*unit, &result_unit, FIND_TYPE);
+			terrainTraversal.Run(enemyUnitFinder);
+			*enemy = result_unit;
 		}
+		// Previous attack finding code before we added TerrainTraversal for all
+		// of these. Here for prosperity
+
+		// } else if (FIND_TYPE == AIATTACK_ALLMAP) {
+		// 	*enemy = AttackUnitsInDistance(*unit, MaxMapWidth);
+		// } else if (FIND_TYPE == AIATTACK_BUILDING) {
+		// 	*enemy = AttackUnitsInDistance(*unit, MaxMapWidth, IsBuildingType());
+		// 	Assert(!*enemy);
+		// 	if (*enemy == NULL || !(*enemy)->Type->Building) {
+		// 		*enemy = AttackUnitsInDistance(*unit, MaxMapWidth);
+		// 	}
+		// } else if (FIND_TYPE == AIATTACK_AGRESSIVE) {
+		// 	*enemy = AttackUnitsInDistance(*unit, MaxMapWidth, IsAggresiveUnit());
+		// 	Assert(!*enemy || (*enemy)->IsAgressive());
+		// 	if (*enemy == NULL) {
+		// 		*enemy = AttackUnitsInDistance(*unit, MaxMapWidth);
+		// 	}
+		// }
 		return *enemy == NULL;
 	}
 private:
@@ -426,10 +498,15 @@ void AiForce::Attack(const Vec2i &pos)
 		return;
 	}
 	if (this->State == AiForceAttackingState_Waiting && isDefenceForce == false) {
-		Vec2i resultPos;
+		Vec2i resultPos {-1, -1};
 		NewRallyPoint(goalPos, &resultPos);
-		this->GoalPos = resultPos;
-		this->State = AiForceAttackingState_GoingToRallyPoint;
+		if (resultPos.x != -1 && resultPos.y != -1) {
+			this->GoalPos = resultPos;
+			this->State = AiForceAttackingState_GoingToRallyPoint;
+		} else {
+			this->GoalPos = goalPos;
+			this->State = AiForceAttackingState_Attacking;
+		}
 	} else {
 		this->GoalPos = goalPos;
 		this->State = AiForceAttackingState_Attacking;
@@ -603,9 +680,9 @@ void AiForceManager::CheckUnits(int *counter)
 			const unsigned int t = aiut.Type->Slot;
 			const int wantedCount = aiut.Want;
 			int e = unit_types_count[t];
-			if (t < AiHelpers.Equiv.size()) {
-				for (unsigned int j = 0; j < AiHelpers.Equiv[t].size(); ++j) {
-					e += unit_types_count[AiHelpers.Equiv[t][j]->Slot];
+			if (t < AiHelpers.Equiv().size()) {
+				for (unsigned int j = 0; j < AiHelpers.Equiv()[t].size(); ++j) {
+					e += unit_types_count[AiHelpers.Equiv()[t][j]->Slot];
 				}
 			}
 			const int requested = wantedCount - (e + counter[t] - attacking[t]);
@@ -1022,10 +1099,15 @@ void AiForce::Update()
 			State = AiForceAttackingState_Waiting;
 			return;
 		} else {
-			Vec2i resultPos;
+			Vec2i resultPos {-1, -1};
 			NewRallyPoint(unit->tilePos, &resultPos);
-			this->GoalPos = resultPos;
-			this->State = AiForceAttackingState_GoingToRallyPoint;
+			if (resultPos.x != -1 && resultPos.y != -1) {
+				this->GoalPos = resultPos;
+				this->State = AiForceAttackingState_GoingToRallyPoint;
+			} else {
+				this->GoalPos = unit->tilePos;
+				this->State = AiForceAttackingState_Attacking;
+			}
 		}
 	}
 	for (size_t i = 0; i != idleUnits.size(); ++i) {
@@ -1055,7 +1137,13 @@ void AiForce::Update()
 
 void AiForceManager::Update()
 {
-	for (unsigned int f = 0; f < forces.size(); ++f) {
+	unsigned int fsize = forces.size();
+	int maxPathing = 2; // reduce load by stopping after issuing a few map searches
+	for (unsigned int f = 0; f < fsize; ++f) {
+		if (maxPathing < 0) {
+			return;
+		}
+
 		AiForce &force = forces[f];
 		//  Look if our defenders still have enemies in range.
 
@@ -1078,15 +1166,23 @@ void AiForceManager::Update()
 					if (force.Units[i]->MapDistanceTo(force.GoalPos) <= nearDist) {
 						//  Look if still enemies in attack range.
 						const CUnit *dummy = NULL;
+						maxPathing--;
 						if (!AiForceEnemyFinder<AIATTACK_RANGE>(force, &dummy).found()) {
 							force.ReturnToHome();
 						}
 					}
 				}
+
+				if (force.Defending == false) {
+					// force is no longer defending
+					return;
+				}
+
 				// Find idle units and order them to defend
 				// Don't attack if there aren't our units near goal point
 				std::vector<CUnit *> nearGoal;
 				const Vec2i offset(15, 15);
+				maxPathing--;
 				Select(force.GoalPos - offset, force.GoalPos + offset, nearGoal,
 					   IsAnAlliedUnitOf(*force.Units[0]->Player));
 				if (nearGoal.empty()) {
@@ -1118,6 +1214,7 @@ void AiForceManager::Update()
 			}
 		} else if (force.Attacking) {
 			force.RemoveDeadUnit();
+			maxPathing--;
 			force.Update();
 		}
 	}
