@@ -34,7 +34,7 @@
 --  Includes
 ----------------------------------------------------------------------------*/
 
-#include <png.h>
+#include "SDL_image.h"
 
 #include "stratagus.h"
 
@@ -56,6 +56,7 @@
 #include "missile.h"
 #include "netconnect.h"
 #include "network.h"
+#include "online_service.h"
 #include "parameters.h"
 #include "pathfinder.h"
 #include "player.h"
@@ -109,26 +110,16 @@ static std::vector<gcn::Container *> Containers;
 */
 void SaveGameSettings(CFile &file)
 {
-	file.printf("\nGameSettings.NetGameType = %d\n", GameSettings.NetGameType);
-	for (int i = 0; i < PlayerMax - 1; ++i) {
-		file.printf("GameSettings.Presets[%d].PlayerColor = %d\n", i, GameSettings.Presets[i].PlayerColor);
-		file.printf("GameSettings.Presets[%d].AIScript = \"%s\"\n", i, GameSettings.Presets[i].AIScript.c_str());
-		file.printf("GameSettings.Presets[%d].Race = %d\n", i, GameSettings.Presets[i].Race);
-		file.printf("GameSettings.Presets[%d].Team = %d\n", i, GameSettings.Presets[i].Team);
-		file.printf("GameSettings.Presets[%d].Type = %d\n", i, GameSettings.Presets[i].Type);
-	}
-	file.printf("GameSettings.Resources = %d\n", GameSettings.Resources);
-	file.printf("GameSettings.Difficulty = %d\n", GameSettings.Difficulty);
-	file.printf("GameSettings.NumUnits = %d\n", GameSettings.NumUnits);
-	file.printf("GameSettings.Opponents = %d\n", GameSettings.Opponents);
-	file.printf("GameSettings.GameType = %d\n", GameSettings.GameType);
-	file.printf("GameSettings.NoFogOfWar = %s\n", GameSettings.NoFogOfWar ? "true" : "false");
-	file.printf("GameSettings.RevealMap = %d\n", GameSettings.RevealMap);
-	file.printf("GameSettings.MapRichness = %d\n", GameSettings.MapRichness);
-	file.printf("GameSettings.Inside = %s\n", GameSettings.Inside ? "true" : "false");
+	file.printf("\n");
+	GameSettings.Save([&](std::string field) {
+		file.printf("GameSettings.%s\n", field.c_str());
+	});
 	file.printf("\n");
 }
 
+/// forward declaration
+void CreateGame(const std::string &filename, CMap *map);
+
 void StartMap(const std::string &filename, bool clean)
 {
 	std::string nc, rc;
@@ -263,156 +254,45 @@ static void LoadStratagusMap(const std::string &smpname, const std::string &mapn
 // Write a small image of map preview
 static void WriteMapPreview(const char *mapname, CMap &map)
 {
-	FILE *fp = fopen(mapname, "wb");
-	if (fp == NULL) {
-		return;
-	}
-
-	png_structp png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
-	if (png_ptr == NULL) {
-		fclose(fp);
-		return;
-	}
-
-	png_infop info_ptr = png_create_info_struct(png_ptr);
-	if (info_ptr == NULL) {
-		fclose(fp);
-		png_destroy_write_struct(&png_ptr, NULL);
-		return;
-	}
-
-	if (setjmp(png_jmpbuf(png_ptr))) {
-		/* If we get here, we had a problem reading the file */
-		fclose(fp);
-		png_destroy_write_struct(&png_ptr, &info_ptr);
-		return;
-	}
-
-	/* set up the output control if you are using standard C streams */
-	png_init_io(png_ptr, fp);
-
-	png_set_IHDR(png_ptr, info_ptr, UI.Minimap.W, UI.Minimap.H, 8,
-				 PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT,
-				 PNG_FILTER_TYPE_DEFAULT);
-
-	png_write_info(png_ptr, info_ptr);
-
 	const int rectSize = 5; // size of rectange used for player start spots
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (UseOpenGL) {
-		unsigned char *pixels = new unsigned char[UI.Minimap.W * UI.Minimap.H * 3];
-		if (!pixels) {
-			fprintf(stderr, "Out of memory\n");
-			exit(1);
-		}
-		// Copy GL map surface to pixel array
-		for (int i = 0; i < UI.Minimap.H; ++i) {
-			for (int j = 0; j < UI.Minimap.W; ++j) {
-				Uint32 c = ((Uint32 *)MinimapSurfaceGL)[j + i * UI.Minimap.W];
-				const int offset = (i * UI.Minimap.W + j) * 3;
-				pixels[offset + 0] = ((c & RMASK) >> RSHIFT);
-				pixels[offset + 1] = ((c & GMASK) >> GSHIFT);
-				pixels[offset + 2] = ((c & BMASK) >> BSHIFT);
-			}
-		}
-		// Add player start spots
-		for (int i = 0; i < PlayerMax - 1; ++i) {
-			if (Players[i].Type != PlayerNobody) {
-				for (int j = -rectSize / 2; j <= rectSize / 2; ++j) {
-					for (int k = -rectSize / 2; k <= rectSize / 2; ++k) {
-						const int miniMapX = Players[i].StartPos.x * UI.Minimap.W / map.Info.MapWidth;
-						const int miniMapY = Players[i].StartPos.y * UI.Minimap.H / map.Info.MapHeight;
-						if (miniMapX + j < 0 || miniMapX + j >= UI.Minimap.W) {
-							continue;
-						}
-						if (miniMapY + k < 0 || miniMapY + k >= UI.Minimap.H) {
-							continue;
-						}
-						const int offset = ((miniMapY + k) * UI.Minimap.H + miniMapX + j) * 3;
-						pixels[offset + 0] = ((Players[i].Color & RMASK) >> RSHIFT);
-						pixels[offset + 1] = ((Players[i].Color & GMASK) >> GSHIFT);
-						pixels[offset + 2] = ((Players[i].Color & BMASK) >> BSHIFT);
-					}
-				}
-			}
-		}
-		// Write everything in PNG
-		for (int i = 0; i < UI.Minimap.H; ++i) {
-			unsigned char *row = new unsigned char[UI.Minimap.W * 3];
-			memcpy(row, pixels + i * UI.Minimap.W * 3, UI.Minimap.W * 3);
-			png_write_row(png_ptr, row);
-			delete[] row;
-		}
-		delete[] pixels;
-	} else
-#endif
-	{
-		unsigned char *row = new unsigned char[UI.Minimap.W * 3];
-		const SDL_PixelFormat *fmt = MinimapSurface->format;
-		SDL_Surface *preview = SDL_CreateRGBSurface(SDL_SWSURFACE,
-													UI.Minimap.W, UI.Minimap.H, 32, fmt->Rmask, fmt->Gmask, fmt->Bmask, 0);
-		SDL_BlitSurface(MinimapSurface, NULL, preview, NULL);
+	const SDL_PixelFormat *fmt = MinimapSurface->format;
+	SDL_Surface *preview = SDL_CreateRGBSurface(SDL_SWSURFACE,
+												UI.Minimap.W, UI.Minimap.H, 32, fmt->Rmask, fmt->Gmask, fmt->Bmask, 0);
+	SDL_BlitSurface(MinimapSurface, NULL, preview, NULL);
 
-		SDL_LockSurface(preview);
+	SDL_LockSurface(preview);
 
-		SDL_Rect rect;
-		for (int i = 0; i < PlayerMax - 1; ++i) {
-			if (Players[i].Type != PlayerNobody) {
-				rect.x = Players[i].StartPos.x * UI.Minimap.W / map.Info.MapWidth - rectSize / 2;
-				rect.y = Players[i].StartPos.y * UI.Minimap.H / map.Info.MapHeight - rectSize / 2;
-				rect.w = rect.h = rectSize;
-				SDL_FillRect(preview, &rect, Players[i].Color);
-			}
+	SDL_Rect rect;
+	for (int i = 0; i < PlayerMax - 1; ++i) {
+		if (Players[i].Type != PlayerTypes::PlayerNobody) {
+			rect.x = Players[i].StartPos.x * UI.Minimap.W / map.Info.MapWidth - rectSize / 2;
+			rect.y = Players[i].StartPos.y * UI.Minimap.H / map.Info.MapHeight - rectSize / 2;
+			rect.w = rect.h = rectSize;
+			SDL_FillRect(preview, &rect, Players[i].Color);
 		}
-
-		for (int i = 0; i < UI.Minimap.H; ++i) {
-			switch (preview->format->BytesPerPixel) {
-				case 1:
-					for (int j = 0; j < UI.Minimap.W; ++j) {
-						Uint8 c = ((Uint8 *)preview->pixels)[j + i * UI.Minimap.W];
-						row[j * 3 + 0] = fmt->palette->colors[c].r;
-						row[j * 3 + 1] = fmt->palette->colors[c].g;
-						row[j * 3 + 2] = fmt->palette->colors[c].b;
-					}
-					break;
-				case 3:
-					memcpy(row, (char *)preview->pixels + i * UI.Minimap.W, UI.Minimap.W * 3);
-					break;
-				case 4:
-					for (int j = 0; j < UI.Minimap.W; ++j) {
-						Uint32 c = ((Uint32 *)preview->pixels)[j + i * UI.Minimap.W];
-						row[j * 3 + 0] = ((c & fmt->Rmask) >> fmt->Rshift);
-						row[j * 3 + 1] = ((c & fmt->Gmask) >> fmt->Gshift);
-						row[j * 3 + 2] = ((c & fmt->Bmask) >> fmt->Bshift);
-					}
-					break;
-			}
-			png_write_row(png_ptr, row);
-		}
-		delete[] row;
-
-		SDL_UnlockSurface(preview);
-		SDL_FreeSurface(preview);
 	}
 
-	png_write_end(png_ptr, info_ptr);
-
-	/* clean up after the write, and free any memory allocated */
-	png_destroy_write_struct(&png_ptr, &info_ptr);
-
-	fclose(fp);
+	SDL_UnlockSurface(preview);
+	IMG_SavePNG(preview, mapname);
+	SDL_FreeSurface(preview);
 }
 
+std::string PlayerTypeNames[static_cast<int>(PlayerTypes::PlayerRescueActive) + 1] = {
+	"",
+	"",
+	"neutral",
+	"nobody",
+	"computer",
+	"person",
+	"rescue-passive",
+	"rescue-active"
+};
 
 // Write the map presentation file
-static int WriteMapPresentation(const std::string &mapname, CMap &map)
+static int WriteMapPresentation(const std::string &mapname, CMap &map, Vec2i newSize)
 {
 	FileWriter *f = NULL;
 
-	const char *type[] = {"", "", "neutral", "nobody",
-						  "computer", "person", "rescue-passive", "rescue-active"
-						 };
-
 	int numplayers = 0;
 	int topplayer = PlayerMax - 2;
 
@@ -424,19 +304,24 @@ static int WriteMapPresentation(const std::string &mapname, CMap &map)
 		f->printf("-- File licensed under the GNU GPL version 2.\n\n");
 
 		f->printf("DefinePlayerTypes(");
-		while (topplayer > 0 && map.Info.PlayerType[topplayer] == PlayerNobody) {
+		while (topplayer > 0 && map.Info.PlayerType[topplayer] == PlayerTypes::PlayerNobody) {
 			--topplayer;
 		}
 		for (int i = 0; i <= topplayer; ++i) {
-			f->printf("%s\"%s\"", (i ? ", " : ""), type[map.Info.PlayerType[i]]);
-			if (map.Info.PlayerType[i] == PlayerPerson) {
+			f->printf("%s\"%s\"", (i ? ", " : ""), PlayerTypeNames[static_cast<int>(map.Info.PlayerType[i])].c_str());
+			if (map.Info.PlayerType[i] == PlayerTypes::PlayerPerson) {
 				++numplayers;
 			}
 		}
 		f->printf(")\n");
 
+		if (newSize.x == 0 || newSize.y == 0) {
+			newSize.x = map.Info.MapWidth;
+			newSize.y = map.Info.MapHeight;
+		}
+
 		f->printf("PresentMap(\"%s\", %d, %d, %d, %d)\n",
-				  map.Info.Description.c_str(), numplayers, map.Info.MapWidth, map.Info.MapHeight,
+				  map.Info.Description.c_str(), numplayers, newSize.x, newSize.y,
 				  map.Info.MapUID + 1);
 
 		if (map.Info.Filename.find(".sms") == std::string::npos && !map.Info.Filename.empty()) {
@@ -460,7 +345,7 @@ static int WriteMapPresentation(const std::string &mapname, CMap &map)
 **  @param map           map to save
 **  @param writeTerrain  write the tiles map in the .sms
 */
-int WriteMapSetup(const char *mapSetup, CMap &map, int writeTerrain)
+int WriteMapSetup(const char *mapSetup, CMap &map, int writeTerrain, Vec2i newSize, Vec2i offset)
 {
 	FileWriter *f = NULL;
 
@@ -472,9 +357,17 @@ int WriteMapSetup(const char *mapSetup, CMap &map, int writeTerrain)
 		// MAPTODO Copyright notice in generated file
 		f->printf("-- File licensed under the GNU GPL version 2.\n\n");
 
+		f->printf("-- preamble\n");
+		f->printf("if CanAccessFile(__file__ .. \".preamble\") then Load(__file__ .. \".preamble\", Editor.Running == 0) end\n\n");
+		if (!Map.Info.Preamble.empty()) {
+			FileWriter *preamble = CreateFileWriter(std::string(mapSetup) + ".preamble");
+			preamble->write(Map.Info.Preamble.c_str(), Map.Info.Preamble.size());
+			delete preamble;
+		}
+
 		f->printf("-- player configuration\n");
 		for (int i = 0; i < PlayerMax; ++i) {
-			if (Map.Info.PlayerType[i] == PlayerNobody) {
+			if (Map.Info.PlayerType[i] == PlayerTypes::PlayerNobody) {
 				continue;
 			}
 			f->printf("SetStartView(%d, %d, %d)\n", i, Players[i].StartPos.x, Players[i].StartPos.y);
@@ -498,6 +391,17 @@ int WriteMapSetup(const char *mapSetup, CMap &map, int writeTerrain)
 		f->printf("LoadTileModels(\"%s\")\n\n", map.TileModelsFileName.c_str());
 
 		if (writeTerrain) {
+		   	if (newSize.x != 0 && newSize.y != 0) {
+				f->printf("for x=0,%d,1 do\n", newSize.x - 1);
+				f->printf("    for y=0,%d,1 do\n", newSize.y - 1);
+				f->printf("        SetTile(%d, x, y, 0)\n", Map.Tileset->getDefaultTileIndex());
+				f->printf("    end\n");
+				f->printf("end\n");
+			} else {
+				newSize.x = map.Info.MapWidth;
+				newSize.y = map.Info.MapHeight;
+			}
+
 			f->printf("-- Tiles Map\n");
 			for (int i = 0; i < map.Info.MapHeight; ++i) {
 				for (int j = 0; j < map.Info.MapWidth; ++j) {
@@ -505,11 +409,20 @@ int WriteMapSetup(const char *mapSetup, CMap &map, int writeTerrain)
 					const int tile = mf.getGraphicTile();
 					const int n = map.Tileset->findTileIndexByTile(tile);
 					const int value = mf.Value;
-					f->printf("SetTile(%3d, %d, %d, %d)\n", n, j, i, value);
+					const int x = j + offset.x;
+					const int y = i + offset.y;
+					if (x < newSize.x && y < newSize.y) {
+						f->printf("SetTile(%3d, %d, %d, %d)\n", n, x, y, value);
+					}
 				}
 			}
 		}
 
+		if (newSize.x == 0 || newSize.y == 0) {
+			newSize.x = map.Info.MapWidth;
+			newSize.y = map.Info.MapHeight;
+		}
+
 		f->printf("\n-- set map default stat and map sound for unit types\n");
 		for (std::vector<CUnitType *>::size_type i = 0; i < UnitTypes.size(); ++i) {
 			const CUnitType &type = *UnitTypes[i];
@@ -572,20 +485,24 @@ int WriteMapSetup(const char *mapSetup, CMap &map, int writeTerrain)
 		f->printf("\n-- place units\n");
 		f->printf("if (MapUnitsInit ~= nil) then MapUnitsInit() end\n");
 		std::vector<CUnit *> teleporters;
-		for (CUnitManager::Iterator it = UnitManager.begin(); it != UnitManager.end(); ++it) {
+		for (CUnitManager::Iterator it = UnitManager->begin(); it != UnitManager->end(); ++it) {
 			const CUnit &unit = **it;
-			f->printf("unit = CreateUnit(\"%s\", %d, {%d, %d})\n",
-					  unit.Type->Ident.c_str(),
-					  unit.Player->Index,
-					  unit.tilePos.x, unit.tilePos.y);
-			if (unit.Type->GivesResource) {
-				f->printf("SetResourcesHeld(unit, %d)\n", unit.ResourcesHeld);
-			}
-			if (!unit.Active) { //Active is true by default
-				f->printf("SetUnitVariable(unit, \"Active\", false)\n");
-			}
-			if (unit.Type->BoolFlag[TELEPORTER_INDEX].value && unit.Goal) {
-				teleporters.push_back(*it);
+			const int x = unit.tilePos.x + offset.x;
+			const int y = unit.tilePos.y + offset.y;
+			if (x < newSize.x && y < newSize.y) {
+				f->printf("unit = CreateUnit(\"%s\", %d, {%d, %d})\n",
+						  unit.Type->Ident.c_str(),
+						  unit.Player->Index,
+						  x, y);
+				if (unit.Type->GivesResource) {
+					f->printf("SetResourcesHeld(unit, %d)\n", unit.ResourcesHeld);
+				}
+				if (!unit.Active) { //Active is true by default
+					f->printf("SetUnitVariable(unit, \"Active\", false)\n");
+				}
+				if (unit.Type->BoolFlag[TELEPORTER_INDEX].value && unit.Goal) {
+					teleporters.push_back(*it);
+				}
 			}
 		}
 		f->printf("\n\n");
@@ -594,6 +511,15 @@ int WriteMapSetup(const char *mapSetup, CMap &map, int writeTerrain)
 			f->printf("SetTeleportDestination(%d, %d)\n", UnitNumber(unit), UnitNumber(*unit.Goal));
 		}
 		f->printf("\n\n");
+
+		f->printf("-- postamble\n");
+		f->printf("if CanAccessFile(__file__ .. \".postamble\") then Load(__file__ .. \".postamble\", Editor.Running == 0) end\n\n");
+		if (!Map.Info.Postamble.empty()) {
+			FileWriter *postamble = CreateFileWriter(std::string(mapSetup) + ".postamble");
+			postamble->write(Map.Info.Postamble.c_str(), Map.Info.Postamble.size());
+			delete postamble;
+		}
+
 	} catch (const FileException &) {
 		fprintf(stderr, "Can't save map setup : '%s' \n", mapSetup);
 		delete f;
@@ -613,7 +539,7 @@ int WriteMapSetup(const char *mapSetup, CMap &map, int writeTerrain)
 **  @param map       map to save
 **  @param writeTerrain   write the tiles map in the .sms
 */
-int SaveStratagusMap(const std::string &mapName, CMap &map, int writeTerrain)
+int SaveStratagusMap(const std::string &mapName, CMap &map, int writeTerrain, Vec2i newSize, Vec2i offset)
 {
 	if (!map.Info.MapWidth || !map.Info.MapHeight) {
 		fprintf(stderr, "%s: invalid Stratagus map\n", mapName.c_str());
@@ -625,6 +551,7 @@ int SaveStratagusMap(const std::string &mapName, CMap &map, int writeTerrain)
 	char *setupExtension = strstr(mapSetup, ".smp");
 	if (!setupExtension) {
 		fprintf(stderr, "%s: invalid Stratagus map filename\n", mapName.c_str());
+		return -1;
 	}
 
 	char previewName[PATH_MAX];
@@ -634,14 +561,13 @@ int SaveStratagusMap(const std::string &mapName, CMap &map, int writeTerrain)
 	WriteMapPreview(previewName, map);
 
 	memcpy(setupExtension, ".sms", 4 * sizeof(char));
-	if (WriteMapPresentation(mapName, map) == -1) {
+	if (WriteMapPresentation(mapName, map, newSize) == -1) {
 		return -1;
 	}
 
-	return WriteMapSetup(mapSetup, map, writeTerrain);
+	return WriteMapSetup(mapSetup, map, writeTerrain, newSize, offset);
 }
 
-
 /**
 **  Load any map.
 **
@@ -806,11 +732,11 @@ static void GameTypeLeftVsRight()
 static void GameTypeManVsMachine()
 {
 	for (int i = 0; i < PlayerMax - 1; ++i) {
-		if (Players[i].Type != PlayerPerson && Players[i].Type != PlayerComputer) {
+		if (Players[i].Type != PlayerTypes::PlayerPerson && Players[i].Type != PlayerTypes::PlayerComputer) {
 			continue;
 		}
 		for (int j = i + 1; j < PlayerMax - 1; ++j) {
-			if (Players[j].Type != PlayerPerson && Players[j].Type != PlayerComputer) {
+			if (Players[j].Type != PlayerTypes::PlayerPerson && Players[j].Type != PlayerTypes::PlayerComputer) {
 				continue;
 			}
 			if (Players[i].Type == Players[j].Type) {
@@ -832,7 +758,7 @@ static void GameTypeManVsMachine()
 static void GameTypeManTeamVsMachine()
 {
 	for (int i = 0; i < PlayerMax - 1; ++i) {
-		if (Players[i].Type != PlayerPerson && Players[i].Type != PlayerComputer) {
+		if (Players[i].Type != PlayerTypes::PlayerPerson && Players[i].Type != PlayerTypes::PlayerComputer) {
 			continue;
 		}
 		for (int j = 0; j < PlayerMax - 1; ++j) {
@@ -845,7 +771,7 @@ static void GameTypeManTeamVsMachine()
 				}
 			}
 		}
-		if (Players[i].Type == PlayerPerson) {
+		if (Players[i].Type == PlayerTypes::PlayerPerson) {
 			Players[i].Team = 2;
 		} else {
 			Players[i].Team = 1;
@@ -853,6 +779,44 @@ static void GameTypeManTeamVsMachine()
 	}
 }
 
+/**
+**  Machine vs Machine
+*/
+static void GameTypeMachineVsMachine()
+{
+	Map.Reveal();
+	for (int i = 0; i < PlayerMax - 1; ++i) {
+		if (Players[i].Type == PlayerTypes::PlayerComputer) {
+			for (int j = i + 1; j < PlayerMax - 1; ++j) {
+				if (Players[j].Type == PlayerTypes::PlayerComputer) {
+					CommandDiplomacy(i, DiplomacyEnemy, j);
+					CommandDiplomacy(j, DiplomacyEnemy, i);
+				} else {
+					CommandDiplomacy(i, DiplomacyNeutral, j);
+					CommandDiplomacy(j, DiplomacyNeutral, i);
+				}
+			}
+		}
+	}
+}
+
+/**
+ ** Machine vs Machine Training
+ */
+static void GameTypeMachineVsMachineTraining()
+{
+	Assert(!IsNetworkGame());
+	GameTypeMachineVsMachine();
+	FastForwardCycle = LONG_MAX;
+	SyncHash = 0;
+	InitSyncRand();
+	SetEffectsEnabled(false);
+	SetMusicEnabled(false);
+	for (int i = 0; i < MyRand() % 100; i++) {
+		SyncRand();
+	}
+}
+
 /*----------------------------------------------------------------------------
 --  Game creation
 ----------------------------------------------------------------------------*/
@@ -895,12 +859,40 @@ void CreateGame(const std::string &filename, CMap *map)
 	}
 
 	for (int i = 0; i < PlayerMax; ++i) {
-		int playertype = (PlayerTypes)Map.Info.PlayerType[i];
-		// Network games only:
-		if (GameSettings.Presets[i].Type != SettingsPresetMapDefault) {
+		PlayerTypes playertype = Map.Info.PlayerType[i];
+		if (GameSettings.Presets[i].Type != PlayerTypes::MapDefault) {
 			playertype = GameSettings.Presets[i].Type;
 		}
 		CreatePlayer(playertype);
+		if (GameSettings.Presets[i].Team != SettingsPresetMapDefault) {
+			// why this calculation? Well. The CreatePlayer function assigns some
+			// default team values, starting from up to PlayerMax + some constant (2 at the time
+			// of this writing). So to not accidentally interfere with those teams, we assign the team
+			// offset by 2 * PlayerMax.
+			Players[i].Team = GameSettings.Presets[i].Team + 2 * PlayerMax;
+			for (int j = 0; j < i; j++) {
+				if (GameSettings.Presets[j].Team != SettingsPresetMapDefault) {
+					if (GameSettings.Presets[j].Team != GameSettings.Presets[i].Team) {
+						Players[i].SetDiplomacyEnemyWith(Players[j]);
+						Players[j].SetDiplomacyEnemyWith(Players[i]);
+					} else {
+						Players[i].SetDiplomacyAlliedWith(Players[j]);
+						Players[j].SetDiplomacyAlliedWith(Players[i]);
+						Players[i].ShareVisionWith(Players[j]);
+						Players[j].ShareVisionWith(Players[i]);
+					}
+				}
+			}
+		}
+	}
+	if (!ThisPlayer && !IsNetworkGame()) {
+		// In demo or kiosk mode, pick first empty slot
+		for (int i = 0; i < PlayerMax; ++i) {
+			if (Players[i].Type == PlayerTypes::PlayerNobody) {
+				ThisPlayer = &Players[i];
+				break;
+			}
+		}
 	}
 
 	if (!filename.empty()) {
@@ -922,66 +914,65 @@ void CreateGame(const std::string &filename, CMap *map)
 	SyncHash = 0;
 	InitSyncRand();
 
-	if (IsNetworkGame()) { // Prepare network play
-		NetworkOnStartGame();
-	} else {
-		const std::string &localPlayerName = Parameters::Instance.LocalPlayerName;
-
-		if (!localPlayerName.empty() && localPlayerName != "Anonymous") {
-			ThisPlayer->SetName(localPlayerName);
-		}
-	}
-
-	CallbackMusicOn();
+	NetworkOnStartGame();
 
 #if 0
 	GamePaused = true;
 #endif
 
-	if (FlagRevealMap) {
-		Map.Reveal();
+	if (FlagRevealMap != MapRevealModes::cHidden) {
+		Map.Reveal(FlagRevealMap);
 	}
 
 	//
 	// Setup game types
 	//
 	// FIXME: implement more game types
-	if (GameSettings.GameType != SettingsGameTypeMapDefault) {
+	if (GameSettings.GameType != GameTypes::SettingsGameTypeMapDefault) {
 		switch (GameSettings.GameType) {
-			case SettingsGameTypeMelee:
+			case GameTypes::SettingsGameTypeMelee:
 				break;
-			case SettingsGameTypeFreeForAll:
+			case GameTypes::SettingsGameTypeFreeForAll:
 				GameTypeFreeForAll();
 				break;
-			case SettingsGameTypeTopVsBottom:
+			case GameTypes::SettingsGameTypeTopVsBottom:
 				GameTypeTopVsBottom();
 				break;
-			case SettingsGameTypeLeftVsRight:
+			case GameTypes::SettingsGameTypeLeftVsRight:
 				GameTypeLeftVsRight();
 				break;
-			case SettingsGameTypeManVsMachine:
+			case GameTypes::SettingsGameTypeManVsMachine:
 				GameTypeManVsMachine();
 				break;
-			case SettingsGameTypeManTeamVsMachine:
+			case GameTypes::SettingsGameTypeManTeamVsMachine:
 				GameTypeManTeamVsMachine();
+				break;
+			case GameTypes::SettingsGameTypeMachineVsMachine:
+				GameTypeMachineVsMachine();
+				break;
+			case GameTypes::SettingsGameTypeMachineVsMachineTraining:
+				GameTypeMachineVsMachineTraining();
+				break;
 
 				// Future game type ideas
 #if 0
-			case SettingsGameTypeOneOnOne:
+			case GameTypes::SettingsGameTypeOneOnOne:
 				break;
-			case SettingsGameTypeCaptureTheFlag:
+			case GameTypes::SettingsGameTypeCaptureTheFlag:
 				break;
-			case SettingsGameTypeGreed:
+			case GameTypes::SettingsGameTypeGreed:
 				break;
-			case SettingsGameTypeSlaughter:
+			case GameTypes::SettingsGameTypeSlaughter:
 				break;
-			case SettingsGameTypeSuddenDeath:
+			case GameTypes::SettingsGameTypeSuddenDeath:
 				break;
-			case SettingsGameTypeTeamMelee:
+			case GameTypes::SettingsGameTypeTeamMelee:
 				break;
-			case SettingsGameTypeTeamCaptureTheFlag:
+			case GameTypes::SettingsGameTypeTeamCaptureTheFlag:
 				break;
 #endif
+            default:
+				break;
 		}
 	}
 
@@ -1068,13 +1059,12 @@ void CreateGame(const std::string &filename, CMap *map)
 	// FIXME: The palette is loaded after the units are created.
 	// FIXME: This loops fixes the colors of the units.
 	//
-	for (CUnitManager::Iterator it = UnitManager.begin(); it != UnitManager.end(); ++it) {
+	for (CUnitManager::Iterator it = UnitManager->begin(); it != UnitManager->end(); ++it) {
 		CUnit &unit = **it;
-		// I don't really think that there can be any rescued units at this point.
-		if (unit.RescuedFrom) {
-			unit.Colors = &unit.RescuedFrom->UnitColors;
-		} else {
-			unit.Colors = &unit.Player->UnitColors;
+		if (unit.Type->OnReady) {
+			unit.Type->OnReady->pushPreamble();
+			unit.Type->OnReady->pushInteger(UnitNumber(unit));
+			unit.Type->OnReady->run();
 		}
 	}
 
@@ -1091,20 +1081,8 @@ void CreateGame(const std::string &filename, CMap *map)
 */
 void InitSettings()
 {
-	for (int i = 0; i < PlayerMax; ++i) {
-		GameSettings.Presets[i].PlayerColor = i;
-		GameSettings.Presets[i].AIScript = "ai-passive";
-		GameSettings.Presets[i].Race = SettingsPresetMapDefault;
-		GameSettings.Presets[i].Team = SettingsPresetMapDefault;
-		GameSettings.Presets[i].Type = SettingsPresetMapDefault;
-	}
-	GameSettings.Resources = SettingsPresetMapDefault;
-	GameSettings.NumUnits = SettingsPresetMapDefault;
-	GameSettings.Opponents = SettingsPresetMapDefault;
-	GameSettings.Difficulty = SettingsPresetMapDefault;
-	GameSettings.GameType = SettingsPresetMapDefault;
-	GameSettings.MapRichness = SettingsPresetMapDefault;
-	GameSettings.NetGameType = SettingsSinglePlayerGame;
+	GameSettings.Init();
+	Preference.InitializeSettingsFromPreferences(GameSettings);
 }
 
 // call the lua function: CleanGame_Lua.
@@ -1147,9 +1125,15 @@ void CleanGame()
 }
 
 /**
+** <b>Description</b>
+**
 **  Return of game name.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>SetGameName</strong>("Wargus Map - Chapter 1")</code></div>
 */
 static int CclSetGameName(lua_State *l)
 {
@@ -1181,11 +1165,20 @@ static int CclSetFullGameName(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Set God mode.
 **
 **  @param l  Lua state.
 **
 **  @return   The old mode.
+**
+** Example:
+**
+** <div class="example"><code>-- God Mode enabled
+**		<strong>SetGodMode</strong>(true)
+**		-- God Mode disabled
+**		<strong>SetGodMode</strong>(false)</code></div>
 */
 static int CclSetGodMode(lua_State *l)
 {
@@ -1195,11 +1188,18 @@ static int CclSetGodMode(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Get God mode.
 **
 **  @param l  Lua state.
 **
 **  @return   God mode.
+**
+** Example:
+**
+** <div class="example"><code>g_mode = <strong>GetGodMode</strong>()
+**		print(g_mode)</code></div>
 */
 static int CclGetGodMode(lua_State *l)
 {
@@ -1452,9 +1452,15 @@ static int ScriptSetUseHPForXp(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Set the local player name
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>SetLocalPlayerName</strong>("Stormreaver Clan")</code></div>
 */
 static int CclSetLocalPlayerName(lua_State *l)
 {
@@ -1464,9 +1470,15 @@ static int CclSetLocalPlayerName(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Get the local player name
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>GetLocalPlayerName</strong>()</code></div>
 */
 static int CclGetLocalPlayerName(lua_State *l)
 {
@@ -1476,7 +1488,14 @@ static int CclGetLocalPlayerName(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Get Stratagus Version
+**
+** Example:
+**
+** <div class="example"><code>version = <strong>GetStratagusVersion</strong>()
+**		print(version)</code></div>
 */
 static int CclGetStratagusVersion(lua_State *l)
 {
@@ -1486,7 +1505,14 @@ static int CclGetStratagusVersion(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Get Stratagus Homepage
+**
+** Example:
+**
+** <div class="example"><code>url = <strong>GetStratagusHomepage</strong>()
+**	print(url)</code></div>
 */
 static int CclGetStratagusHomepage(lua_State *l)
 {
@@ -1596,6 +1622,7 @@ void LuaRegisterModules()
 	UpgradesCclRegister();
 	UserInterfaceCclRegister();
 	VideoCclRegister();
+	OnlineServiceCclRegister();
 }
 
 
