@@ -92,22 +92,6 @@ public:
 	LogEntry *Next;
 };
 
-/**
-**  Multiplayer Player definition
-*/
-class MPPlayer
-{
-public:
-	MPPlayer() : PlayerColor(0), Race(0), Team(0), Type(0) {}
-
-	std::string Name;
-	std::string AIScript;
-	int PlayerColor;
-	int Race;
-	int Team;
-	int Type;
-};
-
 /**
 ** Full replay structure (definition + logs)
 */
@@ -115,10 +99,9 @@ class FullReplay
 {
 public:
 	FullReplay() :
-		MapId(0), Type(0), Race(0), LocalPlayer(0),
-		Resource(0), NumUnits(0), Difficulty(0), NoFow(false), Inside(false), RevealMap(0),
-		MapRichness(0), GameType(0), Opponents(0), Commands(NULL)
+		MapId(0), LocalPlayer(0), Commands(NULL)
 	{
+		ReplaySettings.Init();
 		memset(Engine, 0, sizeof(Engine));
 		memset(Network, 0, sizeof(Network));
 	}
@@ -130,20 +113,10 @@ public:
 	std::string MapPath;
 	unsigned MapId;
 
-	int Type;
-	int Race;
 	int LocalPlayer;
-	MPPlayer Players[PlayerMax];
+	std::string PlayerNames[PlayerMax];
 
-	int Resource;
-	int NumUnits;
-	int Difficulty;
-	bool NoFow;
-	bool Inside;
-	int RevealMap;
-	int MapRichness;
-	int GameType;
-	int Opponents;
+	Settings ReplaySettings;
 	int Engine[3];
 	int Network[3];
 	LogEntry *Commands;
@@ -162,6 +135,7 @@ bool CommandLogDisabled;           /// True if command log is off
 ReplayType ReplayGameType;         /// Replay game type
 static bool DisabledLog;           /// Disabled log for replay
 static CFile *LogFile;             /// Replay log file
+static std::string LastLogFileName;/// Last log file name
 static unsigned long NextLogCycle; /// Next log cycle number
 static int InitReplay;             /// Initialize replay
 static FullReplay *CurrentReplay;
@@ -190,19 +164,26 @@ static FullReplay *StartReplay()
 	replay->Comment1 = "Generated by Stratagus Version " VERSION "";
 	replay->Comment2 = "Visit " HOMEPAGE " for more information";
 
-	if (GameSettings.NetGameType == SettingsSinglePlayerGame) {
-		replay->Type = ReplaySinglePlayer;
-	} else {
-		replay->Type = ReplayMultiPlayer;
-	}
+	replay->ReplaySettings = GameSettings;
 
 	for (int i = 0; i < PlayerMax; ++i) {
-		replay->Players[i].Name = Players[i].Name;
-		replay->Players[i].PlayerColor = GameSettings.Presets[i].PlayerColor;
-		replay->Players[i].AIScript = GameSettings.Presets[i].AIScript;
-		replay->Players[i].Race = GameSettings.Presets[i].Race;
-		replay->Players[i].Team = GameSettings.Presets[i].Team;
-		replay->Players[i].Type = GameSettings.Presets[i].Type;
+		replay->PlayerNames[i] = Players[i].Name;
+		replay->ReplaySettings.Presets[i].PlayerColor = GameSettings.Presets[i].PlayerColor;
+		// Some settings are determined during CreateGame and then stored on the
+		// Players array. For replay, we want to store these,
+		// generally. However, this is not true for teams, since there is
+		// automatic handling of alliance flags for rescueable players.
+		// Additionally, teams are automatically created in CPlayer::Init.
+		// The presets team is actually not used verbatim, but instead used
+		// as the basis of further calculation for the actual player's team
+		// (see game.cpp CreateGame right after the CreatePlayer call where
+		// we check GameSettings.Presets[i].Team != SettingsPresetMapDefault)
+		// So in order for the replay to work, we must not save the actual player
+		// teams, but instead the preset teams.
+		replay->ReplaySettings.Presets[i].AIScript = Players[i].AiName; // GameSettings.Presets[i].AIScript;
+		replay->ReplaySettings.Presets[i].Race = Players[i].Race; // GameSettings.Presets[i].Race;
+		replay->ReplaySettings.Presets[i].Team = GameSettings.Presets[i].Team; // Players[i].Team;
+		replay->ReplaySettings.Presets[i].Type = Players[i].Type; // GameSettings.Presets[i].Type;
 	}
 
 	replay->LocalPlayer = ThisPlayer->Index;
@@ -211,15 +192,6 @@ static FullReplay *StartReplay()
 	replay->Map = Map.Info.Description;
 	replay->MapId = (signed int)Map.Info.MapUID;
 	replay->MapPath = CurrentMapPath;
-	replay->Resource = GameSettings.Resources;
-	replay->NumUnits = GameSettings.NumUnits;
-	replay->Difficulty = GameSettings.Difficulty;
-	replay->NoFow = GameSettings.NoFogOfWar;
-	replay->Inside = GameSettings.Inside;
-	replay->GameType = GameSettings.GameType;
-	replay->RevealMap = GameSettings.RevealMap;
-	replay->MapRichness = GameSettings.MapRichness;
-	replay->Opponents = GameSettings.Opponents;
 
 	replay->Engine[0] = StratagusMajorVersion;
 	replay->Engine[1] = StratagusMinorVersion;
@@ -236,40 +208,26 @@ static FullReplay *StartReplay()
 */
 static void ApplyReplaySettings()
 {
-	if (CurrentReplay->Type == ReplayMultiPlayer) {
+	if (CurrentReplay->ReplaySettings.NetGameType == NetGameTypes::SettingsMultiPlayerGame) {
 		ExitNetwork1();
 		NetPlayers = 2;
-		GameSettings.NetGameType = SettingsMultiPlayerGame;
-
-		ReplayGameType = ReplayMultiPlayer;
 		NetLocalPlayerNumber = CurrentReplay->LocalPlayer;
+		ReplayGameType = ReplayMultiPlayer;
 	} else {
-		GameSettings.NetGameType = SettingsSinglePlayerGame;
 		ReplayGameType = ReplaySinglePlayer;
 	}
-
-	for (int i = 0; i < PlayerMax; ++i) {
-		GameSettings.Presets[i].PlayerColor = CurrentReplay->Players[i].PlayerColor;
-		GameSettings.Presets[i].AIScript = CurrentReplay->Players[i].AIScript;
-		GameSettings.Presets[i].Race = CurrentReplay->Players[i].Race;
-		GameSettings.Presets[i].Team = CurrentReplay->Players[i].Team;
-		GameSettings.Presets[i].Type = CurrentReplay->Players[i].Type;
-	}
+	GameSettings = CurrentReplay->ReplaySettings;
 
 	if (strcpy_s(CurrentMapPath, sizeof(CurrentMapPath), CurrentReplay->MapPath.c_str()) != 0) {
 		fprintf(stderr, "Replay map path is too long\n");
 		// FIXME: need to handle errors better
 		Exit(1);
 	}
-	GameSettings.Resources = CurrentReplay->Resource;
-	GameSettings.NumUnits = CurrentReplay->NumUnits;
-	GameSettings.Difficulty = CurrentReplay->Difficulty;
-	Map.NoFogOfWar = GameSettings.NoFogOfWar = CurrentReplay->NoFow;
-	GameSettings.Inside = CurrentReplay->Inside;
-	GameSettings.GameType = CurrentReplay->GameType;
-	FlagRevealMap = GameSettings.RevealMap = CurrentReplay->RevealMap;
-	GameSettings.MapRichness = CurrentReplay->MapRichness;
-	GameSettings.Opponents = CurrentReplay->Opponents;
+
+	Map.NoFogOfWar = GameSettings.NoFogOfWar;
+	FlagRevealMap = GameSettings.RevealMap;
+
+	GameSettings.Save(+[](std::string s) { DebugPrint("%s\n" _C_ s.c_str()); });
 
 	// FIXME : check engine version
 	// FIXME : FIXME: check network version
@@ -338,33 +296,19 @@ static void SaveFullLog(CFile &file)
 	file.printf("  Map = \"%s\",\n", CurrentReplay->Map.c_str());
 	file.printf("  MapPath = \"%s\",\n", CurrentReplay->MapPath.c_str());
 	file.printf("  MapId = %u,\n", CurrentReplay->MapId);
-	file.printf("  Type = %d,\n", CurrentReplay->Type);
-	file.printf("  Race = %d,\n", CurrentReplay->Race);
 	file.printf("  LocalPlayer = %d,\n", CurrentReplay->LocalPlayer);
 	file.printf("  Players = {\n");
 	for (int i = 0; i < PlayerMax; ++i) {
-		if (!CurrentReplay->Players[i].Name.empty()) {
-			file.printf("\t{ Name = \"%s\",", CurrentReplay->Players[i].Name.c_str());
-		} else {
-			file.printf("\t{");
-		}
-		file.printf(" AIScript = \"%s\",", CurrentReplay->Players[i].AIScript.c_str());
-		file.printf(" PlayerColor = %d,", CurrentReplay->Players[i].PlayerColor);
-		file.printf(" Race = %d,", CurrentReplay->Players[i].Race);
-		file.printf(" Team = %d,", CurrentReplay->Players[i].Team);
-		file.printf(" Type = %d }%s", CurrentReplay->Players[i].Type,
-					i != PlayerMax - 1 ? ",\n" : "\n");
+		file.printf("\t{ Name = \"%s\", ", CurrentReplay->PlayerNames[i].c_str());
+		CurrentReplay->ReplaySettings.Presets[i].Save([&] (std::string field) {
+			file.printf("%s, ", field.c_str());
+		});
+		file.printf("}%s", i != PlayerMax - 1 ? ",\n" : "\n");
 	}
 	file.printf("  },\n");
-	file.printf("  Resource = %d,\n", CurrentReplay->Resource);
-	file.printf("  NumUnits = %d,\n", CurrentReplay->NumUnits);
-	file.printf("  Difficulty = %d,\n", CurrentReplay->Difficulty);
-	file.printf("  NoFow = %s,\n", CurrentReplay->NoFow ? "true" : "false");
-	file.printf("  Inside = %s,\n", CurrentReplay->Inside ? "true" : "false");
-	file.printf("  RevealMap = %d,\n", CurrentReplay->RevealMap);
-	file.printf("  GameType = %d,\n", CurrentReplay->GameType);
-	file.printf("  Opponents = %d,\n", CurrentReplay->Opponents);
-	file.printf("  MapRichness = %d,\n", CurrentReplay->MapRichness);
+	CurrentReplay->ReplaySettings.Save([&] (std::string field) {
+		file.printf("  %s,\n", field.c_str());
+	}, false);
 	file.printf("  Engine = { %d, %d, %d },\n",
 				CurrentReplay->Engine[0], CurrentReplay->Engine[1], CurrentReplay->Engine[2]);
 	file.printf("  Network = { %d, %d, %d }\n",
@@ -425,8 +369,10 @@ void CommandLog(const char *action, const CUnit *unit, int flush,
 	// to the save file name, to test more than one player on one computer.
 	//
 	if (!LogFile) {
+		time_t now;
+		time(&now);
 		struct stat tmp;
-		char buf[16];
+
 		std::string path(Parameters::Instance.GetUserDirectory());
 		if (!GameName.empty()) {
 			path += "/";
@@ -438,10 +384,10 @@ void CommandLog(const char *action, const CUnit *unit, int flush,
 			makedir(path.c_str(), 0777);
 		}
 
-		snprintf(buf, sizeof(buf), "%d", ThisPlayer->Index);
-
 		path += "/log_of_stratagus_";
-		path += buf;
+		path += std::to_string(ThisPlayer->Index);
+		path += "_";
+		path += std::to_string((intmax_t)now);
 		path += ".log";
 
 		LogFile = new CFile;
@@ -452,7 +398,7 @@ void CommandLog(const char *action, const CUnit *unit, int flush,
 			LogFile = NULL;
 			return;
 		}
-
+		LastLogFileName = path;
 		if (CurrentReplay) {
 			SaveFullLog(*LogFile);
 		}
@@ -555,7 +501,7 @@ static int CclLog(lua_State *l)
 		} else if (!strcmp(value, "Num")) {
 			log->Num = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "SyncRandSeed")) {
-			log->SyncRandSeed = LuaToUnsignedNumber(l, -1);
+			log->SyncRandSeed = lua_tointeger(l, -1);
 		} else {
 			LuaError(l, "Unsupported key: %s" _C_ value);
 		}
@@ -608,10 +554,6 @@ static int CclReplayLog(lua_State *l)
 			replay->MapPath = LuaToString(l, -1);
 		} else if (!strcmp(value, "MapId")) {
 			replay->MapId = LuaToNumber(l, -1);
-		} else if (!strcmp(value, "Type")) {
-			replay->Type = LuaToNumber(l, -1);
-		} else if (!strcmp(value, "Race")) {
-			replay->Race = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "LocalPlayer")) {
 			replay->LocalPlayer = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "Players")) {
@@ -630,17 +572,17 @@ static int CclReplayLog(lua_State *l)
 				while (lua_next(l, top) != 0) {
 					value = LuaToString(l, -2);
 					if (!strcmp(value, "Name")) {
-						replay->Players[j].Name = LuaToString(l, -1);
+						replay->PlayerNames[j] = LuaToString(l, -1);
 					} else if (!strcmp(value, "AIScript")) {
-						replay->Players[j].AIScript = LuaToString(l, -1);
+						replay->ReplaySettings.Presets[j].AIScript = LuaToString(l, -1);
 					} else if (!strcmp(value, "PlayerColor")) {
-						replay->Players[j].PlayerColor = LuaToNumber(l, -1);
+						replay->ReplaySettings.Presets[j].PlayerColor = LuaToNumber(l, -1);
 					} else if (!strcmp(value, "Race")) {
-						replay->Players[j].Race = LuaToNumber(l, -1);
+						replay->ReplaySettings.Presets[j].Race = LuaToNumber(l, -1);
 					} else if (!strcmp(value, "Team")) {
-						replay->Players[j].Team = LuaToNumber(l, -1);
+						replay->ReplaySettings.Presets[j].Team = LuaToNumber(l, -1);
 					} else if (!strcmp(value, "Type")) {
-						replay->Players[j].Type = LuaToNumber(l, -1);
+						replay->ReplaySettings.Presets[j].Type = static_cast<PlayerTypes>(LuaToNumber(l, -1));
 					} else {
 						LuaError(l, "Unsupported key: %s" _C_ value);
 					}
@@ -648,24 +590,6 @@ static int CclReplayLog(lua_State *l)
 				}
 				lua_pop(l, 1);
 			}
-		} else if (!strcmp(value, "Resource")) {
-			replay->Resource = LuaToNumber(l, -1);
-		} else if (!strcmp(value, "NumUnits")) {
-			replay->NumUnits = LuaToNumber(l, -1);
-		} else if (!strcmp(value, "Difficulty")) {
-			replay->Difficulty = LuaToNumber(l, -1);
-		} else if (!strcmp(value, "NoFow")) {
-			replay->NoFow = LuaToBoolean(l, -1);
-		} else if (!strcmp(value, "Inside")) {
-			replay->Inside = LuaToBoolean(l, -1);
-		} else if (!strcmp(value, "RevealMap")) {
-			replay->RevealMap = LuaToNumber(l, -1);
-		} else if (!strcmp(value, "GameType")) {
-			replay->GameType = LuaToNumber(l, -1);
-		} else if (!strcmp(value, "Opponents")) {
-			replay->Opponents = LuaToNumber(l, -1);
-		} else if (!strcmp(value, "MapRichness")) {
-			replay->MapRichness = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "Engine")) {
 			if (!lua_istable(l, -1) || lua_rawlen(l, -1) != 3) {
 				LuaError(l, "incorrect argument");
@@ -681,7 +605,9 @@ static int CclReplayLog(lua_State *l)
 			replay->Network[1] = LuaToNumber(l, -1, 2);
 			replay->Network[2] = LuaToNumber(l, -1, 3);
 		} else {
-			LuaError(l, "Unsupported key: %s" _C_ value);
+			if (!replay->ReplaySettings.SetField(value, LuaToNumber(l, -1))) {
+				LuaError(l, "Unsupported key: %s" _C_ value);
+			}
 		}
 		lua_pop(l, 1);
 	}
@@ -725,7 +651,6 @@ int LoadReplay(const std::string &name)
 {
 	CleanReplayLog();
 	ReplayGameType = ReplaySinglePlayer;
-
 	LuaLoadFile(name);
 
 	NextLogCycle = ~0UL;
@@ -763,7 +688,7 @@ void CleanReplayLog()
 {
 	if (CurrentReplay) {
 		DeleteReplay(CurrentReplay);
-		CurrentReplay = 0;
+		CurrentReplay = nullptr;
 	}
 	ReplayStep = NULL;
 
@@ -795,8 +720,8 @@ static void DoNextReplay()
 	const Vec2i pos(ReplayStep->PosX, ReplayStep->PosY);
 	const int arg1 = ReplayStep->PosX;
 	const int arg2 = ReplayStep->PosY;
-	CUnit *unit = unitSlot != -1 ? &UnitManager.GetSlotUnit(unitSlot) : NULL;
-	CUnit *dunit = (ReplayStep->DestUnitNumber != -1 ? &UnitManager.GetSlotUnit(ReplayStep->DestUnitNumber) : NULL);
+	CUnit *unit = unitSlot != -1 ? &UnitManager->GetSlotUnit(unitSlot) : NULL;
+	CUnit *dunit = (ReplayStep->DestUnitNumber != -1 ? &UnitManager->GetSlotUnit(ReplayStep->DestUnitNumber) : NULL);
 	const char *val = ReplayStep->Value.c_str();
 	const int num = ReplayStep->Num;
 
@@ -850,6 +775,8 @@ static void DoNextReplay()
 		SendCommandUnload(*unit, pos, dunit, flags);
 	} else if (!strcmp(action, "build")) {
 		SendCommandBuildBuilding(*unit, pos, *UnitTypeByIdent(val), flags);
+	} else if (!strcmp(action, "explore")) {
+		SendCommandExplore(*unit, flags);
 	} else if (!strcmp(action, "dismiss")) {
 		SendCommandDismiss(*unit);
 	} else if (!strcmp(action, "resource-loc")) {
@@ -922,8 +849,8 @@ static void ReplayEachCycle()
 	}
 	if (InitReplay) {
 		for (int i = 0; i < PlayerMax; ++i) {
-			if (!CurrentReplay->Players[i].Name.empty()) {
-				Players[i].SetName(CurrentReplay->Players[i].Name);
+			if (!CurrentReplay->PlayerNames[i].empty()) {
+				Players[i].SetName(CurrentReplay->PlayerNames[i]);
 			}
 		}
 		ReplayStep = CurrentReplay->Commands;
@@ -982,7 +909,6 @@ int SaveReplay(const std::string &filename)
 {
 	FILE *fd;
 	char *buf;
-	std::ostringstream logfile;
 	std::string destination;
 	struct stat sb;
 	size_t size;
@@ -994,9 +920,7 @@ int SaveReplay(const std::string &filename)
 
 	destination = Parameters::Instance.GetUserDirectory() + "/" + GameName + "/logs/" + filename;
 
-	logfile << Parameters::Instance.GetUserDirectory() << "/" << GameName << "/logs/log_of_stratagus_" << ThisPlayer->Index << ".log";
-
-	if (stat(logfile.str().c_str(), &sb)) {
+	if (!LastLogFileName.empty() && stat(LastLogFileName.c_str(), &sb)) {
 		fprintf(stderr, "stat failed\n");
 		return -1;
 	}
@@ -1005,7 +929,7 @@ int SaveReplay(const std::string &filename)
 		fprintf(stderr, "Out of memory\n");
 		return -1;
 	}
-	fd = fopen(logfile.str().c_str(), "rb");
+	fd = fopen(LastLogFileName.c_str(), "rb");
 	if (!fd) {
 		fprintf(stderr, "fopen failed\n");
 		delete[] buf;
