@@ -70,6 +70,7 @@ enum UnitAction {
 	UnitActionUnload,       /// unit leaving transporter
 	UnitActionPatrol,       /// unit paroling area
 	UnitActionBuild,        /// unit builds building
+	UnitActionExplore,      /// unit explores map
 
 	UnitActionRepair,       /// unit repairing
 	UnitActionResource,     /// unit harvesting resources
@@ -93,7 +94,7 @@ struct lua_State;
 class COrder
 {
 public:
-	explicit COrder(int action) : Goal(), Action(action), Finished(false)
+	explicit COrder(int action) : Goal(), Action(action), Finished(false), Instant(false)
 	{
 	}
 	virtual ~COrder();
@@ -133,6 +134,7 @@ public:
 	static COrder *NewActionBuilt(CUnit &builder, CUnit &unit);
 	static COrder *NewActionDefend(CUnit &dest);
 	static COrder *NewActionDie();
+	static COrder *NewActionExplore(const CUnit &unit);
 	static COrder *NewActionFollow(CUnit &dest);
 	static COrder *NewActionMove(const Vec2i &pos);
 	static COrder *NewActionPatrol(const Vec2i &currentPos, const Vec2i &dest);
@@ -148,7 +150,7 @@ public:
 	static COrder *NewActionTrain(CUnit &trainer, CUnitType &type);
 	static COrder *NewActionTransformInto(CUnitType &type);
 	static COrder *NewActionUnload(const Vec2i &pos, CUnit *what);
-	static COrder *NewActionUpgradeTo(CUnit &unit, CUnitType &type);
+	static COrder *NewActionUpgradeTo(CUnit &unit, CUnitType &type, bool instant = false);
 
 protected:
 	void UpdatePathFinderData_NotCalled(PathFinderInput &input);
@@ -158,6 +160,7 @@ private:
 public:
 	const unsigned char Action;   /// global action
 	bool Finished; /// true when order is finish
+	bool Instant; /// true to ignore TimeCost
 };
 
 typedef COrder *COrderPtr;
