@@ -34,6 +34,7 @@
 
 #include <vector>
 #include <string>
+#include <tuple>
 #include "icons.h"
 #ifndef __VEC2I_H__
 #include "vec2i.h"
@@ -66,9 +67,13 @@ public:
 	~CEditor() {}
 
 	void Init();
-	/// Make random map
-	void CreateRandomMap() const;
 
+	/// Make random map
+	void CreateRandomMap(bool shuffleTransitions = false) const;
+	/// Variables for random map creation
+	int BaseTileIndex; /// Tile to fill the map with initially;
+	std::vector<std::tuple<int, int, int>> RandomTiles; /// other tiles to fill randomly. (tile, count, area size)
+	std::vector<std::tuple<std::string, int, int, int>> RandomUnits; /// neutral units to add randomly. (name, count, initial resources, tile under unit)
 
 	std::vector<std::string> UnitTypes;             /// Sorted editor unit-type table.
 	std::vector<const CUnitType *> ShownUnitTypes;  /// Shown editor unit-type table.
@@ -109,6 +114,7 @@ public:
 
 extern CEditor Editor;
 
+extern bool TileToolNoFixup;
 extern char TileToolRandom;
 
 /*----------------------------------------------------------------------------
@@ -125,6 +131,7 @@ extern void EditorUpdateDisplay();
 
 /// Save a map from editor
 extern int EditorSaveMap(const std::string &file);
+extern int EditorSaveMapWithResize(const std::string &file, Vec2i sz = {0, 0}, Vec2i off = {0, 0});
 
 /// Register ccl features
 extern void EditorCclRegister();
@@ -132,7 +139,7 @@ extern void EditorCclRegister();
 /// Update surroundings for tile changes
 extern void EditorTileChanged(const Vec2i &pos);
 
-extern void EditorChangeTile(const Vec2i &pos, int tileIndex, int d);
+extern void EditorChangeTile(const Vec2i &pos, int tileIndex, int d, bool changeSurroundings);
 
 //@}
 
