@@ -96,9 +96,6 @@ public:
 
 	void Load();
 	void Reload() const;
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	void FreeOpenGL();
-#endif
 	void Clean();
 
 	CGraphic *GetFontColorGraphic(const CFontColor &fontColor) const;
@@ -109,9 +106,6 @@ public:
 	void DynamicLoad() const;
 
 private:
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	void MakeFontColorTextures() const;
-#endif
 	void MeasureWidths();
 
 private:
@@ -133,7 +127,7 @@ public:
 	static CFontColor *Get(const std::string &ident);
 
 	std::string Ident;
-	CColor Colors[MaxFontColors];
+	SDL_Color *Colors;
 };
 
 /*----------------------------------------------------------------------------
@@ -161,6 +155,7 @@ extern CFont &GetSmallFont();  /// Small font used in stats
 extern CFont &GetGameFont();   /// Normal font used in game
 extern bool IsGameFontReady(); /// true when GameFont is provided
 
+extern int FontCodePage;
 
 /// Set the default text colors for normal and reverse text
 extern void SetDefaultTextColors(const std::string &normal, const std::string &reverse);
@@ -175,13 +170,6 @@ extern int GetHotKey(const std::string &text);
 /// Load and initialize the fonts
 extern void LoadFonts();
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-/// Free OpenGL fonts
-extern void FreeOpenGLFonts();
-/// Reload OpenGL fonts
-extern void ReloadFonts();
-#endif
-
 /// Cleanup the font module
 extern void CleanFonts();
 
