@@ -63,6 +63,7 @@ enum ButtonCmd {
 	ButtonHarvest,        /// order harvest
 	ButtonBuild,          /// order build
 	ButtonPatrol,         /// order patrol
+	ButtonExplore,        /// order explore
 	ButtonAttackGround,   /// order attack ground
 	ButtonSpellCast,      /// order cast spell
 	ButtonUnload,         /// order unload unit
@@ -126,7 +127,8 @@ enum _button_area_ {
 enum _menu_button_under_ {
 	ButtonUnderMenu,              /// Menu button
 	ButtonUnderNetworkMenu,       /// Network menu button
-	ButtonUnderNetworkDiplomacy   /// Diplomacy button
+	ButtonUnderNetworkDiplomacy,  /// Diplomacy button
+	ButtonUnderFreeWorkers        /// Free workers icon
 };
 
 /// current interface state
@@ -342,6 +344,9 @@ extern void UiFindIdleWorker();
 /// Track unit, the viewport follows the unit.
 extern void UiTrackUnit();
 
+/// Scroll margins on screen
+extern void SetScrollMargins(unsigned int top, unsigned int right, unsigned int bottom, unsigned int left);
+
 /// Handle cheats
 extern int HandleCheats(const std::string &input);
 
@@ -374,6 +379,8 @@ extern void CenterOnMessage();
 extern void CleanMessages();
 /// show/hide messages
 extern void ToggleShowMessages();
+/// max message count
+extern void SetMaxMessageCount(int newMax);
 
 /// Draw the timer
 extern void DrawTimer();
@@ -406,8 +413,10 @@ extern bool ButtonCheckUnitVariable(const CUnit &unit, const ButtonAction &butto
 extern bool ButtonCheckUnitsOr(const CUnit &unit, const ButtonAction &button);
 /// Check if allowed units exists
 extern bool ButtonCheckUnitsAnd(const CUnit &unit, const ButtonAction &button);
-/// Check if units don't exist
+/// Check if not one unit exist
 extern bool ButtonCheckUnitsNot(const CUnit &unit, const ButtonAction &button);
+/// Check if none of the units exist
+extern bool ButtonCheckUnitsNor(const CUnit &unit, const ButtonAction &button);
 /// Check if have network play
 extern bool ButtonCheckNetwork(const CUnit &unit, const ButtonAction &button);
 /// Check if don't have network play
@@ -424,6 +433,8 @@ extern bool ButtonCheckUpgradeTo(const CUnit &unit, const ButtonAction &button);
 extern bool ButtonCheckResearch(const CUnit &unit, const ButtonAction &button);
 /// Check if all requirements for a single research are meet
 extern bool ButtonCheckSingleResearch(const CUnit &unit, const ButtonAction &button);
+/// Check for button enabled, if requested condition passes check. Used for debug purposes
+extern bool ButtonCheckDebug(const CUnit &unit, const ButtonAction &button);
 
 //
 // in ccl_ui.c
