@@ -59,14 +59,12 @@ public:
 		WithTerrain(false), ShowSelected(false),
 		Transparent(false), UpdateCache(false) {}
 
+	void SetFogOpacityLevels(const uint8_t explored, const uint8_t revealed, const uint8_t unseen);
+
 	void UpdateXY(const Vec2i &pos);
 	void UpdateSeenXY(const Vec2i &) {}
 	void Update();
 	void Create();
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	void FreeOpenGL();
-	void Reload();
-#endif
 	void Destroy();
 	void Draw() const;
 	void DrawViewportArea(const CViewport &viewport) const;
@@ -87,19 +85,21 @@ public:
 	bool ShowSelected;
 	bool Transparent;
 	bool UpdateCache;
+private:
+	struct MinimapSettings
+	{
+		/// used to draw fog in on the minimap
+		uint8_t FogVisibleOpacity	{0x00};
+		uint8_t FogExploredOpacity  {0x55};
+		uint8_t FogRevealedOpacity  {0xAA};
+		uint8_t FogUnseenOpacity 	{0xFF};		
+	} Settings;
 };
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-// Minimap surface with units (for OpenGL)
-extern unsigned char *MinimapSurfaceGL;
-// Minimap surface with terrain only (for OpenGL)
-extern unsigned char *MinimapTerrainSurfaceGL;
-#endif
-
 // Minimap surface with units (for software)
 extern SDL_Surface *MinimapSurface;
 // Minimap surface with terrain only (for software)
-extern SDL_Surface *MinimapTerrainSurface;
+//extern SDL_Surface *MinimapTerrainSurface;
 
 //@}
 
