@@ -51,6 +51,7 @@
 # include <netdb.h>
 # include <sys/socket.h>
 # include <sys/ioctl.h>
+# include <ifaddrs.h>
 # ifndef __BEOS__
 #  include <net/if.h>
 #  include <arpa/inet.h>
@@ -111,8 +112,10 @@ extern void NetExit();
 
 /// Resolve host in name or or colon dot notation.
 extern unsigned long NetResolveHost(const std::string &host);
-/// Get local IP from network file descriptor
-extern int NetSocketAddr(const Socket sock, unsigned long *ips, int maxAddr);
+/// Get local IPs
+extern int NetSocketAddr(unsigned long *ips, int maxAddr);
+/// Get local hostname
+extern std::string NetGetHostname();
 
 /// Open a UDP Socket port. (param in network format)
 extern Socket NetOpenUDP(unsigned long ip, int port);
