@@ -32,6 +32,7 @@
 //@{
 
 #include "net_message.h"
+#include "network/netsockets.h"
 
 class CHost;
 
@@ -80,7 +81,8 @@ enum _net_client_con_state_ {
 	ccs_goahead,              /// Server wants to start game
 	ccs_started,              /// Server has started game
 	ccs_incompatibleengine,   /// Incompatible engine version
-	ccs_incompatibleluafiles  /// Incompatible lua files
+	ccs_incompatibleluafiles, /// Incompatible lua files
+	ccs_needmap,              /// Client needs to be sent the map
 };
 
 /*----------------------------------------------------------------------------
@@ -89,7 +91,6 @@ enum _net_client_con_state_ {
 
 extern int NetPlayers;                /// Network players
 
-extern int HostsCount;                /// Number of hosts.
 extern CNetworkHost Hosts[PlayerMax]; /// Host, port, and number of all players.
 
 extern int NetConnectRunning;              /// Network menu: Setup mode active
@@ -97,9 +98,14 @@ extern int NetConnectType;              /// Network menu: Setup mode active
 extern int NetLocalHostsSlot;              /// Network menu: Slot # in Hosts array of local client
 extern int NetLocalPlayerNumber;           /// Player number of local client
 
+extern std::string NetworkMapName;         /// Name of the map received with ICMMap
+extern std::string NetworkMapFragmentName; /// Name of the map currently loading via ICMMapNeeded
+
 extern CServerSetup ServerSetupState;      /// Network menu: Multiplayer Server Menu selections state
 extern CServerSetup LocalSetupState;       /// Network menu: Multiplayer Client Menu selections local state
 
+extern bool NoRandomPlacementMultiplayer; /// Disable the random placement of players in muliplayer mode
+
 /*----------------------------------------------------------------------------
 --  Functions
 ----------------------------------------------------------------------------*/
@@ -119,6 +125,8 @@ extern void NetworkProcessServerRequest();  /// Menu Loop: Send out server reque
 extern void NetworkServerResyncClients();   /// Menu Loop: Server: Mark clients state to send stateinfo message
 extern void NetworkDetachFromServer();      /// Menu Loop: Client: Send GoodBye to the server and detach
 
+extern void NetworkSendICMessage(CUDPSocket &socket, const CHost &host, const CInitMessage_Header &msg);
+
 //@}
 
 #endif // !__NETCONNECT_H__
