@@ -199,6 +199,8 @@ extern int AStarMovingUnitCrossingCost;
 extern bool AStarKnowUnseenTerrain;
 /// Cost of using a square we haven't seen before.
 extern int AStarUnknownTerrainCost;
+/// Maximum number of iterations of A* before giving up.
+extern int AStarMaxSearchIterations;
 
 //
 //  Convert heading into direction.
@@ -218,11 +220,14 @@ extern void FreePathfinder();
 
 /// Returns the next element of the path
 extern int NextPathElement(CUnit &unit, short int *xdp, short int *ydp);
-/// Return distance to unit.
-extern int UnitReachable(const CUnit &unit, const CUnit &dst, int range);
+/// Return path length to unit 'dst'.
+extern int UnitReachable(const CUnit &src, const CUnit &dst, int range, bool from_outside_container);
+/// Return path length to unit 'dst' or error code.
+extern int CalcPathLengthToUnit(const CUnit &src, const CUnit &dst,
+						  const int minrange, const int range);
 /// Can the unit 'src' reach the place x,y
 extern int PlaceReachable(const CUnit &src, const Vec2i &pos, int w, int h,
-						  int minrange, int maxrange);
+						  int minrange, int maxrange, bool from_outside_container);
 
 //
 // in astar.cpp
@@ -237,6 +242,9 @@ extern int GetAStarMovingUnitCrossingCost();
 extern void SetAStarUnknownTerrainCost(int cost);
 extern int GetAStarUnknownTerrainCost();
 
+extern void SetAStarFixedEnemyUnitsUnpassable(const bool value);
+extern bool GetAStarFixedEnemyUnitsUnpassable();
+
 extern void PathfinderCclRegister();
 
 //@}
