@@ -36,13 +36,17 @@
 --  Includes
 ----------------------------------------------------------------------------*/
 
+#include <set>
 #include <string>
+#include <vector>
 
 #include "color.h"
 #include "upgrade_structs.h"
 
 #include "vec2i.h"
 
+#include "settings.h"
+
 class CGraphic;
 
 /*----------------------------------------------------------------------------
@@ -78,11 +82,33 @@ enum _diplomacy_ {
 ///  Player structure
 class CPlayer
 {
+public:
+	static inline RevealTypes RevelationFor { RevealTypes::cNoRevelation }; /// type of revelation (when player lost their last main facility)
+
+public:
+	/// Check if relevation enabled
+	static const bool IsRevelationEnabled()
+	{
+		// By default there is no revelation. Can be changed in lua-script
+		return CPlayer::RevelationFor != RevealTypes::cNoRevelation; 
+	}
+	/// Change revelation type
+	static void SetRevelationType(const RevealTypes type);
+	/// Get revealed players list
+	static const std::vector<const CPlayer *> &GetRevealedPlayers()
+	{
+		return CPlayer::RevealedPlayers;
+	}
+
+private:
+	/// List of players revealed after losing their last Town Hall
+	static inline std::vector<const CPlayer *> RevealedPlayers;	
+	
 public:
 	int Index;          /// player as number
 	std::string Name;   /// name of non computer
 
-	int   Type;         /// type of player (human,computer,...)
+	PlayerTypes Type;   /// type of player (human,computer,...)
 	int   Race;         /// race of player (orc,human,...)
 	std::string AiName; /// AI for computer
 
@@ -128,13 +154,12 @@ public:
 	int    TotalResources[MaxCosts];
 	int    TotalRazings;
 	int    TotalKills;      /// How many unit killed
-
+	
+	int 	LostMainFacilityTimer { 0 };/// The timer for when the player lost the last town hall 
+									/// (to make the player's units be revealed)
+	
 	IntColor Color;           /// color of units on minimap
 
-	CUnitColors UnitColors; /// Unit colors for new units
-
-	std::vector<CUnit *> FreeWorkers;	/// Container for free workers
-
 	// Upgrades/Allows:
 	CAllow Allow;                 /// Allowed for player
 	CUpgradeTimers UpgradeTimers; /// Timer for the upgrades
@@ -149,14 +174,28 @@ public:
 	std::vector<CUnit *>::iterator UnitBegin();
 	std::vector<CUnit *>::const_iterator UnitEnd() const;
 	std::vector<CUnit *>::iterator UnitEnd();
-
+	
+	const std::vector<CUnit *> &GetUnits() const {
+		return this->Units;
+	}
 	CUnit &GetUnit(int index) const;
 	int GetUnitCount() const;
 
 	void AddUnit(CUnit &unit);
 	void RemoveUnit(CUnit &unit);
+
+	std::vector<CUnit *>::const_iterator FreeWorkersBegin() const;
+	std::vector<CUnit *>::const_iterator FreeWorkersEnd() const;
+	std::vector<CUnit *>::iterator FreeWorkersBegin();
+	std::vector<CUnit *>::iterator FreeWorkersEnd();
+
+	CUnit *GetFreeWorker(int index) const;
+	int GetFreeWorkersCount() const;
 	void UpdateFreeWorkers();
 
+	void ClearUnitColors();
+	void SetUnitColors(std::vector<CColor> &colors);
+
 	/// Get a resource of the player
 	int GetResource(const int resource, const int type);
 	/// Adds/subtracts some resources to/from the player store
@@ -208,15 +247,31 @@ public:
 		return (Index != index && (Enemy & (1 << index)) != 0);
 	}
 
+	/**
+	**  Check if the player index is an enemy
+	*/
+	bool IsAllied(const int index) const
+	{
+		return (Index != index && (Allied & (1 << index)) != 0);
+	}
+
 	bool IsEnemy(const CPlayer &player) const;
 	bool IsEnemy(const CUnit &unit) const;
 	bool IsAllied(const CPlayer &player) const;
 	bool IsAllied(const CUnit &unit) const;
 	bool IsVisionSharing() const;
-	bool IsSharedVision(const CPlayer &player) const;
-	bool IsSharedVision(const CUnit &unit) const;
-	bool IsBothSharedVision(const CPlayer &player) const;
-	bool IsBothSharedVision(const CUnit &unit) const;
+	const std::set<uint8_t> &GetSharedVision() const
+	{
+		return this->HasVisionFrom;
+	}
+	const std::set<uint8_t> &GetGaveVisionTo() const
+	{
+		return this->GaveVisionTo;
+	}
+	bool HasSharedVisionWith(const CPlayer &player) const
+	{
+		return (this->GaveVisionTo.find(player.Index) != this->GaveVisionTo.end());
+	}
 	bool IsTeamed(const CPlayer &player) const;
 	bool IsTeamed(const CUnit &unit) const;
 
@@ -225,18 +280,33 @@ public:
 	void SetDiplomacyEnemyWith(const CPlayer &player);
 	void SetDiplomacyCrazyWith(const CPlayer &player);
 
-	void ShareVisionWith(const CPlayer &player);
-	void UnshareVisionWith(const CPlayer &player);
+	void ShareVisionWith(CPlayer &player);
+	void EnableSharedVisionFrom(const CPlayer &player);
+	void UnshareVisionWith(CPlayer &player);
+	void DisableSharedVisionFrom(const CPlayer &player);
 
-	void Init(/* PlayerTypes */ int type);
+	void Init(PlayerTypes type);
 	void Save(CFile &file) const;
 	void Load(lua_State *l);
+	
+	bool IsRevealed() const
+	{
+		return this->isRevealed;
+	}
+	void SetRevealed(const bool revealed);
 
 private:
-	std::vector<CUnit *> Units; /// units of this player
-	unsigned int Enemy;         /// enemy bit field for this player
-	unsigned int Allied;        /// allied bit field for this player
-	unsigned int SharedVision;  /// shared vision bit field
+	CUnitColors UnitColors;            /// Unit colors for new units
+	std::vector<CUnit *> Units;        /// units of this player
+	std::vector<CUnit *> FreeWorkers;  /// Container for free workers
+	unsigned int Enemy;                /// enemy bit field for this player
+	unsigned int Allied;               /// allied bit field for this player
+	std::set<uint8_t> HasVisionFrom;   /// set of player indexes that have shared their vision with this player
+	std::set<uint8_t> GaveVisionTo;    /// set of player indexes that this player has shared vision with
+
+	bool isRevealed { false }; 	/// whether the player has been revealed (i.e. after losing the last Town Hall)
+
+	friend void CleanPlayers();
 };
 
 /**
@@ -261,61 +331,6 @@ public:
 	unsigned int Count;             /// number of races
 };
 
-
-enum PlayerRacesOld {
-	PlayerRaceHuman = 0,  /// belongs to human
-	PlayerRaceOrc  = 1    /// belongs to orc
-};
-
-/**
-**  Types for the player
-**
-**  #PlayerNeutral
-**
-**    This player is controlled by the computer doing nothing.
-**
-**  #PlayerNobody
-**
-**    This player is unused. Nobody controls this player.
-**
-**  #PlayerComputer
-**
-**    This player is controlled by the computer. CPlayer::AiNum
-**    selects the AI strategy.
-**
-**  #PlayerPerson
-**
-**    This player is contolled by a person. This can be the player
-**    sitting on the local computer or player playing over the
-**    network.
-**
-**  #PlayerRescuePassive
-**
-**    This player does nothing, the game pieces just sit in the game
-**    (being passive)... when a person player moves next to a
-**    PassiveRescue unit/building, then it is "rescued" and becomes
-**    part of that persons team. If the city center is rescued, than
-**    all units of this player are rescued.
-**
-**  #PlayerRescueActive
-**
-**    This player is controlled by the computer. CPlayer::AiNum
-**    selects the AI strategy. Until it is rescued it plays like
-**    an ally. The first person which reaches units of this player,
-**    can rescue them. If the city center is rescued, than all units
-**    of this player are rescued.
-*/
-enum PlayerTypes {
-	PlayerNeutral = 2,        /// neutral
-	PlayerNobody  = 3,        /// unused slot
-	PlayerComputer = 4,       /// computer player
-	PlayerPerson = 5,         /// human player
-	PlayerRescuePassive = 6,  /// rescued passive
-	PlayerRescueActive = 7    /// rescued  active
-};
-
-#define PlayerNumNeutral (PlayerMax - 1)  /// this is the neutral player slot
-
 /**
 **  Notify types. Noties are send to the player.
 */
@@ -333,9 +348,9 @@ extern int NumPlayers;             /// How many player slots used
 extern CPlayer Players[PlayerMax];  /// All players
 extern CPlayer *ThisPlayer;         /// Player on local computer
 extern bool NoRescueCheck;          /// Disable rescue check
-extern std::vector<CColor> PlayerColorsRGB[PlayerMax]; /// Player colors
-extern std::vector<IntColor> PlayerColors[PlayerMax]; /// Player colors
-extern std::string PlayerColorNames[PlayerMax];  /// Player color names
+extern std::vector<std::vector<CColor>> PlayerColorsRGB; /// Player colors
+extern std::vector<std::vector<SDL_Color>> PlayerColorsSDL; /// Player colors
+extern std::vector<std::string> PlayerColorNames;  /// Player color names
 
 extern PlayerRace PlayerRaces;  /// Player races
 
@@ -357,7 +372,7 @@ extern void CleanPlayers();
 extern void SavePlayers(CFile &file);
 
 /// Create a new player
-extern void CreatePlayer(int type);
+extern void CreatePlayer(PlayerTypes type);
 
 
 /// Initialize the computer opponent AI
@@ -367,8 +382,8 @@ extern void PlayersEachCycle();
 /// Called each second for a given player handler (AI)
 extern void PlayersEachSecond(int player);
 
-/// Change current color set to new player of the sprite
-extern void GraphicPlayerPixels(CPlayer &player, const CGraphic &sprite);
+/// Change current color set to the player color of the sprite
+extern void GraphicPlayerPixels(int colorIndex, const CGraphic &sprite);
 
 /// Output debug information for players
 extern void DebugPlayers();
