@@ -37,12 +37,15 @@
 ----------------------------------------------------------------------------*/
 
 #include "vec2i.h"
+#include <string>
 #include <vector>
 
 struct lua_State;
 
 // Not used until now:
-#define MapFieldSpeedMask 0x0007  /// Move faster on this tile
+#define MapFieldSpeedMask 0x0003  /// Move faster on this tile
+
+#define MapFieldOpaque 0x0004  /// Units can't see through this field (FoW)
 
 #define MapFieldHuman 0x0008  /// Human is owner of the field (walls)
 
@@ -54,13 +57,18 @@ struct lua_State;
 #define MapFieldUnpassable 0x0100  /// Field is movement blocked
 #define MapFieldWall       0x0200  /// Field contains wall
 #define MapFieldRocks      0x0400  /// Field contains rocks
-#define MapFieldForest     0x0800  /// Field contains forest
+#define MapFieldForest     0x0800  /// Field contains forest or other harvestable resource
 
 #define MapFieldLandUnit 0x1000  /// Land unit on field
 #define MapFieldAirUnit  0x2000  /// Air unit on field
 #define MapFieldSeaUnit  0x4000  /// Water unit on field
 #define MapFieldBuilding 0x8000  /// Building on field
 
+#define MapFieldDecorative 0x10000  /// A field that needs no mixing with the surroundings, for the editor
+#define MapFieldCost4 (0x20000 | MapFieldForest)  		/// This field is terrain harvestable, but gives Cost4 instead of wood
+#define MapFieldCost5 (0x40000 | MapFieldForest)  		/// This field is terrain harvestable, but gives Cost5 instead of wood
+#define MapFieldCost6 (0x80000 | MapFieldForest)  		/// This field is terrain harvestable, but gives Cost6 instead of wood
+
 /**
 **  These are used for lookup tiles types
 **  mainly used for the FOW implementation of the seen woods/rocks
@@ -110,7 +118,7 @@ public:
 
 public:
 	unsigned short tile;  /// graphical pos
-	unsigned short flag;  /// Flag
+	unsigned int flag;    /// Flag
 	CTileInfo tileinfo;   /// Tile descriptions
 };
 
@@ -123,6 +131,7 @@ public:
 	unsigned int getTileCount() const { return tiles.size(); }
 
 	unsigned int getDefaultTileIndex() const;
+	unsigned int getDefaultWoodTileIndex() const;
 
 	bool isAWallTile(unsigned tile) const;
 	bool isARaceWallTile(unsigned tile, bool human) const;
@@ -163,10 +172,11 @@ public:
 
 	void parse(lua_State *l);
 	void buildTable(lua_State *l);
+	int parseTilesetTileFlags(lua_State *l, int *back, int *j);
+	int findTileIndex(unsigned char baseTerrain, unsigned char mixTerrain = 0) const;
 
 private:
 	unsigned int getOrAddSolidTileIndexByName(const std::string &name);
-	int findTileIndex(unsigned char baseTerrain, unsigned char mixTerrain = 0) const;
 	int getTileIndex(unsigned char baseTerrain, unsigned char mixTerrain, unsigned int quad) const;
 	void buildWallReplacementTable();
 	void parseSlots(lua_State *l, int t);
