@@ -169,10 +169,10 @@ public:
 	bool Contains(const PixelPos &screenPos) const;
 
 public:
-	int X;                          /// Screen pixel left corner x coordinate
-	int Y;                          /// Screen pixel upper corner y coordinate
-	int EndX;                       /// Screen pixel right x coordinate
-	int EndY;                       /// Screen pixel bottom y coordinate
+	int X;                          /// Screen pixel left corner x coordinate adjusted for current map size
+	int Y;                          /// Screen pixel upper corner y coordinate adjusted for current map size 
+	int EndX;                       /// Screen pixel right x coordinate adjusted for current map size
+	int EndY;                       /// Screen pixel bottom y coordinate adjusted for current map size
 	int ScrollPaddingLeft;          /// Scrollable area past the left of map
 	int ScrollPaddingRight;         /// Scrollable area past the right of map
 	int ScrollPaddingTop;           /// Scrollable area past the top of map
@@ -288,6 +288,7 @@ private:
 	void DoClicked_Unload(int button);
 	void DoClicked_SpellCast(int button);
 	void DoClicked_Repair(int button);
+	void DoClicked_Explore();
 	void DoClicked_Return();
 	void DoClicked_Stop();
 	void DoClicked_StandGround();
@@ -299,7 +300,7 @@ private:
 	void DoClicked_Train(int button);
 	void DoClicked_UpgradeTo(int button);
 	void DoClicked_Research(int button);
-	void DoClicked_CallbackAction(int button);
+	void DoClicked_CallbackAction(int button, int clickingPlayer);
 
 
 public:
@@ -407,7 +408,7 @@ public:
 
 	CInfoPanel InfoPanel;               /// Info panel
 	std::vector<CUnitInfoPanel *> InfoPanelContents;/// Info panel contents
-
+	std::string DefaultUnitPortrait;    /// Name of the unit portrait to show when there is no selection
 	std::vector<CPopup *> ButtonPopups; /// Popup windows for buttons
 
 	CUIButton *SingleSelectedButton;    /// Button for single selected unit
@@ -435,6 +436,13 @@ public:
 
 	std::vector<CUIButton> TransportingButtons;/// Button info for transporting
 
+        std::vector<std::string> LifeBarColorNames;
+        std::vector<int> LifeBarPercents;
+        std::vector<IntColor> LifeBarColorsInt;
+        int8_t LifeBarYOffset;
+		int8_t LifeBarPadding;
+        bool LifeBarBorder;
+
 	// Completed bar
 	CColor CompletedBarColorRGB;     /// color for completed bar
 	IntColor CompletedBarColor;      /// color for completed bar
@@ -474,6 +482,12 @@ public:
 	// Game timer
 	CUITimer Timer;                     /// game timer
 
+	// Offsets used by editor
+	Vec2i EditorSettingsAreaTopLeft;
+	Vec2i EditorSettingsAreaBottomRight;
+	Vec2i EditorButtonAreaTopLeft;
+	Vec2i EditorButtonAreaBottomRight;
+
 	// Offsets for 640x480 center used by menus
 	int Offset640X;                     /// Offset for 640x480 X position
 	int Offset480Y;                     /// Offset for 640x480 Y position
@@ -597,6 +611,8 @@ extern void CallHandler(unsigned int handle, int value);
 
 /// Show load progress
 extern void ShowLoadProgress(const char *fmt, ...) PRINTF_VAARG_ATTRIBUTE(1, 2);
+/// Check if Demo/Attract mode is in progress
+extern bool IsDemoMode();
 
 //@}
 
