@@ -36,6 +36,7 @@
 --  Includes
 ----------------------------------------------------------------------------*/
 
+#include "settings.h"
 #include <vector>
 
 #ifndef __UNITTYPE_H__
@@ -76,15 +77,30 @@ typedef COrder *COrderPtr;
 
 /*
 ** Configuration of the small (unit) AI.
+** sPPP PPdd dddd dddd 0000 0000 0hhh hhhh
+** s... .ppp pppp p... .... .... .... ....
+** s... ...I .... .... iiii i... .... ....
+** s... .... .... c... .... .... .... ....
 */
-#define PRIORITY_FACTOR   0x00080000
-#define HEALTH_FACTOR     0x00000001
-#define DISTANCE_FACTOR   0x00010000
-#define INRANGE_FACTOR    0x00008000
-#define INRANGE_BONUS     0x01000000
-#define CANATTACK_BONUS   0x00080000
-#define AIPRIORITY_BONUS  0x04000000
+#define PRIORITY_FACTOR   0x00080000   /// p
+#define HEALTH_FACTOR     0x00000001   /// h (0..100)%
+#define DISTANCE_FACTOR   0x00010000   /// d (0..1023)
+#define INRANGE_FACTOR    0x00008000   /// i (0..31)
+#define INRANGE_BONUS     0x01000000   /// I
+#define CANATTACK_BONUS   0x00080000   /// c
+#define AIPRIORITY_BONUS  0x04000000   /// P (0..31)
 
+/*
+** Same for alternate (simplified) implementation of the small (unit) AI.
+** sAT0 0000  0ppp pppp  pddd dddd  dhhh hhhh
+*/
+#define AT_ATTACKED_BY_FACTOR 0x40000000 /// A (attacker is under attack by target)
+#define AT_THREAT_FACTOR      0x20000000 /// T
+#define AT_PRIORITY_OFFSET    15         /// p (0..255)
+#define AT_DISTANCE_OFFSET    7          /// d (0..255) 
+#define AT_PRIORITY_MASK_HI   0xFFFF8000 /// Mask for checking only priority (without distance part)
+
+#define AT_FARAWAY_REDUCE_OFFSET 14      /// Priority reduce offset for far away targets (AT_THREAT_FACTOR must be preserved if present)
 
 /// Called whenever the selected unit was updated
 extern void SelectedUnitChanged();
@@ -119,7 +135,9 @@ enum _directions_ {
 class CUnit
 {
 public:
-	CUnit() : tilePos(-1, -1), pathFinderData(NULL), SavedOrder(NULL), NewOrder(NULL), CriticalOrder(NULL) { Init(); }
+	CUnit() : tilePos(-1, -1), pathFinderData(NULL), SavedOrder(NULL), NewOrder(NULL), CriticalOrder(NULL), Colors(-1),
+				AutoCastSpell(NULL), SpellCoolDownTimers(NULL), Variable(NULL) { Init(); }
+	~CUnit();
 
 	void Init();
 
@@ -177,8 +195,8 @@ public:
 
 	inline bool IsInvisibile(const CPlayer &player) const
 	{
-		return (&player != Player && !!Variable[INVISIBLE_INDEX].Value
-				&& !player.IsBothSharedVision(*Player));
+		return (&player != Player && Variable[INVISIBLE_INDEX].Value > 0
+				&& !Player->HasSharedVisionWith(player));
 	}
 
 	/**
@@ -217,7 +235,7 @@ public:
 		if (this->Type->BoolFlag[REVEALER_INDEX].value) {
 			return false;
 		}
-		if ((player.Type == PlayerComputer && !this->Type->BoolFlag[PERMANENTCLOAK_INDEX].value)
+		if ((player.Type == PlayerTypes::PlayerComputer && !this->Type->BoolFlag[PERMANENTCLOAK_INDEX].value)
 			|| IsVisible(player) || IsVisibleOnRadar(player)) {
 			return IsAliveOnMap();
 		} else {
@@ -253,10 +271,6 @@ public:
 	bool IsEnemy(const CUnit &unit) const;
 	bool IsAllied(const CPlayer &player) const;
 	bool IsAllied(const CUnit &unit) const;
-	bool IsSharedVision(const CPlayer &player) const;
-	bool IsSharedVision(const CUnit &unit) const;
-	bool IsBothSharedVision(const CPlayer &player) const;
-	bool IsBothSharedVision(const CUnit &unit) const;
 	bool IsTeamed(const CPlayer &player) const;
 	bool IsTeamed(const CUnit &unit) const;
 
@@ -338,7 +352,7 @@ public:
 
 	// DISPLAY:
 	int         Frame;      /// Image frame: <0 is mirrored
-	CUnitColors *Colors;    /// Player colors
+	int  Colors;            /// custom colors
 	bool IndividualUpgrades[UpgradeMax];      /// individual upgrades which the unit has
 
 	signed char IX;         /// X image displacement to map position
@@ -349,6 +363,7 @@ public:
 
 	unsigned char DamagedType;   /// Index of damage type of unit which damaged this unit
 	unsigned long Attacked;      /// gamecycle unit was last attacked
+	unsigned long Summoned;      /// GameCycle unit was summoned using spells
 	unsigned Blink : 3;          /// Let selection rectangle blink
 	unsigned Moving : 1;         /// The unit is moving
 	unsigned ReCast : 1;         /// Recast again next cycle
@@ -364,7 +379,6 @@ public:
 	unsigned Boarded : 1;        /// Unit is on board a transporter.
 	unsigned CacheLock : 1;      /// Unit is on lock by unitcache operations.
 
-	unsigned Summoned : 1;       /// Unit is summoned using spells.
 	unsigned Waiting : 1;        /// Unit is waiting and playing its still animation
 	unsigned MineLow : 1;        /// This mine got a notification about its resources being low
 
@@ -396,6 +410,7 @@ unsigned    ByPlayer : PlayerMax;   /// Track unit seen by player
 
 	unsigned int Wait;          /// action counter
 	int Threshold;              /// The counter while ai unit couldn't change target.
+	int UnderAttack;			/// The counter while small ai can ignore non aggressive targets if searching attacker.
 
 	struct _unit_anim_ {
 		const CAnimation *Anim;      /// Anim
@@ -430,36 +445,63 @@ class CPreference
 {
 public:
 	CPreference() : ShowSightRange(false), ShowReactionRange(false),
-		ShowAttackRange(false), ShowMessages(true), BigScreen(false),
-		PauseOnLeave(true), AiExplores(true), GrayscaleIcons(false),
+		ShowAttackRange(false), ShowMessages(true), ShowNoSelectionStats(true), BigScreen(false),
+		PauseOnLeave(true), GrayscaleIcons(false),
 		IconsShift(false), StereoSound(true), MineNotifications(false),
 		DeselectInMine(false), NoStatusLineTooltips(false),
-		IconFrameG(NULL), PressedIconFrameG(NULL),
+		SelectionRectangleIndicatesDamage(false),
+		IconFrameG(NULL), PressedIconFrameG(NULL), HardwareCursor(false),
 		ShowOrders(0), ShowNameDelay(0), ShowNameTime(0), AutosaveMinutes(5) {};
 
 	bool ShowSightRange;       /// Show sight range.
 	bool ShowReactionRange;    /// Show reaction range.
 	bool ShowAttackRange;      /// Show attack range.
 	bool ShowMessages;		   /// Show messages.
+	bool ShowNoSelectionStats; /// Show stats when no selection is active
 	bool BigScreen;			   /// If true, shows the big screen(without panels)
 	bool PauseOnLeave;         /// If true, game pauses when cursor is gone
-	bool AiExplores;           /// If true, AI sends explorers to search for resources (almost useless thing)
 	bool GrayscaleIcons;       /// Use grayscaled icons for unavailable units, upgrades, etc
 	bool IconsShift;           /// Shift icons slightly when you press on them
 	bool StereoSound;          /// Enables/disables stereo sound effects
 	bool MineNotifications;    /// Show mine is running low/depleted messages
 	bool DeselectInMine;       /// Deselect peasants in mines
 	bool NoStatusLineTooltips; /// Don't show messages on status line
+	bool HardwareCursor;    /// If true, uses the hardware to draw the cursor. Shaders do no longer apply to the cursor, but this way it's decoupled from the game refresh rate
+	bool SelectionRectangleIndicatesDamage; /// If true, the selection rectangle interpolates color to indicate damage
+
+	int FrameSkip;          /// Mask used to skip rendering frames (useful for slow renderers that keep up with the game logic, but not the rendering to screen like e.g. original Raspberry Pi)
 
 	int ShowOrders;			/// How many second show orders of unit on map.
 	int ShowNameDelay;		/// How many cycles need to wait until unit's name popup will appear.
 	int ShowNameTime;		/// How many cycles need to show unit's name popup.
 	int AutosaveMinutes;	/// Autosave the game every X minutes; autosave is disabled if the value is 0
-
-	std::string SF2Soundfont;/// Path to SF2 soundfont
-
 	CGraphic *IconFrameG;
 	CGraphic *PressedIconFrameG;
+
+	// these are "preferences" in the sense that the user wants to set these
+	// persistently across launches for single player games. However, they are
+	// relevant for network sync and replays, so the actually relevant storage
+	// is in GameSettings, and we just have properties for the lua code here and
+	// to initialize GameSettings to the user preferred defaults
+	void InitializeSettingsFromPreferences(Settings &s) {
+		s.AiExplores = AiExplores;
+		s.SimplifiedAutoTargeting = SimplifiedAutoTargeting;
+		s.AiChecksDependencies = AiChecksDependencies;
+	}
+private:
+	bool AiExplores = true;
+	bool SimplifiedAutoTargeting = false;
+	bool AiChecksDependencies = false;
+
+#if USING_TOLUAPP
+public:
+	bool get_AiExplores() { return AiExplores; }
+	void set_AiExplores(bool v) { AiExplores = GameSettings.AiExplores = v; }
+	bool get_SimplifiedAutoTargeting() { return SimplifiedAutoTargeting; }
+	void set_SimplifiedAutoTargeting(bool v) { SimplifiedAutoTargeting = GameSettings.SimplifiedAutoTargeting = v; }
+	bool get_AiChecksDependencies() { return AiChecksDependencies; }
+	void set_AiChecksDependencies(bool v) { AiChecksDependencies = GameSettings.AiChecksDependencies = v; }
+#endif
 };
 
 extern CPreference Preference;
@@ -566,6 +608,20 @@ extern void LetUnitDie(CUnit &unit, bool suicide = false);
 extern void DestroyAllInside(CUnit &source);
 /// Calculate some value to measure the unit's priority for AI
 extern int ThreatCalculate(const CUnit &unit, const CUnit &dest);
+extern int TargetPriorityCalculate(const CUnit *const attacker, const CUnit *const dest);
+
+/// Is target within reaction range of this unit?
+extern bool InReactRange(const CUnit &unit, const CUnit &target);
+/// Is target within attack range of this unit?
+extern bool InAttackRange(const CUnit &unit, const CUnit &target);
+/// Is tile within attack range of this unit?
+extern bool InAttackRange(const CUnit &unit, const Vec2i &tilePos);
+/// Return randomly selected position in direction (to/from) dirUnit from srcPos 
+extern Vec2i GetRndPosInDirection(const Vec2i &srcPos, const CUnit &dirUnit, const bool dirFrom, const int minRange, const int devRadius, const int rangeDev = 3);
+/// Return randomly selected position in direction (to/from) dirPos from srcPos 
+extern Vec2i GetRndPosInDirection(const Vec2i &srcPos, const Vec2i &dirPos, const bool dirFrom, const int minRange, const int devRadius, const int rangeDev = 3);
+
+
 /// Hit unit with damage, if destroyed give attacker the points
 extern void HitUnit(CUnit *attacker, CUnit &target, int damage, const Missile *missile = NULL);
 
@@ -606,6 +662,8 @@ extern void DrawSelectionRectangle(IntColor, int, int, int, int);
 extern void DrawSelectionRectangleWithTrans(IntColor, int, int, int, int);
 /// Draw corners around unit
 extern void DrawSelectionCorners(IntColor, int, int, int, int);
+/// Draw ellipse around unit
+extern void (*DrawSelectionEllipse(float factor))(IntColor, int, int, int, int);
 
 /// Register CCL decorations features
 extern void DecorationCclRegister();
