@@ -100,7 +100,7 @@ public:
 	unsigned WaitAtDepot;           /// Cycles the unit waits while returning.
 	unsigned ResourceId;            /// Id of the resource harvested. Redundant.
 	unsigned FinalResource;         /// Convert resource when delivered.
-	unsigned char TerrainHarvester;    /// Unit will harvest terrain(wood only for now).
+	unsigned char TerrainHarvester;    /// Unit will harvest terrain.
 	unsigned char LoseResources;       /// The unit will lose it's resource when distracted.
 	unsigned char HarvestFromOutside;  /// Unit harvests without entering the building.
 	unsigned char RefineryHarvester;   /// Unit have to build Refinery buildings for harvesting.
@@ -118,13 +118,14 @@ public:
 class CVariable
 {
 public:
-	CVariable() : Max(0), Value(0), Increase(0), Enable(0) {}
+	CVariable() : Max(0), Value(0), Increase(0), IncreaseFrequency(0), Enable(0) {}
 
 	bool operator ==(const CVariable &rhs) const
 	{
 		return this->Max == rhs.Max
 			   && this->Value == rhs.Value
 			   && this->Increase == rhs.Increase
+			   && this->IncreaseFrequency == rhs.IncreaseFrequency
 			   && this->Enable == rhs.Enable;
 	}
 	bool operator !=(const CVariable &rhs) const { return !(*this == rhs); }
@@ -133,7 +134,8 @@ public:
 	int Max;        /// Maximum for the variable. (Assume min is 0.)
 	int Value;      /// Current (or initial) value of the variable (or initial value).
 	char Increase;  /// Number to increase(decrease) Value by second.
-	char Enable;    /// True if the unit doesn't have this variable. (f.e shield)
+	unsigned char IncreaseFrequency:7;    /// Every how many seconds we should apply the increase
+	unsigned char Enable:1;    /// True if the unit doesn't have this variable. (f.e shield)
 };
 
 // Index for boolflag already defined
@@ -170,7 +172,11 @@ enum {
 	NORANDOMPLACING_INDEX,			/// Don't use random frame rotation
 	ORGANIC_INDEX,					/// Organic unit (used for death coil spell)
 	SIDEATTACK_INDEX,
+	SKIRMISHER_INDEX,
+	ALWAYSTHREAT_INDEX,				/// Unit always considered as threat for auto targeting algorihm, useful for unit without main attack ability, but which can cast spells (f.e. defiler in SC:BW)
+	ELEVATED_INDEX,					/// Unit is elevated and can see over opaque tiles placed in the same ground level with the unit.
 	NOFRIENDLYFIRE_INDEX,           /// Unit accepts friendly fire for splash attacks
+	MAINFACILITY_INDEX,				/// Unit is a main building (Town Hall f. ex.)
 	NBARALREADYDEFINED
 };
 
@@ -196,6 +202,8 @@ enum {
 	BASICDAMAGE_INDEX,
 	POSX_INDEX,
 	POSY_INDEX,
+	POS_RIGHT_INDEX,
+	POS_BOTTOM_INDEX,
 	TARGETPOSX_INDEX,
 	TARGETPOSY_INDEX,
 	RADAR_INDEX,
@@ -241,6 +249,8 @@ public:
 	/// function to draw the decorations.
 	virtual void Draw(int x, int y, const CUnitType &type, const CVariable &var) const = 0;
 
+	bool BoolFlagMatches(const CUnitType &type) const;
+
 	unsigned int Index;     /// Index of the variable. @see DefineVariables
 
 	int OffsetX;            /// Offset in X coord.
@@ -261,12 +271,15 @@ public:
 	bool HideNeutral;       /// if true, don't show for neutral unit.
 	bool HideAllied;        /// if true, don't show for allied unit. (but show own units)
 	bool ShowOpponent;      /// if true, show for opponent unit.
+
+	bool BoolFlagInvert;    /// if 1, invert the bool flag check
+	int BoolFlag;           /// if !=-1, show only for units with this flag
 };
 
 class CDecoVarBar : public CDecoVar
 {
 public:
-	/// function to draw the decorations.
+	CDecoVarBar() : MinValue(0), MaxValue(100), Invert(false) {};
 	virtual void Draw(int x, int y, const CUnitType &type, const CVariable &var) const;
 
 	bool IsVertical;            /// if true, vertical bar, else horizontal.
@@ -274,12 +287,25 @@ public:
 	int Height;                 /// Height of the bar.
 	int Width;                  /// Width of the bar.
 	bool ShowFullBackground;    /// if true, show background like value equal to max.
+	bool Invert;                /// if true, invert length
+	int MinValue;               /// show only above percent
+	int MaxValue;               /// show only below percent
 	char BorderSize;            /// Size of the border, 0 for no border.
 	// FIXME color depend of percent (red, Orange, Yellow, Green...)
 	IntColor Color;             /// Color of bar.
 	IntColor BColor;            /// Color of background.
 };
 
+class CDecoVarFrame : public CDecoVar
+{
+public:
+	CDecoVarFrame() : Thickness(0), ColorIndex(-1) {};
+	virtual void Draw(int x, int y, const CUnitType &type, const CVariable &var) const;
+
+	int Thickness;
+	int ColorIndex;
+};
+
 class CDecoVarText : public CDecoVar
 {
 public:
@@ -318,6 +344,21 @@ public:
 	int FadeValue; /// if variable's value is below than FadeValue, it drawn transparent.
 };
 
+/// use to show specific frame in a sprite.
+class CDecoVarAnimatedSprite : public CDecoVar
+{
+public:
+	CDecoVarAnimatedSprite() : NSprite(-1), n(0), WaitFrames(0) {}
+	/// function to draw the decorations.
+	virtual void Draw(int x, int y, const CUnitType &type, const CVariable &var) const;
+
+	char NSprite;    /// Index of sprite. (@see DefineSprites and @see GetSpriteIndex)
+	char WaitFrames; /// Frames to wait between each sprite animation step
+private:
+	char lastFrame; /// last update
+	int n;         /// identifiant in SpellSprite
+};
+
 enum UnitTypeType {
 	UnitTypeLand,  /// Unit lives on land
 	UnitTypeFly,   /// Unit lives in air
@@ -484,6 +525,7 @@ public:
 	std::string Name;               /// Pretty name shown from the engine
 	int Slot;                       /// Type as number
 	std::string File;               /// Sprite files
+	std::string AltFile;            /// Alternative sprite files
 	std::string ShadowFile;         /// Shadow file
 
 	int Width;                                            /// Sprite width
@@ -495,6 +537,9 @@ public:
 	int ShadowHeight;                                     /// Shadow sprite height
 	int ShadowOffsetX;                                    /// Shadow horizontal offset
 	int ShadowOffsetY;                                    /// Shadow vertical offset
+	char ShadowScale;                                     /// Shadow scale-down factor
+	char ShadowSpriteFrame;                               /// If > 0, the shadow is a simple sprite without
+	                                                      /// directions and this selects which frame to use
 	PixelPos MissileOffsets[UnitSides][MaxAttackPos];     /// Attack offsets for missiles
 
 	CAnimations *Animations;        /// Animation scripts
@@ -505,6 +550,7 @@ public:
 	struct _portrait_ {
 		std::string *Files;
 		int Num;
+		int Talking; /// offset into portraits for talking portraits
 		Mng **Mngs;
 		mutable int CurrMng;
 		mutable int NumIterations;
@@ -514,11 +560,12 @@ public:
 	MissileConfig Explosion;                         /// Missile for unit explosion
 	MissileConfig Impact[ANIMATIONS_DEATHTYPES + 2]; /// Missiles spawned if unit is hit(+shield)
 
-	LuaCallback *DeathExplosion;
+	LuaCallback *OnDeath;           /// lua function called when unit is about to die, receives x,y,unit
 	LuaCallback *OnHit;             /// lua function called when unit is hit
 	LuaCallback *OnEachCycle;       /// lua function called every cycle
 	LuaCallback *OnEachSecond;      /// lua function called every second
 	LuaCallback *OnInit;            /// lua function called on unit init
+	LuaCallback *OnReady;           /// lua function called when unit ready/built
 
 	int TeleportCost;               /// mana used for teleportation
 	LuaCallback *TeleportEffectIn;   /// lua function to create effects before teleportation
@@ -586,6 +633,7 @@ public:
 	unsigned Neutral : 1;           /// Unit is neutral, used by the editor
 
 	unsigned SideAttack : 1;            /// Unit turns for attack (used for ships)
+	unsigned Skirmisher : 1;            /// Unit will try to shoot from max range
 
 	CUnitStats DefaultStat;
 	CUnitStats MapDefaultStat;
@@ -618,6 +666,7 @@ public:
 	CUnitStats Stats[PlayerMax];     /// Unit status for each player
 
 	CPlayerColorGraphic *Sprite;     /// Sprite images
+	CPlayerColorGraphic *AltSprite;  /// Alternative sprite images
 	CGraphic *ShadowSprite;          /// Shadow sprite image
 };
 
@@ -768,7 +817,7 @@ extern void SaveUnitTypes(CFile &file);              /// Save the unit-type tabl
 extern CUnitType *NewUnitTypeSlot(const std::string &ident);/// Allocate an empty unit-type slot
 /// Draw the sprite frame of unit-type
 extern void DrawUnitType(const CUnitType &type, CPlayerColorGraphic *sprite,
-						 int player, int frame, const PixelPos &screenPos);
+						 int colorIndex, int frame, const PixelPos &screenPos);
 
 extern void InitUnitTypes(int reset_player_stats);   /// Init unit-type table
 extern void LoadUnitTypeSprite(CUnitType &unittype); /// Load the sprite for a unittype
