@@ -32,13 +32,6 @@
 
 //@{
 
-#ifdef USE_WIN32
-#undef NOUSER
-#ifndef _WIN32_WINNT
-#define _WIN32_WINNT 0x0400
-#endif
-#endif
-
 #include <cstdlib>
 #include <cstdint>
 #include <string>
@@ -140,6 +133,7 @@ int getopt(int argc, char *const argv[], const char *optstring);
 #include <string>
 
 int GetClipboard(std::string &str);
+void SetClipboard(std::string &str);
 
 /*----------------------------------------------------------------------------
 --  UTF8
@@ -148,6 +142,14 @@ int GetClipboard(std::string &str);
 int UTF8GetNext(const std::string &text, int curpos);
 int UTF8GetPrev(const std::string &text, int curpos);
 
+/*----------------------------------------------------------------------------
+--  SIMD support
+----------------------------------------------------------------------------*/
+bool supportsSSE2();
+bool supportsAVX();
+void *aligned_malloc(size_t alignment, size_t size);
+void aligned_free(void *block);
+
 //@}
 
 #endif /* __UTIL_H__ */
