@@ -34,19 +34,7 @@
 //@{
 
 #include "SDL.h"
-
-#ifdef USE_GLES
-#include "GLES/gl.h"
-#endif
-
-#ifdef USE_OPENGL
-#ifdef __APPLE__
-#define GL_GLEXT_PROTOTYPES 1
-#endif
-#include "SDL_opengl.h"
 #include "shaders.h"
-#endif
-
 #include "guichan.h"
 
 #include "color.h"
@@ -54,16 +42,55 @@
 
 class CFont;
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-extern char ForceUseOpenGL;
-extern bool UseOpenGL;
-extern bool ZoomNoResize;
-extern bool GLShaderPipelineSupported;
+/// The SDL screen
+extern SDL_Window *TheWindow;
+extern SDL_Renderer *TheRenderer;
+extern SDL_Surface *TheScreen;
+extern SDL_Texture *TheTexture;
+
+#if SDL_BYTEORDER == SDL_LIL_ENDIAN
+#define RSHIFT  16
+#define GSHIFT  8
+#define BSHIFT  0
+#define ASHIFT  24
+#define RMASK   0x00ff0000
+#define GMASK   0x0000ff00
+#define BMASK   0x000000ff
+#define AMASK   0xff000000
+#else
+#define RSHIFT  8
+#define GSHIFT  16
+#define BSHIFT  24
+#define ASHIFT  0
+#define RMASK   0x0000ff00
+#define GMASK   0x00ff0000
+#define BMASK   0xff000000
+#define AMASK   0x000000ff
 #endif
 
+using pixelModifier = uint32_t(*)(const uint32_t, const uint32_t, const uint32_t); // type alias
+
+/// Class for modifiers for custom pixel manipulations
+class PixelModifier
+{
+public:
+	/// This one returns srcRGB+A(modulated) only if srcA is present. Otherwise returns dstRGBA. 
+	/// Used to copy (without alpha modulating) those pixels which has alpha chanel values
+	static uint32_t CopyWithSrcAlphaKey(const uint32_t srcPixel, const uint32_t dstPixel, const uint32_t reqAlpha)
+	{
+		uint32_t srcAlpha = (srcPixel >> ASHIFT) & 0xFF;
+		if (srcAlpha) {
+			srcAlpha = (srcAlpha * reqAlpha) >> 8;
+			return (srcPixel - (srcPixel & AMASK)) + (srcAlpha << ASHIFT);
+		}
+		return dstPixel;
+	}
+	/// Add more modifiers here
+};
+
 class CGraphic : public gcn::Image
 {
-
+public:
 	struct frame_pos_t {
 		short int x;
 		short int y;
@@ -73,10 +100,6 @@ protected:
 	CGraphic() : Surface(NULL), SurfaceFlip(NULL), frame_map(NULL),
 		Width(0), Height(0), NumFrames(1), GraphicWidth(0), GraphicHeight(0),
 		Refs(1), Resized(false)
-#if defined(USE_OPENGL) || defined(USE_GLES)
-		, TextureWidth(0.f), TextureHeight(0.f), Textures(NULL), NumTextures(0),
-		ColorCyclingTextures(NULL), NumColorCycles(0)
-#endif
 	{
 		frameFlip_map = NULL;
 	}
@@ -84,31 +107,54 @@ protected:
 
 public:
 	// Draw
-	void DrawClip(int x, int y) const;
-	void DrawSub(int gx, int gy, int w, int h, int x, int y) const;
-	void DrawSubClip(int gx, int gy, int w, int h, int x, int y) const;
+	void DrawClip(int x, int y,
+				  SDL_Surface *surface = TheScreen) const;
+	void DrawSub(int gx, int gy, int w, int h, int x, int y,
+				 SDL_Surface *surface = TheScreen) const;
+
+	void DrawSubCustomMod(int gx, int gy, int w, int h, int x, int y,
+					      pixelModifier modifier, 
+						  const uint32_t param,
+						  SDL_Surface *surface = TheScreen) const;
+
+	void DrawSubClip(int gx, int gy, int w, int h, int x, int y,
+					 SDL_Surface *surface = TheScreen) const;
 	void DrawSubTrans(int gx, int gy, int w, int h, int x, int y,
-					  unsigned char alpha) const;
+					  unsigned char alpha,
+					  SDL_Surface *surface = TheScreen) const;
 	void DrawSubClipTrans(int gx, int gy, int w, int h, int x, int y,
-						  unsigned char alpha) const;
+						  unsigned char alpha, 
+						  SDL_Surface *surface = TheScreen) const;
+
+	void DrawSubClipCustomMod(int gx, int gy, int w, int h, int x, int y,
+							  pixelModifier modifier, 
+							  const uint32_t param,
+							  SDL_Surface *surface = TheScreen) const;
 
 	// Draw frame
-	void DrawFrame(unsigned frame, int x, int y) const;
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	void DoDrawFrameClip(GLuint *textures, unsigned frame, int x, int y) const;
-#endif
-	void DrawFrameClip(unsigned frame, int x, int y) const;
-	void DrawFrameTrans(unsigned frame, int x, int y, int alpha) const;
-	void DrawFrameClipTrans(unsigned frame, int x, int y, int alpha) const;
+	void DrawFrame(unsigned frame, int x, int y,
+				   SDL_Surface *surface = TheScreen) const;
+	void DrawFrameClip(unsigned frame, int x, int y,
+					   SDL_Surface *surface = TheScreen) const;
+	void DrawFrameTrans(unsigned frame, int x, int y, int alpha,
+						SDL_Surface *surface = TheScreen) const;
+	void DrawFrameClipTrans(unsigned frame, int x, int y, int alpha, 
+							SDL_Surface *surface = TheScreen) const;
+
+	void DrawFrameClipCustomMod(unsigned frame, int x, int y, 
+								pixelModifier modifier, 
+								const uint32_t param,
+								SDL_Surface *surface = TheScreen) const;
 
 	// Draw frame flipped horizontally
-	void DrawFrameX(unsigned frame, int x, int y) const;
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	void DoDrawFrameClipX(GLuint *textures, unsigned frame, int x, int y) const;
-#endif
-	void DrawFrameClipX(unsigned frame, int x, int y) const;
-	void DrawFrameTransX(unsigned frame, int x, int y, int alpha) const;
-	void DrawFrameClipTransX(unsigned frame, int x, int y, int alpha) const;
+	void DrawFrameX(unsigned frame, int x, int y,
+					SDL_Surface *surface = TheScreen) const;
+	void DrawFrameClipX(unsigned frame, int x, int y,
+						SDL_Surface *surface = TheScreen) const;
+	void DrawFrameTransX(unsigned frame, int x, int y, int alpha,
+						 SDL_Surface *surface = TheScreen) const;
+	void DrawFrameClipTransX(unsigned frame, int x, int y, int alpha,
+							 SDL_Surface *surface = TheScreen) const;
 
 
 	static CGraphic *New(const std::string &file, int w = 0, int h = 0);
@@ -116,19 +162,19 @@ public:
 	static CGraphic *Get(const std::string &file);
 
 	static void Free(CGraphic *g);
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	bool DeleteColorCyclingTextures();
-#endif
 
 	void Load(bool grayscale = false);
 	void Flip();
-	void UseDisplayFormat();
 	void Resize(int w, int h);
 	void SetOriginalSize();
 	bool TransparentPixel(int x, int y);
-	void MakeShadow();
+	void SetPaletteColor(int idx, int r, int g, int b);
+	void MakeShadow(int xOffset, int yOffset);
 
-	inline bool IsLoaded() const { return Surface != NULL; }
+	// minor programmatic editing features
+	void OverlayGraphic(CGraphic *other, bool mask = false);
+
+	inline bool IsLoaded(bool flipped = false) const { return Surface != NULL && (!flipped || SurfaceFlip != NULL); }
 
 	//guichan
 	virtual void *_getData() const { return Surface; }
@@ -150,15 +196,6 @@ public:
 	int Refs;                  /// Uses of this graphic
 	bool Resized;              /// Image has been resized
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	GLfloat TextureWidth;      /// Width of the texture
-	GLfloat TextureHeight;     /// Height of the texture
-	GLuint *Textures;          /// Texture names
-	int NumTextures;           /// Number of textures
-	GLuint **ColorCyclingTextures; /// Texture names
-	int NumColorCycles; /// Number of color cycled texture groups
-#endif
-
 	friend class CFont;
 };
 
@@ -167,38 +204,57 @@ class CPlayerColorGraphic : public CGraphic
 protected:
 	CPlayerColorGraphic()
 	{
-#if defined(USE_OPENGL) || defined(USE_GLES)
-		memset(PlayerColorTextures, 0, sizeof(PlayerColorTextures));
-#endif
 	}
 
 public:
-	void DrawPlayerColorFrameClipX(int player, unsigned frame, int x, int y);
-	void DrawPlayerColorFrameClip(int player, unsigned frame, int x, int y);
+	void DrawPlayerColorFrameClipX(int colorIndex, unsigned frame, int x, int y,
+								   SDL_Surface *surface = TheScreen);
+	void DrawPlayerColorFrameClip(int colorIndex, unsigned frame, int x, int y,
+								  SDL_Surface *surface = TheScreen);
 
 	static CPlayerColorGraphic *New(const std::string &file, int w = 0, int h = 0);
 	static CPlayerColorGraphic *ForceNew(const std::string &file, int w = 0, int h = 0);
 	static CPlayerColorGraphic *Get(const std::string &file);
 
 	CPlayerColorGraphic *Clone(bool grayscale = false) const;
-
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	GLuint *PlayerColorTextures[PlayerMax];/// Textures with player colors
-#endif
 };
 
 #ifdef USE_MNG
+#ifdef WIN32
+#ifdef HAVE_STDDEF_H
+#undef HAVE_STDDEF_H
+#endif
+#endif
 #include <libmng.h>
+#ifdef WIN32
+#ifndef HAVE_STDDEF_H
+#undef HAVE_STDDEF_H
+#endif
+#endif
 
-class Mng
+class Mng : public gcn::Image
 {
-public:
 	Mng();
 	~Mng();
-	int Load(const std::string &name);
+
+	uint32_t refcnt = 0;
+
+public:
+	static Mng *New(const std::string &name);
+	static void Free(Mng *mng);
+	bool Load();
 	void Reset();
 	void Draw(int x, int y);
 
+	//guichan
+	virtual void *_getData() const;
+	virtual int getWidth() const { return surface->w; }
+	virtual int getHeight() const { return surface->h; }
+	virtual bool isDirty() const { return true; }
+
+	static uint32_t MaxFPS;
+
+	mutable bool is_dirty;
 	std::string name;
 	FILE *fd;
 	mng_handle handle;
@@ -206,11 +262,27 @@ public:
 	unsigned char *buffer;
 	unsigned long ticks;
 	int iteration;
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	GLfloat texture_width;   /// Width of the texture
-	GLfloat texture_height;  /// Height of the texture
-	GLuint texture_name;     /// Texture name
-#endif
+};
+#else
+/// empty class for lua scripts
+class Mng : public gcn::Image
+{
+	Mng() {};
+	~Mng() {};
+public:
+	static Mng *New(const std::string &name) { return NULL; }
+	static void Free(Mng *mng) {};
+	bool Load() { return false; };
+	void Reset() {};
+	void Draw(int x, int y) {};
+
+	//guichan
+	virtual void *_getData() const { return NULL; };
+	virtual int getWidth() const { return 0; };
+	virtual int getHeight() const { return 0; };
+	virtual bool isDirty() const { return false; };
+
+	static inline uint32_t MaxFPS = 15;
 };
 #endif
 
@@ -243,31 +315,11 @@ struct EventCallback {
 
 };
 
-#if SDL_BYTEORDER == SDL_LIL_ENDIAN
-#define RSHIFT  0
-#define GSHIFT  8
-#define BSHIFT  16
-#define ASHIFT  24
-#define RMASK   0x000000ff
-#define GMASK   0x0000ff00
-#define BMASK   0x00ff0000
-#define AMASK   0xff000000
-#else
-#define RSHIFT  24
-#define GSHIFT  16
-#define BSHIFT  8
-#define ASHIFT  0
-#define RMASK   0xff000000
-#define GMASK   0x00ff0000
-#define BMASK   0x0000ff00
-#define AMASK   0x000000ff
-#endif
-
 
 class CVideo
 {
 public:
-	CVideo() : Width(0), Height(0), Depth(0), FullScreen(false) {}
+	CVideo() : Width(0), Height(0), WindowWidth(0), WindowHeight(0), VerticalPixelSize(1), Depth(0), FullScreen(false) {}
 
 	void LockScreen();
 	void UnlockScreen();
@@ -308,6 +360,8 @@ public:
 	void DrawCircleClip(Uint32 color, int x, int y, int r);
 	void DrawTransCircleClip(Uint32 color, int x, int y, int r, unsigned char alpha);
 
+	void DrawEllipseClip(Uint32 color, int x, int y, int rx, int ry);
+
 	void FillCircle(Uint32 color, int x, int y, int radius);
 	void FillTransCircle(Uint32 color, int x, int y, int radius, unsigned char alpha);
 	void FillCircleClip(Uint32 color, const PixelPos &screenPos, int radius);
@@ -315,14 +369,7 @@ public:
 
 	inline Uint32 MapRGB(SDL_PixelFormat *f, Uint8 r, Uint8 g, Uint8 b)
 	{
-#if defined(USE_OPENGL) || defined(USE_GLES)
-		if (UseOpenGL) {
-			return MapRGBA(f, r, g, b, 0xFF);
-		} else
-#endif
-		{
-			return SDL_MapRGB(f, r, g, b);
-		}
+		return SDL_MapRGB(f, r, g, b);
 	}
 	inline Uint32 MapRGB(SDL_PixelFormat *f, const CColor &color)
 	{
@@ -330,14 +377,7 @@ public:
 	}
 	inline Uint32 MapRGBA(SDL_PixelFormat *f, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
 	{
-#if defined(USE_OPENGL) || defined(USE_GLES)
-		if (UseOpenGL) {
-			return ((r << RSHIFT) | (g << GSHIFT) | (b << BSHIFT) | (a << ASHIFT));
-		} else
-#endif
-		{
-			return SDL_MapRGBA(f, r, g, b, a);
-		}
+		return SDL_MapRGBA(f, r, g, b, a);
 	}
 	inline Uint32 MapRGBA(SDL_PixelFormat *f, const CColor &color)
 	{
@@ -345,41 +385,19 @@ public:
 	}
 	inline void GetRGB(Uint32 c, SDL_PixelFormat *f, Uint8 *r, Uint8 *g, Uint8 *b)
 	{
-#if defined(USE_OPENGL) || defined(USE_GLES)
-		if (UseOpenGL) {
-			*r = (c >> RSHIFT) & 0xff;
-			*g = (c >> GSHIFT) & 0xff;
-			*b = (c >> BSHIFT) & 0xff;
-		} else
-#endif
-		{
-			SDL_GetRGB(c, f, r, g, b);
-		}
+		SDL_GetRGB(c, f, r, g, b);
 	}
 	inline void GetRGBA(Uint32 c, SDL_PixelFormat *f, Uint8 *r, Uint8 *g, Uint8 *b, Uint8 *a)
 	{
-#if defined(USE_OPENGL) || defined(USE_GLES)
-		if (UseOpenGL) {
-			*r = (c >> RSHIFT) & 0xff;
-			*g = (c >> GSHIFT) & 0xff;
-			*b = (c >> BSHIFT) & 0xff;
-			*a = (c >> ASHIFT) & 0xff;
-		} else
-#endif
-		{
-			SDL_GetRGBA(c, f, r, g, b, a);
-		}
+		SDL_GetRGBA(c, f, r, g, b, a);
 	}
 
 	int Width;
 	int Height;
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	int ViewportWidth;         /// Actual width of the window
-	int ViewportHeight;        /// Actual height of the window
-#endif
-#if defined(USE_TOUCHSCREEN) && defined(USE_WIN32)
+	int WindowWidth;
+	int WindowHeight;
+	double VerticalPixelSize;
 	SDL_Cursor *blankCursor;
-#endif
 	int Depth;
 	bool FullScreen;
 };
@@ -413,23 +431,15 @@ extern unsigned long SlowFrameCounter;
 /// (bring Players[] in sync with Pixels[])
 extern void SetPlayersPalette();
 
-/// The SDL screen
-extern SDL_Surface *TheScreen;
-
-#if defined(USE_OPENGL) || defined(USE_GLES)
-/// Max texture size supported on the video card
-extern GLint GLMaxTextureSize;
-/// User-specified limit for ::GLMaxTextureSize
-extern GLint GLMaxTextureSizeOverride;
-/// Is OpenGL texture compression supported
-extern bool GLTextureCompressionSupported;
-/// Use OpenGL texture compression
-extern bool UseGLTextureCompression;
-#endif
-
 /// register lua function
 extern void VideoCclRegister();
 
+/// initialize the image loaders part
+extern void InitImageLoaders();
+
+/// deinitialize the image loaders
+extern void DeInitImageLoaders();
+
 /// initialize the video part
 extern void InitVideo();
 
@@ -439,28 +449,6 @@ void DeInitVideo();
 /// Check if a resolution is valid
 extern int VideoValidResolution(int w, int h);
 
-/// Load graphic from PNG file
-extern int LoadGraphicPNG(CGraphic *g);
-
-#if defined(USE_OPENGL) || defined(USE_GLES)
-
-/// Make an OpenGL texture
-extern void MakeTexture(CGraphic *graphic);
-/// Make an OpenGL texture of the player color pixels only.
-extern void MakePlayerColorTexture(CPlayerColorGraphic *graphic, int player);
-
-/// Regenerate Window screen if needed
-extern void ValidateOpenGLScreen();
-
-/// Free OpenGL graphics
-extern void FreeOpenGLGraphics();
-/// Reload OpenGL graphics
-extern void ReloadGraphics();
-/// Reload OpenGL
-extern void ReloadOpenGL();
-
-#endif
-
 /// Initializes video synchronization.
 extern void SetVideoSync();
 
@@ -531,25 +519,31 @@ extern Uint32 ColorRed;
 extern Uint32 ColorGreen;
 extern Uint32 ColorYellow;
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-void DrawTexture(const CGraphic *g, GLuint *textures, int sx, int sy,
-				 int ex, int ey, int x, int y, int flip);
-#endif
+inline Uint32 IndexToColor(unsigned int index) {
+    // FIXME: this only works after video was initialized, so we do it dynamically
+    static const Uint32 ColorValues[] = {ColorRed, ColorYellow, ColorGreen, ColorLightGray,
+                                         ColorGray, ColorDarkGray, ColorWhite, ColorOrange,
+                                         ColorLightBlue, ColorBlue, ColorDarkGreen, ColorBlack};
+    return ColorValues[index];
+}
+
+static const char *ColorNames[] = {"red", "yellow", "green", "light-gray",
+                                   "gray", "dark-gray", "white", "orange",
+                                   "light-blue", "blue", "dark-green", "black", NULL};
+
+inline int GetColorIndexByName(const char *colorName) {
+    int i = 0;
+    while (ColorNames[i] != NULL) {
+        if (!strcmp(colorName, ColorNames[i])) {
+            return i;
+        }
+        i++;
+    }
+    return -1;
+}
 
 extern void FreeGraphics();
 
-
-// ARB_texture_compression
-#if defined(USE_OPENGL) && !defined(__APPLE__)
-extern PFNGLCOMPRESSEDTEXIMAGE3DARBPROC    glCompressedTexImage3DARB;
-extern PFNGLCOMPRESSEDTEXIMAGE2DARBPROC    glCompressedTexImage2DARB;
-extern PFNGLCOMPRESSEDTEXIMAGE1DARBPROC    glCompressedTexImage1DARB;
-extern PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC glCompressedTexSubImage3DARB;
-extern PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC glCompressedTexSubImage2DARB;
-extern PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC glCompressedTexSubImage1DARB;
-extern PFNGLGETCOMPRESSEDTEXIMAGEARBPROC   glGetCompressedTexImageARB;
-#endif
-
 //
 //  Color Cycling stuff
 //
@@ -558,12 +552,17 @@ extern void VideoPaletteListAdd(SDL_Surface *surface);
 extern void VideoPaletteListRemove(SDL_Surface *surface);
 extern void ClearAllColorCyclingRange();
 extern void AddColorCyclingRange(unsigned int begin, unsigned int end);
+extern unsigned int SetColorCycleSpeed(unsigned int speed);
 extern void SetColorCycleAll(bool value);
 extern void RestoreColorCyclingSurface();
 
 /// Does ColorCycling..
 extern void ColorCycle();
 
+/// Blit a surface into another with alpha blending
+extern void BlitSurfaceAlphaBlending_32bpp(const SDL_Surface *srcSurface, const SDL_Rect *srcRect, 
+												 SDL_Surface *dstSurface, const SDL_Rect *dstRect, const bool enableMT = true);
+
 //@}
 
 #endif // !__VIDEO_H__
