@@ -38,6 +38,7 @@
 
 #include "map.h"
 
+#include "fov.h"
 #include "iolib.h"
 #include "player.h"
 #include "tileset.h"
@@ -52,9 +53,10 @@
 ----------------------------------------------------------------------------*/
 
 CMap Map;                   /// The current map
-int FlagRevealMap;          /// Flag must reveal the map
+MapRevealModes FlagRevealMap;          /// Flag must reveal the map
 int ReplayRevealMap;        /// Reveal Map is replay
-int ForestRegeneration;     /// Forest regeneration
+unsigned int ForestRegeneration;     /// Forest regeneration
+int ForestRegenerationFrequency;     /// Forest regeneration frequency (every how many seconds we apply 1 point of regeneration)
 char CurrentMapPath[1024];  /// Path of the current map
 
 /*----------------------------------------------------------------------------
@@ -128,31 +130,40 @@ void CMap::MarkSeenTile(CMapField &mf)
 /**
 **  Reveal the entire map.
 */
-void CMap::Reveal()
+void CMap::Reveal(MapRevealModes mode /* = MapRevealModes::cKnown */ )
 {
-	//  Mark every explored tile as visible. 1 turns into 2.
-	for (int i = 0; i != this->Info.MapWidth * this->Info.MapHeight; ++i) {
-		CMapField &mf = *this->Field(i);
-		CMapFieldPlayerInfo &playerInfo = mf.playerInfo;
-		for (int p = 0; p < PlayerMax; ++p) {
-			playerInfo.Visible[p] = std::max<unsigned short>(1, playerInfo.Visible[p]);
-		}
-		MarkSeenTile(mf);
+	/// We can't do "unrevealing" yet
+	if (static_cast<int>(mode) < static_cast<int>(GameSettings.RevealMap)
+		|| static_cast<int>(mode) >= static_cast<int>(MapRevealModes::cNumOfModes)) {
+		return;
 	}
-	//  Global seen recount. Simple and effective.
-	for (CUnitManager::Iterator it = UnitManager.begin(); it != UnitManager.end(); ++it) {
-		CUnit &unit = **it;
-		//  Reveal neutral buildings. Gold mines:)
-		if (unit.Player->Type == PlayerNeutral) {
+
+	//  Mark every explored tile as visible. 1 turns into 2.
+	if (static_cast<int>(mode) >= static_cast<int>(MapRevealModes::cExplored)) {
+		for (int i = 0; i != this->Info.MapWidth * this->Info.MapHeight; ++i) {
+			CMapField &mf = *this->Field(i);
+			CMapFieldPlayerInfo &playerInfo = mf.playerInfo;
 			for (int p = 0; p < PlayerMax; ++p) {
-				if (Players[p].Type != PlayerNobody && (!(unit.Seen.ByPlayer & (1 << p)))) {
-					UnitGoesOutOfFog(unit, Players[p]);
-					UnitGoesUnderFog(unit, Players[p]);
+				playerInfo.Visible[p] = std::max<unsigned short>(1, playerInfo.Visible[p]);
+			}
+			MarkSeenTile(mf);
+		}
+	}
+
+	//  Global seen recount. Simple and effective.
+	for (CUnit *unit : UnitManager->GetUnits()) {
+		//  Reveal neutral buildings. Gold mines:)
+		if (unit->Player->Type == PlayerTypes::PlayerNeutral) {
+			for (const CPlayer &player : Players) {
+				if (player.Type != PlayerTypes::PlayerNobody && (!(unit->Seen.ByPlayer & (1 << player.Index )))) {
+					UnitGoesOutOfFog(*unit, player);
+					UnitGoesUnderFog(*unit, player);
 				}
 			}
 		}
-		UnitCountSeen(unit);
+		UnitCountSeen(*unit);
 	}
+	GameSettings.RevealMap = mode; 
 }
 
 /*----------------------------------------------------------------------------
@@ -245,8 +256,10 @@ bool UnitTypeCanBeAt(const CUnitType &type, const Vec2i &pos)
 
 	for (int addy = 0; addy < type.TileHeight; ++addy) {
 		for (int addx = 0; addx < type.TileWidth; ++addx) {
-			if (Map.Info.IsPointOnMap(pos.x + addx, pos.y + addy) == false
-				|| Map.Field(pos.x + addx + index)->CheckMask(mask) == true) {
+			if (!Map.Info.IsPointOnMap(pos.x + addx, pos.y + addy)) {
+				return false;
+			}
+			if (!type.BoolFlag[NONSOLID_INDEX].value && Map.Field(pos.x + addx + index)->CheckMask(mask)) {
 				return false;
 			}
 		}
@@ -266,9 +279,6 @@ bool UnitTypeCanBeAt(const CUnitType &type, const Vec2i &pos)
 bool UnitCanBeAt(const CUnit &unit, const Vec2i &pos)
 {
 	Assert(unit.Type);
-	if (unit.Type->BoolFlag[NONSOLID_INDEX].value) {
-		return true;
-	}
 	return UnitTypeCanBeAt(*unit.Type, pos);
 }
 
@@ -308,9 +318,8 @@ void CMapInfo::Clear()
 	this->MapUID = 0;
 }
 
-CMap::CMap() : Fields(NULL), NoFogOfWar(false), TileGraphic(NULL)
+CMap::CMap() : Fields(NULL), NoFogOfWar(false), TileGraphic(NULL), Tileset(NULL)
 {
-	Tileset = new CTileset;
 }
 
 CMap::~CMap()
@@ -318,6 +327,12 @@ CMap::~CMap()
 	delete Tileset;
 }
 
+void CMap::AllocateTileset()
+{
+	Assert(!Tileset);
+	Tileset = new CTileset();
+}
+
 /**
 **  Alocate and initialise map table
 */
@@ -334,13 +349,14 @@ void CMap::Create()
 */
 void CMap::Init()
 {
-	InitFogOfWar();
+	FogOfWar->Init();
+	this->isMapInitialized = true;
 }
 
 /**
 **  Cleanup the map module.
 */
-void CMap::Clean()
+void CMap::Clean(const bool isHardClean /* = false*/)
 {
 	delete[] this->Fields;
 
@@ -354,10 +370,19 @@ void CMap::Clean()
 	CGraphic::Free(this->TileGraphic);
 	this->TileGraphic = NULL;
 
-	FlagRevealMap = 0;
+	FlagRevealMap = MapRevealModes::cHidden;
 	ReplayRevealMap = 0;
 
 	UI.Minimap.Destroy();
+
+	FieldOfView.Clean();
+	
+	FogOfWar->Clean(isHardClean);
+	for (CViewport *vp = UI.Viewports; vp < UI.Viewports + UI.NumViewports; ++vp) {
+		vp->Clean();
+	}
+
+	this->isMapInitialized = false;
 }
 
 /**
@@ -431,7 +456,7 @@ void CMap::FixTile(unsigned short type, int seen, const Vec2i &pos)
 	int flags;
 	if (type == MapFieldForest) {
 		removedtile = this->Tileset->getRemovedTreeTile();
-		flags = (MapFieldForest | MapFieldUnpassable);
+		flags = (MapFieldCost4 | MapFieldCost5 | MapFieldCost6 | MapFieldForest | MapFieldUnpassable);
 	} else { // (type == MapFieldRocks)
 		removedtile = this->Tileset->getRemovedRockTile();
 		flags = (MapFieldRocks | MapFieldUnpassable);
@@ -490,7 +515,7 @@ void CMap::FixTile(unsigned short type, int seen, const Vec2i &pos)
 	}
 
 	//maybe isExplored
-	if (mf.playerInfo.IsExplored(*ThisPlayer)) {
+	if (mf.playerInfo.IsTeamVisible(*ThisPlayer)) {
 		UI.Minimap.UpdateSeenXY(pos);
 		if (!seen) {
 			MarkSeenTile(mf);
@@ -516,20 +541,49 @@ void CMap::FixNeighbors(unsigned short type, int seen, const Vec2i &pos)
 	}
 }
 
+
+/**
+**  Remove Wood, Rock or Wall tile from the map
+**  It checks if field is opaque and recalc vision for nearby units
+**
+**  @param tilePos   Map tile-position.
+*/
+void CMap::ClearTile(const Vec2i &tilePos)
+{
+	Assert(Map.Info.IsPointOnMap(tilePos));
+	
+	CMapField &mapField = *this->Field(tilePos);
+
+	const bool isOpaque = Map.Field(tilePos)->isOpaque();
+	if (isOpaque) {
+		MapRefreshUnitsSight(tilePos, true);
+		mapField.Flags &= ~MapFieldOpaque;
+	}
+	if (mapField.ForestOnMap()) {
+		ClearWoodTile(tilePos);
+	} else if (mapField.RockOnMap()) {
+		ClearRockTile(tilePos);
+	} else if (mapField.isAWall()) {
+		RemoveWall(tilePos);
+	}
+	if (isOpaque) {
+		MapRefreshUnitsSight(tilePos);
+	} 	
+}
 /// Remove wood from the map.
 void CMap::ClearWoodTile(const Vec2i &pos)
 {
 	CMapField &mf = *this->Field(pos);
 
 	mf.setGraphicTile(this->Tileset->getRemovedTreeTile());
-	mf.Flags &= ~(MapFieldForest | MapFieldUnpassable);
+	mf.Flags &= ~(MapFieldCost4 | MapFieldCost5 | MapFieldCost6 | MapFieldForest | MapFieldUnpassable);
 	mf.Value = 0;
 
 	UI.Minimap.UpdateXY(pos);
 	FixNeighbors(MapFieldForest, 0, pos);
 
 	//maybe isExplored
-	if (mf.playerInfo.IsExplored(*ThisPlayer)) {
+	if (mf.playerInfo.IsTeamVisible(*ThisPlayer)) {
 		UI.Minimap.UpdateSeenXY(pos);
 		MarkSeenTile(mf);
 	}
@@ -547,7 +601,7 @@ void CMap::ClearRockTile(const Vec2i &pos)
 	FixNeighbors(MapFieldRocks, 0, pos);
 
 	//maybe isExplored
-	if (mf.playerInfo.IsExplored(*ThisPlayer)) {
+	if (mf.playerInfo.IsTeamVisible(*ThisPlayer)) {
 		UI.Minimap.UpdateSeenXY(pos);
 		MarkSeenTile(mf);
 	}
@@ -589,19 +643,19 @@ void CMap::RegenerateForestTile(const Vec2i &pos)
 		&& topMf.Value >= ForestRegeneration
 		&& !(topMf.Flags & occupedFlag)) {
 		DebugPrint("Real place wood\n");
-		topMf.setTileIndex(*Map.Tileset, Map.Tileset->getTopOneTreeTile(), 0);
+		topMf.setTileIndex(*Map.Tileset, Map.Tileset->getDefaultWoodTileIndex(), 0);
 		topMf.setGraphicTile(Map.Tileset->getTopOneTreeTile());
 		topMf.playerInfo.SeenTile = topMf.getGraphicTile();
-		topMf.Value = 0;
+		topMf.Value = 100; // TODO: Should be DefaultResourceAmounts[WoodCost] once all games are migrated
 		topMf.Flags |= MapFieldForest | MapFieldUnpassable;
 		UI.Minimap.UpdateSeenXY(pos + offset);
 		UI.Minimap.UpdateXY(pos + offset);
 
 
-		mf.setTileIndex(*Map.Tileset, Map.Tileset->getBottomOneTreeTile(), 0);
+		mf.setTileIndex(*Map.Tileset, Map.Tileset->getDefaultWoodTileIndex(), 0);
 		mf.setGraphicTile(Map.Tileset->getBottomOneTreeTile());
 		mf.playerInfo.SeenTile = mf.getGraphicTile();
-		mf.Value = 0;
+		mf.Value = 100; // TODO: Should be DefaultResourceAmounts[WoodCost] once all games are migrated
 		mf.Flags |= MapFieldForest | MapFieldUnpassable;
 		UI.Minimap.UpdateSeenXY(pos);
 		UI.Minimap.UpdateXY(pos);
@@ -624,6 +678,9 @@ void CMap::RegenerateForest()
 	if (!ForestRegeneration) {
 		return;
 	}
+	if (ForestRegenerationFrequency != 1 && (GameCycle / CYCLES_PER_SECOND % ForestRegenerationFrequency) != 0) {
+		return; // not this second
+	}
 	Vec2i pos;
 	for (pos.y = 0; pos.y < Info.MapHeight; ++pos.y) {
 		for (pos.x = 0; pos.x < Info.MapWidth; ++pos.x) {
@@ -637,8 +694,9 @@ void CMap::RegenerateForest()
 **  Load the map presentation
 **
 **  @param mapname  map filename
+**  @return true on successful load, else false
 */
-void LoadStratagusMapInfo(const std::string &mapname)
+bool LoadStratagusMapInfo(const std::string &mapname)
 {
 	// Set the default map setup by replacing .smp with .sms
 	size_t loc = mapname.find(".smp");
@@ -647,7 +705,7 @@ void LoadStratagusMapInfo(const std::string &mapname)
 		Map.Info.Filename.replace(loc, 4, ".sms");
 	}
 
-	LuaLoadFile(mapname);
+	return LuaLoadFile(mapname) == 0;
 }
 
 //@}
