@@ -53,31 +53,25 @@
 --  Defines
 ----------------------------------------------------------------------------*/
 
-#define MINIMAP_FAC (16 * 3)  /// integer scale factor
-
+/// integer scale factor
+static constexpr int MINIMAP_FAC           {16 * 3};  
 /// unit attacked are shown red for at least this amount of cycles
-#define ATTACK_RED_DURATION (1 * CYCLES_PER_SECOND)
+static constexpr int ATTACK_RED_DURATION   {1 * CYCLES_PER_SECOND};
 /// unit attacked are shown blinking for this amount of cycles
-#define ATTACK_BLINK_DURATION (7 * CYCLES_PER_SECOND)
+static constexpr int ATTACK_BLINK_DURATION {7 * CYCLES_PER_SECOND};
+
+static constexpr int SCALE_PRECISION       {100};
+
 
-#define SCALE_PRECISION 100
 
 
 /*----------------------------------------------------------------------------
 --  Variables
 ----------------------------------------------------------------------------*/
 
-SDL_Surface *MinimapSurface;        /// generated minimap
-SDL_Surface *MinimapTerrainSurface; /// generated minimap terrain
-
-#if defined(USE_OPENGL) || defined(USE_GLES)
-unsigned char *MinimapSurfaceGL;
-unsigned char *MinimapTerrainSurfaceGL;
-
-static GLuint MinimapTexture;
-static int MinimapTextureWidth;
-static int MinimapTextureHeight;
-#endif
+SDL_Surface 	   *MinimapSurface{nullptr};        /// generated minimap
+static SDL_Surface *MinimapTerrainSurface{nullptr}; /// generated minimap terrain
+static SDL_Surface *MinimapFogSurface{nullptr};		/// generated minimap fog of war
 
 static int *Minimap2MapX;                  /// fast conversion table
 static int *Minimap2MapY;                  /// fast conversion table
@@ -105,25 +99,6 @@ int NumMinimapEvents;
 ----------------------------------------------------------------------------*/
 
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-/**
-**  Create the minimap texture
-*/
-static void CreateMinimapTexture()
-{
-	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
-	glGenTextures(1, &MinimapTexture);
-	glBindTexture(GL_TEXTURE_2D, MinimapTexture);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, MinimapTextureWidth,
-				 MinimapTextureHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE,
-				 MinimapSurfaceGL);
-}
-#endif
-
 /**
 **  Create a mini-map from the tiles of the map.
 **
@@ -166,47 +141,21 @@ void CMinimap::Create()
 	}
 
 	// Palette updated from UpdateMinimapTerrain()
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (UseOpenGL) {
-		for (MinimapTextureWidth = 1; MinimapTextureWidth < W; MinimapTextureWidth <<= 1) {
-		}
-		for (MinimapTextureHeight = 1; MinimapTextureHeight < H; MinimapTextureHeight <<= 1) {
-		}
-		MinimapTerrainSurfaceGL = new unsigned char[MinimapTextureWidth * MinimapTextureHeight * 4];
-		MinimapSurfaceGL = new unsigned char[MinimapTextureWidth * MinimapTextureHeight * 4];
-		memset(MinimapSurfaceGL, 0, MinimapTextureWidth * MinimapTextureHeight * 4);
-		CreateMinimapTexture();
-	} else
-#endif
-	{
-		SDL_PixelFormat *f = Map.TileGraphic->Surface->format;
-		MinimapTerrainSurface = SDL_CreateRGBSurface(SDL_SWSURFACE, W, H, f->BitsPerPixel, f->Rmask, f->Gmask, f->Bmask, f->Amask);
-		MinimapSurface = SDL_CreateRGBSurface(SDL_SWSURFACE,  W, H, 32, TheScreen->format->Rmask, TheScreen->format->Gmask, TheScreen->format->Bmask, 0);
-	}
+	SDL_PixelFormat *f 	  = Map.TileGraphic->Surface->format;
+	MinimapTerrainSurface = SDL_CreateRGBSurface(SDL_SWSURFACE, W, H, f->BitsPerPixel, f->Rmask, f->Gmask, f->Bmask, f->Amask);
+	MinimapSurface 		  = SDL_CreateRGBSurface(SDL_SWSURFACE, W, H, 32, RMASK, GMASK, BMASK, 0);
+	MinimapFogSurface 	  = SDL_CreateRGBSurface(SDL_SWSURFACE, W, H, 32, RMASK, GMASK, BMASK, AMASK);
 
+    SDL_SetSurfaceBlendMode(MinimapFogSurface, SDL_BLENDMODE_BLEND);
+	
+	const uint32_t fogColorSolid = FogOfWar->GetFogColorSDL() | (uint32_t(0xFF) << ASHIFT);
+	SDL_FillRect(MinimapFogSurface, NULL, fogColorSolid);
+	
 	UpdateTerrain();
 
 	NumMinimapEvents = 0;
 }
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-/**
-**  Free OpenGL minimap
-*/
-void CMinimap::FreeOpenGL()
-{
-	glDeleteTextures(1, &MinimapTexture);
-}
-
-/**
-**  Reload OpenGL minimap
-*/
-void CMinimap::Reload()
-{
-	CreateMinimapTexture();
-}
-#endif
-
 /**
 **  Calculate the tile graphic pixel
 */
@@ -233,26 +182,15 @@ void CMinimap::UpdateTerrain()
 	}
 	const int bpp = Map.TileGraphic->Surface->format->BytesPerPixel;
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (!UseOpenGL)
-#endif
-	{
-		if (bpp == 1) {
-			SDL_SetPalette(MinimapTerrainSurface, SDL_LOGPAL,
-						   Map.TileGraphic->Surface->format->palette->colors, 0, 256);
-		}
+	if (bpp == 1) {
+		SDL_SetPaletteColors(MinimapTerrainSurface->format->palette,
+							 Map.TileGraphic->Surface->format->palette->colors, 0, 256);
 	}
 
 	const int tilepitch = Map.TileGraphic->Surface->w / PixelTileSize.x;
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (UseOpenGL) {
-		SDL_LockSurface(Map.TileGraphic->Surface);
-	} else
-#endif
-	{
-		SDL_LockSurface(MinimapTerrainSurface);
-	}
+	Assert(SDL_MUSTLOCK(MinimapTerrainSurface) == 0);
+	Assert(SDL_MUSTLOCK(Map.TileGraphic->Surface) == 0);
 
 	//
 	//  Pixel 7,6 7,14, 15,6 15,14 are taken for the minimap picture.
@@ -263,50 +201,38 @@ void CMinimap::UpdateTerrain()
 			const int xofs = PixelTileSize.x * (tile % tilepitch);
 			const int yofs = PixelTileSize.y * (tile / tilepitch);
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-			if (UseOpenGL) {
-				Uint32 c;
-
-				if (bpp == 1) {
-					SDL_Color color = Map.TileGraphic->Surface->format->palette->colors[
-										  *GetTileGraphicPixel(xofs, yofs, mx, my, scalex, scaley, bpp)];
-					c = Video.MapRGB(0, color.r, color.g, color.b);
-				} else {
-					SDL_PixelFormat *f = Map.TileGraphic->Surface->format;
-					c = *(Uint32 *)GetTileGraphicPixel(xofs, yofs, mx, my, scalex, scaley, bpp);
-					c = Video.MapRGB(0,
-									 ((c & f->Rmask) >> f->Rshift),
-									 ((c & f->Gmask) >> f->Gshift),
-									 ((c & f->Bmask) >> f->Bshift));
-				}
-				*(Uint32 *)&(MinimapTerrainSurfaceGL[(mx + my * MinimapTextureWidth) * 4]) = c;
-			} else
-#endif
-			{
-				if (bpp == 1) {
-					((Uint8 *)MinimapTerrainSurface->pixels)[mx + my * MinimapTerrainSurface->pitch] =
-						*GetTileGraphicPixel(xofs, yofs, mx, my, scalex, scaley, bpp);
-				} else if (bpp == 3) {
-					Uint8 *d = &((Uint8 *)MinimapTerrainSurface->pixels)[mx * bpp + my * MinimapTerrainSurface->pitch];
-					Uint8 *s = GetTileGraphicPixel(xofs, yofs, mx, my, scalex, scaley, bpp);
-					*d++ = *s++;
-					*d++ = *s++;
-					*d++ = *s++;
-				} else {
-					*(Uint32 *)&((Uint8 *)MinimapTerrainSurface->pixels)[mx * bpp + my * MinimapTerrainSurface->pitch] =
-						*(Uint32 *)GetTileGraphicPixel(xofs, yofs, mx, my, scalex, scaley, bpp);
-				}
+			if (bpp == 1) {
+				((Uint8 *)MinimapTerrainSurface->pixels)[mx + my * MinimapTerrainSurface->pitch] =
+					*GetTileGraphicPixel(xofs, yofs, mx, my, scalex, scaley, bpp);
+			} else if (bpp == 3) {
+				Uint8 *d = &((Uint8 *)MinimapTerrainSurface->pixels)[mx * bpp + my * MinimapTerrainSurface->pitch];
+				Uint8 *s = GetTileGraphicPixel(xofs, yofs, mx, my, scalex, scaley, bpp);
+				*d++ = *s++;
+				*d++ = *s++;
+				*d++ = *s++;
+			} else {
+				*(Uint32 *)&((Uint8 *)MinimapTerrainSurface->pixels)[mx * bpp + my * MinimapTerrainSurface->pitch] =
+					*(Uint32 *)GetTileGraphicPixel(xofs, yofs, mx, my, scalex, scaley, bpp);
 			}
 
 		}
 	}
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (!UseOpenGL)
-#endif
-	{
-		SDL_UnlockSurface(MinimapTerrainSurface);
-	}
-	SDL_UnlockSurface(Map.TileGraphic->Surface);
+}
+
+
+/** 
+** Set fog of war opacity (alpha chanel values) for different levels of visibility
+** 
+** @param explored  alpha channel value for explored tiles
+** @param revealed  alpha channel value for revealed tiles (when the map revealed)
+** @param unseen    alpha channel value for unseen tiles
+** 
+*/
+void CMinimap::SetFogOpacityLevels(const uint8_t explored, const uint8_t revealed, const uint8_t unseen)
+{
+    this->Settings.FogExploredOpacity = explored;
+    this->Settings.FogRevealedOpacity = revealed;
+    this->Settings.FogUnseenOpacity   = unseen;
 }
 
 /**
@@ -316,17 +242,8 @@ void CMinimap::UpdateTerrain()
 */
 void CMinimap::UpdateXY(const Vec2i &pos)
 {
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (UseOpenGL) {
-		if (!MinimapTerrainSurfaceGL) {
-			return;
-		}
-	} else
-#endif
-	{
-		if (!MinimapTerrainSurface) {
-			return;
-		}
+	if (!MinimapTerrainSurface) {
+		return;
 	}
 
 	int scalex = MinimapScaleX * SCALE_PRECISION / MINIMAP_FAC;
@@ -344,13 +261,6 @@ void CMinimap::UpdateXY(const Vec2i &pos)
 	//
 	//  Pixel 7,6 7,14, 15,6 15,14 are taken for the minimap picture.
 	//
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (!UseOpenGL)
-#endif
-	{
-		SDL_LockSurface(MinimapTerrainSurface);
-	}
-	SDL_LockSurface(Map.TileGraphic->Surface);
 
 	const int ty = pos.y * Map.Info.MapWidth;
 	const int tx = pos.x;
@@ -381,51 +291,21 @@ void CMinimap::UpdateXY(const Vec2i &pos)
 			const int xofs = PixelTileSize.x * (tile % tilepitch);
 			const int yofs = PixelTileSize.y * (tile / tilepitch);
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-			if (UseOpenGL) {
-				Uint32 c;
+			const int index = mx * bpp + my * MinimapTerrainSurface->pitch;
+			Uint8 *s = GetTileGraphicPixel(xofs, yofs, mx, my, scalex, scaley, bpp);
+			if (bpp == 1) {
+				((Uint8 *)MinimapTerrainSurface->pixels)[index] = *s;
+			} else if (bpp == 3) {
+				Uint8 *d = &((Uint8 *)MinimapTerrainSurface->pixels)[index];
 
-				if (bpp == 1) {
-					const int colorIndex = *GetTileGraphicPixel(xofs, yofs, mx, my, scalex, scaley, bpp);
-					const SDL_Color color = Map.TileGraphic->Surface->format->palette->colors[colorIndex];
-
-					c = Video.MapRGB(0, color.r, color.g, color.b);
-				} else {
-					SDL_PixelFormat *f = Map.TileGraphic->Surface->format;
-
-					c = *(Uint32 *)GetTileGraphicPixel(xofs, yofs, mx, my, scalex, scaley, bpp);
-					c = Video.MapRGB(0,
-									 ((c & f->Rmask) >> f->Rshift),
-									 ((c & f->Gmask) >> f->Gshift),
-									 ((c & f->Bmask) >> f->Bshift));
-				}
-				*(Uint32 *)&(MinimapTerrainSurfaceGL[(mx + my * MinimapTextureWidth) * 4]) = c;
-			} else
-#endif
-			{
-				const int index = mx * bpp + my * MinimapTerrainSurface->pitch;
-				Uint8 *s = GetTileGraphicPixel(xofs, yofs, mx, my, scalex, scaley, bpp);
-				if (bpp == 1) {
-					((Uint8 *)MinimapTerrainSurface->pixels)[index] = *s;
-				} else if (bpp == 3) {
-					Uint8 *d = &((Uint8 *)MinimapTerrainSurface->pixels)[index];
-
-					*d++ = *s++;
-					*d++ = *s++;
-					*d++ = *s++;
-				} else {
-					*(Uint32 *)&((Uint8 *)MinimapTerrainSurface->pixels)[index] = *(Uint32 *)s;
-				}
+				*d++ = *s++;
+				*d++ = *s++;
+				*d++ = *s++;
+			} else {
+				*(Uint32 *)&((Uint8 *)MinimapTerrainSurface->pixels)[index] = *(Uint32 *)s;
 			}
 		}
 	}
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (!UseOpenGL)
-#endif
-	{
-		SDL_UnlockSurface(MinimapTerrainSurface);
-	}
-	SDL_UnlockSurface(Map.TileGraphic->Surface);
 }
 
 /**
@@ -435,7 +315,7 @@ static void DrawUnitOn(CUnit &unit, int red_phase)
 {
 	const CUnitType *type;
 
-	if (Editor.Running || ReplayRevealMap || unit.IsVisible(*ThisPlayer)) {
+	if (Editor.Running || ReplayRevealMap || unit.IsVisible(*ThisPlayer) || unit.Player->IsRevealed()) {
 		type = unit.Type;
 	} else {
 		type = unit.Seen.Type;
@@ -459,7 +339,7 @@ static void DrawUnitOn(CUnit &unit, int red_phase)
 			color = ColorGreen;
 		}
 	} else {
-		color = PlayerColors[GameSettings.Presets[unit.Player->Index].PlayerColor][0];
+		color = PlayerColorsRGB[GameSettings.Presets[unit.Player->Index].PlayerColor][0];
 	}
 
 	int mx = 1 + UI.Minimap.XOffset + Map2MinimapX[unit.tilePos.x];
@@ -473,29 +353,17 @@ static void DrawUnitOn(CUnit &unit, int red_phase)
 		h0 = UI.Minimap.H - my;
 	}
 	int bpp = 0;
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (!UseOpenGL)
-#endif
-	{
-		SDL_Color c;
-		bpp = MinimapSurface->format->BytesPerPixel;
-		SDL_GetRGB(color, TheScreen->format, &c.r, &c.g, &c.b);
-	}
+	SDL_Color c;
+	bpp = MinimapSurface->format->BytesPerPixel;
+	SDL_GetRGB(color, TheScreen->format, &c.r, &c.g, &c.b);
 	while (w-- >= 0) {
 		int h = h0;
 		while (h-- >= 0) {
-#if defined(USE_OPENGL) || defined(USE_GLES)
-			if (UseOpenGL) {
-				*(Uint32 *)&(MinimapSurfaceGL[((mx + w) + (my + h) * MinimapTextureWidth) * 4]) = color;
-			} else
-#endif
-			{
-				const unsigned int index = (mx + w) * bpp + (my + h) * MinimapSurface->pitch;
-				if (bpp == 2) {
-					*(Uint16 *)&((Uint8 *)MinimapSurface->pixels)[index] = color;
-				} else {
-					*(Uint32 *)&((Uint8 *)MinimapSurface->pixels)[index] = color;
-				}
+			const unsigned int index = (mx + w) * bpp + (my + h) * MinimapSurface->pitch;
+			if (bpp == 2) {
+				*(Uint16 *)&((Uint8 *)MinimapSurface->pixels)[index] = color;
+			} else {
+				*(Uint32 *)&((Uint8 *)MinimapSurface->pixels)[index] = color;
 			}
 		}
 	}
@@ -515,99 +383,46 @@ void CMinimap::Update()
 
 	// Clear Minimap background if not transparent
 	if (!Transparent) {
-#if defined(USE_OPENGL) || defined(USE_GLES)
-		if (UseOpenGL) {
-			memset(MinimapSurfaceGL, 0, MinimapTextureWidth * MinimapTextureHeight * 4);
-		} else
-#endif
-		{
-			SDL_FillRect(MinimapSurface, NULL, SDL_MapRGB(MinimapSurface->format, 0, 0, 0));
-		}
-	}
-
-	int bpp;
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (UseOpenGL) {
-		bpp = 0;
-	} else
-#endif
-	{
-		bpp = MinimapSurface->format->BytesPerPixel;
+		SDL_FillRect(MinimapSurface, NULL, SDL_MapRGB(MinimapSurface->format, 0, 0, 0));
 	}
 
 	//
 	// Draw the terrain
 	//
 	if (WithTerrain) {
-#if defined(USE_OPENGL) || defined(USE_GLES)
-		if (UseOpenGL) {
-			memcpy(MinimapSurfaceGL, MinimapTerrainSurfaceGL, MinimapTextureWidth * MinimapTextureHeight * 4);
-		} else
-#endif
-		{
-			SDL_BlitSurface(MinimapTerrainSurface, NULL, MinimapSurface, NULL);
-		}
+		SDL_BlitSurface(MinimapTerrainSurface, NULL, MinimapSurface, NULL);
 	}
+	const uint32_t fogColorSDL = FogOfWar->GetFogColorSDL();
+	if (!ReplayRevealMap) {
+		uint32_t *const minimapFog = static_cast<uint32_t *>(MinimapFogSurface->pixels);
+		size_t index = 0;
+		for (uint16_t my = 0; my < H; ++my) {
+			for (uint16_t mx = 0; mx < W; ++mx) {
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (!UseOpenGL)
-#endif
-	{
-		SDL_LockSurface(MinimapSurface);
-		SDL_LockSurface(MinimapTerrainSurface);
-	}
-
-	for (int my = 0; my < H; ++my) {
-		for (int mx = 0; mx < W; ++mx) {
-			int visiontype; // 0 unexplored, 1 explored, >1 visible.
-
-			if (ReplayRevealMap) {
-				visiontype = 2;
-			} else {
 				const Vec2i tilePos(Minimap2MapX[mx], Minimap2MapY[my] / Map.Info.MapWidth);
-				visiontype = Map.Field(tilePos)->playerInfo.TeamVisibilityState(*ThisPlayer);
-			}
+				const uint8_t vis = FogOfWar->GetVisibilityForTile(tilePos); 
 
-			if (visiontype == 0 || (visiontype == 1 && ((mx & 1) != (my & 1)))) {
-#if defined(USE_OPENGL) || defined(USE_GLES)
-				if (UseOpenGL) {
-					*(Uint32 *)&(MinimapSurfaceGL[(mx + my * MinimapTextureWidth) * 4]) = Video.MapRGB(0, 0, 0, 0);
-				} else
-#endif
-				{
-					const int index = mx * bpp + my * MinimapSurface->pitch;
-					if (bpp == 2) {
-						*(Uint16 *)&((Uint8 *)MinimapSurface->pixels)[index] = ColorBlack;
-					} else {
-						*(Uint32 *)&((Uint8 *)MinimapSurface->pixels)[index] = ColorBlack;
-					}
-				}
+				const uint32_t fogAlpha = vis == 0 ? (GameSettings.RevealMap != MapRevealModes::cHidden ? Settings.FogRevealedOpacity : Settings.FogUnseenOpacity)
+											   	   : vis == 1 ? Settings.FogExploredOpacity 
+										   		   			  : Settings.FogVisibleOpacity;
+
+				minimapFog[index++] = fogColorSDL | (fogAlpha << ASHIFT);
 			}
 		}
+		/// Alpha blending the fog of war texture to minimap
+		/// TODO: switch to hardware rendering
+		const SDL_Rect fogRect {0, 0, W, H};
+		BlitSurfaceAlphaBlending_32bpp(MinimapFogSurface, &fogRect, MinimapSurface, &fogRect);
 	}
-
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (!UseOpenGL)
-#endif
-	{
-		SDL_UnlockSurface(MinimapTerrainSurface);
-	}
-
 	//
 	// Draw units on map
 	//
-	for (CUnitManager::Iterator it = UnitManager.begin(); it != UnitManager.end(); ++it) {
+	for (CUnitManager::Iterator it = UnitManager->begin(); it != UnitManager->end(); ++it) {
 		CUnit &unit = **it;
-		if (unit.IsVisibleOnMinimap()) {
+		if (unit.IsVisibleOnMinimap() && !unit.Removed && !unit.Type->BoolFlag[REVEALER_INDEX].value) {
 			DrawUnitOn(unit, red_phase);
 		}
 	}
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (!UseOpenGL)
-#endif
-	{
-		SDL_UnlockSurface(MinimapSurface);
-	}
 }
 
 /**
@@ -634,55 +449,8 @@ static void DrawEvents()
 */
 void CMinimap::Draw() const
 {
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (UseOpenGL) {
-		glBindTexture(GL_TEXTURE_2D, MinimapTexture);
-		glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, MinimapTextureWidth, MinimapTextureHeight,
-						GL_RGBA, GL_UNSIGNED_BYTE, MinimapSurfaceGL);
-
-#ifdef USE_GLES
-		float texCoord[] = {
-			0.0f, 0.0f,
-			(float)W / MinimapTextureWidth, 0.0f,
-			0.0f, (float)H / MinimapTextureHeight,
-			(float)W / MinimapTextureWidth, (float)H / MinimapTextureHeight
-		};
-
-		float vertex[] = {
-			2.0f / (GLfloat)Video.Width *X - 1.0f, -2.0f / (GLfloat)Video.Height *Y + 1.0f,
-			2.0f / (GLfloat)Video.Width *(X + W) - 1.0f, -2.0f / (GLfloat)Video.Height *Y + 1.0f,
-			2.0f / (GLfloat)Video.Width *X - 1.0f, -2.0f / (GLfloat)Video.Height *(Y + H) + 1.0f,
-			2.0f / (GLfloat)Video.Width *(X + W) - 1.0f, -2.0f / (GLfloat)Video.Height *(Y + H) + 1.0f
-		};
-
-		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-		glEnableClientState(GL_VERTEX_ARRAY);
-
-		glTexCoordPointer(2, GL_FLOAT, 0, texCoord);
-		glVertexPointer(2, GL_FLOAT, 0, vertex);
-		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-
-		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-		glDisableClientState(GL_VERTEX_ARRAY);
-#endif
-#ifdef USE_OPENGL
-		glBegin(GL_QUADS);
-		glTexCoord2f(0.0f, 0.0f);
-		glVertex2i(X, Y);
-		glTexCoord2f(0.0f, (float)H / MinimapTextureHeight);
-		glVertex2i(X, Y + H);
-		glTexCoord2f((float)W / MinimapTextureWidth, (float)H / MinimapTextureHeight);
-		glVertex2i(X + W, Y + H);
-		glTexCoord2f((float)W / MinimapTextureWidth, 0.0f);
-		glVertex2i(X + W, Y);
-		glEnd();
-#endif
-	} else
-#endif
-	{
-		SDL_Rect drect = {Sint16(X), Sint16(Y), 0, 0};
-		SDL_BlitSurface(MinimapSurface, NULL, TheScreen, &drect);
-	}
+	SDL_Rect drect = {Sint16(X), Sint16(Y), 0, 0};
+	SDL_BlitSurface(MinimapSurface, NULL, TheScreen, &drect);
 
 	DrawEvents();
 }
@@ -722,26 +490,19 @@ PixelPos CMinimap::TilePosToScreenPos(const Vec2i &tilePos) const
 */
 void CMinimap::Destroy()
 {
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (UseOpenGL) {
-		delete[] MinimapTerrainSurfaceGL;
-		MinimapTerrainSurfaceGL = NULL;
-		if (MinimapSurfaceGL) {
-			glDeleteTextures(1, &MinimapTexture);
-			delete[] MinimapSurfaceGL;
-			MinimapSurfaceGL = NULL;
-		}
-	} else
-#endif
-	{
+	if (MinimapTerrainSurface) {
 		VideoPaletteListRemove(MinimapTerrainSurface);
 		SDL_FreeSurface(MinimapTerrainSurface);
 		MinimapTerrainSurface = NULL;
-		if (MinimapSurface) {
-			VideoPaletteListRemove(MinimapSurface);
-			SDL_FreeSurface(MinimapSurface);
-			MinimapSurface = NULL;
-		}
+	}
+	if (MinimapSurface) {
+		VideoPaletteListRemove(MinimapSurface);
+		SDL_FreeSurface(MinimapSurface);
+		MinimapSurface = NULL;
+	}
+	if (MinimapFogSurface && MinimapFogSurface->format != NULL) {
+		SDL_FreeSurface(MinimapFogSurface);
+		MinimapFogSurface = NULL;
 	}
 	delete[] Minimap2MapX;
 	Minimap2MapX = NULL;
