@@ -196,6 +196,7 @@
 */
 PixelSize PixelTileSize(32, 32);
 
+static const int TILE_PATH_MAX  = 6;
 
 /*----------------------------------------------------------------------------
 -- Functions
@@ -226,8 +227,44 @@ void CTileset::clear()
 
 unsigned int CTileset::getDefaultTileIndex() const
 {
-	// TODO: remove hardcoded value.
-	return 0x50;
+	const int n = tiles.size();
+    int solid = 0;
+    for (int i = 0; i < n;) {
+		const CTile &tile = tiles[i];
+		const CTileInfo &tileinfo = tile.tileinfo;
+		if (tileinfo.BaseTerrain && tileinfo.MixTerrain) {
+			i += 256;
+		} else {
+			if (tileinfo.BaseTerrain != 0 && tileinfo.MixTerrain == 0) {
+				if (tile.flag & MapFieldLandAllowed) {
+					return i;
+				}
+			}
+			i += 16;
+		}
+    }
+    return 0x50;
+}
+
+unsigned int CTileset::getDefaultWoodTileIndex() const
+{
+    const int n = tiles.size();
+    int solid = 0;
+    for (int i = 0; i < n;) {
+	const CTile &tile = tiles[i];
+	const CTileInfo &tileinfo = tile.tileinfo;
+	if (tileinfo.BaseTerrain && tileinfo.MixTerrain) {
+	    i += 256;
+	} else {
+	    if (tileinfo.BaseTerrain != 0 && tileinfo.MixTerrain == 0) {
+		if (tile.flag & MapFieldForest) {
+		    solid = i;
+		}
+	    }
+	    i += 16;
+	}
+    }
+    return solid;
 }
 
 bool CTileset::isAWallTile(unsigned tile) const
@@ -351,8 +388,11 @@ int CTileset::findTilePath(int base, int goal, int length, std::vector<char> &ma
 		*tileIndex = tileres;
 		return length;
 	}
+	if (length >= TILE_PATH_MAX) {
+		return TILE_PATH_MAX;
+	}
 	// Find any mixed tile
-	int l = INT_MAX;
+	int l = TILE_PATH_MAX;
 	for (size_t i = 0; i != tiles.size();) {
 		int j = 0;
 		if (base == tiles[i].tileinfo.BaseTerrain) {
@@ -396,7 +436,8 @@ int CTileset::tileFromQuad(unsigned fixed, unsigned quad) const
 	while (!(type1 = (fixed & 0xFF))) {
 		fixed >>= 8;
 		if (!fixed) {
-			ExitFatal(-1);
+			DebugPrint("WARNING: No fixed tile found for %x" _C_ fixed);
+			return 0;
 		}
 	}
 	fixed >>= 8;
@@ -446,7 +487,7 @@ int CTileset::tileFromQuad(unsigned fixed, unsigned quad) const
 	std::vector<char> marks;
 	marks.resize(getSolidTerrainCount(), 0);
 	marks[type1] = type1;
-	if (findTilePath(type1, type2, 0, marks, &tileIndex) == INT_MAX) {
+	if (findTilePath(type1, type2, 0, marks, &tileIndex) == TILE_PATH_MAX) {
 		DebugPrint("Huch, no mix found!!!!!!!!!!!\n");
 		const int res = findTileIndex(type1);
 		Assert(res != -1);
@@ -564,14 +605,15 @@ unsigned int CTileset::getTileNumber(int basic, bool random, bool filler) const
 				++n;
 			}
 		}
-		n = SyncRand() % n;
+		n = MyRand() % n;
 		int i = -1;
 		do {
 			while (++i < 16 && !tiles[tile + i].tile) {
 			}
 		} while (i < 16 && n--);
-		Assert(i != 16);
-		return tile + i;
+		if (i != 16) {
+			return tile + i;
+		}
 	}
 	if (filler) {
 		int i = 0;
@@ -635,11 +677,16 @@ unsigned CTileset::getQuadFromTile(unsigned int tile) const
 
 void CTileset::fillSolidTiles(std::vector<unsigned int> *solidTiles) const
 {
-	for (size_t i = 16; i < tiles.size(); i += 16) {
-		const CTileInfo &info = tiles[i].tileinfo;
+	std::vector<int> seen_types;
+	seen_types.resize(solidTerrainTypes.size(), 0);
 
+	for (size_t i = 16; i < tiles.size(); i++) {
+		const CTileInfo &info = tiles[i].tileinfo;
 		if (info.BaseTerrain && info.MixTerrain == 0) {
-			solidTiles->push_back(tiles[i].tile);
+			if (seen_types[info.BaseTerrain] == 0) {
+				seen_types[info.BaseTerrain] = 1;
+				solidTiles->push_back(tiles[i].tile);
+			}
 		}
 	}
 }
