@@ -83,13 +83,14 @@ extern NumberDesc *Damage;                   /// Damage calculation for missile.
 */
 void MissileType::LoadMissileSprite()
 {
-	if (this->G && !this->G->IsLoaded()) {
+	if (this->G && !this->G->IsLoaded(this->Flip)) {
 		this->G->Load();
 		if (this->Flip) {
 			this->G->Flip();
 		}
 
 		// Correct the number of frames in graphic
+		DebugPrint("%s - %d>=%d\n" _C_ this->G->File.c_str() _C_ this->G->NumFrames _C_ this->SpriteFrames);
 		Assert(this->G->NumFrames >= this->SpriteFrames);
 		this->G->NumFrames = this->SpriteFrames;
 		// FIXME: Don't use NumFrames as number of frames.
@@ -396,10 +397,11 @@ void FireMissile(CUnit &unit, CUnit *goal, const Vec2i &goalPos)
 	// If Firing from inside a Bunker
 	CUnit *from = GetFirstContainer(unit);
 	const int dir = ((unit.Direction + NextDirection / 2) & 0xFF) / NextDirection;
-	const PixelPos startPixelPos = Map.TilePosToMapPixelPos_Center(from->tilePos)
+	const PixelPos startPixelPos = from->GetMapPixelPosCenter()
 								   + unit.Type->MissileOffsets[dir][0];
 
 	Vec2i dpos;
+	bool divisionOffset = false;
 	if (goal) {
 		Assert(goal->Type);  // Target invalid?
 		// Moved out of attack range?
@@ -415,6 +417,8 @@ void FireMissile(CUnit &unit, CUnit *goal, const Vec2i &goalPos)
 		if (unit.Container) {
 			NearestOfUnit(*goal, GetFirstContainer(unit)->tilePos, &dpos);
 		} else {
+			if (goal->Type->TileWidth % 2 == 0)
+				divisionOffset = true;
 			dpos = goal->tilePos + goal->Type->GetHalfTileSize();
 		}
 	} else {
@@ -423,6 +427,8 @@ void FireMissile(CUnit &unit, CUnit *goal, const Vec2i &goalPos)
 	}
 
 	PixelPos destPixelPos = Map.TilePosToMapPixelPos_Center(dpos);
+	if (divisionOffset)
+		destPixelPos -= PixelTileSize/2;
 	Missile *missile = MakeMissile(*unit.Type->Missile.Missile, startPixelPos, destPixelPos);
 	//
 	// Damage of missile
@@ -489,8 +495,8 @@ static int MissileVisibleInViewport(const CViewport &vp, const Missile &missile)
 void MissileType::DrawMissileType(int frame, const PixelPos &pos) const
 {
 #ifdef DYNAMIC_LOAD
-	if (!this->G->IsLoaded()) {
-		LoadMissileSprite(this);
+	if (!this->G->IsLoaded(this->Flip)) {
+		((MissileType*)this)->LoadMissileSprite();
 	}
 #endif
 
@@ -534,8 +540,8 @@ void Missile::DrawMissile(const CViewport &vp) const
 	// FIXME: I should copy SourcePlayer for second level missiles.
 	if (sunit && sunit->Player) {
 #ifdef DYNAMIC_LOAD
-		if (!this->Type->Sprite) {
-			LoadMissileSprite(this->Type);
+		if (!this->Type->G->IsLoaded(this->Type->Flip)) {
+			((MissileType*)this->Type)->LoadMissileSprite();
 		}
 #endif
 	}
@@ -632,9 +638,10 @@ void Missile::MissileNewHeadingFromXY(const PixelPos &delta)
 **
 **  @return         true if goal is reached, false else.
 */
-bool MissileInitMove(Missile &missile)
+bool MissileInitMove(Missile &missile, bool pointToPoint)
 {
-	const PixelPos heading = missile.destination - missile.position;
+	const PixelPos source = pointToPoint ? missile.source : missile.position;
+	const PixelPos heading = missile.destination - source;
 
 	missile.MissileNewHeadingFromXY(heading);
 	if (!(missile.State & 1)) {
@@ -740,7 +747,7 @@ bool MissileHandleBlocking(Missile &missile, const PixelPos &position)
 */
 bool PointToPointMissile(Missile &missile)
 {
-	MissileInitMove(missile);
+	MissileInitMove(missile, true);
 	if (missile.TotalStep == 0) {
 		return true;
 	}
