@@ -38,6 +38,7 @@
 #include "commands.h"
 
 #include "actions.h"
+#include "fov.h"
 #include "net_message.h"
 #include "network.h"
 #include "replay.h"
@@ -57,7 +58,7 @@
 */
 void SendCommandStopUnit(CUnit &unit)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("stop", &unit, FlushCommands, -1, -1, NoUnitP, NULL, -1);
 		CommandStopUnit(unit);
 	} else {
@@ -73,7 +74,7 @@ void SendCommandStopUnit(CUnit &unit)
 */
 void SendCommandStandGround(CUnit &unit, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("stand-ground", &unit, flush, -1, -1, NoUnitP, NULL, -1);
 		CommandStandGround(unit, flush);
 	} else {
@@ -90,7 +91,7 @@ void SendCommandStandGround(CUnit &unit, int flush)
 */
 void SendCommandDefend(CUnit &unit, CUnit &dest, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("defend", &unit, flush, -1, -1, &dest, NULL, -1);
 		CommandDefend(unit, dest, flush);
 	} else {
@@ -107,7 +108,7 @@ void SendCommandDefend(CUnit &unit, CUnit &dest, int flush)
 */
 void SendCommandFollow(CUnit &unit, CUnit &dest, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("follow", &unit, flush, -1, -1, &dest, NULL, -1);
 		CommandFollow(unit, dest, flush);
 	} else {
@@ -124,7 +125,7 @@ void SendCommandFollow(CUnit &unit, CUnit &dest, int flush)
 */
 void SendCommandMove(CUnit &unit, const Vec2i &pos, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("move", &unit, flush, pos.x, pos.y, NoUnitP, NULL, -1);
 		CommandMove(unit, pos, flush);
 	} else {
@@ -142,7 +143,7 @@ void SendCommandMove(CUnit &unit, const Vec2i &pos, int flush)
 */
 void SendCommandRepair(CUnit &unit, const Vec2i &pos, CUnit *dest, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("repair", &unit, flush, pos.x, pos.y, dest, NULL, -1);
 		CommandRepair(unit, pos, dest, flush);
 	} else {
@@ -158,7 +159,7 @@ void SendCommandRepair(CUnit &unit, const Vec2i &pos, CUnit *dest, int flush)
 */
 void SendCommandAutoRepair(CUnit &unit, int on)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("auto-repair", &unit, FlushCommands, on, -1, NoUnitP, NULL, 0);
 		CommandAutoRepair(unit, on);
 	} else {
@@ -176,7 +177,7 @@ void SendCommandAutoRepair(CUnit &unit, int on)
 */
 void SendCommandAttack(CUnit &unit, const Vec2i &pos, CUnit *attack, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("attack", &unit, flush, pos.x, pos.y, attack, NULL, -1);
 		CommandAttack(unit, pos, attack, flush);
 	} else {
@@ -193,7 +194,7 @@ void SendCommandAttack(CUnit &unit, const Vec2i &pos, CUnit *attack, int flush)
 */
 void SendCommandAttackGround(CUnit &unit, const Vec2i &pos, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("attack-ground", &unit, flush, pos.x, pos.y, NoUnitP, NULL, -1);
 		CommandAttackGround(unit, pos, flush);
 	} else {
@@ -210,7 +211,7 @@ void SendCommandAttackGround(CUnit &unit, const Vec2i &pos, int flush)
 */
 void SendCommandPatrol(CUnit &unit, const Vec2i &pos, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("patrol", &unit, flush, pos.x, pos.y, NoUnitP, NULL, -1);
 		CommandPatrolUnit(unit, pos, flush);
 	} else {
@@ -227,7 +228,7 @@ void SendCommandPatrol(CUnit &unit, const Vec2i &pos, int flush)
 */
 void SendCommandBoard(CUnit &unit, CUnit &dest, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("board", &unit, flush, -1, -1, &dest, NULL, -1);
 		CommandBoard(unit, dest, flush);
 	} else {
@@ -245,7 +246,7 @@ void SendCommandBoard(CUnit &unit, CUnit &dest, int flush)
 */
 void SendCommandUnload(CUnit &unit, const Vec2i &pos, CUnit *what, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("unload", &unit, flush, pos.x, pos.y, what, NULL, -1);
 		CommandUnload(unit, pos, what, flush);
 	} else {
@@ -263,7 +264,7 @@ void SendCommandUnload(CUnit &unit, const Vec2i &pos, CUnit *what, int flush)
 */
 void SendCommandBuildBuilding(CUnit &unit, const Vec2i &pos, CUnitType &what, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("build", &unit, flush, pos.x, pos.y, NoUnitP, what.Ident.c_str(), -1);
 		CommandBuildBuilding(unit, pos, what, flush);
 	} else {
@@ -271,6 +272,22 @@ void SendCommandBuildBuilding(CUnit &unit, const Vec2i &pos, CUnitType &what, in
 	}
 }
 
+/**
+** Send command: Unit explores the map.
+**
+** @param unit     pointer to unit.
+** @param flush    Flag flush all pending commands.
+*/
+void SendCommandExplore(CUnit &unit, int flush)
+{
+	if (IsReplayGame()) {
+		CommandLog("explore", &unit, flush, -1, -1, NoUnitP, NULL, -1);
+		CommandExplore(unit, flush);
+	} else {
+		NetworkSendCommand(MessageCommandExplore, unit, 0, 0, NoUnitP, 0, flush);
+	}
+}
+
 /**
 **  Send command: Cancel this building construction.
 **
@@ -279,7 +296,7 @@ void SendCommandBuildBuilding(CUnit &unit, const Vec2i &pos, CUnitType &what, in
 void SendCommandDismiss(CUnit &unit)
 {
 	// FIXME: currently unit and worker are same?
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("dismiss", &unit, FlushCommands, -1, -1, NULL, NULL, -1);
 		CommandDismiss(unit);
 	} else {
@@ -296,7 +313,7 @@ void SendCommandDismiss(CUnit &unit)
 */
 void SendCommandResourceLoc(CUnit &unit, const Vec2i &pos, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("resource-loc", &unit, flush, pos.x, pos.y, NoUnitP, NULL, -1);
 		CommandResourceLoc(unit, pos, flush);
 	} else {
@@ -313,7 +330,7 @@ void SendCommandResourceLoc(CUnit &unit, const Vec2i &pos, int flush)
 */
 void SendCommandResource(CUnit &unit, CUnit &dest, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("resource", &unit, flush, -1, -1, &dest, NULL, -1);
 		CommandResource(unit, dest, flush);
 	} else {
@@ -330,7 +347,7 @@ void SendCommandResource(CUnit &unit, CUnit &dest, int flush)
 */
 void SendCommandReturnGoods(CUnit &unit, CUnit *goal, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("return", &unit, flush, -1, -1, goal, NULL, -1);
 		CommandReturnGoods(unit, goal, flush);
 	} else {
@@ -347,7 +364,7 @@ void SendCommandReturnGoods(CUnit &unit, CUnit *goal, int flush)
 */
 void SendCommandTrainUnit(CUnit &unit, CUnitType &what, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("train", &unit, flush, -1, -1, NoUnitP, what.Ident.c_str(), -1);
 		CommandTrainUnit(unit, what, flush);
 	} else {
@@ -364,7 +381,7 @@ void SendCommandTrainUnit(CUnit &unit, CUnitType &what, int flush)
 */
 void SendCommandCancelTraining(CUnit &unit, int slot, const CUnitType *type)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("cancel-train", &unit, FlushCommands, -1, -1, NoUnitP,
 				   type ? type->Ident.c_str() : NULL, slot);
 		CommandCancelTraining(unit, slot, type);
@@ -383,7 +400,7 @@ void SendCommandCancelTraining(CUnit &unit, int slot, const CUnitType *type)
 */
 void SendCommandUpgradeTo(CUnit &unit, CUnitType &what, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("upgrade-to", &unit, flush, -1, -1, NoUnitP, what.Ident.c_str(), -1);
 		CommandUpgradeTo(unit, what, flush);
 	} else {
@@ -398,7 +415,7 @@ void SendCommandUpgradeTo(CUnit &unit, CUnitType &what, int flush)
 */
 void SendCommandCancelUpgradeTo(CUnit &unit)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("cancel-upgrade-to", &unit, FlushCommands, -1, -1, NoUnitP, NULL, -1);
 		CommandCancelUpgradeTo(unit);
 	} else {
@@ -416,7 +433,7 @@ void SendCommandCancelUpgradeTo(CUnit &unit)
 */
 void SendCommandResearch(CUnit &unit, CUpgrade &what, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("research", &unit, flush, -1, -1, NoUnitP, what.Ident.c_str(), -1);
 		CommandResearch(unit, what, flush);
 	} else {
@@ -432,7 +449,7 @@ void SendCommandResearch(CUnit &unit, CUpgrade &what, int flush)
 */
 void SendCommandCancelResearch(CUnit &unit)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("cancel-research", &unit, FlushCommands, -1, -1, NoUnitP, NULL, -1);
 		CommandCancelResearch(unit);
 	} else {
@@ -452,7 +469,7 @@ void SendCommandCancelResearch(CUnit &unit)
 */
 void SendCommandSpellCast(CUnit &unit, const Vec2i &pos, CUnit *dest, int spellid, int flush)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("spell-cast", &unit, flush, pos.x, pos.y, dest, NULL, spellid);
 		CommandSpellCast(unit, pos, dest, *SpellTypeTable[spellid], flush);
 	} else {
@@ -470,7 +487,7 @@ void SendCommandSpellCast(CUnit &unit, const Vec2i &pos, CUnit *dest, int spelli
 */
 void SendCommandAutoSpellCast(CUnit &unit, int spellid, int on)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		CommandLog("auto-spell-cast", &unit, FlushCommands, on, -1, NoUnitP, NULL, spellid);
 		CommandAutoSpellCast(unit, spellid, on);
 	} else {
@@ -488,7 +505,7 @@ void SendCommandAutoSpellCast(CUnit &unit, int spellid, int on)
 */
 void SendCommandDiplomacy(int player, int state, int opponent)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		switch (state) {
 			case DiplomacyNeutral:
 				CommandLog("diplomacy", NoUnitP, 0, player, opponent,
@@ -523,7 +540,7 @@ void SendCommandDiplomacy(int player, int state, int opponent)
 */
 void SendCommandSharedVision(int player, bool state, int opponent)
 {
-	if (!IsNetworkGame()) {
+	if (IsReplayGame()) {
 		if (state == false) {
 			CommandLog("shared-vision", NoUnitP, 0, player, opponent,
 					   NoUnitP, "0", -1);
@@ -559,7 +576,7 @@ void SendCommandSharedVision(int player, bool state, int opponent)
 void ExecCommand(unsigned char msgnr, UnitRef unum,
 				 unsigned short x, unsigned short y, UnitRef dstnr)
 {
-	CUnit &unit = UnitManager.GetSlotUnit(unum);
+	CUnit &unit = UnitManager->GetSlotUnit(unum);
 	const Vec2i pos(x, y);
 	const int arg1 = x;
 	const int arg2 = y;
@@ -593,7 +610,7 @@ void ExecCommand(unsigned char msgnr, UnitRef unum,
 			break;
 		case MessageCommandDefend: {
 			if (dstnr != (unsigned short)0xFFFF) {
-				CUnit &dest = UnitManager.GetSlotUnit(dstnr);
+				CUnit &dest = UnitManager->GetSlotUnit(dstnr);
 				Assert(dest.Type);
 				CommandLog("defend", &unit, status, -1, -1, &dest, NULL, -1);
 				CommandDefend(unit, dest, status);
@@ -602,7 +619,7 @@ void ExecCommand(unsigned char msgnr, UnitRef unum,
 		}
 		case MessageCommandFollow: {
 			if (dstnr != (unsigned short)0xFFFF) {
-				CUnit &dest = UnitManager.GetSlotUnit(dstnr);
+				CUnit &dest = UnitManager->GetSlotUnit(dstnr);
 				Assert(dest.Type);
 				CommandLog("follow", &unit, status, -1, -1, &dest, NULL, -1);
 				CommandFollow(unit, dest, status);
@@ -616,7 +633,7 @@ void ExecCommand(unsigned char msgnr, UnitRef unum,
 		case MessageCommandRepair: {
 			CUnit *dest = NoUnitP;
 			if (dstnr != (unsigned short)0xFFFF) {
-				dest = &UnitManager.GetSlotUnit(dstnr);
+				dest = &UnitManager->GetSlotUnit(dstnr);
 				Assert(dest && dest->Type);
 			}
 			CommandLog("repair", &unit, status, pos.x, pos.y, dest, NULL, -1);
@@ -630,7 +647,7 @@ void ExecCommand(unsigned char msgnr, UnitRef unum,
 		case MessageCommandAttack: {
 			CUnit *dest = NoUnitP;
 			if (dstnr != (unsigned short)0xFFFF) {
-				dest = &UnitManager.GetSlotUnit(dstnr);
+				dest = &UnitManager->GetSlotUnit(dstnr);
 				Assert(dest && dest->Type);
 			}
 			CommandLog("attack", &unit, status, pos.x, pos.y, dest, NULL, -1);
@@ -647,7 +664,7 @@ void ExecCommand(unsigned char msgnr, UnitRef unum,
 			break;
 		case MessageCommandBoard: {
 			if (dstnr != (unsigned short)0xFFFF) {
-				CUnit &dest = UnitManager.GetSlotUnit(dstnr);
+				CUnit &dest = UnitManager->GetSlotUnit(dstnr);
 				Assert(dest.Type);
 				CommandLog("board", &unit, status, arg1, arg2, &dest, NULL, -1);
 				CommandBoard(unit, dest, status);
@@ -657,7 +674,7 @@ void ExecCommand(unsigned char msgnr, UnitRef unum,
 		case MessageCommandUnload: {
 			CUnit *dest = NULL;
 			if (dstnr != (unsigned short)0xFFFF) {
-				dest = &UnitManager.GetSlotUnit(dstnr);
+				dest = &UnitManager->GetSlotUnit(dstnr);
 				Assert(dest && dest->Type);
 			}
 			CommandLog("unload", &unit, status, pos.x, pos.y, dest, NULL, -1);
@@ -668,6 +685,10 @@ void ExecCommand(unsigned char msgnr, UnitRef unum,
 			CommandLog("build", &unit, status, pos.x, pos.y, NoUnitP, UnitTypes[dstnr]->Ident.c_str(), -1);
 			CommandBuildBuilding(unit, pos, *UnitTypes[dstnr], status);
 			break;
+		case MessageCommandExplore:
+			CommandLog("explore", &unit, status, -1, -1, NoUnitP, NULL, -1);
+			CommandExplore(unit, status);
+			break;
 		case MessageCommandDismiss:
 			CommandLog("dismiss", &unit, FlushCommands, -1, -1, NULL, NULL, -1);
 			CommandDismiss(unit);
@@ -678,7 +699,7 @@ void ExecCommand(unsigned char msgnr, UnitRef unum,
 			break;
 		case MessageCommandResource: {
 			if (dstnr != (unsigned short)0xFFFF) {
-				CUnit &dest = UnitManager.GetSlotUnit(dstnr);
+				CUnit &dest = UnitManager->GetSlotUnit(dstnr);
 				Assert(dest.Type);
 				CommandLog("resource", &unit, status, -1, -1, &dest, NULL, -1);
 				CommandResource(unit, dest, status);
@@ -686,7 +707,7 @@ void ExecCommand(unsigned char msgnr, UnitRef unum,
 			break;
 		}
 		case MessageCommandReturn: {
-			CUnit *dest = (dstnr != (unsigned short)0xFFFF) ? &UnitManager.GetSlotUnit(dstnr) : NULL;
+			CUnit *dest = (dstnr != (unsigned short)0xFFFF) ? &UnitManager->GetSlotUnit(dstnr) : NULL;
 			CommandLog("return", &unit, status, -1, -1, dest, NULL, -1);
 			CommandReturnGoods(unit, dest, status);
 			break;
@@ -729,7 +750,7 @@ void ExecCommand(unsigned char msgnr, UnitRef unum,
 			if (arg2 != (unsigned short)0xFFFF) {
 				CUnit *dest = NULL;
 				if (dstnr != (unsigned short)0xFFFF) {
-					dest = &UnitManager.GetSlotUnit(dstnr);
+					dest = &UnitManager->GetSlotUnit(dstnr);
 					Assert(dest && dest->Type);
 				}
 				CommandLog("spell-cast", &unit, status, pos.x, pos.y, dest, NULL, id);
@@ -782,6 +803,47 @@ void ExecExtendedCommand(unsigned char type, int status,
 			}
 			CommandSharedVision(arg2, arg3 ? true : false, arg4);
 			break;
+		case ExtendedMessageAutoTargetingDB:
+			/// arg1: 0:true / 1:false
+			if (arg1 == 0 || arg1 == 1) {
+				GameSettings.SimplifiedAutoTargeting = arg1 ? true : false;
+				/// CommandLog(...);
+			} else {
+				/// CommandLog(...);
+			}
+			break;
+		case ExtendedMessageFieldOfViewDB:
+			{
+				/// arg1: 0:cShadowCasting / 1:cSimpleRadial
+				FieldOfViewTypes fovType = arg1 == 0 ? FieldOfViewTypes::cShadowCasting
+										 : arg1 == 1 ? FieldOfViewTypes::cSimpleRadial
+													 : FieldOfViewTypes::NumOfTypes;
+				if (fovType < FieldOfViewTypes::NumOfTypes) {
+					FieldOfView.SetType(fovType);
+					/// CommandLog(...);
+				} else {
+					/// CommandLog(...);
+				}
+			}
+			break;
+		case ExtendedMessageMapFieldsOpacityDB:
+			/// Arg2: Opaque fields flags
+			FieldOfView.SetOpaqueFields(arg2);
+			/// CommandLog(...);
+			break;
+		case ExtendedMessageRevealMapDB:
+			Map.Reveal(static_cast<MapRevealModes>(arg1));
+			/// CommandLog(...);
+			break;
+		case ExtendedMessageFogOfWarDB:
+			/// arg1: enable/disable fog
+			Map.NoFogOfWar = arg1 == 0 ? true : false;
+			/// arg2: request for update 
+			if (arg2 != 0) {
+				UpdateFogOfWarChange();
+			}
+			/// CommandLog(...);
+			break;
 		default:
 			DebugPrint("Unknown extended message %u/%s %u %u %u %u\n" _C_
 					   type _C_ status ? "flush" : "-" _C_
