@@ -240,85 +240,79 @@ size_t CServerSetup::Serialize(unsigned char *buf) const
 {
 	unsigned char *p = buf;
 
-	p += serialize8(p, this->ResourcesOption);
-	p += serialize8(p, this->UnitsOption);
-	p += serialize8(p, this->FogOfWar);
-	p += serialize8(p, this->Inside);
-	p += serialize8(p, this->RevealMap);
-	p += serialize8(p, this->TilesetSelection);
-	p += serialize8(p, this->GameTypeOption);
-	p += serialize8(p, this->Difficulty);
-	p += serialize8(p, this->MapRichness);
-	p += serialize8(p, this->Opponents);
+	p += serialize8(p, static_cast<int8_t>(this->ServerGameSettings.DefeatReveal));
+	p += serialize8(p, static_cast<int8_t>(this->ServerGameSettings.Difficulty));
+	p += serialize8(p, static_cast<int8_t>(this->ServerGameSettings.FoV));
+	p += serialize8(p, static_cast<int8_t>(this->ServerGameSettings.GameType));
+	// Inside is part of the bitfield
+	// NetGameType is not needed
+	// NoFogOfWar is part of the bitfield
+	p += serialize8(p, static_cast<int8_t>(this->ServerGameSettings.NumUnits));
+	p += serialize8(p, static_cast<int8_t>(this->ServerGameSettings.Opponents));
+	p += serialize8(p, static_cast<int8_t>(this->ServerGameSettings.Resources));
+	p += serialize8(p, static_cast<int8_t>(this->ServerGameSettings.RevealMap));
+	// The bitfield contains Inside and NoFogOfWar, as well as game-defined settings
+	p += serialize32(p, this->ServerGameSettings._Bitfield);
+
 	for (int i = 0; i < PlayerMax; ++i) {
-		p += serialize8(p, this->CompOpt[i]);
+		p += serialize8(p, static_cast<int8_t>(this->ServerGameSettings.Presets[i].Race));
+		p += serialize8(p, static_cast<int8_t>(this->ServerGameSettings.Presets[i].PlayerColor));
+		p += serialize8(p, static_cast<int8_t>(this->ServerGameSettings.Presets[i].Team));
+		p += serialize8(p, static_cast<int8_t>(this->ServerGameSettings.Presets[i].Type));
+	}
+	for (int i = 0; i < PlayerMax; ++i) {
+		p += serialize8(p, static_cast<int8_t>(this->CompOpt[i]));
 	}
 	for (int i = 0; i < PlayerMax; ++i) {
 		p += serialize8(p, this->Ready[i]);
 	}
-	for (int i = 0; i < PlayerMax; ++i) {
-		p += serialize8(p, this->Race[i]);
-	}
 	return p - buf;
 }
 
 size_t CServerSetup::Deserialize(const unsigned char *p)
 {
 	const unsigned char *buf = p;
-	p += deserialize8(p, &this->ResourcesOption);
-	p += deserialize8(p, &this->UnitsOption);
-	p += deserialize8(p, &this->FogOfWar);
-	p += deserialize8(p, &this->Inside);
-	p += deserialize8(p, &this->RevealMap);
-	p += deserialize8(p, &this->TilesetSelection);
-	p += deserialize8(p, &this->GameTypeOption);
-	p += deserialize8(p, &this->Difficulty);
-	p += deserialize8(p, &this->MapRichness);
-	p += deserialize8(p, &this->Opponents);
+	p += deserialize8(p, reinterpret_cast<int8_t*>(&this->ServerGameSettings.DefeatReveal));
+	p += deserialize8(p, reinterpret_cast<int8_t*>(&this->ServerGameSettings.Difficulty));
+	p += deserialize8(p, reinterpret_cast<int8_t*>(&this->ServerGameSettings.FoV));
+	p += deserialize8(p, reinterpret_cast<int8_t*>(&this->ServerGameSettings.GameType));
+	// Inside is part of the bitfield
+	// NetGameType is not needed
+	// NoFogOfWar is part of the bitfield
+	p += deserialize8(p, reinterpret_cast<int8_t*>(&this->ServerGameSettings.NumUnits));
+	p += deserialize8(p, reinterpret_cast<int8_t*>(&this->ServerGameSettings.Opponents));
+	p += deserialize8(p, reinterpret_cast<int8_t*>(&this->ServerGameSettings.Resources));
+	p += deserialize8(p, reinterpret_cast<int8_t*>(&this->ServerGameSettings.RevealMap));
+	// The bitfield contains Inside and NoFogOfWar, as well as game-defined settings
+	p += deserialize32(p, reinterpret_cast<uint32_t*>(&this->ServerGameSettings._Bitfield));
+
 	for (int i = 0; i < PlayerMax; ++i) {
-		p += deserialize8(p, &this->CompOpt[i]);
+		p += deserialize8(p, reinterpret_cast<int8_t*>(&this->ServerGameSettings.Presets[i].Race));
+		p += deserialize8(p, reinterpret_cast<int8_t*>(&this->ServerGameSettings.Presets[i].PlayerColor));
+		p += deserialize8(p, reinterpret_cast<int8_t*>(&this->ServerGameSettings.Presets[i].Team));
+		p += deserialize8(p, reinterpret_cast<int8_t*>(&this->ServerGameSettings.Presets[i].Type));
+	}
+	for (int i = 0; i < PlayerMax; ++i) {
+		p += deserialize8(p, reinterpret_cast<int8_t*>(&this->CompOpt[i]));
 	}
 	for (int i = 0; i < PlayerMax; ++i) {
 		p += deserialize8(p, &this->Ready[i]);
 	}
-	for (int i = 0; i < PlayerMax; ++i) {
-		p += deserialize8(p, &this->Race[i]);
-	}
 	return p - buf;
 }
 
 void CServerSetup::Clear()
 {
-	ResourcesOption = 0;
-	UnitsOption = 0;
-	FogOfWar = 0;
-	Inside = 0;
-	RevealMap = 0;
-	TilesetSelection = 0;
-	GameTypeOption = 0;
-	Difficulty = 0;
-	MapRichness = 0;
-	Opponents = 0;
+	ServerGameSettings.Init();
 	memset(CompOpt, 0, sizeof(CompOpt));
 	memset(Ready, 0, sizeof(Ready));
-	memset(Race, 0, sizeof(Race));
 }
 
 bool CServerSetup::operator == (const CServerSetup &rhs) const
 {
-	return (ResourcesOption == rhs.ResourcesOption
-			&& UnitsOption == rhs.UnitsOption
-			&& FogOfWar == rhs.FogOfWar
-			&& Inside == rhs.Inside
-			&& RevealMap == rhs.RevealMap
-			&& TilesetSelection == rhs.TilesetSelection
-			&& GameTypeOption == rhs.GameTypeOption
-			&& Difficulty == rhs.Difficulty
-			&& MapRichness == rhs.MapRichness
-			&& Opponents == rhs.Opponents
+	return (ServerGameSettings == rhs.ServerGameSettings
 			&& memcmp(CompOpt, rhs.CompOpt, sizeof(CompOpt)) == 0
-			&& memcmp(Ready, rhs.Ready, sizeof(Ready)) == 0
-			&& memcmp(Race, rhs.Race, sizeof(Race)) == 0);
+			&& memcmp(Ready, rhs.Ready, sizeof(Ready)) == 0);
 }
 
 //
@@ -388,9 +382,8 @@ const unsigned char *CInitMessage_Config::Serialize() const
 	unsigned char *p = buf;
 
 	p += header.Serialize(p);
-	p += serialize8(p, this->clientIndex);
-	p += serialize8(p, this->hostsCount);
-	for (int i = 0; i != PlayerMax; ++i) {
+	p += serialize8(p, clientIndex);
+	for (int i = 0; i < PlayerMax; ++i) {
 		p += this->hosts[i].Serialize(p);
 	}
 	return buf;
@@ -399,9 +392,8 @@ const unsigned char *CInitMessage_Config::Serialize() const
 void CInitMessage_Config::Deserialize(const unsigned char *p)
 {
 	p += header.Deserialize(p);
-	p += deserialize8(p, &this->clientIndex);
-	p += deserialize8(p, &this->hostsCount);
-	for (int i = 0; i != PlayerMax; ++i) {
+	p += deserialize8(p, &clientIndex);
+	for (int i = 0; i < PlayerMax; ++i) {
 		p += this->hosts[i].Deserialize(p);
 	}
 }
@@ -478,6 +470,7 @@ const unsigned char *CInitMessage_Welcome::Serialize() const
 	for (int i = 0; i < PlayerMax; ++i) {
 		p += this->hosts[i].Serialize(p);
 	}
+	p += serialize16(p, this->NetHostSlot);
 	p += serialize32(p, this->Lag);
 	p += serialize32(p, this->gameCyclesPerUpdate);
 	return buf;
@@ -489,6 +482,7 @@ void CInitMessage_Welcome::Deserialize(const unsigned char *p)
 	for (int i = 0; i < PlayerMax; ++i) {
 		p += this->hosts[i].Deserialize(p);
 	}
+	p += deserialize16(p, &this->NetHostSlot);
 	p += deserialize32(p, &this->Lag);
 	p += deserialize32(p, &this->gameCyclesPerUpdate);
 }
@@ -522,6 +516,53 @@ void CInitMessage_Map::Deserialize(const unsigned char *p)
 	p += deserialize32(p, &this->MapUID);
 }
 
+//
+// CInitMessage_MapFileFragment
+//
+
+CInitMessage_MapFileFragment::CInitMessage_MapFileFragment(uint32_t fragment) :
+	header(MessageInit_FromClient, ICMMapNeeded)
+{
+	this->PathSize = 0;
+	this->DataSize = 0;
+	this->FragmentIndex = fragment;
+}
+
+CInitMessage_MapFileFragment::CInitMessage_MapFileFragment(const char *path, const char *data, uint32_t dataSize, uint32_t fragment) :
+	header(MessageInit_FromServer, ICMMapNeeded)
+{
+	int pathSize = strlen(path);
+	Assert(pathSize <= 256);
+	Assert(sizeof(this->Data) >= pathSize + dataSize);
+	this->PathSize = pathSize;
+	this->DataSize = dataSize;
+	memcpy(this->Data, path, pathSize);
+	memcpy(this->Data + pathSize, data, dataSize);
+	this->FragmentIndex = fragment;
+}
+
+const unsigned char *CInitMessage_MapFileFragment::Serialize() const
+{
+	unsigned char *buf = new unsigned char[Size()];
+	unsigned char *p = buf;
+
+	p += header.Serialize(p);
+	p += serialize32(p, this->FragmentIndex);
+	p += serialize16(p, this->DataSize);
+	p += serialize8(p, this->PathSize);
+	p += serialize(p, this->Data);
+	return buf;
+}
+
+void CInitMessage_MapFileFragment::Deserialize(const unsigned char *p)
+{
+	p += header.Deserialize(p);
+	p += deserialize32(p, &this->FragmentIndex);
+	p += deserialize16(p, &this->DataSize);
+	p += deserialize8(p, &this->PathSize);
+	p += deserialize(p, this->Data);
+}
+
 //
 // CInitMessage_State
 //
@@ -686,8 +727,14 @@ size_t CNetworkCommandQuit::Serialize(unsigned char *buf) const
 size_t CNetworkCommandQuit::Deserialize(const unsigned char *buf)
 {
 	const unsigned char *p = buf;
-	p += deserialize16(p, &this->player);
-	return p - buf;
+	if (p) {
+		p += deserialize16(p, &this->player);
+		return p - buf;
+	} else {
+		// can happen when the other end crashed hard
+		this->player = -1;
+		return 0;
+	}
 }
 
 //
