@@ -51,10 +51,10 @@
 ----------------------------------------------------------------------------*/
 
 struct Node {
-	int CostFromStart;  /// Real costs to reach this point
-	short int CostToGoal;     /// Estimated cost to goal
-	char InGoal;        /// is this point in the goal
-	char Direction;     /// Direction for trace back
+	int32_t CostFromStart;  /// Real costs to reach this point
+	int16_t CostToGoal;     /// Estimated cost to goal
+	int8_t InGoal;        /// is this point in the goal
+	int8_t Direction;     /// Direction for trace back
 };
 
 struct Open {
@@ -88,9 +88,6 @@ const int XY2Heading[3][3] = { {7, 6, 5}, {0, 0, 4}, {1, 2, 3}};
 static Node *AStarMatrix;
 
 /// a list of close nodes, helps to speed up the matrix cleaning
-static int *CloseSet;
-static int CloseSetSize;
-static int Threshold;
 static int OpenSetMaxSize;
 static int AStarMatrixSize;
 #define MAX_CLOSE_SET_RATIO 4
@@ -99,11 +96,15 @@ static int AStarMatrixSize;
 /// see pathfinder.h
 int AStarFixedUnitCrossingCost;// = MaxMapWidth * MaxMapHeight;
 int AStarMovingUnitCrossingCost = 5;
+int AStarMaxSearchIterations = INT_MAX;
 bool AStarKnowUnseenTerrain = false;
 int AStarUnknownTerrainCost = 2;
+/// Used to temporary make enemy units unpassable (needs for correct path lenght calculating for automatic targeting alorithm)
+static bool AStarFixedEnemyUnitsUnpassable = false;
 
 static int AStarMapWidth;
 static int AStarMapHeight;
+static int AStarMapMax;
 
 static int AStarGoalX;
 static int AStarGoalY;
@@ -118,8 +119,9 @@ static Open *OpenSet;
 /// The size of the open node set
 static int OpenSetSize;
 
-static int *CostMoveToCache;
-static const int CacheNotSet = -5;
+static int32_t *CostMoveToCache;
+static int CostMoveToCacheSize;
+static constexpr int CacheNotSet = -1;
 
 /*----------------------------------------------------------------------------
 --  Profile
@@ -255,18 +257,23 @@ void InitAStar(int mapWidth, int mapHeight)
 
 	AStarMapWidth = mapWidth;
 	AStarMapHeight = mapHeight;
+	AStarMapMax =  AStarMapWidth * AStarMapHeight;
 
-	AStarMatrixSize = sizeof(Node) * AStarMapWidth * AStarMapHeight;
-	AStarMatrix = new Node[AStarMapWidth * AStarMapHeight];
+	// align the matrix, the open set, and the cost to move cache
+	// on 64-byte boundary, in case the memset/memmove operations
+	// of the libc we're using has a 128/256/512bit SIMD vector
+	// instruction branch, since we might be clearing 8M of
+	// memory for a 2048x2048 map
+	AStarMatrixSize = sizeof(Node) * AStarMapMax;
+	AStarMatrix = (Node *)aligned_malloc(64, AStarMatrixSize);
 	memset(AStarMatrix, 0, AStarMatrixSize);
 
-	Threshold = AStarMapWidth * AStarMapHeight / MAX_CLOSE_SET_RATIO;
-	CloseSet = new int[Threshold];
+	OpenSetMaxSize = AStarMapMax / MAX_OPEN_SET_RATIO;
+	OpenSet = (Open *)aligned_malloc(64, OpenSetMaxSize * sizeof(Open));
 
-	OpenSetMaxSize = AStarMapWidth * AStarMapHeight / MAX_OPEN_SET_RATIO;
-	OpenSet = new Open[OpenSetMaxSize];
-
-	CostMoveToCache = new int[AStarMapWidth * AStarMapHeight];
+	CostMoveToCacheSize = sizeof(int32_t) * AStarMapMax;
+	CostMoveToCache = (int32_t*)aligned_malloc(64, CostMoveToCacheSize);
+	memset(CostMoveToCache, CacheNotSet, CostMoveToCacheSize);
 
 	for (int i = 0; i < 9; ++i) {
 		Heading2O[i] = Heading2Y[i] * AStarMapWidth;
@@ -280,15 +287,12 @@ void InitAStar(int mapWidth, int mapHeight)
 */
 void FreeAStar()
 {
-	delete[] AStarMatrix;
+	aligned_free(AStarMatrix);
 	AStarMatrix = NULL;
-	delete[] CloseSet;
-	CloseSet = NULL;
-	CloseSetSize = 0;
-	delete[] OpenSet;
+	aligned_free(OpenSet);
 	OpenSet = NULL;
 	OpenSetSize = 0;
-	delete[] CostMoveToCache;
+	aligned_free(CostMoveToCache);
 	CostMoveToCache = NULL;
 
 	ProfilePrint();
@@ -305,63 +309,18 @@ static void AStarPrepare()
 /**
 **  Clean up A*
 */
+static void CostMoveToCacheCleanUp();
 static void AStarCleanUp()
 {
 	ProfileBegin("AStarCleanUp");
-
-	if (CloseSetSize >= Threshold) {
-		AStarPrepare();
-	} else {
-		for (int i = 0; i < CloseSetSize; ++i) {
-			AStarMatrix[CloseSet[i]].CostFromStart = 0;
-			AStarMatrix[CloseSet[i]].InGoal = 0;
-		}
-	}
+	AStarPrepare();
+	CostMoveToCacheCleanUp();
 	ProfileEnd("AStarCleanUp");
 }
 
 static void CostMoveToCacheCleanUp()
 {
-	ProfileBegin("CostMoveToCacheCleanUp");
-	int AStarMapMax =  AStarMapWidth * AStarMapHeight;
-#if 1
-	int *ptr = CostMoveToCache;
-#ifdef __x86_64__
-	union {
-		intptr_t d;
-		int i[2];
-	} conv;
-	conv.i[0] = CacheNotSet;
-	conv.i[1] = CacheNotSet;
-
-	if (((uintptr_t)ptr) & 4) {
-		*ptr++ = CacheNotSet;
-		--AStarMapMax;
-	}
-#endif
-	while (AStarMapMax > 3) {
-#ifdef __x86_64__
-		*((intptr_t *)ptr) = conv.d;
-		*((intptr_t *)(ptr + 2)) = conv.d;
-		ptr += 4;
-#else
-		*ptr++ = CacheNotSet;
-		*ptr++ = CacheNotSet;
-		*ptr++ = CacheNotSet;
-		*ptr++ = CacheNotSet;
-#endif
-		AStarMapMax -= 4;
-	};
-	while (AStarMapMax) {
-		*ptr++ = CacheNotSet;
-		--AStarMapMax;
-	}
-#else
-	for (int i = 0; i < AStarMapMax; ++i) {
-		CostMoveToCache[i] = CacheNotSet;
-	}
-#endif
-	ProfileEnd("CostMoveToCacheCleanUp");
+	memset(CostMoveToCache, CacheNotSet, CostMoveToCacheSize);
 }
 
 /**
@@ -490,16 +449,6 @@ static int AStarFindNode(int eo)
 	return -1;
 }
 
-/**
-**  Add a node to the closed set
-*/
-static void AStarAddToClose(int node)
-{
-	if (CloseSetSize < Threshold) {
-		CloseSet[CloseSetSize++] = node;
-	}
-}
-
 #define GetIndex(x, y) (x) + (y) * AStarMapWidth
 
 /* build-in costmoveto code */
@@ -528,12 +477,18 @@ static int CostMoveToCallBack_Default(unsigned int index, const CUnit &unit)
 			if (flag && (AStarKnowUnseenTerrain || mf->playerInfo.IsExplored(*unit.Player))) {
 				if (flag & ~(MapFieldLandUnit | MapFieldAirUnit | MapFieldSeaUnit)) {
 					// we can't cross fixed units and other unpassable things
+#ifdef DEBUG
+					const_cast<CMapField *>(mf)->lastAStarCost = -1;
+#endif
 					return -1;
 				}
 				CUnit *goal = mf->UnitCache.find(unit_finder);
 				if (!goal) {
 					// Shouldn't happen, mask says there is something on this tile
 					Assert(0);
+#ifdef DEBUG
+					const_cast<CMapField *>(mf)->lastAStarCost = -1;
+#endif
 					return -1;
 				}
 				if (goal->Moving)  {
@@ -542,11 +497,20 @@ static int CostMoveToCallBack_Default(unsigned int index, const CUnit &unit)
 				} else {
 					// for non moving unit Always Fail unless goal is unit, or unit can attack the target
 					if (&unit != goal) {
+						if (GetAStarFixedEnemyUnitsUnpassable() == true) {
+#ifdef DEBUG
+							const_cast<CMapField *>(mf)->lastAStarCost = -1;
+#endif
+							return -1;
+						}
 						if (goal->Player->IsEnemy(unit) && unit.IsAgressive() && CanTarget(*unit.Type, *goal->Type)
 							&& goal->Variable[UNHOLYARMOR_INDEX].Value == 0 && goal->IsVisibleAsGoal(*unit.Player)) {
 								cost += 2 * AStarMovingUnitCrossingCost;
 						} else {
 						// FIXME: Need support for moving a fixed unit to add cost
+#ifdef DEBUG
+							const_cast<CMapField *>(mf)->lastAStarCost = -1;
+#endif
 							return -1;
 						}
 						//cost += AStarFixedUnitCrossingCost;
@@ -560,6 +524,9 @@ static int CostMoveToCallBack_Default(unsigned int index, const CUnit &unit)
 			}
 			// Add tile movement cost
 			cost += mf->getCost();
+#ifdef DEBUG
+			const_cast<CMapField *>(mf)->lastAStarCost = cost;
+#endif
 			++mf;
 		} while (--i);
 		index += AStarMapWidth;
@@ -581,12 +548,19 @@ static int CostMoveToCallBack_Default(unsigned int index, const CUnit &unit)
 */
 static inline int CostMoveTo(unsigned int index, const CUnit &unit)
 {
-	int *c = &CostMoveToCache[index];
+	int32_t *c = &CostMoveToCache[index];
 	if (*c != CacheNotSet) {
-		return *c;
+		// for performance reasons, CostMoveToCache uses -1 to
+		// indicate it is unset, but the algorithm is simpler
+		// if the range of costs is [-1, INT_MAX]. so we always
+		// store everything +1
+		return *c - 1;
 	}
-	*c = CostMoveToCallBack_Default(index, unit);
-	return *c;
+	*c = CostMoveToCallBack_Default(index, unit) + 1;
+#ifdef DEBUG
+	Assert(*c >= 0);
+#endif
+	return *c - 1;
 }
 
 class AStarGoalMarker
@@ -602,7 +576,6 @@ public:
 			AStarMatrix[offset].InGoal = 1;
 			*goal_reachable = true;
 		}
-		AStarAddToClose(offset);
 	}
 private:
 	const CUnit &unit;
@@ -931,10 +904,8 @@ int AStarFindPath(const Vec2i &startPos, const Vec2i &goalPos, int gw, int gh,
 
 	//  Initialize
 	AStarCleanUp();
-	CostMoveToCacheCleanUp();
 
 	OpenSetSize = 0;
-	CloseSetSize = 0;
 
 	if (!AStarMarkGoal(goalPos, gw, gh, tilesizex, tilesizey, minrange, maxrange, unit)) {
 		// goal is not reachable
@@ -958,7 +929,6 @@ int AStarFindPath(const Vec2i &startPos, const Vec2i &goalPos, int gw, int gh,
 		ProfileEnd("AStarFindPath");
 		return ret;
 	}
-	AStarAddToClose(OpenSet[0].O);
 	if (AStarMatrix[eo].InGoal) {
 		ret = PF_REACHED;
 		ProfileEnd("AStarFindPath");
@@ -966,6 +936,8 @@ int AStarFindPath(const Vec2i &startPos, const Vec2i &goalPos, int gw, int gh,
 	}
 	Vec2i endPos;
 
+	int counter = AStarMaxSearchIterations;
+
 	//  Begin search
 	while (1) {
 		// Find the best node of from the open set
@@ -983,17 +955,13 @@ int AStarFindPath(const Vec2i &startPos, const Vec2i &goalPos, int gw, int gh,
 			break;
 		}
 
-#if 0
 		// If we have looked too long, then exit.
 		if (!counter--) {
-			// FIXME: Select a "good" point from the open set.
-			// Nearest point to goal.
+			// TODO: Select a "good" point from the open set.
 			AstarDebugPrint("way too long\n");
-			ret = PF_FAILED;
 			ProfileEnd("AStarFindPath");
-			return ret;
+			return PF_UNREACHABLE;
 		}
-#endif
 
 		// Generate successors of this node.
 
@@ -1044,8 +1012,6 @@ int AStarFindPath(const Vec2i &startPos, const Vec2i &goalPos, int gw, int gh,
 					ProfileEnd("AStarFindPath");
 					return ret;
 				}
-				// we add the point to the close set
-				AStarAddToClose(eo);
 			} else if (new_cost < AStarMatrix[eo].CostFromStart) {
 				// Already visited node, but we have here a better path
 				// I know, it's redundant (but simpler like this)
@@ -1164,4 +1130,13 @@ int GetAStarUnknownTerrainCost()
 	return AStarUnknownTerrainCost;
 }
 
+void SetAStarFixedEnemyUnitsUnpassable(const bool value)
+{
+	AStarFixedEnemyUnitsUnpassable = value;
+}
+
+bool GetAStarFixedEnemyUnitsUnpassable()
+{
+	return AStarFixedEnemyUnitsUnpassable;
+}
 //@}
