@@ -268,6 +268,8 @@ static Target *SelectTargetUnitsOfAutoCast(CUnit &caster, const SpellType &spell
 	// Select all units aroung the caster
 	std::vector<CUnit *> table;
 	SelectAroundUnit(caster, range, table, OutOfMinRange(minRange, caster.tilePos));
+	if (minRange == 0)
+		table.push_back(&caster); // Allow self as target (we check conditions later)
 
 	// Check generic conditions. FIXME: a better way to do this?
 	if (autocast->Combat != CONDITION_TRUE) {
@@ -344,7 +346,7 @@ static Target *SelectTargetUnitsOfAutoCast(CUnit &caster, const SpellType &spell
 			for (size_t i = 0; i != table.size(); ++i) {
 				// Check if unit in battle
 				if (autocast->Attacker == CONDITION_ONLY) {
-					const int range = table[i]->Player->Type == PlayerPerson ? table[i]->Type->ReactRangePerson : table[i]->Type->ReactRangeComputer;
+					const int range = table[i]->Player->Type == PlayerTypes::PlayerPerson ? table[i]->Type->ReactRangePerson : table[i]->Type->ReactRangeComputer;
 					if ((table[i]->CurrentAction() != UnitActionAttack
 						 && table[i]->CurrentAction() != UnitActionAttackGround
 						 && table[i]->CurrentAction() != UnitActionSpellCast)
