@@ -44,6 +44,9 @@
 #include "parameters.h"
 #include "util.h"
 
+#include "SDL.h"
+
+#include <map>
 #include <stdarg.h>
 #include <stdio.h>
 
@@ -191,6 +194,48 @@ int CFile::printf(const char *format, ...)
 	return ret;
 }
 
+static Sint64 sdl_size(SDL_RWops * context) {
+	CFile *self = reinterpret_cast<CFile*>(context->hidden.unknown.data1);
+	long currentPosition = self->tell();
+	self->seek(0, SEEK_END);
+	long size = self->tell();
+	self->seek(currentPosition, SEEK_SET);
+	return size;
+}
+
+static Sint64 sdl_seek(SDL_RWops * context, Sint64 offset, int whence) {
+	CFile *self = reinterpret_cast<CFile*>(context->hidden.unknown.data1);
+	return self->seek(offset, whence);
+}
+
+static size_t sdl_read(SDL_RWops * context, void *ptr, size_t size, size_t maxnum) {
+	CFile *self = reinterpret_cast<CFile*>(context->hidden.unknown.data1);
+	return self->read(ptr, size * maxnum) / size;
+}
+
+static size_t sdl_write(SDL_RWops * context, const void *ptr, size_t size, size_t num) {
+	return 0;
+}
+
+static int sdl_close(SDL_RWops * context) {
+	CFile *self = reinterpret_cast<CFile*>(context->hidden.unknown.data1);
+	free(context);
+	return self->close();
+}
+
+SDL_RWops * CFile::as_SDL_RWops()
+{
+	SDL_RWops *ops = (SDL_RWops *) calloc(1, sizeof(SDL_RWops));
+	ops->type = SDL_RWOPS_UNKNOWN;
+	ops->hidden.unknown.data1 = this;
+	ops->size = sdl_size;
+	ops->seek = sdl_seek;
+	ops->read = sdl_read;
+	ops->write = sdl_write;
+	ops->close = sdl_close;
+	return ops;
+}
+
 //
 //  Implementation.
 //
@@ -512,17 +557,17 @@ static bool FindFileWithExtension(char(&file)[PATH_MAX])
 		return true;
 	}
 #if defined(USE_ZLIB) || defined(USE_BZ2LIB)
-	char buf[PATH_MAX];
+	char buf[PATH_MAX + 4];
 #endif
 #ifdef USE_ZLIB // gzip or bzip2 in global shared directory
-	sprintf(buf, "%s.gz", file);
+	snprintf(buf, PATH_MAX, "%s.gz", file);
 	if (!access(buf, R_OK)) {
 		strcpy_s(file, PATH_MAX, buf);
 		return true;
 	}
 #endif
 #ifdef USE_BZ2LIB
-	sprintf(buf, "%s.bz2", file);
+	snprintf(buf, PATH_MAX, "%s.bz2", file);
 	if (!access(buf, R_OK)) {
 		strcpy_s(file, PATH_MAX, buf);
 		return true;
@@ -631,9 +676,17 @@ static void LibraryFileName(const char *file, char(&buffer)[PATH_MAX])
 
 extern std::string LibraryFileName(const char *file)
 {
-	char buffer[PATH_MAX];
-	LibraryFileName(file, buffer);
-	return buffer;
+	static std::unordered_map<std::string, std::string> FileNameMap;
+	auto result = FileNameMap.find(file);
+	if (result == std::end(FileNameMap)) {
+		char buffer[PATH_MAX];
+		LibraryFileName(file, buffer);
+		std::string r(buffer);
+		FileNameMap[file] = r;
+		return r;
+	} else {
+		return result->second;
+	}
 }
 
 bool CanAccessFile(const char *filename)
@@ -679,8 +732,8 @@ int ReadDataDirectory(const char *dirname, std::vector<FileList> &fl)
 #else
 	strcat_s(buffer, sizeof(buffer), "*.*");
 	struct _finddata_t fileinfo;
-	long hFile = _findfirst(buffer, &fileinfo);
-	if (hFile != -1L) {
+	intptr_t hFile = _findfirst(buffer, &fileinfo);
+	if (hFile != -1) {
 		do {
 			filename = fileinfo.name;
 #endif
@@ -793,4 +846,46 @@ FileWriter *CreateFileWriter(const std::string &filename)
 	}
 }
 
+/**
+ * Quote arguments for usage in calls to system(), popen() and similar.
+ * Really only needed on Windows, where all these calls just concatenate
+ * all arguments with a space and pass the full string to the next process.
+ */
+std::vector<std::string> QuoteArguments(std::vector<std::string> args)
+{
+	std::vector<std::string> outArgs;
+	for (auto arg : args) {
+#ifdef WIN32
+		if (!arg.empty() && arg.find_first_of(" \t\n\v\"") == std::string::npos) {
+			outArgs.push_back(arg);
+		} else {
+			// Windows always needs argument quoting around arguments with spaces
+			std::string ss = "\"";
+			for (auto ch = arg.begin(); ; ch++) {
+				int backslashes = 0;
+				while (ch != arg.end() && *ch == '\\') {
+					ch++;
+					backslashes++;
+				}
+				if (ch == arg.end()) {
+					ss.append(backslashes * 2, '\\');
+					break;
+				} else if (*ch == '"') {
+					ss.append(backslashes * 2 + 1, '\\');
+					ss.push_back(*ch);
+				} else {
+					ss.append(backslashes, '\\');
+					ss.push_back(*ch);
+				}
+			}
+			ss.push_back('"');
+			outArgs.push_back(ss);
+		}
+#else
+		outArgs.push_back(arg);
+#endif
+	}
+	return outArgs;
+}
+
 //@}
