@@ -139,7 +139,7 @@
 **
 **  CPlayer::SharedVision
 **
-**    A bit field which contains shared vision for this player.
+**    Contains shared vision for this player.
 **    Shared vision only works when it's activated both ways. Really.
 **
 **  CPlayer::StartX CPlayer::StartY
@@ -318,10 +318,10 @@ bool NoRescueCheck;               /// Disable rescue check
 /**
 **  Colors used for minimap.
 */
-std::vector<CColor> PlayerColorsRGB[PlayerMax];
-std::vector<IntColor> PlayerColors[PlayerMax];
+std::vector<std::vector<CColor>> PlayerColorsRGB;
+std::vector<std::vector<SDL_Color>> PlayerColorsSDL;
 
-std::string PlayerColorNames[PlayerMax];
+std::vector<std::string> PlayerColorNames;
 
 /**
 **  Which indexes to replace with player color
@@ -333,6 +333,14 @@ int PlayerColorIndexCount;
 --  Functions
 ----------------------------------------------------------------------------*/
 
+/**
+**  Change revelation type
+*/
+void CPlayer::SetRevelationType(const RevealTypes type)
+{
+	CPlayer::RevelationFor = type;
+}
+
 /**
 **  Clean up the PlayerRaces names.
 */
@@ -364,11 +372,8 @@ void InitPlayers()
 {
 	for (int p = 0; p < PlayerMax; ++p) {
 		Players[p].Index = p;
-		if (!Players[p].Type) {
-			Players[p].Type = PlayerNobody;
-		}
-		for (int x = 0; x < PlayerColorIndexCount; ++x) {
-			PlayerColors[p][x] = Video.MapRGB(TheScreen->format, PlayerColorsRGB[p][x]);
+		if (Players[p].Type == PlayerTypes::PlayerUnset) {
+			Players[p].Type = PlayerTypes::PlayerNobody;
 		}
 	}
 }
@@ -379,6 +384,7 @@ void InitPlayers()
 void CleanPlayers()
 {
 	ThisPlayer = NULL;
+	CPlayer::RevealedPlayers.clear();
 	for (unsigned int i = 0; i < PlayerMax; ++i) {
 		Players[i].Clear();
 	}
@@ -389,10 +395,10 @@ void CleanPlayers()
 void FreePlayerColors()
 {
 	for (int i = 0; i < PlayerMax; ++i) {
-		Players[i].UnitColors.Colors.clear();
-		PlayerColorsRGB[i].clear();
-		PlayerColors[i].clear();
+		Players[i].ClearUnitColors();
 	}
+	PlayerColorsRGB.clear();
+	PlayerColorsSDL.clear();
 }
 
 /**
@@ -415,6 +421,25 @@ void SavePlayers(CFile &file)
 	file.printf("SetThisPlayer(%d)\n\n", ThisPlayer->Index);
 }
 
+/**
+ **	Add/remove players to/from list of revealed players
+ */
+void CPlayer::SetRevealed(const bool revealed)
+{
+	if (revealed == this->IsRevealed()) {
+		return;
+	}
+	this->isRevealed = revealed;
+	
+	std::vector<const CPlayer *> &revealedPlayers = CPlayer::RevealedPlayers;
+	if (revealed) {
+		revealedPlayers.push_back(this);
+	} else {
+		/// Remove element from vector;
+		revealedPlayers.erase(std::remove(revealedPlayers.begin(), revealedPlayers.end(), this), 
+							  revealedPlayers.end());
+	}
+}
 
 void CPlayer::Save(CFile &file) const
 {
@@ -423,13 +448,13 @@ void CPlayer::Save(CFile &file) const
 	file.printf("  \"name\", \"%s\",\n", p.Name.c_str());
 	file.printf("  \"type\", ");
 	switch (p.Type) {
-		case PlayerNeutral:       file.printf("\"neutral\",");         break;
-		case PlayerNobody:        file.printf("\"nobody\",");          break;
-		case PlayerComputer:      file.printf("\"computer\",");        break;
-		case PlayerPerson:        file.printf("\"person\",");          break;
-		case PlayerRescuePassive: file.printf("\"rescue-passive\","); break;
-		case PlayerRescueActive:  file.printf("\"rescue-active\","); break;
-		default:                  file.printf("%d,", p.Type); break;
+		case PlayerTypes::PlayerNeutral:       file.printf("\"neutral\",");         break;
+		case PlayerTypes::PlayerNobody:        file.printf("\"nobody\",");          break;
+		case PlayerTypes::PlayerComputer:      file.printf("\"computer\",");        break;
+		case PlayerTypes::PlayerPerson:        file.printf("\"person\",");          break;
+		case PlayerTypes::PlayerRescuePassive: file.printf("\"rescue-passive\","); break;
+		case PlayerTypes::PlayerRescueActive:  file.printf("\"rescue-active\","); break;
+		default:                  file.printf("%d,", (int)p.Type); break;
 	}
 	file.printf(" \"race\", \"%s\",", PlayerRaces.Name[p.Race].c_str());
 	file.printf(" \"ai-name\", \"%s\",\n", p.AiName.c_str());
@@ -445,7 +470,7 @@ void CPlayer::Save(CFile &file) const
 	}
 	file.printf("\", \"shared-vision\", \"");
 	for (int j = 0; j < PlayerMax; ++j) {
-		file.printf("%c", (p.SharedVision & (1 << j)) ? 'X' : '_');
+		file.printf("%c", (p.GaveVisionTo.find(j) != p.GaveVisionTo.end()) ? 'X' : '_');
 	}
 	file.printf("\",\n  \"start\", {%d, %d},\n", p.StartPos.x, p.StartPos.y);
 
@@ -503,6 +528,9 @@ void CPlayer::Save(CFile &file) const
 	// TotalNumUnits done by load units.
 	// NumBuildings done by load units.
 
+	if (p.IsRevealed()) {
+		file.printf(" \"revealed\",");
+	}
 	file.printf(" \"supply\", %d,", p.Supply);
 	file.printf(" \"unit-limit\", %d,", p.UnitLimit);
 	file.printf(" \"building-limit\", %d,", p.BuildingLimit);
@@ -522,6 +550,9 @@ void CPlayer::Save(CFile &file) const
 	file.printf("\n  \"total-razings\", %d,", p.TotalRazings);
 	file.printf("\n  \"total-kills\", %d,", p.TotalKills);
 
+	if (p.LostMainFacilityTimer != 0) {
+		file.printf("\n  \"lost-main-facility-timer\", %d,", p.LostMainFacilityTimer);
+	}
 	file.printf("\n  \"speed-resource-harvest\", {");
 	for (int j = 0; j < MaxCosts; ++j) {
 		if (j) {
@@ -570,7 +601,7 @@ void CPlayer::Save(CFile &file) const
 **
 **  @param type  Player type (Computer,Human,...).
 */
-void CreatePlayer(int type)
+void CreatePlayer(PlayerTypes type)
 {
 	if (NumPlayers == PlayerMax) { // already done for bigmaps!
 		return;
@@ -581,18 +612,18 @@ void CreatePlayer(int type)
 	player.Init(type);
 }
 
-void CPlayer::Init(/* PlayerTypes */ int type)
+void CPlayer::Init(PlayerTypes type)
 {
 	std::vector<CUnit *>().swap(this->Units);
 	std::vector<CUnit *>().swap(this->FreeWorkers);
 
 	//  Take first slot for person on this computer,
 	//  fill other with computer players.
-	if (type == PlayerPerson && !NetPlayers) {
+	if (type == PlayerTypes::PlayerPerson && !NetPlayers) {
 		if (!ThisPlayer) {
 			ThisPlayer = this;
 		} else {
-			type = PlayerComputer;
+			type = PlayerTypes::PlayerComputer;
 		}
 	}
 	if (NetPlayers && NumPlayers == NetLocalPlayerNumber) {
@@ -613,22 +644,22 @@ void CPlayer::Init(/* PlayerTypes */ int type)
 	//  All person players are enemies.
 	int team;
 	switch (type) {
-		case PlayerNeutral:
-		case PlayerNobody:
+		case PlayerTypes::PlayerNeutral:
+		case PlayerTypes::PlayerNobody:
 		default:
 			team = 0;
 			this->SetName("Neutral");
 			break;
-		case PlayerComputer:
+		case PlayerTypes::PlayerComputer:
 			team = 1;
 			this->SetName("Computer");
 			break;
-		case PlayerPerson:
+		case PlayerTypes::PlayerPerson:
 			team = 2 + NumPlayers;
 			this->SetName("Person");
 			break;
-		case PlayerRescuePassive:
-		case PlayerRescueActive:
+		case PlayerTypes::PlayerRescuePassive:
+		case PlayerTypes::PlayerRescueActive:
 			// FIXME: correct for multiplayer games?
 			this->SetName("Computer");
 			team = 2 + NumPlayers;
@@ -646,43 +677,43 @@ void CPlayer::Init(/* PlayerTypes */ int type)
 	//  Calculate enemy/allied mask.
 	for (int i = 0; i < NumPlayers; ++i) {
 		switch (type) {
-			case PlayerNeutral:
-			case PlayerNobody:
+			case PlayerTypes::PlayerNeutral:
+			case PlayerTypes::PlayerNobody:
 			default:
 				break;
-			case PlayerComputer:
+			case PlayerTypes::PlayerComputer:
 				// Computer allied with computer and enemy of all persons.
-				if (Players[i].Type == PlayerComputer) {
+				if (Players[i].Type == PlayerTypes::PlayerComputer) {
 					this->Allied |= (1 << i);
 					Players[i].Allied |= (1 << NumPlayers);
-				} else if (Players[i].Type == PlayerPerson || Players[i].Type == PlayerRescueActive) {
+				} else if (Players[i].Type == PlayerTypes::PlayerPerson || Players[i].Type == PlayerTypes::PlayerRescueActive) {
 					this->Enemy |= (1 << i);
 					Players[i].Enemy |= (1 << NumPlayers);
 				}
 				break;
-			case PlayerPerson:
+			case PlayerTypes::PlayerPerson:
 				// Humans are enemy of all?
-				if (Players[i].Type == PlayerComputer || Players[i].Type == PlayerPerson) {
+				if (Players[i].Type == PlayerTypes::PlayerComputer || Players[i].Type == PlayerTypes::PlayerPerson) {
 					this->Enemy |= (1 << i);
 					Players[i].Enemy |= (1 << NumPlayers);
-				} else if (Players[i].Type == PlayerRescueActive || Players[i].Type == PlayerRescuePassive) {
+				} else if (Players[i].Type == PlayerTypes::PlayerRescueActive || Players[i].Type == PlayerTypes::PlayerRescuePassive) {
 					this->Allied |= (1 << i);
 					Players[i].Allied |= (1 << NumPlayers);
 				}
 				break;
-			case PlayerRescuePassive:
+			case PlayerTypes::PlayerRescuePassive:
 				// Rescue passive are allied with persons
-				if (Players[i].Type == PlayerPerson) {
+				if (Players[i].Type == PlayerTypes::PlayerPerson) {
 					this->Allied |= (1 << i);
 					Players[i].Allied |= (1 << NumPlayers);
 				}
 				break;
-			case PlayerRescueActive:
+			case PlayerTypes::PlayerRescueActive:
 				// Rescue active are allied with persons and enemies of computer
-				if (Players[i].Type == PlayerComputer) {
+				if (Players[i].Type == PlayerTypes::PlayerComputer) {
 					this->Enemy |= (1 << i);
 					Players[i].Enemy |= (1 << NumPlayers);
-				} else if (Players[i].Type == PlayerPerson) {
+				} else if (Players[i].Type == PlayerTypes::PlayerPerson) {
 					this->Allied |= (1 << i);
 					Players[i].Allied |= (1 << NumPlayers);
 				}
@@ -708,13 +739,15 @@ void CPlayer::Init(/* PlayerTypes */ int type)
 	this->NumBuildings = 0;
 	this->Score = 0;
 
-	this->Color = PlayerColors[NumPlayers][0];
+	this->Color = PlayerColorsRGB[NumPlayers][0];
 
-	if (Players[NumPlayers].Type == PlayerComputer || Players[NumPlayers].Type == PlayerRescueActive) {
+	if (Players[NumPlayers].Type == PlayerTypes::PlayerComputer || Players[NumPlayers].Type == PlayerTypes::PlayerRescueActive) {
 		this->AiEnabled = true;
 	} else {
 		this->AiEnabled = false;
 	}
+	this->LostMainFacilityTimer = 0;
+	this->isRevealed = false;
 	++NumPlayers;
 }
 
@@ -738,13 +771,14 @@ void CPlayer::Clear()
 {
 	Index = 0;
 	Name.clear();
-	Type = 0;
+	Type = PlayerTypes::PlayerUnset;
 	Race = 0;
 	AiName.clear();
 	Team = 0;
 	Enemy = 0;
 	Allied = 0;
-	SharedVision = 0;
+	HasVisionFrom.clear();
+	GaveVisionTo.clear();
 	StartPos.x = 0;
 	StartPos.y = 0;
 	memset(Resources, 0, sizeof(Resources));
@@ -772,6 +806,7 @@ void CPlayer::Clear()
 	memset(TotalResources, 0, sizeof(TotalResources));
 	TotalRazings = 0;
 	TotalKills = 0;
+	this->LostMainFacilityTimer = 0;
 	Color = 0;
 	UpgradeTimers.Clear();
 	for (int i = 0; i < MaxCosts; ++i) {
@@ -782,12 +817,19 @@ void CPlayer::Clear()
 	SpeedTrain = SPEEDUP_FACTOR;
 	SpeedUpgrade = SPEEDUP_FACTOR;
 	SpeedResearch = SPEEDUP_FACTOR;
+	this->isRevealed = false;
 }
 
 
 void CPlayer::AddUnit(CUnit &unit)
 {
-	Assert(unit.Player != this);
+	if (unit.Player == this) {
+		return;
+	}
+	if (unit.PlayerSlot != static_cast<size_t>(-1)) {
+		// unit is registered with another player
+		unit.Player->RemoveUnit(unit);
+	}
 	Assert(unit.PlayerSlot == static_cast<size_t>(-1));
 	unit.PlayerSlot = this->Units.size();
 	this->Units.push_back(&unit);
@@ -810,6 +852,36 @@ void CPlayer::RemoveUnit(CUnit &unit)
 	Assert(last == &unit || this->Units[last->PlayerSlot] == last);
 }
 
+std::vector<CUnit *>::const_iterator CPlayer::FreeWorkersBegin() const
+{
+	return FreeWorkers.begin();
+}
+
+std::vector<CUnit *>::iterator CPlayer::FreeWorkersBegin()
+{
+	return FreeWorkers.begin();
+}
+
+std::vector<CUnit *>::const_iterator CPlayer::FreeWorkersEnd() const
+{
+	return FreeWorkers.end();
+}
+
+std::vector<CUnit *>::iterator CPlayer::FreeWorkersEnd()
+{
+	return FreeWorkers.end();
+}
+
+CUnit *CPlayer::GetFreeWorker(int index) const
+{
+	return FreeWorkers[index];
+}
+
+int CPlayer::GetFreeWorkersCount() const
+{
+	return static_cast<int>(FreeWorkers.size());
+}
+
 void CPlayer::UpdateFreeWorkers()
 {
 	FreeWorkers.clear();
@@ -823,7 +895,7 @@ void CPlayer::UpdateFreeWorkers()
 
 	for (int i = 0; i < nunits; ++i) {
 		CUnit &unit = this->GetUnit(i);
-		if (unit.IsAlive() && unit.Type->BoolFlag[HARVESTER_INDEX].value && unit.Type->ResInfo && !unit.Removed) {
+		if (unit.IsAlive() && unit.Type->BoolFlag[HARVESTER_INDEX].value && !unit.Removed) {
 			if (unit.CurrentAction() == UnitActionStill) {
 				FreeWorkers.push_back(&unit);
 			}
@@ -831,7 +903,6 @@ void CPlayer::UpdateFreeWorkers()
 	}
 }
 
-
 std::vector<CUnit *>::const_iterator CPlayer::UnitBegin() const
 {
 	return Units.begin();
@@ -960,7 +1031,7 @@ int CPlayer::GetUnitTotalCount(const CUnitType &type) const
 	for (std::vector<CUnit *>::const_iterator it = this->UnitBegin(); it != this->UnitEnd(); ++it) {
 		CUnit &unit = **it;
 
-		if (unit.CurrentAction() == UnitActionUpgradeTo) {
+		if (GameRunning && unit.CurrentAction() == UnitActionUpgradeTo) {
 			COrder_UpgradeTo &order = dynamic_cast<COrder_UpgradeTo &>(*unit.CurrentOrder());
 			if (order.GetUnitType().Slot == type.Slot) {
 				++count;
@@ -1163,7 +1234,18 @@ void PlayersEachCycle()
 {
 	for (int player = 0; player < NumPlayers; ++player) {
 		CPlayer &p = Players[player];
-
+		if (CPlayer::IsRevelationEnabled()) {
+			if (p.LostMainFacilityTimer && !p.IsRevealed() && p.LostMainFacilityTimer < ((int) GameCycle)) {
+				p.SetRevealed(true);
+				for (int j = 0; j < NumPlayers; ++j) {
+					if (player != j && Players[j].Type != PlayerTypes::PlayerNobody) {
+						Players[j].Notify(_("%s has not rebuilt their base and is being revealed!"), p.Name.c_str());
+					} else {
+						Players[j].Notify("%s", _("You have not rebuilt your base and have been revealed!"));
+					}
+				}
+			}
+		}
 		if (p.AiEnabled) {
 			AiEachCycle(p);
 		}
@@ -1201,17 +1283,18 @@ void PlayersEachSecond(int playerIdx)
 **  @param player  Pointer to player.
 **  @param sprite  The sprite in which the colors should be changed.
 */
-void GraphicPlayerPixels(CPlayer &player, const CGraphic &sprite)
+void GraphicPlayerPixels(int colorIndex, const CGraphic &sprite)
 {
 	Assert(PlayerColorIndexCount);
 
-	SDL_LockSurface(sprite.Surface);
-	std::vector<SDL_Color> sdlColors(player.UnitColors.Colors.begin(), player.UnitColors.Colors.end());
-	SDL_SetColors(sprite.Surface, &sdlColors[0], PlayerColorIndexStart, PlayerColorIndexCount);
+	Assert(SDL_MUSTLOCK(sprite.Surface) == 0);
+	// TODO: This vector allocation is costly in profiles
+	std::vector<SDL_Color> sdlColors = PlayerColorsSDL[colorIndex];
+	Assert(!sprite.Surface->format->palette || sprite.Surface->format->palette->ncolors > PlayerColorIndexStart + PlayerColorIndexCount);
+	SDL_SetPaletteColors(sprite.Surface->format->palette, &sdlColors[0], PlayerColorIndexStart, PlayerColorIndexCount);
 	if (sprite.SurfaceFlip) {
-		SDL_SetColors(sprite.SurfaceFlip, &sdlColors[0], PlayerColorIndexStart, PlayerColorIndexCount);
+		SDL_SetPaletteColors(sprite.SurfaceFlip->format->palette, &sdlColors[0], PlayerColorIndexStart, PlayerColorIndexCount);
 	}
-	SDL_UnlockSurface(sprite.Surface);
 }
 
 /**
@@ -1222,7 +1305,7 @@ void GraphicPlayerPixels(CPlayer &player, const CGraphic &sprite)
 void SetPlayersPalette()
 {
 	for (int i = 0; i < PlayerMax; ++i) {
-		Players[i].UnitColors.Colors = PlayerColorsRGB[i];
+		Players[i].SetUnitColors(PlayerColorsRGB[i]);
 	}
 }
 
@@ -1235,20 +1318,19 @@ void DebugPlayers()
 	DebugPrint("Nr   Color   I Name     Type         Race    Ai\n");
 	DebugPrint("--  -------- - -------- ------------ ------- -----\n");
 	for (int i = 0; i < PlayerMax; ++i) {
-		if (Players[i].Type == PlayerNobody) {
+		if (Players[i].Type == PlayerTypes::PlayerNobody) {
 			continue;
 		}
 		const char *playertype;
 
 		switch (Players[i].Type) {
-			case 0: playertype = "Don't know 0"; break;
-			case 1: playertype = "Don't know 1"; break;
-			case 2: playertype = "neutral     "; break;
-			case 3: playertype = "nobody      "; break;
-			case 4: playertype = "computer    "; break;
-			case 5: playertype = "person      "; break;
-			case 6: playertype = "rescue pas. "; break;
-			case 7: playertype = "rescue akt. "; break;
+			case PlayerTypes::PlayerUnset: playertype = "unset     "; break;
+			case PlayerTypes::PlayerNeutral: playertype = "neutral     "; break;
+			case PlayerTypes::PlayerNobody: playertype = "nobody      "; break;
+			case PlayerTypes::PlayerComputer: playertype = "computer    "; break;
+			case PlayerTypes::PlayerPerson: playertype = "person      "; break;
+			case PlayerTypes::PlayerRescuePassive: playertype = "rescue pas. "; break;
+			case PlayerTypes::PlayerRescueActive: playertype = "rescue akt. "; break;
 			default : playertype = "?unknown?   "; break;
 		}
 		DebugPrint("%2d: %8.8s %c %-8.8s %s %7s %s\n" _C_ i _C_ PlayerColorNames[i].c_str() _C_
@@ -1258,6 +1340,11 @@ void DebugPlayers()
 				   PlayerRaces.Name[Players[i].Race].c_str() _C_
 				   Players[i].AiName.c_str());
 	}
+	DebugPrint("GameSettings\n");
+	DebugPrint("--  -------- - -------- ------------ ------- -----\n");
+	GameSettings.Save(+[](std::string f) {
+		DebugPrint("%s\n" _C_ f.c_str());
+	}, true);
 #endif
 }
 
@@ -1361,14 +1448,26 @@ void CPlayer::SetDiplomacyCrazyWith(const CPlayer &player)
 	this->Allied |= 1 << player.Index;
 }
 
-void CPlayer::ShareVisionWith(const CPlayer &player)
+void CPlayer::ShareVisionWith(CPlayer &player)
 {
-	this->SharedVision |= (1 << player.Index);
+	this->GaveVisionTo.insert(player.Index);
+	player.EnableSharedVisionFrom(*this);
 }
 
-void CPlayer::UnshareVisionWith(const CPlayer &player)
+void CPlayer::EnableSharedVisionFrom(const CPlayer &player)
 {
-	this->SharedVision &= ~(1 << player.Index);
+	this->HasVisionFrom.insert(player.Index);
+}
+
+void CPlayer::UnshareVisionWith(CPlayer &player)
+{
+	this->GaveVisionTo.erase(player.Index);
+	player.DisableSharedVisionFrom(*this);
+}
+
+void CPlayer::DisableSharedVisionFrom(const CPlayer &player)
+{
+	this->HasVisionFrom.erase(player.Index);
 }
 
 
@@ -1393,7 +1492,7 @@ bool CPlayer::IsEnemy(const CUnit &unit) const
 */
 bool CPlayer::IsAllied(const CPlayer &player) const
 {
-	return (Allied & (1 << player.Index)) != 0;
+	return IsAllied(player.Index);
 }
 
 /**
@@ -1407,48 +1506,16 @@ bool CPlayer::IsAllied(const CUnit &unit) const
 
 bool CPlayer::IsVisionSharing() const
 {
-	return SharedVision != 0;
+	return !this->HasVisionFrom.empty();
 }
 
-/**
-**  Check if the player shares vision with the player
-*/
-bool CPlayer::IsSharedVision(const CPlayer &player) const
-{
-	return (SharedVision & (1 << player.Index)) != 0;
-}
-
-/**
-**  Check if the player shares vision with the unit
-*/
-bool CPlayer::IsSharedVision(const CUnit &unit) const
-{
-	return IsSharedVision(*unit.Player);
-}
-
-/**
-**  Check if the both players share vision
-*/
-bool CPlayer::IsBothSharedVision(const CPlayer &player) const
-{
-	return (SharedVision & (1 << player.Index)) != 0
-		   && (player.SharedVision & (1 << Index)) != 0;
-}
-
-/**
-**  Check if the player and the unit share vision
-*/
-bool CPlayer::IsBothSharedVision(const CUnit &unit) const
-{
-	return IsBothSharedVision(*unit.Player);
-}
 
 /**
 **  Check if the player is teamed
 */
 bool CPlayer::IsTeamed(const CPlayer &player) const
 {
-	return Team == player.Team;
+	return this->Team == player.Team;
 }
 
 /**
@@ -1456,7 +1523,17 @@ bool CPlayer::IsTeamed(const CPlayer &player) const
 */
 bool CPlayer::IsTeamed(const CUnit &unit) const
 {
-	return IsTeamed(*unit.Player);
+	return this->IsTeamed(*unit.Player);
+}
+
+void CPlayer::ClearUnitColors()
+{
+	UnitColors.Clear();
+}
+
+void CPlayer::SetUnitColors(std::vector<CColor> &colors)
+{
+	UnitColors.Set(colors);
 }
 
 //@}
