@@ -33,6 +33,16 @@
 --  Includes
 ----------------------------------------------------------------------------*/
 
+#if __has_include(<filesystem>)
+#include <filesystem>
+namespace fs = std::filesystem;
+#elif __has_include(<experimental/filesystem>)
+#include <experimental/filesystem>
+namespace fs = std::experimental::filesystem;
+#else
+error "Missing the <filesystem> header."
+#endif
+
 #include <signal.h>
 
 #include "stratagus.h"
@@ -122,6 +132,9 @@ static int report(int status, bool exitOnError)
 		fprintf(stderr, "%s\n", msg);
 		if (exitOnError) {
 			::exit(1);
+		} else {
+			lua_pushstring(Lua, msg);
+			lua_setglobal(Lua, "__last_error__");
 		}
 		lua_pop(Lua, 1);
 	}
@@ -159,12 +172,37 @@ static int luatraceback(lua_State *L)
 int LuaCall(int narg, int clear, bool exitOnError)
 {
 	const int base = lua_gettop(Lua) - narg;  // function index
-	lua_pushcfunction(Lua, luatraceback);  // push traceback function
-	lua_insert(Lua, base);  // put it under chunk and args
+	return LuaCall(Lua, narg, clear ? 0 : LUA_MULTRET, base, exitOnError);
+}
+
+/**
+**  Call a lua function
+**
+**  @param L            Pointer to Lua state
+**  @param narg         Number of arguments
+**  @param nresults     Number of return values
+**  @param base         Stack index of the function to call
+**  @param exitOnError  Exit the program when an error occurs
+**
+**  @return             0 in success, else exit.
+*/
+int LuaCall(lua_State *L, int narg, int nresults, int base, bool exitOnError)
+{
+#if 0
+	lua_getglobal(L, "debug");
+	lua_getfield(L, -1, "traceback");
+	lua_call(L, 0, 1); 
+	const char *str = lua_tostring(L, -1);
+	lua_pop(L, 2);
+	fprintf(stderr, "\n===============\n%s\n\n", str);
+#endif
+
+	lua_pushcfunction(L, luatraceback);  // push traceback function
+	lua_insert(L, base);  // put it under chunk and args
 	signal(SIGINT, laction);
-	const int status = lua_pcall(Lua, narg, (clear ? 0 : LUA_MULTRET), base);
+	const int status = lua_pcall(L, narg, nresults, base);
 	signal(SIGINT, SIG_DFL);
-	lua_remove(Lua, base);  // remove traceback function
+	lua_remove(L, base);  // remove traceback function
 
 	return report(status, exitOnError);
 }
@@ -207,9 +245,9 @@ static bool GetFileContent(const std::string &file, std::string &content)
 **  @param file  File to load and execute
 **  @param nargs Number of arguments that caller has put on the stack
 **
-**  @return      0 for success, else exit.
+**  @return      0 for success, -1 if the file was not found, else exit.
 */
-int LuaLoadFile(const std::string &file, const std::string &strArg)
+int LuaLoadFile(const std::string &file, const std::string &strArg, bool exitOnError)
 {
 	DebugPrint("Loading '%s'\n" _C_ file.c_str());
 
@@ -217,21 +255,29 @@ int LuaLoadFile(const std::string &file, const std::string &strArg)
 	if (GetFileContent(file, content) == false) {
 		return -1;
 	}
-	if (file.rfind(".lua") != -1) {
+	if (file.rfind("stratagus.lua") != -1) {
 		FileChecksums ^= fletcher32(content);
+		DebugPrint("FileChecksums after loading %s: %x\n" _C_ file.c_str() _C_ FileChecksums);
 	}
+	// save the current __file__
+	lua_getglobal(Lua, "__file__");
+
 	const int status = luaL_loadbuffer(Lua, content.c_str(), content.size(), file.c_str());
 
 	if (!status) {
+		lua_pushstring(Lua, fs::absolute(fs::path(file)).generic_u8string().c_str());
+		lua_setglobal(Lua, "__file__");
 		if (!strArg.empty()) {
 			lua_pushstring(Lua, strArg.c_str());
-			LuaCall(1, 1);
+			LuaCall(1, 1, exitOnError);
 		} else {
-			LuaCall(0, 1);
+			LuaCall(0, 1, exitOnError);
 		}
 	} else {
-		report(status, true);
+		report(status, exitOnError);
 	}
+	// restore the old __file__
+	lua_setglobal(Lua, "__file__");
 	return status;
 }
 
@@ -256,9 +302,13 @@ static int CclSavePreferences(lua_State *l)
 */
 static int CclLoad(lua_State *l)
 {
-	LuaCheckArgs(l, 1);
+	const int arg = lua_gettop(l);
+	if (arg < 1 || arg > 2) {
+		LuaError(l, "incorrect argument");
+	}
 	const std::string filename = LibraryFileName(LuaToString(l, 1));
-	if (LuaLoadFile(filename) == -1) {
+	bool exitOnError = arg == 2 ? LuaToBoolean(l, 2) : true;
+	if (LuaLoadFile(filename, "", exitOnError) == -1) {
 		DebugPrint("Load failed: %s\n" _C_ filename.c_str());
 	}
 	return 0;
@@ -316,6 +366,21 @@ int LuaToNumber(lua_State *l, int narg)
 	return static_cast<int>(lua_tonumber(l, narg));
 }
 
+/**
+**  Convert lua number in C float.
+**  It checks also type and exit in case of error.
+**
+**  @param l     Lua state.
+**  @param narg  Argument number.
+**
+**  @return      C number from lua.
+*/
+float LuaToFloat(lua_State *l, int narg)
+{
+	luaL_checktype(l, narg, LUA_TNUMBER);
+	return static_cast<float>(lua_tonumber(l, narg));
+}
+
 /**
 **  Convert lua number in C unsigned int.
 **  It checks also type and exit in case of error.
@@ -767,7 +832,7 @@ NumberDesc *CclParseNumberDesc(lua_State *l)
 			lua_pop(l, 1); // table.
 		} else {
 			lua_pop(l, 1);
-			LuaError(l, "unknow condition '%s'"_C_ key);
+			LuaError(l, "unknow condition '%s'" _C_ key);
 		}
 	} else {
 		LuaError(l, "Parse Error in ParseNumber");
@@ -881,7 +946,7 @@ StringDesc *CclParseStringDesc(lua_State *l)
 			res->D.PlayerName = CclParseNumberDesc(l);
 		} else {
 			lua_pop(l, 1);
-			LuaError(l, "unknow condition '%s'"_C_ key);
+			LuaError(l, "unknow condition '%s'" _C_ key);
 		}
 	} else {
 		LuaError(l, "Parse Error in ParseString");
@@ -1735,6 +1800,10 @@ static int CclInverseVideo(lua_State *l)
 **  @param l  Lua state.
 **
 **  @return   equivalent lua table.
+**
+** Example:
+**
+** <div class="example"><code>u_data = <strong>UnitType</strong>("unit-footman")</code></div>
 */
 static int CclUnitName(lua_State *l)
 {
@@ -1963,8 +2032,8 @@ static int CclFilteredListDirectory(lua_State *l, int type, int mask)
 	if (args < 1 || args > 2) {
 		LuaError(l, "incorrect argument");
 	}
-	const char *userdir = lua_tostring(l, 1);
-	const int rel = args > 1 ? lua_toboolean(l, 2) : 0;
+	const char *userdir = LuaToString(l, 1);
+	const bool rel = args > 1 ? lua_toboolean(l, 2) : false;
 	int n = strlen(userdir);
 
 	int pathtype = 0; // path relative to stratagus dir
@@ -1990,11 +2059,9 @@ static int CclFilteredListDirectory(lua_State *l, int type, int mask)
 	} else if (rel) {
 		std::string dir = LibraryFileName(userdir);
 		snprintf(directory, sizeof(directory), "%s", dir.c_str());
-		lua_pop(l, 1);
 	} else {
 		snprintf(directory, sizeof(directory), "%s/%s", StratagusLibPath.c_str(), userdir);
 	}
-	lua_pop(l, 1);
 	lua_newtable(l);
 	std::vector<FileList> flp;
 	n = ReadDataDirectory(directory, flp);
@@ -2003,7 +2070,7 @@ static int CclFilteredListDirectory(lua_State *l, int type, int mask)
 		if ((flp[i].type & mask) == type) {
 			lua_pushnumber(l, j + 1);
 			lua_pushstring(l, flp[i].name.c_str());
-			lua_settable(l, 1);
+			lua_settable(l, -3);
 			++j;
 		}
 	}
@@ -2065,12 +2132,53 @@ static int CclDebugPrint(lua_State *l)
 	lua_Debug ar;
 	lua_getstack(l, 1, &ar);
 	lua_getinfo(l, "nSl", &ar);
-	fprintf(stdout, "%s:%d: %s: %s", ar.source, ar.currentline, ar.what, LuaToString(l, 1));
+	fprintf(stdout, "%s:%d: %s: %s\n", ar.source, ar.currentline, ar.what, LuaToString(l, 1));
 #endif
 
 	return 1;
 }
 
+/**
+ * Restart the entire game. This function only returns when an error happens.
+ */
+static int CclRestartStratagus(lua_State *l)
+{
+	LuaCheckArgs(l, 0);
+#ifdef WIN32
+	char executable_path[MAX_PATH];
+	memset(executable_path, 0, sizeof(executable_path));
+	GetModuleFileName(NULL, executable_path, sizeof(executable_path)-1);
+#else
+	char *executable_path = const_cast<char*>(OriginalArgv[0].c_str());
+#endif
+	bool insertRestartArgument = true;
+	for (auto arg : OriginalArgv) {
+		if (arg == "-r") {
+			insertRestartArgument = false;
+			break;
+		}
+	}
+	std::vector<std::string> quotedArgs = QuoteArguments(OriginalArgv);
+
+	int newArgc = quotedArgs.size() + (insertRestartArgument ? 2 : 1);
+	char **argv = new char*[newArgc];
+	for (unsigned int i = 0; i < quotedArgs.size(); i++) {
+		argv[i] = const_cast<char*>(quotedArgs[i].c_str());
+	}
+	if (insertRestartArgument) {
+		argv[newArgc - 2] = (char*)"-r";
+	}
+	argv[newArgc - 1] = (char *)0;
+#ifdef WIN32
+	_execv(executable_path, argv);
+#else
+	execvp(executable_path, argv);
+#endif
+	delete[] argv;
+
+	return 0;
+}
+
 /*............................................................................
 ..  Commands
 ............................................................................*/
@@ -2106,9 +2214,11 @@ void InitLua()
 	// For security we don't load all libs
 	static const luaL_Reg lualibs[] = {
 		{"", luaopen_base},
-		//{LUA_LOADLIBNAME, luaopen_package},
 		{LUA_TABLIBNAME, luaopen_table},
-		//{LUA_IOLIBNAME, luaopen_io},
+#ifdef DEBUG
+		{LUA_IOLIBNAME, luaopen_io},
+		{LUA_LOADLIBNAME, luaopen_package},
+#endif
 		{LUA_OSLIBNAME, luaopen_os},
 		{LUA_STRLIBNAME, luaopen_string},
 		{LUA_MATHLIBNAME, luaopen_math},
@@ -2119,10 +2229,29 @@ void InitLua()
 	Lua = luaL_newstate();
 
 	for (const luaL_Reg *lib = lualibs; lib->func; ++lib) {
+#if LUA_VERSION_NUM == 503 || LUA_VERSION_NUM == 502
+		luaL_requiref(Lua, lib->name, lib->func, 1);
+		lua_pop(Lua, 1);
+#else
 		lua_pushcfunction(Lua, lib->func);
 		lua_pushstring(Lua, lib->name);
 		lua_call(Lua, 1, 0);
+#endif
 	}
+#if defined(DEBUG) && !defined(WIN32)
+	static const char* mobdebug =
+#include "./lua/mobdebug.luaheader"
+;
+	int status = luaL_loadbuffer(Lua, mobdebug, strlen(mobdebug), "mobdebug.lua");
+	if (!status) {
+		status = LuaCall(0, 0, false);
+		if (!status) {
+			fprintf(stderr, "mobdebug loaded and available via mobdebug.start()\n");
+			lua_setglobal(Lua, "mobdebug");
+		}
+	}
+	report(status, false);
+#endif
 	tolua_stratagus_open(Lua);
 	lua_settop(Lua, 0);  // discard any results
 }
@@ -2326,11 +2455,15 @@ static std::string SaveGlobal(lua_State *l, bool is_root, std::vector<std::strin
 					key = key + "[" + lua_tostring(l, -1) + "]";
 					lua_pop(l, 1);
 				}
-				lua_pushvalue(l, -1);
 				//res += "if (" + lhsLine + " == nil) then " + lhsLine + " = {} end\n";
-				blockTableNames.push_back(key);
-				res += SaveGlobal(l, false, blockTableNames);
-				blockTableNames.pop_back();
+				if(std::find(blockTableNames.begin(), blockTableNames.end(), key) == blockTableNames.end()
+				 && key != "[]")
+				{
+					lua_pushvalue(l, -1);
+					blockTableNames.push_back(key);
+					res += SaveGlobal(l, false, blockTableNames);
+					blockTableNames.pop_back();
+				}
 			}
 		}
 		lua_pop(l, 1); /* pop the value */
@@ -2410,6 +2543,164 @@ void LoadCcl(const std::string &filename, const std::string &luaArgStr)
 	LuaGarbageCollect();
 }
 
+#ifdef WIN32
+// copied from msdn
+fs::path GetAVolumePath(__in PWCHAR VolumeName)
+{
+    DWORD  CharCount = MAX_PATH + 1;
+    PWCHAR Names     = NULL;
+    PWCHAR NameIdx   = NULL;
+    BOOL   Success   = FALSE;
+
+    for (;;) {
+        Names = (PWCHAR) new BYTE [CharCount * sizeof(WCHAR)];
+
+        if (!Names) {
+            ExitFatal(1);
+        }
+
+        Success = GetVolumePathNamesForVolumeNameW(VolumeName, Names, CharCount, &CharCount);
+        if (Success) {
+            break;
+        }
+
+        if (GetLastError() != ERROR_MORE_DATA) {
+            break;
+        }
+
+        delete [] Names;
+        Names = NULL;
+    }
+
+	for (NameIdx = Names; NameIdx[0] != L'\0'; NameIdx += wcslen(NameIdx) + 1) {	
+		fs::path result(NameIdx);
+		delete [] Names;
+		Names = NULL;
+		return result;
+	}
+
+    return fs::path();
+}
+
+std::vector<fs::path> getVolumes()
+{
+    DWORD  CharCount            = 0;
+    WCHAR  DeviceName[MAX_PATH] = L"";
+    DWORD  Error                = ERROR_SUCCESS;
+    HANDLE FindHandle           = INVALID_HANDLE_VALUE;
+    BOOL   Found                = FALSE;
+    size_t Index                = 0;
+    BOOL   Success              = FALSE;
+    WCHAR  VolumeName[MAX_PATH] = L"";
+
+	int curIndex = 0;
+	std::vector<fs::path> result;
+
+    //  Enumerate all volumes in the system.
+    FindHandle = FindFirstVolumeW(VolumeName, ARRAYSIZE(VolumeName));
+
+    if (FindHandle == INVALID_HANDLE_VALUE) {
+        Error = GetLastError();
+        wprintf(L"FindFirstVolumeW failed with error code %d\n", Error);
+        ExitFatal(1);
+    }
+
+    for (;;) {
+        //  Skip the \\?\ prefix and remove the trailing backslash.
+        Index = wcslen(VolumeName) - 1;
+
+        if (VolumeName[0]     != L'\\' ||
+            VolumeName[1]     != L'\\' ||
+            VolumeName[2]     != L'?'  ||
+            VolumeName[3]     != L'\\' ||
+            VolumeName[Index] != L'\\') {
+            Error = ERROR_BAD_PATHNAME;
+            wprintf(L"FindFirstVolumeW/FindNextVolumeW returned a bad path: %s\n", VolumeName);
+            ExitFatal(1);
+        }
+
+        //  QueryDosDeviceW does not allow a trailing backslash,
+        //  so temporarily remove it.
+        VolumeName[Index] = L'\0';
+        CharCount = QueryDosDeviceW(&VolumeName[4], DeviceName, ARRAYSIZE(DeviceName)); 
+        VolumeName[Index] = L'\\';
+
+        if (CharCount == 0) {
+            Error = GetLastError();
+            wprintf(L"QueryDosDeviceW failed with error code %d\n", Error);
+            ExitFatal(1);
+        }
+
+        wprintf(L"\nFound a device:\n %s", DeviceName);
+        wprintf(L"\nVolume name: %s", VolumeName);
+		fs::path r = GetAVolumePath(VolumeName);
+		if (!r.empty()) {
+			result.push_back(r);
+ 	       	wprintf(L"\nPath: %s", r.wstring().c_str());
+		}
+
+        //  Move on to the next volume.
+        Success = FindNextVolumeW(FindHandle, VolumeName, ARRAYSIZE(VolumeName));
+
+        if (!Success) {
+            Error = GetLastError();
+
+            if (Error != ERROR_NO_MORE_FILES) {
+                wprintf(L"FindNextVolumeW failed with error code %d\n", Error);
+                ExitFatal(1);
+            }
+
+            //  Finished iterating through all the volumes.
+            Error = ERROR_SUCCESS;
+            break;
+        }
+    }
+
+    FindVolumeClose(FindHandle);
+    FindHandle = INVALID_HANDLE_VALUE;
+
+    return result;
+}
+#endif
+
+static int CclListFilesystem(lua_State *l)
+{
+	LuaCheckArgs(l, 1);
+	const char *dir = LuaToString(l, 1);
+
+#ifdef WIN32
+	if (strcmp(dir, "/") == 0) {
+		std::vector<fs::path> vols = getVolumes();
+		lua_newtable(l);	
+		int j = 0;
+		for (auto const& vol: vols) {
+			if (!access(vol.generic_u8string().c_str(), R_OK)) {
+				lua_pushnumber(l, ++j);
+				lua_pushstring(l, vol.generic_u8string().c_str());
+				lua_settable(l, -3);
+			}
+		}
+		return 1;
+	}
+#endif
+
+	lua_newtable(l);
+	int j = 0;
+	for (auto const& dir_entry: fs::directory_iterator(fs::path(dir))) {
+		if ((fs::is_regular_file(dir_entry.path()) || fs::is_directory(dir_entry.path())) && !access(dir_entry.path().generic_u8string().c_str(), R_OK)) {
+			std::string name = dir_entry.path().generic_u8string();
+			if (fs::is_directory(dir_entry.path())) {
+				name += "/";
+			}
+			lua_pushnumber(l, ++j);
+			lua_pushstring(l, name.c_str());
+			lua_settable(l, -3);
+		}
+	}
+
+	return 1;
+}
+
 void ScriptRegister()
 {
 	AliasRegister();
@@ -2419,6 +2710,9 @@ void ScriptRegister()
 	lua_register(Lua, "ListFilesInDirectory", CclListFilesInDirectory);
 	lua_register(Lua, "ListDirsInDirectory", CclListDirsInDirectory);
 
+	// TODO: Only allow this when extractor tool runs
+	// lua_register(Lua, "ListFilesystem", CclListFilesystem);
+
 	lua_register(Lua, "SetDamageFormula", CclSetDamageFormula);
 
 	lua_register(Lua, "SavePreferences", CclSavePreferences);
@@ -2426,6 +2720,8 @@ void ScriptRegister()
 	lua_register(Lua, "LoadBuffer", CclLoadBuffer);
 
 	lua_register(Lua, "DebugPrint", CclDebugPrint);
+
+	lua_register(Lua, "RestartStratagus", CclRestartStratagus);
 }
 
 //@}
