@@ -94,8 +94,8 @@ void CPlayer::Load(lua_State *l)
 {
 	const int args = lua_gettop(l);
 
-	this->Units.resize(0);
-	this->FreeWorkers.resize(0);
+	this->Units.clear();
+	this->FreeWorkers.clear();
 
 	// j = 0 represent player Index.
 	for (int j = 1; j < args; ++j) {
@@ -107,17 +107,17 @@ void CPlayer::Load(lua_State *l)
 		} else if (!strcmp(value, "type")) {
 			value = LuaToString(l, j + 1);
 			if (!strcmp(value, "neutral")) {
-				this->Type = PlayerNeutral;
+				this->Type = PlayerTypes::PlayerNeutral;
 			} else if (!strcmp(value, "nobody")) {
-				this->Type = PlayerNobody;
+				this->Type = PlayerTypes::PlayerNobody;
 			} else if (!strcmp(value, "computer")) {
-				this->Type = PlayerComputer;
+				this->Type = PlayerTypes::PlayerComputer;
 			} else if (!strcmp(value, "person")) {
-				this->Type = PlayerPerson;
+				this->Type = PlayerTypes::PlayerPerson;
 			} else if (!strcmp(value, "rescue-passive")) {
-				this->Type = PlayerRescuePassive;
+				this->Type = PlayerTypes::PlayerRescuePassive;
 			} else if (!strcmp(value, "rescue-active")) {
-				this->Type = PlayerRescueActive;
+				this->Type = PlayerTypes::PlayerRescueActive;
 			} else {
 				LuaError(l, "Unsupported tag: %s" _C_ value);
 			}
@@ -152,10 +152,13 @@ void CPlayer::Load(lua_State *l)
 		} else if (!strcmp(value, "shared-vision")) {
 			value = LuaToString(l, j + 1);
 			for (int i = 0; i < PlayerMax && *value; ++i, ++value) {
+				if (i == this->Index) {
+					continue;
+				}
 				if (*value == '-' || *value == '_' || *value == ' ') {
-					this->SharedVision &= ~(1 << i);
+					this->UnshareVisionWith(Players[i]);
 				} else {
-					this->SharedVision |= (1 << i);
+					this->ShareVisionWith(Players[i]);
 				}
 			}
 		} else if (!strcmp(value, "start")) {
@@ -235,6 +238,9 @@ void CPlayer::Load(lua_State *l)
 		} else if (!strcmp(value, "ai-disabled")) {
 			this->AiEnabled = false;
 			--j;
+		} else if (!strcmp(value, "revealed")) {
+			this->SetRevealed(true); 
+			--j;	
 		} else if (!strcmp(value, "supply")) {
 			this->Supply = LuaToNumber(l, j + 1);
 		} else if (!strcmp(value, "demand")) {
@@ -255,6 +261,8 @@ void CPlayer::Load(lua_State *l)
 			this->TotalRazings = LuaToNumber(l, j + 1);
 		} else if (!strcmp(value, "total-kills")) {
 			this->TotalKills = LuaToNumber(l, j + 1);
+		} else if (!strcmp(value, "lost-main-facility-timer")) {
+			this->LostMainFacilityTimer = LuaToNumber(l, j + 1);
 		} else if (!strcmp(value, "total-resources")) {
 			if (!lua_istable(l, j + 1)) {
 				LuaError(l, "incorrect argument");
@@ -328,33 +336,176 @@ void CPlayer::Load(lua_State *l)
 }
 
 /**
-**  Change unit owner
+** <b>Description</b>
+**
+**  Change all units owned by one player or change only specific units owned by one player
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code>-- Changes all units owned by player 0 and give to player 1
+**		<strong>ChangeUnitsOwner</strong>({16, 17}, {30, 32}, 0, 1)
+**		-- Changes all farms owned by player 0 and give to player 1
+**		<strong>ChangeUnitsOwner</strong>({16, 17}, {30, 32}, 0, 1, "unit-farm")</code></div>
+**
 */
 static int CclChangeUnitsOwner(lua_State *l)
 {
-	LuaCheckArgs(l, 4);
+    int args = lua_gettop(l);
+    if (args != 4 && args != 5) {
+        LuaError(l, "incorrect argument count, need 4 or 5 args");
+    }
 
-	Vec2i pos1;
-	Vec2i pos2;
-	CclGetPos(l, &pos1.x, &pos1.y, 1);
-	CclGetPos(l, &pos2.x, &pos2.y, 2);
-	const int oldp = LuaToNumber(l, 3);
-	const int newp = LuaToNumber(l, 4);
-	std::vector<CUnit *> table;
-
-	Select(pos1, pos2, table, HasSamePlayerAs(Players[oldp]));
-	for (size_t i = 0; i != table.size(); ++i) {
-		table[i]->ChangeOwner(Players[newp]);
+    Vec2i pos1;
+    Vec2i pos2;
+    CclGetPos(l, &pos1.x, &pos1.y, 1);
+    CclGetPos(l, &pos2.x, &pos2.y, 2);
+	if (pos1.x > pos2.x) {
+		std::swap(pos1.x, pos2.x);
 	}
-	return 0;
+	if (pos1.y > pos2.y) {
+		std::swap(pos1.y, pos2.y);
+	}
+
+    const int oldp = LuaToNumber(l, 3);
+    const int newp = LuaToNumber(l, 4);
+    std::vector<CUnit *> table;
+	// Change all units
+    if (args == 4) {
+        Select(pos1, pos2, table, HasSamePlayerAs(Players[oldp]));
+    } else { //Change only specific units by the type.
+        CUnitType *type = UnitTypeByIdent(LuaToString(l, 5));
+        Select(pos1, pos2, table, HasSamePlayerAndTypeAs(Players[oldp], *type));
+    }
+    for (auto unit : table) {
+        unit->ChangeOwner(Players[newp]);
+    }
+    return 0;
 }
 
 /**
+** <b>Description</b>
+**
+**  <strong>GiveUnitsToPlayer(amount, type, fromPlayer, toPlayer)</strong>
+**  <strong>GiveUnitsToPlayer(amount, type, topLeft, bottomRight, fromPlayer, toPlayer)</strong>
+**  Give some units of a specific type from a player to another player. Optionally only inside a rectangle.
+**  Returns number of units actually assigned. This can be smaller than the requested amount if the
+**  <code>fromPlayer</code> did not have enough units.<br/>
+**
+**  Instead of a number you can pass "all" as the first argument, to hand over all units.<br/>
+**  
+**  Instead of a unit type name, you can pass "any", "unit", "building" as the second argument,
+**  to hand over anything, and unit, or any building.
+**
+**  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code>
+**   -- Give 2 peasants from player 4 to player 2
+**   GiveUnitsToPlayer(2, "unit-peasant", 4, 2)
+**   -- Give 4 knights from player 5 to player 1 inside the rectangle 2,2 - 14,14
+**   GiveUnitsToPlayer(2, "unit-peasant", {2,2}, {14,14}, 4, 2)
+**   -- Give any 4 units from player 5 to player 1 inside the rectangle 2,2 - 14,14
+**   GiveUnitsToPlayer(2, "any", 4, 2)
+** </code></div>
+**
+*/
+static int CclGiveUnitsToPlayer(lua_State *l)
+{
+	int args = lua_gettop(l);
+	if (args != 4 && args != 6) {
+		LuaError(l, "incorrect argument count for GiveUnitsToPlayer, need 4 or 6 args");
+	}
+
+	int cnt;
+	if (lua_isnumber(l, 1)) {
+		cnt = LuaToNumber(l, 1);
+	} else {
+		std::string cntStr = std::string(LuaToString(l, 1));
+		if (cntStr != "all") {
+			LuaError(l, "incorrect 1st argument to GiveUnitsToPlayer. Must be number or 'all'");
+		}
+		cnt = std::numeric_limits<int>::max();
+	}
+
+	const int oldp = LuaToNumber(l, args == 4 ? 3 : 5);
+	const int newp = LuaToNumber(l, args == 4 ? 4 : 6);
+
+	std::string typestr = std::string(LuaToString(l, 2));
+	int assignedCnt = 0;
+
+	CUnitType *type = nullptr;
+	bool any = false;
+	bool onlyUnits = false;
+	bool onlyBuildings = false;
+	if ((any = (typestr == "any"))) {
+	} else if ((onlyUnits = (typestr == "unit"))) {
+	} else if ((onlyBuildings = (typestr == "building"))) {
+	} else {
+		type = UnitTypeByIdent(LuaToString(l, 2));
+		if (!type) {
+			LuaError(l, "incorrect 2nd argument to GiveUnitsToPlayer. Must be a unit type or 'any', 'unit', or 'building'");
+		}
+	}
+
+	if (cnt > 0) {
+		std::vector<CUnit *> table;
+		if (args == 6) {
+			Vec2i pos1;
+			Vec2i pos2;
+			CclGetPos(l, &pos1.x, &pos1.y, 3);
+			CclGetPos(l, &pos2.x, &pos2.y, 4);
+			if (pos1.x > pos2.x) {
+				std::swap(pos1.x, pos2.x);
+			}
+			if (pos1.y > pos2.y) {
+				std::swap(pos1.y, pos2.y);
+			}
+			if (any) {
+				Select(pos1, pos2, table, HasSamePlayerAs(Players[oldp]));
+			} else if (onlyUnits) {
+				Select(pos1, pos2, table, AndPredicate(HasSamePlayerAs(Players[oldp]), NotPredicate(IsBuildingType())));
+			} else if (onlyBuildings) {
+				Select(pos1, pos2, table, AndPredicate(HasSamePlayerAs(Players[oldp]), IsBuildingType()));
+			} else {
+				Select(pos1, pos2, table, HasSamePlayerAndTypeAs(Players[oldp], *type));
+			}
+			for (size_t i = 0; i != table.size() && cnt > 0; ++i) {
+				table[i]->ChangeOwner(Players[newp]);
+				assignedCnt++;
+				cnt--;
+			}
+		} else {
+			std::vector<CUnit *> table;
+			for (std::vector<CUnit *>::const_iterator it = Players[oldp].UnitBegin(); it != Players[oldp].UnitEnd() && cnt > 0; ++it) {
+				CUnit *unit = *it;
+				if (any || (onlyUnits && !unit->Type->Building) || (onlyBuildings && unit->Type->Building) || (type == unit->Type)) {
+					table.push_back(unit);
+				}
+			}
+			for (auto unit : table) {
+				unit->ChangeOwner(Players[newp]);
+			}
+			assignedCnt = table.size();
+		}
+	}
+
+	lua_pushnumber(l, assignedCnt);
+	return 1;
+}
+
+/**
+** <b>Description</b>
+**
 **  Get ThisPlayer.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>GetThisPlayer</strong>()</code></div>
 */
 static int CclGetThisPlayer(lua_State *l)
 {
@@ -368,6 +519,8 @@ static int CclGetThisPlayer(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Set ThisPlayer.
 **
 **  @param l  Lua state.
@@ -383,9 +536,20 @@ static int CclSetThisPlayer(lua_State *l)
 }
 
 /**
-**  Set MaxSelectable
+** <b>Description</b>
+**
+**  Set the maximum amount of units that can be selected.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code>-- 9 units can be selected together.
+**		  <strong>SetMaxSelectable</strong>(9)
+**		  -- 18 units can be selected together.
+**		  <strong>SetMaxSelectable</strong>(18)
+**		  -- 50 units can be selected together.
+**		  <strong>SetMaxSelectable</strong>(50)</code></div>
 */
 static int CclSetMaxSelectable(lua_State *l)
 {
@@ -397,9 +561,15 @@ static int CclSetMaxSelectable(lua_State *l)
 }
 
 /**
-**  Set player unit limit.
+** <b>Description</b>
+**
+**  Set players units limit.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>SetAllPlayersUnitLimit</strong>(200)</code></div>
 */
 static int CclSetAllPlayersUnitLimit(lua_State *l)
 {
@@ -413,9 +583,15 @@ static int CclSetAllPlayersUnitLimit(lua_State *l)
 }
 
 /**
-**  Set player unit limit.
+** <b>Description</b>
+**
+**  Set players buildings limit.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>SetAllPlayersBuildingLimit</strong>(200)</code></div>
 */
 static int CclSetAllPlayersBuildingLimit(lua_State *l)
 {
@@ -429,9 +605,15 @@ static int CclSetAllPlayersBuildingLimit(lua_State *l)
 }
 
 /**
-**  Set player unit limit.
+** <b>Description</b>
+**
+**  Set players total units limit.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>SetAllPlayersTotalUnitLimit</strong>(400)</code></div>
 */
 static int CclSetAllPlayersTotalUnitLimit(lua_State *l)
 {
@@ -445,11 +627,21 @@ static int CclSetAllPlayersTotalUnitLimit(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Change the diplomacy from player to another player.
 **
 **  @param l  Lua state.
 **
-**  @return          FIXME: should return old state.
+**  @return FIXME: should return old state.
+**
+** Example:
+**
+** <div class="example"><code><strong>SetDiplomacy</strong>(0,"allied",1)
+**		<strong>SetDiplomacy</strong>(1,"allied",0)
+**		
+**		<strong>SetDiplomacy</strong>(0,"enemy",2)
+**		<strong>SetDiplomacy</strong>(1,"enemy",2)</code></div>
 */
 static int CclSetDiplomacy(lua_State *l)
 {
@@ -483,11 +675,46 @@ static int CclDiplomacy(lua_State *l)
 }
 
 /**
+**  Get diplomacy from one player to another. Returns the strings "allied",
+**  "enemy", "neutral", or "crazy".
+*/
+static int CclGetDiplomacy(lua_State *l)
+{
+	LuaCheckArgs(l, 2);
+	const int base = LuaToNumber(l, 1);
+	const int plynr = LuaToNumber(l, 2);
+
+	if (Players[base].IsEnemy(plynr)) {
+		if (Players[base].IsAllied(plynr)) {
+			lua_pushstring(l, "crazy");
+		} else {
+			lua_pushstring(l, "enemy");
+		}
+	} else if (Players[base].IsAllied(plynr)) {
+		lua_pushstring(l, "allied");
+	} else {
+		lua_pushstring(l, "neutral");
+	}
+	return 1;
+}
+
+/**
+** <b>Description</b>
+**
 **  Change the shared vision from player to another player.
 **
+**
 **  @param l  Lua state.
 **
-**  @return   FIXME: should return old state.
+**  @return FIXME: should return old state.
+**
+** Example:
+**
+** <div class="example"><code><strong>SetSharedVision</strong>(0,true,1)
+**		<strong>SetSharedVision</strong>(1,true,0)
+**		
+**		<strong>SetSharedVision</strong>(0,false,2)
+**		<strong>SetSharedVision</strong>(1,false,2)</code></div>
 */
 static int CclSetSharedVision(lua_State *l)
 {
@@ -515,6 +742,41 @@ static int CclSharedVision(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
+**  Change the players revelation type - reveal all units, only buidings or don't reveal anything
+**
+**  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>SetRevelationType</strong>("no-revelation")
+** 		<strong>SetRevelationType</strong>("buildings-only")
+** 		<strong>SetRevelationType</strong>("all-units")</code></div>
+**
+*/
+static int CclSetRevelationType(lua_State *l)
+{
+	LuaCheckArgs(l, 1);
+
+	const char *revel_type = LuaToString(l, 1);
+	if (!strcmp(revel_type, "no-revelation")) {
+		CPlayer::SetRevelationType(RevealTypes::cNoRevelation);
+	} else if (!strcmp(revel_type, "all-units")) {
+		CPlayer::SetRevelationType(RevealTypes::cAllUnits);
+	} else if (!strcmp(revel_type, "buildings-only")) {
+		CPlayer::SetRevelationType(RevealTypes::cBuildingsOnly);
+	} else {
+		PrintFunction();
+		fprintf(stdout, "Accessible types of revelation \"no-revelation\", \"all-units\" and \"buildings-only\".\n");
+		return 1;
+	}
+	return 0;
+}
+
+/**
+** <b>Description</b>
+**
 **  Define race names in addition to those already there.
 **
 **  @param l  Lua state.
@@ -564,20 +826,59 @@ static int CclDefineRaceNames(lua_State *l)
 }
 
 /**
-**  Define player colors
+** <b>Description</b>
+**
+**  Define player colors. Pass "false" as an optional second
+**  argument to add the colors to the existing ones.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>DefinePlayerColors</strong>({
+**		"red", {{164, 0, 0}, {124, 0, 0}, {92, 4, 0}, {68, 4, 0}},
+**		"blue", {{12, 72, 204}, {4, 40, 160}, {0, 20, 116}, {0, 4, 76}},
+**		"green", {{44, 180, 148}, {20, 132, 92}, {4, 84, 44}, {0, 40, 12}},
+**		"violet", {{152, 72, 176}, {116, 44, 132}, {80, 24, 88}, {44, 8, 44}},
+**		"orange", {{248, 140, 20}, {200, 96, 16}, {152, 60, 16}, {108, 32, 12}},
+**		"black", {{40, 40, 60}, {28, 28, 44}, {20, 20, 32}, {12, 12, 20}},
+**		"white", {{224, 224, 224}, {152, 152, 180}, {84, 84, 128}, {36, 40, 76}},
+**		"yellow", {{252, 252, 72}, {228, 204, 40}, {204, 160, 16}, {180, 116, 0}},
+**		"red", {{164, 0, 0}, {124, 0, 0}, {92, 4, 0}, {68, 4, 0}},
+**		"blue", {{12, 72, 204}, {4, 40, 160}, {0, 20, 116}, {0, 4, 76}},
+**		"green", {{44, 180, 148}, {20, 132, 92}, {4, 84, 44}, {0, 40, 12}},
+**		"violet", {{152, 72, 176}, {116, 44, 132}, {80, 24, 88}, {44, 8, 44}},
+**		"orange", {{248, 140, 20}, {200, 96, 16}, {152, 60, 16}, {108, 32, 12}},
+**		"black", {{40, 40, 60}, {28, 28, 44}, {20, 20, 32}, {12, 12, 20}},
+**		"white", {{224, 224, 224}, {152, 152, 180}, {84, 84, 128}, {36, 40, 76}},
+**		"yellow", {{252, 252, 72}, {228, 204, 40}, {204, 160, 16}, {180, 116, 0}},
+**	})</code></div>
 */
 static int CclDefinePlayerColors(lua_State *l)
 {
-	LuaCheckArgs(l, 1);
+	int nargs = lua_gettop(l);
+	if (nargs < 1 || nargs > 2) {
+		LuaError(l, "wrong number of arguments");
+	}
 	if (!lua_istable(l, 1)) {
 		LuaError(l, "incorrect argument");
 	}
-
+	bool defaultNeutralPlayer = false;
 	const int args = lua_rawlen(l, 1);
+	if (nargs < 2 || LuaToBoolean(l, 2)) {
+		PlayerColorNames.clear();
+		PlayerColorsRGB.clear();
+		PlayerColorsSDL.clear();
+		if (args / 2 < PlayerMax - 1) { // accept no color for neutral player
+			LuaError(l, "You need to define at least %d colors" _C_ PlayerMax - 1);
+		}
+		if (args / 2 < PlayerMax) {
+			defaultNeutralPlayer = true;
+		}
+	}
+
 	for (int i = 0; i < args; ++i) {
-		PlayerColorNames[i / 2] = LuaToString(l, 1, i + 1);
+		PlayerColorNames.push_back(LuaToString(l, 1, i + 1));
 		++i;
 		lua_rawgeti(l, 1, i + 1);
 		if (!lua_istable(l, -1)) {
@@ -587,15 +888,31 @@ static int CclDefinePlayerColors(lua_State *l)
 		if (numcolors != PlayerColorIndexCount) {
 			LuaError(l, "You should use %d colors (See DefinePlayerColorIndex())" _C_ PlayerColorIndexCount);
 		}
+		std::vector<CColor> newColors;
 		for (int j = 0; j < numcolors; ++j) {
 			lua_rawgeti(l, -1, j + 1);
-			PlayerColorsRGB[i / 2][j].Parse(l);
+			CColor newColor;
+			newColor.Parse(l);
+			newColors.push_back(newColor);
 			lua_pop(l, 1);
 		}
+		PlayerColorsRGB.push_back(newColors);
+		PlayerColorsSDL.push_back(std::vector<SDL_Color>(newColors.begin(), newColors.end()));
+	}
+
+	if (defaultNeutralPlayer) {
+		PlayerColorNames.push_back("neutral-black");
+		std::vector<CColor> neutralColors;
+		for (int j = 0; j < PlayerColorIndexCount; ++j) {
+			CColor neutralColor;
+			neutralColors.push_back(neutralColor);
+		}
+		PlayerColorsRGB.push_back(neutralColors);
+		PlayerColorsSDL.push_back(std::vector<SDL_Color>(neutralColors.begin(), neutralColors.end()));
 	}
 
 	return 0;
-}
+} 
 
 /**
 **  Make new player colors
@@ -621,21 +938,23 @@ static int CclDefinePlayerColorIndex(lua_State *l)
 	PlayerColorIndexStart = LuaToNumber(l, 1);
 	PlayerColorIndexCount = LuaToNumber(l, 2);
 
-	for (int i = 0; i < PlayerMax; ++i) {
-		PlayerColorsRGB[i].clear();
-		PlayerColorsRGB[i].resize(PlayerColorIndexCount);
-		PlayerColors[i].clear();
-		PlayerColors[i].resize(PlayerColorIndexCount, 0);
-	}
+	PlayerColorsRGB.clear();
+	PlayerColorsSDL.clear();
 	return 0;
 }
 
 // ----------------------------------------------------------------------------
 
 /**
+** <b>Description</b>
+**
 **  Get player data.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>GetPlayerData</strong>(0,"TotalNumUnits")</code></div>
 */
 static int CclGetPlayerData(lua_State *l)
 {
@@ -788,9 +1107,20 @@ static int CclGetPlayerData(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Set player data.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code>
+**  <strong>SetPlayerData</strong>(0,"Name","Nation of Stromgarde") -- set the name of this player
+**	<strong>SetPlayerData</strong>(0,"RaceName","human") -- the the race to human
+** 	<strong>SetPlayerData</strong>(0,"Resources","gold",1700) -- set the player to have 1700 gold
+**  <strong>SetPlayerData</strong>(0, "Allow", "upgrade-paladin", "R") -- give the player the Paladin upgrade
+** </code></div>
 */
 static int CclSetPlayerData(lua_State *l)
 {
@@ -896,9 +1226,16 @@ static int CclSetPlayerData(lua_State *l)
 }
 
 /**
+** <b>Description</b> 
+** 
 **  Set ai player algo.
 **
 **  @param l  Lua state.
+** 
+** Example: 
+** 
+** <div class="example"> <code> -- Player 1 has a passive A.I 
+**		  <strong>SetAiType</strong>(1, "Passive")</code></div>
 */
 static int CclSetAiType(lua_State *l)
 {
@@ -925,6 +1262,7 @@ void PlayerCclRegister()
 {
 	lua_register(Lua, "Player", CclPlayer);
 	lua_register(Lua, "ChangeUnitsOwner", CclChangeUnitsOwner);
+	lua_register(Lua, "GiveUnitsToPlayer", CclGiveUnitsToPlayer);
 	lua_register(Lua, "GetThisPlayer", CclGetThisPlayer);
 	lua_register(Lua, "SetThisPlayer", CclSetThisPlayer);
 
@@ -935,9 +1273,12 @@ void PlayerCclRegister()
 	lua_register(Lua, "SetAllPlayersTotalUnitLimit", CclSetAllPlayersTotalUnitLimit);
 
 	lua_register(Lua, "SetDiplomacy", CclSetDiplomacy);
+	lua_register(Lua, "GetDiplomacy", CclGetDiplomacy);
 	lua_register(Lua, "Diplomacy", CclDiplomacy);
 	lua_register(Lua, "SetSharedVision", CclSetSharedVision);
 	lua_register(Lua, "SharedVision", CclSharedVision);
+	
+	lua_register(Lua, "SetRevelationType", CclSetRevelationType);
 
 	lua_register(Lua, "DefineRaceNames", CclDefineRaceNames);
 	lua_register(Lua, "DefineNewRaceNames", CclDefineRaceNames);
