@@ -210,6 +210,8 @@ extern void beos_init(int argc, char **argv);
 #include <stdexcept>
 #include <stacktrace/call_stack.hpp>
 #include <stacktrace/stack_exception.hpp>
+#else
+#include "st_backtrace.h"
 #endif
 
 #include <stdlib.h>
@@ -244,6 +246,17 @@ std::string MenuRace;
 bool EnableDebugPrint;           /// if enabled, print the debug messages
 bool EnableAssert;               /// if enabled, halt on assertion failures
 bool EnableUnitDebug;            /// if enabled, a unit info dump will be created
+bool IsRestart;                  /// if true, the game skips some things like title screens
+
+std::vector<std::string> OriginalArgv;
+
+#ifdef DEBUG
+bool IsDebugEnabled {true};      /// Is debug enabled? Flag to pass into lua code. 
+#else
+bool IsDebugEnabled {false};     /// Is debug enabled? Flag to pass into lua code.
+#endif
+bool EnableWallsInSinglePlayer {false}; /// Enables ability to build walls in the single player games
+										/// used for debug purposes
 
 /*============================================================================
 ==  MAIN
@@ -326,21 +339,6 @@ static void PrintHeader()
 #ifdef USE_THEORA
 		"THEORA "
 #endif
-#ifdef USE_FLUIDSYNTH
-		"FLUIDSYNTH "
-#endif
-#ifdef USE_MIKMOD
-		"MIKMOD "
-#endif
-#ifdef USE_MNG
-		"MNG "
-#endif
-#ifdef USE_OPENGL
-		"OPENGL "
-#endif
-#ifdef USE_GLES
-		"GLES "
-#endif
 #ifdef USE_WIN32
 		"WIN32 "
 #endif
@@ -355,12 +353,6 @@ static void PrintHeader()
 #endif
 #ifdef USE_MAC
 		"MAC "
-#endif
-#ifdef USE_X11
-		"X11 "
-#endif
-#ifdef USE_TOUCHSCREEN
-		"TOUCHSCREEN "
 #endif
 		"";
 
@@ -413,12 +405,20 @@ void Exit(int err)
 	FreeButtonStyles();
 	FreeAllContainers();
 	freeGuichan();
-	DebugPrint("Frames %lu, Slow frames %d = %ld%%\n" _C_
-			   FrameCounter _C_ SlowFrameCounter _C_
+	fprintf(stdout, "Frames %lu, Slow frames %ld = %ld%%\n",
+			   FrameCounter, SlowFrameCounter,
 			   (SlowFrameCounter * 100) / (FrameCounter ? FrameCounter : 1));
 	lua_settop(Lua, 0);
 	lua_close(Lua);
 	DeInitVideo();
+	DeInitImageLoaders();
+
+	if (UnitManager) {
+		delete UnitManager;
+	}
+	if (FogOfWar) {
+		delete FogOfWar;
+	}
 
 	fprintf(stdout, "%s", _("Thanks for playing Stratagus.\n"));
 	exit(err);
@@ -434,6 +434,8 @@ void ExitFatal(int err)
 {
 #ifdef USE_STACKTRACE
 	throw stacktrace::stack_runtime_error((const char*)err);
+#else
+	print_backtrace();
 #endif
 	exit(err);
 }
@@ -447,36 +449,28 @@ static void Usage()
 	printf(
 		"\n\nUsage: %s [OPTIONS] [map.smp|map.smp.gz]\n"
 		"\t-a\t\tEnables asserts check in engine code (for debugging)\n"
+		"\t-b\t\tBenchmark mode. Runs as fast as possible and reports FPS.\n"
 		"\t-c file.lua\tConfiguration start file (default stratagus.lua)\n"
 		"\t-d datapath\tPath to stratagus data (default current directory)\n"
 		"\t-D depth\tVideo mode depth = pixel per point\n"
 		"\t-e\t\tStart editor (instead of game)\n"
 		"\t-E file.lua\tEditor configuration start file (default editor.lua)\n"
 		"\t-F\t\tFull screen video mode\n"
+		"\t-g\t\tForce software rendering (implies no shaders)\n"
 		"\t-G \"options\"\tGame options (passed to game scripts)\n"
 		"\t-h\t\tHelp shows this page\n"
 		"\t-i\t\tEnables unit info dumping into log (for debugging)\n"
 		"\t-I addr\t\tNetwork address to use\n"
 		"\t-l\t\tDisable command log\n"
 		"\t-N name\t\tName of the player\n"
-#if defined(USE_OPENGL) || defined(USE_GLES)
-		"\t-o\t\tDo not use OpenGL or OpenGL ES 1.1\n"
-		"\t-O\t\tUse OpenGL or OpenGL ES 1.1\n"
-#endif
 		"\t-p\t\tEnables debug messages printing in console\n"
 		"\t-P port\t\tNetwork port to use\n"
+		"\t-r\t\tIndicate a rapid start. Skips a few things like title screens\n"
 		"\t-s sleep\tNumber of frames for the AI to sleep before it starts\n"
 		"\t-S speed\tSync speed (100 = 30 frames/s)\n"
-		"\t-u userpath\tPath where stratagus saves preferences, log and savegame\n"
+		"\t-u userpath\tPath where stratagus saves preferences, log and savegame. Use 'userhome' to force platform-default userhome directory.\n"
 		"\t-v mode\t\tVideo mode resolution in format <xres>x<yres>\n"
-		"\t-W\t\tWindowed video mode\n"
-#if defined(USE_OPENGL) || defined(USE_GLES)
-		"\t-x idx\t\tControls fullscreen scaling if your graphics card supports shaders.\n"\
-		"\t  \t\tPass 1 for nearest-neigubour, 2 for EPX/AdvMame, 3 for HQx, 4 for SAL, 5 for SuperEagle\n"\
-		"\t  \t\tYou can also use Ctrl+Alt+/ to cycle between these scaling algorithms at runtime.\n"
-		"\t  \t\tPass -1 to force old-school nearest neighbour scaling without shaders\n"\
-		"\t-Z\t\tUse OpenGL to scale the screen to the viewport (retro-style). Implies -O.\n"
-#endif
+		"\t-W\t\tWindowed video mode. Optionally takes a window size in <xres>x<yres>\n"
 		"map is relative to StratagusLibPath=datapath, use ./map for relative to cwd\n",
 		Parameters::Instance.applicationName.c_str());
 }
@@ -521,11 +515,22 @@ static void RedirectOutput()
 
 void ParseCommandLine(int argc, char **argv, Parameters &parameters)
 {
+#ifdef DEBUG
+	fprintf(stderr, "optind(%d), argc(%d) at startup\n", optind, argc);
+	for (int i = 0; i < argc; i++) {
+		fprintf(stderr, "%s ", argv[i]);
+	}
+	fprintf(stderr, "\n");
+#endif
+	char *sep;
 	for (;;) {
-		switch (getopt(argc, argv, "ac:d:D:eE:FG:hiI:lN:oOP:ps:S:u:v:Wx:Z?-")) {
+		switch (getopt(argc, argv, "abc:d:D:eE:FgG:hiI:lN:oOP:prs:S:u:v:W?-")) {
 			case 'a':
 				EnableAssert = true;
 				continue;
+			case 'b':
+				parameters.benchmark = true;
+				continue;
 			case 'c':
 				parameters.luaStartFilename = optarg;
 				if (strlen(optarg) > 4 &&
@@ -554,6 +559,9 @@ void ParseCommandLine(int argc, char **argv, Parameters &parameters)
 				VideoForceFullScreen = 1;
 				Video.FullScreen = 1;
 				continue;
+			case 'g':
+				SDL_SetHintWithPriority(SDL_HINT_RENDER_DRIVER, "software", SDL_HINT_OVERRIDE);
+				continue;
 			case 'G':
 				parameters.luaScriptArguments = optarg;
 				continue;
@@ -569,27 +577,15 @@ void ParseCommandLine(int argc, char **argv, Parameters &parameters)
 			case 'N':
 				parameters.LocalPlayerName = optarg;
 				continue;
-#if defined(USE_OPENGL) || defined(USE_GLES)
-			case 'o':
-				ForceUseOpenGL = 1;
-				UseOpenGL = 0;
-				if (ZoomNoResize) {
-					fprintf(stderr, "Error: -Z only works with OpenGL enabled\n");
-					Usage();
-					ExitFatal(-1);
-				}
-				continue;
-			case 'O':
-				ForceUseOpenGL = 1;
-				UseOpenGL = 1;
-				continue;
-#endif
 			case 'P':
 				CNetworkParameter::Instance.localPort = atoi(optarg);
 				continue;
 			case 'p':
 				EnableDebugPrint = true;
 				continue;
+			case 'r':
+				IsRestart = true;
+				continue;
 			case 's':
 				AiSleepCycles = atoi(optarg);
 				continue;
@@ -597,14 +593,18 @@ void ParseCommandLine(int argc, char **argv, Parameters &parameters)
 				VideoSyncSpeed = atoi(optarg);
 				continue;
 			case 'u':
-				Parameters::Instance.SetUserDirectory(optarg);
+				if (!strcmp(optarg, "userhome")) {
+					Parameters::Instance.SetUserDirectory("");
+				} else {
+					Parameters::Instance.SetUserDirectory(optarg);
+				}
 				continue;
 			case 'v': {
-				char *sep = strchr(optarg, 'x');
+				sep = strchr(optarg, 'x');
 				if (!sep || !*(sep + 1)) {
 					fprintf(stderr, "%s: incorrect format of video mode resolution -- '%s'\n", argv[0], optarg);
 					Usage();
-					ExitFatal(-1);
+					exit(-1);
 				}
 				Video.Height = atoi(sep + 1);
 				*sep = 0;
@@ -612,46 +612,40 @@ void ParseCommandLine(int argc, char **argv, Parameters &parameters)
 				if (!Video.Height || !Video.Width) {
 					fprintf(stderr, "%s: incorrect format of video mode resolution -- '%sx%s'\n", argv[0], optarg, sep + 1);
 					Usage();
-					ExitFatal(-1);
+					exit(-1);
 				}
-#if defined(USE_OPENGL) || defined(USE_GLES)
-				if (ZoomNoResize) {
-					Video.ViewportHeight = Video.Height;
-					Video.ViewportWidth = Video.Width;
-					Video.Height = 0;
-					Video.Width = 0;
-				}
-#endif
 				continue;
 			}
 			case 'W':
+				if (optind < argc && argv[optind] && argv[optind][0] != '-') {
+					// allow -W to take an optional argument in a POSIX compliant way
+					optarg = argv[optind];
+					sep = strchr(optarg, 'x');
+					if (!sep || !*(sep + 1)) {
+						fprintf(stderr, "%s: incorrect window size -- '%s'\n", argv[0], optarg);
+						Usage();
+						exit(-1);
+					}
+					Video.WindowHeight = atoi(sep + 1);
+					*sep = 0;
+					Video.WindowWidth = atoi(optarg);
+					if (!Video.WindowHeight || !Video.WindowWidth) {
+						fprintf(stderr, "%s: incorrect window size -- '%sx%s'\n", argv[0], optarg, sep + 1);
+						Usage();
+						exit(-1);
+					}
+					optind += 1; // skip the optional window size argument
+				}
 				VideoForceFullScreen = 1;
 				Video.FullScreen = 0;
 				continue;
-#if defined(USE_OPENGL) || defined(USE_GLES)
-			case 'x':
-				ShaderIndex = atoi(optarg);
-				if (atoi(optarg) == -1) {
-					GLShaderPipelineSupported = false;
-				}
-				continue;
-			case 'Z':
-				ForceUseOpenGL = 1;
-				UseOpenGL = 1;
-				ZoomNoResize = 1;
-				Video.ViewportHeight = Video.Height;
-				Video.ViewportWidth = Video.Width;
-				Video.Height = 0;
-				Video.Width = 0;
-				continue;
-#endif
 			case -1:
 				break;
 			case '?':
 			case 'h':
 			default:
 				Usage();
-				ExitFatal(-1);
+				exit(-1);
 		}
 		break;
 	}
@@ -697,6 +691,9 @@ static LONG WINAPI CreateDumpFile(EXCEPTION_POINTERS *ExceptionInfo)
 */
 int stratagusMain(int argc, char **argv)
 {
+	for (int i = 0; i < argc; i++) {
+		OriginalArgv.push_back(std::string(argv[i]));
+	}
 #ifdef USE_BEOS
 	//  Parse arguments for BeOS
 	beos_init(argc, argv);
@@ -722,72 +719,77 @@ int stratagusMain(int argc, char **argv)
 	Assert(pathPtr);
 	StratagusLibPath = pathPtr;
 #endif
-#ifdef USE_STACKTRACE
 	try {
-#endif
-	Parameters &parameters = Parameters::Instance;
-	parameters.SetDefaultValues();
-	parameters.SetLocalPlayerNameFromEnv();
+		Parameters &parameters = Parameters::Instance;
+		parameters.SetDefaultValues();
+		parameters.SetLocalPlayerNameFromEnv();
 
 #ifdef REDIRECT_OUTPUT
-	RedirectOutput();
+		RedirectOutput();
 #endif
 
-	if (argc > 0) {
-		parameters.applicationName = argv[0];
-	}
+		if (argc > 0) {
+			parameters.applicationName = argv[0];
+		}
 
-	// FIXME: Parse options before or after scripts?
-	ParseCommandLine(argc, argv, parameters);
-	// Init the random number generator.
-	InitSyncRand();
+		// FIXME: Parse options before or after scripts?
+		ParseCommandLine(argc, argv, parameters);
+		// Init the random number generator.
+		InitSyncRand();
 
-	makedir(parameters.GetUserDirectory().c_str(), 0777);
+		makedir(parameters.GetUserDirectory().c_str(), 0777);
 
-	// Init Lua and register lua functions!
-	InitLua();
-	LuaRegisterModules();
+		// Init Lua and register lua functions!
+		InitLua();
+		LuaRegisterModules();
 
-	// Initialise AI module
-	InitAiModule();
+		// Initialise AI module
+		InitAiModule();
 
-	LoadCcl(parameters.luaStartFilename, parameters.luaScriptArguments);
+		// Setup sound card, must be done before loading sounds, so that
+		// SDL_mixer can auto-convert to the target format
+		if (!InitSound()) {
+			InitMusic();
+		}
 
-	PrintHeader();
-	PrintLicense();
+		// init globals
+		Map.AllocateTileset();
+		UnitManager = new CUnitManager();
+		FogOfWar = new CFogOfWar();
 
-	// Setup video display
-	InitVideo();
+		LoadCcl(parameters.luaStartFilename, parameters.luaScriptArguments);
 
-	// Setup sound card
-	if (!InitSound()) {
-		InitMusic();
-	}
+		// Setup video display
+		InitVideo();
+
+		PrintHeader();
+		PrintLicense();
 
 #ifndef DEBUG           // For debug it's better not to have:
-	srand(time(NULL));  // Random counter = random each start
+		srand(time(NULL));  // Random counter = random each start
 #endif
 
-	//  Show title screens.
-	SetDefaultTextColors(FontYellow, FontWhite);
-	LoadFonts();
-	SetClipping(0, 0, Video.Width - 1, Video.Height - 1);
-	Video.ClearScreen();
-	ShowTitleScreens();
+		//  Show title screens.
+		SetDefaultTextColors(FontYellow, FontWhite);
+		LoadFonts();
+		SetClipping(0, 0, Video.Width - 1, Video.Height - 1);
+		Video.ClearScreen();
+		if (!IsRestart) {
+			ShowTitleScreens();
+		}
 
-	// Init player data
-	ThisPlayer = NULL;
-	//Don't clear the Players structure as it would erase the allowed units.
-	// memset(Players, 0, sizeof(Players));
-	NumPlayers = 0;
+		// Init player data
+		ThisPlayer = NULL;
+		//Don't clear the Players structure as it would erase the allowed units.
+		// memset(Players, 0, sizeof(Players));
+		NumPlayers = 0;
 
-	UnitManager.Init(); // Units memory management
-	PreMenuSetup();     // Load everything needed for menus
+		UnitManager->Init(); // Units memory management
+		PreMenuSetup();     // Load everything needed for menus
 
-	MenuLoop();
+		MenuLoop();
 
-	Exit(0);
-#ifdef USE_STACKTRACE
+		Exit(0);
 	} catch (const std::exception &e) {
 		fprintf(stderr, "Stratagus crashed!\n");
 		fprintf(stderr, "Please send this call stack to our bug tracker: https://github.com/Wargus/stratagus/issues\n");
@@ -796,7 +798,6 @@ int stratagusMain(int argc, char **argv)
 		fprintf(stderr, "%s", e.what());
 		exit(1);
 	}
-#endif
 	return 0;
 }
 
