@@ -249,6 +249,9 @@ static int GetButtonStatus(const ButtonAction &button, int UnderCursor)
 		case ButtonPatrol:
 			action = UnitActionPatrol;
 			break;
+		case ButtonExplore:
+			action = UnitActionExplore;
+			break;
 		case ButtonHarvest:
 		case ButtonReturn:
 			action = UnitActionResource;
@@ -771,7 +774,7 @@ void CButtonPanel::Draw()
 		//  Tutorial show command key in icons
 		//
 		if (ShowCommandKey) {
-			if (buttons[i].Key == gcn::Key::K_ESCAPE) {
+			if (buttons[i].Key == gcn::Key::K_ESCAPE || buttons[i].Key == 27) {
 				strcpy_s(buf, sizeof(buf), "ESC");
 			} else {
 				buf[0] = toupper(buttons[i].Key);
@@ -884,21 +887,17 @@ bool IsButtonAllowed(const CUnit &unit, const ButtonAction &buttonaction)
 			res = unit.Type->RepairRange > 0;
 			break;
 		case ButtonPatrol:
+		case ButtonExplore:
 			res = unit.CanMove();
 			break;
 		case ButtonHarvest:
 			if (!unit.CurrentResource
-				|| !(unit.ResourcesHeld > 0 && !unit.Type->ResInfo[unit.CurrentResource]->LoseResources)
-				|| (unit.ResourcesHeld != unit.Type->ResInfo[unit.CurrentResource]->ResourceCapacity
-					&& unit.Type->ResInfo[unit.CurrentResource]->LoseResources)) {
+				|| unit.ResourcesHeld < unit.Type->ResInfo[unit.CurrentResource]->ResourceCapacity) {
 				res = true;
 			}
 			break;
 		case ButtonReturn:
-			if (!(!unit.CurrentResource
-				  || !(unit.ResourcesHeld > 0 && !unit.Type->ResInfo[unit.CurrentResource]->LoseResources)
-				  || (unit.ResourcesHeld != unit.Type->ResInfo[unit.CurrentResource]->ResourceCapacity
-					  && unit.Type->ResInfo[unit.CurrentResource]->LoseResources))) {
+		    if (unit.CurrentResource && unit.ResourcesHeld > 0) {
 				res = true;
 			}
 			break;
@@ -1082,8 +1081,8 @@ void CButtonPanel::Update()
 	}
 
 	CUnit &unit = *Selected[0];
-	// foreign unit
-	if (unit.Player != ThisPlayer && !ThisPlayer->IsTeamed(unit)) {
+	// foreign unit, but not *the* neutral player
+	if (unit.Player != ThisPlayer && !ThisPlayer->IsTeamed(unit) && unit.Player->Index != PlayerMax - 1) {
 		CurrentButtons.clear();
 		return;
 	}
@@ -1203,6 +1202,13 @@ void CButtonPanel::DoClicked_Repair(int button)
 	DoClicked_SelectTarget(button);
 }
 
+void CButtonPanel::DoClicked_Explore()
+{
+	for (size_t i = 0; i != Selected.size(); ++i) {
+		SendCommandExplore(*Selected[i], !(KeyModifiers & ModifierShift));
+	}
+}
+
 void CButtonPanel::DoClicked_Return()
 {
 	for (size_t i = 0; i != Selected.size(); ++i) {
@@ -1380,11 +1386,12 @@ void CButtonPanel::DoClicked_Research(int button)
 	}
 }
 
-void CButtonPanel::DoClicked_CallbackAction(int button)
+void CButtonPanel::DoClicked_CallbackAction(int button, int clickingPlayer)
 {
 	LuaCallback* callback = (LuaCallback*)(CurrentButtons[button].Payload);
 	callback->pushPreamble();
 	callback->pushInteger(UnitNumber(*Selected[0]));
+	callback->pushInteger(clickingPlayer);
 	callback->run();
 }
 
@@ -1407,7 +1414,7 @@ void CButtonPanel::DoClicked(int button)
 	//  Button not available.
 	//  or Not Teamed
 	//
-	if (CurrentButtons[button].Pos == -1 || !ThisPlayer->IsTeamed(*Selected[0])) {
+	if (CurrentButtons[button].Pos == -1 || !(ThisPlayer->IsTeamed(*Selected[0]) || Selected[0]->Player->Index == PlayerMax - 1)) {
 		return;
 	}
 	PlayGameSound(GameSounds.Click.Sound, MaxSampleVolume);
@@ -1420,6 +1427,7 @@ void CButtonPanel::DoClicked(int button)
 		case ButtonUnload: { DoClicked_Unload(button); break; }
 		case ButtonSpellCast: { DoClicked_SpellCast(button); break; }
 		case ButtonRepair: { DoClicked_Repair(button); break; }
+		case ButtonExplore: { DoClicked_Explore(); break; }
 		case ButtonMove:    // Follow Next
 		case ButtonPatrol:  // Follow Next
 		case ButtonHarvest: // Follow Next
@@ -1437,7 +1445,7 @@ void CButtonPanel::DoClicked(int button)
 		case ButtonTrain: { DoClicked_Train(button); break; }
 		case ButtonUpgradeTo: { DoClicked_UpgradeTo(button); break; }
 		case ButtonResearch: { DoClicked_Research(button); break; }
-		case ButtonCallbackAction: { DoClicked_CallbackAction(button); break; }
+		case ButtonCallbackAction: { DoClicked_CallbackAction(button, ThisPlayer->Index); break; }
 	}
 }
 
@@ -1450,9 +1458,9 @@ void CButtonPanel::DoClicked(int button)
 */
 int CButtonPanel::DoKey(int key)
 {
-	SDL_keysym keysym;
+	SDL_Keysym keysym;
 	memset(&keysym, 0, sizeof(keysym));
-	keysym.sym = (SDLKey)key;
+	keysym.sym = (SDL_Keycode)key;
 	gcn::Key k = gcn::SDLInput::convertKeyCharacter(keysym);
 	key = k.getValue();
 
