@@ -257,8 +257,31 @@ static const CUnit *GetUnitRef(const CUnit &unit, EnumUnit e)
 	const CUnit *unitToDraw = GetUnitRef(unit, this->UnitRef);
 
 	if (unitToDraw && unitToDraw->Type->Icon.Icon) {
-		unitToDraw->Type->Icon.Icon->DrawUnitIcon(*UI.SingleSelectedButton->Style, 0, this->Pos, "",
-			unitToDraw->RescuedFrom ? unitToDraw->RescuedFrom->Index : unitToDraw->Player->Index);
+		if (this->ButtonIcon) {
+			unitToDraw->Type->Icon.Icon->DrawUnitIcon(*UI.SingleSelectedButton->Style, 0, this->Pos, "",
+				unitToDraw->RescuedFrom ? unitToDraw->RescuedFrom->Index : unitToDraw->Player->Index);
+		} else if (this->SingleSelectionIcon) {
+			unitToDraw->Type->Icon.Icon->DrawSingleSelectionIcon(*UI.SingleSelectedButton->Style, 0, this->Pos, "", *unitToDraw);
+		} else if (this->GroupSelectionIcon) {
+			unitToDraw->Type->Icon.Icon->DrawGroupSelectionIcon(*UI.SingleSelectedButton->Style, 0, this->Pos, "", *unitToDraw);
+		} else if (this->TransportIcon) {
+			unitToDraw->Type->Icon.Icon->DrawContainedIcon(*UI.SingleSelectedButton->Style, 0, this->Pos, "", *unitToDraw);
+		}
+	}
+}
+
+/**
+**  Draw the graphic
+*/
+/* virtual */ void CContentTypeGraphic::Draw(const CUnit &, CFont *) const
+{
+	CGraphic *g = CGraphic::Get(this->graphic);
+	if (g) {
+		if (this->frame) {
+			g->DrawFrameClip(this->frame, this->Pos.x, this->Pos.y);
+		} else {
+			g->DrawClip(this->Pos.x, this->Pos.y);
+		}
 	}
 }
 
@@ -273,27 +296,40 @@ static const CUnit *GetUnitRef(const CUnit &unit, EnumUnit e)
 */
 /* virtual */ void CContentTypeLifeBar::Draw(const CUnit &unit, CFont *) const
 {
-	Assert((unsigned int) this->Index < UnitTypeVar.GetNumberVariable());
-	if (!unit.Variable[this->Index].Max) {
+	Uint32 color;
+	int f;
+	if (this->Index != -1) {
+		Assert((unsigned int) this->Index < UnitTypeVar.GetNumberVariable());
+		if (!unit.Variable[this->Index].Max) {
+			return;
+		}
+		f = (100 * unit.Variable[this->Index].Value) / unit.Variable[this->Index].Max;
+	} else {
+		f = (100 * EvalNumber(this->ValueFunc)) / this->ValueMax;
+		f = f > 100 ? 100 : f;
+	}
+	if (f < 0) {
 		return;
 	}
+	int i = 0;
 
-	Uint32 color;
-	int f = (100 * unit.Variable[this->Index].Value) / unit.Variable[this->Index].Max;
-
-	if (f > 75) {
-		color = ColorDarkGreen;
-	} else if (f > 50) {
-		color = ColorYellow;
-	} else if (f > 25) {
-		color = ColorOrange;
-	} else {
-		color = ColorRed;
+	// get to right color
+	while (static_cast<unsigned int>(f) < this->values[i]) {
+		i++;
 	}
+	color = IndexToColor(this->colors[i]);
 
-	// Border
-	Video.FillRectangleClip(ColorBlack, this->Pos.x - 2, this->Pos.y - 2,
-							this->Width + 3, this->Height + 3);
+	if (this->hasBorder) {
+		// Border. We have a simple heuristic to determine how big it is...
+		// TODO: make configurable?
+		if (this->Height <= 6) {
+			Video.FillRectangleClip(this->hasBorder, this->Pos.x - 2, this->Pos.y - 2,
+									this->Width + 2, this->Height + 2);
+		} else {
+			Video.FillRectangleClip(this->hasBorder, this->Pos.x - 2, this->Pos.y - 2,
+									this->Width + 3, this->Height + 3);
+		}
+	}
 
 	Video.FillRectangleClip(color, this->Pos.x - 1, this->Pos.y - 1,
 							(f * this->Width) / 100, this->Height);
@@ -320,11 +356,7 @@ static const CUnit *GetUnitRef(const CUnit &unit, EnumUnit e)
 	int h = this->height;
 	Assert(w > 0);
 	Assert(h > 4);
-	const Uint32 colors[] = {ColorRed, ColorYellow, ColorGreen, ColorLightGray,
-							 ColorGray, ColorDarkGray, ColorWhite, ColorOrange,
-							 ColorLightBlue, ColorBlue, ColorDarkGreen, ColorBlack
-							};
-	const Uint32 color = (colorIndex != -1) ? colors[colorIndex] : UI.CompletedBarColor;
+	const Uint32 color = (colorIndex != -1) ? IndexToColor(colorIndex) : UI.CompletedBarColor;
 	const int f = (100 * unit.Variable[this->varIndex].Value) / unit.Variable[this->varIndex].Max;
 
 	if (!this->hasBorder) {
@@ -484,30 +516,133 @@ static EnumUnit Str2EnumUnit(lua_State *l, const char *s)
 
 /* virtual */ void CContentTypeIcon::Parse(lua_State *l)
 {
+	SingleSelectionIcon = GroupSelectionIcon = TransportIcon = 0;
+	ButtonIcon = 1;
 	for (lua_pushnil(l); lua_next(l, -2); lua_pop(l, 1)) {
 		const char *key = LuaToString(l, -2);
 		if (!strcmp(key, "Unit")) {
 			this->UnitRef = Str2EnumUnit(l, LuaToString(l, -1));
+		} else if (!strcmp(key, "SingleSelection")) {
+			bool flag = LuaToBoolean(l, -1);
+			if (!ButtonIcon && flag) {
+				LuaError(l, "Only one of SingleSelection, GroupSelection, TransportSelection can be chosen");
+			}
+			ButtonIcon = !flag;
+			SingleSelectionIcon = flag;
+		} else if (!strcmp(key, "GroupSelection")) {
+			bool flag = LuaToBoolean(l, -1);
+			if (!ButtonIcon && flag) {
+				LuaError(l, "Only one of SingleSelection, GroupSelection, TransportSelection can be chosen");
+			}
+			ButtonIcon = !flag;
+			GroupSelectionIcon = flag;
+		} else if (!strcmp(key, "TransportSelection")) {
+			bool flag = LuaToBoolean(l, -1);
+			if (!ButtonIcon && flag) {
+				LuaError(l, "Only one of SingleSelection, GroupSelection, TransportSelection can be chosen");
+			}
+			ButtonIcon = !flag;
+			TransportIcon = flag;
 		} else {
 			LuaError(l, "'%s' invalid for method 'Icon' in DefinePanelContents" _C_ key);
 		}
 	}
 }
 
+/* virtual */ void CContentTypeGraphic::Parse(lua_State *l)
+{
+	if (lua_isstring(l, -1)) {
+		this->graphic = LuaToString(l, -1);
+		this->frame = 0;
+	} else if (lua_istable(l, -1)) {
+		for (lua_pushnil(l); lua_next(l, -2); lua_pop(l, 1)) {
+			const char *key = LuaToString(l, -2);
+			if (!strcmp(key, "Graphic")) {
+				this->graphic = LuaToString(l, -1);
+			} else if (!strcmp(key, "Frame")) {
+				this->frame = LuaToNumber(l, -1);
+			}
+		}
+	}
+}
+
 /* virtual */ void CContentTypeLifeBar::Parse(lua_State *l)
 {
 	for (lua_pushnil(l); lua_next(l, -2); lua_pop(l, 1)) {
 		const char *key = LuaToString(l, -2);
 		if (!strcmp(key, "Variable")) {
-			const char *const name = LuaToString(l, -1);
-			this->Index = UnitTypeVar.VariableNameLookup[name];
-			if (this->Index == -1) {
-				LuaError(l, "unknown variable '%s'" _C_ name);
+			if (lua_isstring(l, -1)) {
+				const char *const name = LuaToString(l, -1);
+				this->Index = UnitTypeVar.VariableNameLookup[name];
+				if (this->Index == -1) {
+					LuaError(l, "unknown variable '%s'" _C_ name);
+				}
+			} else {
+				if (!lua_istable(l, -1)) {
+					LuaError(l, "incorrect argument, need list of size 2 with {function, max} or a string with the name of a unit variable");
+				}
+				for (lua_pushnil(l); lua_next(l, -2); lua_pop(l, 1)) {
+					const char *key = LuaToString(l, -2);
+					if (!strcmp(key, "Max")) {
+						this->ValueMax = LuaToNumber(l, -1);
+					} else if (!strcmp(key, "Value")) {
+						this->ValueFunc = CclParseNumberDesc(l);
+						lua_pushnil(l); // ParseStringDesc eat token
+					} else {
+						lua_pop(l, 1);
+						LuaError(l, "unknow value '%s'" _C_ key);
+					}
+				}
+				if (this->ValueMax == -1) {
+					this->ValueMax = 100;
+				}
+				if (this->ValueFunc == NULL) {
+					LuaError(l, "didn't set a value function");
+				}
 			}
 		} else if (!strcmp(key, "Height")) {
 			this->Height = LuaToNumber(l, -1);
 		} else if (!strcmp(key, "Width")) {
 			this->Width = LuaToNumber(l, -1);
+		} else if (!strcmp(key, "Colors")) {
+			if (!lua_istable(l, -1)) {
+				LuaError(l, "incorrect argument, need list");
+			}
+		    const int color_len = lua_rawlen(l, -1);
+			if (color_len == 0) {
+				LuaError(l, "need at least one {percentage, color} pair, got 0");
+			}
+			this->colors = new unsigned int[color_len];
+			this->values = new unsigned int[color_len];
+			int i = 0;
+			for (lua_pushnil(l); lua_next(l, -2); lua_pop(l, 1)) {
+				if (!lua_istable(l, -1) || lua_rawlen(l, -1) != 2) {
+					LuaError(l, "incorrect argument, need list of size 2 with {percentage, color}");
+				}
+				this->values[i] = LuaToNumber(l, -1, 1);
+				const char *const colorName = LuaToString(l, -1, 2);
+				this->colors[i] = GetColorIndexByName(colorName);
+				if (this->colors[i] == -1) {
+					LuaError(l, "incorrect color: '%s' " _C_ colorName);
+				}
+				i++;
+			}
+			if (this->values[color_len - 1] != 0) {
+				LuaError(l, "the last {percentage, color} pair must be for 0%%");
+			}
+		} else if (!strcmp(key, "Border")) {
+			if (lua_isboolean(l, -1)) {
+				if (LuaToBoolean(l, -1)) {
+					this->hasBorder = 1;
+				} else {
+					this->hasBorder = 0;
+				}
+			} else {
+				this->hasBorder = LuaToUnsignedNumber(l, -1);
+				if (this->hasBorder == 0) {
+					this->hasBorder = 1; // complete black is set to 1
+				}
+			}
 		} else {
 			LuaError(l, "'%s' invalid for method 'LifeBar' in DefinePanelContents" _C_ key);
 		}
@@ -519,25 +654,23 @@ static EnumUnit Str2EnumUnit(lua_State *l, const char *s)
 	if (this->Width <= 0) {
 		this->Width = 50; // Default value.
 	}
-	if (this->Index == -1) {
+	if (this->Index == -1 && this->ValueFunc == NULL) {
 		LuaError(l, "variable undefined for LifeBar");
 	}
-}
+	if (this->colors == NULL || this->values == NULL) {
+		this->colors = new unsigned int[4];
+		this->values = new unsigned int[4];
 
-static int GetColorIndexByName(const char *colorName)
-{
-	//FIXME: need more general way
-	const char *names[] = {
-		"red", "yellow", "green", "light-gray", "gray", "dark-gray",
-		"white", "orange", "light-blue", "blue", "dark-green", "black"
-	};
+		this->values[0] = 75;
+		this->values[1] = 50;
+		this->values[2] = 25;
+		this->values[3] = 0;
 
-	for (unsigned int i = 0; i != sizeof(names) / sizeof(names[0]); ++i) {
-		if (!strcmp(colorName, names[i])) {
-			return i;
-		}
+		this->colors[0] = GetColorIndexByName("dark-green");
+		this->colors[1] = GetColorIndexByName("yellow");
+		this->colors[2] = GetColorIndexByName("orange");;
+		this->colors[3] = GetColorIndexByName("red");
 	}
-	return -1;
 }
 
 /* virtual */ void CContentTypeCompleteBar::Parse(lua_State *l)
