@@ -59,23 +59,25 @@
 --  Defines
 ----------------------------------------------------------------------------*/
 
-/// Scrolling area (<= 15 y)
-#define SCROLL_UP     15
-/// Scrolling area (>= VideoHeight - 16 y)
-#define SCROLL_DOWN   (Video.Height - 16)
-/// Scrolling area (<= 15 y)
-#define SCROLL_LEFT   15
-/// Scrolling area (>= VideoWidth - 16 x)
-#define SCROLL_RIGHT  (Video.Width - 16)
-
 /*----------------------------------------------------------------------------
 --  Variables
 ----------------------------------------------------------------------------*/
 
+int ScrollMarginTop = 15;
+int ScrollMarginBottom = 16;
+int ScrollMarginLeft = 15;
+int ScrollMarginRight = 16;
+
+const char Cursor[] = "~!_";         /// Input cursor
 static Vec2i SavedMapPosition[3];    /// Saved map position
 static char Input[80];               /// line input for messages/long commands
 static int InputIndex;               /// current index into input
 static char InputStatusLine[99];     /// Last input status line
+constexpr int MaxInputHistorySize = 16;  /// Max history of inputs
+static char InputHistory[MaxInputHistorySize * sizeof(Input)] = { '\0' }; /// History of inputs
+static int InputHistoryIdx = 0;      /// Next history idx
+static int InputHistoryPos = 0;      /// Current position in history
+static int InputHistorySize = 0;     /// History fill size
 const char DefaultGroupKeys[] = "0123456789`";/// Default group keys
 std::string UiGroupKeys = DefaultGroupKeys;/// Up to 11 keys, last unselect. Default for qwerty
 bool GameRunning;                    /// Current running state
@@ -93,12 +95,27 @@ CUnit *LastIdleWorker;               /// Last called idle worker
 --  Functions
 ----------------------------------------------------------------------------*/
 
+static void moveInputContent(int targetPos, int srcPos) {
+	memmove(Input + targetPos, Input + srcPos, sizeof(Input) - std::max(targetPos, srcPos));
+}
+
+static void removeCursorFromInput() {
+	// remove cursor, which is at InputIndex. there might be other chars behind it, if we're in the middle
+	moveInputContent(InputIndex, InputIndex + strlen(Cursor));
+}
+
+static void addCursorToInput() {
+	// insert cursor at pos
+	moveInputContent(InputIndex + strlen(Cursor), InputIndex);
+	strncpy(Input + InputIndex, Cursor, strlen(Cursor));
+}
+
 /**
 **  Show input.
 */
 static void ShowInput()
 {
-	snprintf(InputStatusLine, sizeof(InputStatusLine), _("MESSAGE:%s~!_"), Input);
+	snprintf(InputStatusLine, sizeof(InputStatusLine), _("MESSAGE:%s"), Input);
 	char *input = InputStatusLine;
 	// FIXME: This is slow!
 	while (UI.StatusLine.Font->Width(input) > UI.StatusLine.Width) {
@@ -116,8 +133,9 @@ static void ShowInput()
 static void UiBeginInput()
 {
 	KeyState = KeyStateInput;
-	Input[0] = '\0';
+	memset(Input, 0, sizeof(Input));
 	InputIndex = 0;
+	addCursorToInput();
 	UI.StatusLine.ClearCosts();
 	ShowInput();
 }
@@ -169,7 +187,7 @@ static void UiCenterOnGroup(unsigned group, GroupSelectionMode mode = SELECTABLE
 **
 **  @param group  Group number to select.
 */
-static void UiSelectGroup(unsigned group, GroupSelectionMode mode = SELECTABLE_BY_RECTANGLE_ONLY)
+static void UiSelectGroup(unsigned group, GroupSelectionMode mode = SELECT_ALL)
 {
 	SelectGroup(group, mode);
 	SelectionChanged();
@@ -238,7 +256,7 @@ static void UiToggleSound()
 		} else {
 			SetEffectsEnabled(true);
 			SetMusicEnabled(true);
-			CheckMusicFinished(true);
+			CallbackMusicTrigger();
 		}
 	}
 
@@ -427,18 +445,18 @@ void UiToggleTerrain()
 */
 void UiFindIdleWorker()
 {
-	if (ThisPlayer->FreeWorkers.empty()) {
+	if (ThisPlayer->GetFreeWorkersCount() == 0) {
 		return;
 	}
-	CUnit *unit = ThisPlayer->FreeWorkers[0];
+	CUnit *unit = ThisPlayer->GetFreeWorker(0);
 	if (LastIdleWorker) {
-		const std::vector<CUnit *> &freeWorkers = ThisPlayer->FreeWorkers;
-		std::vector<CUnit *>::const_iterator it = std::find(freeWorkers.begin(),
-															freeWorkers.end(),
-															LastIdleWorker);
-		if (it != ThisPlayer->FreeWorkers.end()) {
-			if (*it != ThisPlayer->FreeWorkers.back()) {
-				unit = *(++it);
+		bool found = false;
+		for(auto it = ThisPlayer->FreeWorkersBegin(); it != ThisPlayer->FreeWorkersEnd(); ++it) {
+			if (found) {
+				unit = *it;
+				break;
+			} else if (*it == unit) {
+				found = true;
 			}
 		}
 	}
@@ -584,7 +602,7 @@ static bool CommandKey(int key)
 		// Unselect everything
 		case SDLK_CARET:
 		case SDLK_BACKQUOTE:
-			UiUnselectAll();
+			// UiUnselectAll();
 			break;
 
 		// Group selection
@@ -714,22 +732,21 @@ static bool CommandKey(int key)
 			break;
 
 		case SDLK_UP:
-		case SDLK_KP8:
+		case SDLK_KP_8:
 			KeyScrollState |= ScrollUp;
 			break;
 		case SDLK_DOWN:
-		case SDLK_KP2:
+		case SDLK_KP_2:
 			KeyScrollState |= ScrollDown;
 			break;
 		case SDLK_LEFT:
-		case SDLK_KP4:
+		case SDLK_KP_4:
 			KeyScrollState |= ScrollLeft;
 			break;
 		case SDLK_RIGHT:
-		case SDLK_KP6:
+		case SDLK_KP_6:
 			KeyScrollState |= ScrollRight;
 			break;
-
 		default:
 			if (HandleCommandKey(key)) {
 				break;
@@ -761,7 +778,7 @@ int HandleCheats(const std::string &input)
 #endif
 		} else {
 			ThisPlayer->AiEnabled = true;
-			ThisPlayer->Type = PlayerComputer;
+			ThisPlayer->Type = PlayerTypes::PlayerComputer;
 			if (!ThisPlayer->Ai) {
 				AiInit(*ThisPlayer);
 			}
@@ -821,13 +838,23 @@ static void ReplaceTildeBy2Tilde(char *s)
 **  Handle keys in input mode.
 **
 **  @param key  Key scancode.
-**  @return     True input finished.
 */
-static int InputKey(int key)
+static void InputKey(int key)
 {
 	switch (key) {
 		case SDLK_RETURN:
 		case SDLK_KP_ENTER: { // RETURN
+			removeCursorFromInput();		
+			// save to history
+			strncpy(InputHistory + (InputHistoryIdx * sizeof(Input)), Input, sizeof(Input));
+			if (InputHistorySize < MaxInputHistorySize - 1) {
+				InputHistorySize++;
+				InputHistoryIdx = InputHistorySize;
+			} else {
+				InputHistoryIdx = ((InputHistoryIdx + 1) % InputHistorySize + InputHistorySize) % InputHistorySize;
+			}
+			InputHistoryPos = InputHistoryIdx;
+
 			// Replace ~~ with ~
 			Replace2TildeByTilde(Input);
 #ifdef DEBUG
@@ -839,7 +866,7 @@ static int InputKey(int key)
 			} else
 #endif
 				if (!IsNetworkGame()) {
-					if (!GameObserve && !GamePaused && !GameEstablishing) {
+					if (!GameObserve && !GameEstablishing) {
 						if (HandleCheats(Input)) {
 							CommandLog("input", NoUnitP, FlushCommands, -1, -1, NoUnitP, Input, -1);
 						}
@@ -870,73 +897,122 @@ static int InputKey(int key)
 		case SDLK_ESCAPE:
 			KeyState = KeyStateCommand;
 			UI.StatusLine.Clear();
-			return 1;
+			break;
 
 #ifdef USE_MAC
 		case SDLK_DELETE:
 #endif
-		case SDLK_BACKSPACE:
+		case SDLK_BACKSPACE: {
 			if (InputIndex) {
+				InputHistoryPos = InputHistoryIdx;
+				removeCursorFromInput();
 				if (Input[InputIndex - 1] == '~') {
-					Input[--InputIndex] = '\0';
+					moveInputContent(InputIndex - 1, InputIndex);
+					InputIndex--;
 				}
-				InputIndex = UTF8GetPrev(Input, InputIndex);
-				if (InputIndex >= 0) {
-					Input[InputIndex] = '\0';
-					ShowInput();
+				int prevIndex = UTF8GetPrev(Input, InputIndex);
+				if (prevIndex >= 0) {
+					moveInputContent(prevIndex, InputIndex);
+					InputIndex = prevIndex;
 				}
+				addCursorToInput();
+				ShowInput();
 			}
-			return 1;
+			break;
+		}
+		case SDLK_UP:
+			removeCursorFromInput();
+			strncpy(InputHistory + (InputHistoryPos * sizeof(Input)), Input, sizeof(Input));
+			if (InputHistorySize == 0) {
+				break;
+			}
+			InputHistoryPos = ((InputHistoryPos - 1) % InputHistorySize + InputHistorySize) % InputHistorySize;
+			strncpy(Input, InputHistory + (InputHistoryPos * sizeof(Input)), sizeof(Input));
+			InputIndex = strlen(Input);
+			addCursorToInput();
+			ShowInput();
+			break;
+
+		case SDLK_DOWN:
+			removeCursorFromInput();
+			strncpy(InputHistory + (InputHistoryPos * sizeof(Input)), Input, sizeof(Input));
+			if (InputHistorySize == 0) {
+				break;
+			}
+			InputHistoryPos = ((InputHistoryPos + 1) % InputHistorySize + InputHistorySize) % InputHistorySize;
+			strncpy(Input, InputHistory + (InputHistoryPos * sizeof(Input)), sizeof(Input));
+			InputIndex = strlen(Input);
+			addCursorToInput();
+			ShowInput();
+			break;
+
+		case SDLK_LEFT:
+			if (InputIndex) {
+				removeCursorFromInput();
+				InputIndex = UTF8GetPrev(Input, InputIndex);
+				addCursorToInput();
+				ShowInput();
+			}
+			break;
+
+		case SDLK_RIGHT:
+			removeCursorFromInput();
+			InputIndex = UTF8GetNext(Input, InputIndex);
+			addCursorToInput();
+			ShowInput();
+			break;
 
 		case SDLK_TAB: {
+			InputHistoryPos = InputHistoryIdx;
+			removeCursorFromInput();
 			char *namestart = strrchr(Input, ' ');
 			if (namestart) {
 				++namestart;
 			} else {
 				namestart = Input;
 			}
-			if (!strlen(namestart)) {
-				return 1;
-			}
-			for (int i = 0; i < PlayerMax; ++i) {
-				if (Players[i].Type != PlayerPerson) {
-					continue;
-				}
-				if (!strncasecmp(namestart, Players[i].Name.c_str(), strlen(namestart))) {
-					InputIndex += strlen(Players[i].Name.c_str()) - strlen(namestart);
-					strcpy_s(namestart, sizeof(Input) - (namestart - Input), Players[i].Name.c_str());
-					if (namestart == Input) {
-						InputIndex += 2;
-						strcat_s(namestart, sizeof(Input) - (namestart - Input), ": ");
+			if (strlen(namestart)) {
+				for (int i = 0; i < PlayerMax; ++i) {
+					if (Players[i].Type != PlayerTypes::PlayerPerson) {
+						continue;
+					}
+					if (!strncasecmp(namestart, Players[i].Name.c_str(), strlen(namestart))) {
+						InputIndex += strlen(Players[i].Name.c_str()) - strlen(namestart);
+						strcpy_s(namestart, sizeof(Input) - (namestart - Input), Players[i].Name.c_str());
+						if (namestart == Input) {
+							InputIndex += 2;
+							strcat_s(namestart, sizeof(Input) - (namestart - Input), ": ");
+						}
 					}
-					ShowInput();
 				}
 			}
-			return 1;
+			addCursorToInput();
+			ShowInput();
+			break;
 		}
 		default:
 			if (key >= ' ') {
+				InputHistoryPos = InputHistoryIdx;
+				removeCursorFromInput();
 				gcn::Key k(key);
 				std::string kstr = k.toString();
 				if (key == '~') {
 					if (InputIndex < (int)sizeof(Input) - 2) {
+						moveInputContent(InputIndex + 2, InputIndex);
 						Input[InputIndex++] = key;
 						Input[InputIndex++] = key;
-						Input[InputIndex] = '\0';
-						ShowInput();
 					}
 				} else if (InputIndex < (int)(sizeof(Input) - kstr.size())) {
+					moveInputContent(InputIndex + kstr.size(), InputIndex);
 					for (size_t i = 0; i < kstr.size(); ++i) {
 						Input[InputIndex++] = kstr[i];
 					}
-					Input[InputIndex] = '\0';
-					ShowInput();
 				}
-				return 1;
+				addCursorToInput();
+				ShowInput();
 			}
 			break;
 	}
-	return 0;
 }
 
 /**
@@ -979,20 +1055,18 @@ int HandleKeyModifiersDown(unsigned key, unsigned)
 			return 1;
 		case SDLK_LALT:
 		case SDLK_RALT:
-		case SDLK_LMETA:
-		case SDLK_RMETA:
 			KeyModifiers |= ModifierAlt;
 			// maxy: disabled
 			if (InterfaceState == IfaceStateNormal) {
 				SelectedUnitChanged(); // VLADI: to allow alt-buttons
 			}
 			return 1;
-		case SDLK_LSUPER:
-		case SDLK_RSUPER:
+		case SDLK_LGUI:
+		case SDLK_RGUI:
 			KeyModifiers |= ModifierSuper;
 			return 1;
 		case SDLK_SYSREQ:
-		case SDLK_PRINT:
+		case SDLK_PRINTSCREEN:
 			Screenshot();
 			if (GameRunning) {
 				SetMessage("%s", _("Screenshot made."));
@@ -1025,16 +1099,14 @@ int HandleKeyModifiersUp(unsigned key, unsigned)
 			return 1;
 		case SDLK_LALT:
 		case SDLK_RALT:
-		case SDLK_LMETA:
-		case SDLK_RMETA:
 			KeyModifiers &= ~ModifierAlt;
 			// maxy: disabled
 			if (InterfaceState == IfaceStateNormal) {
 				SelectedUnitChanged(); // VLADI: to allow alt-buttons
 			}
 			return 1;
-		case SDLK_LSUPER:
-		case SDLK_RSUPER:
+		case SDLK_LGUI:
+		case SDLK_RGUI:
 			KeyModifiers &= ~ModifierSuper;
 			return 1;
 	}
@@ -1046,8 +1118,8 @@ int HandleKeyModifiersUp(unsigned key, unsigned)
 */
 static bool IsKeyPad(unsigned key, unsigned *kp)
 {
-	if (key >= SDLK_KP0 && key <= SDLK_KP9) {
-		*kp = SDLK_0 + (key - SDLK_KP0);
+	if (key >= SDLK_KP_0 && key <= SDLK_KP_9) {
+		*kp = SDLK_0 + (key - SDLK_KP_0);
 	} else if (key == SDLK_KP_PERIOD) {
 		*kp = SDLK_PERIOD;
 	} else if (key == SDLK_KP_DIVIDE) {
@@ -1062,6 +1134,8 @@ static bool IsKeyPad(unsigned key, unsigned *kp)
 		*kp = SDLK_RETURN;
 	} else if (key == SDLK_KP_EQUALS) {
 		*kp = SDLK_EQUALS;
+	} else if (key == SDLK_UP || key == SDLK_DOWN || key == SDLK_LEFT || key == SDLK_RIGHT) {
+		*kp = key;
 	} else  {
 		*kp = SDLK_UNKNOWN;
 		return false;
@@ -1077,6 +1151,13 @@ static bool IsKeyPad(unsigned key, unsigned *kp)
 */
 void HandleKeyDown(unsigned key, unsigned keychar)
 {
+	if (IsDemoMode()) {
+		// If we are in "demo mode", exit no matter which key we hit.
+		void ActionDraw();
+		ActionDraw();
+		return;
+	}
+
 	if (HandleKeyModifiersDown(key, keychar)) {
 		return;
 	}
@@ -1114,19 +1195,19 @@ void HandleKeyUp(unsigned key, unsigned keychar)
 
 	switch (key) {
 		case SDLK_UP:
-		case SDLK_KP8:
+		case SDLK_KP_8:
 			KeyScrollState &= ~ScrollUp;
 			break;
 		case SDLK_DOWN:
-		case SDLK_KP2:
+		case SDLK_KP_2:
 			KeyScrollState &= ~ScrollDown;
 			break;
 		case SDLK_LEFT:
-		case SDLK_KP4:
+		case SDLK_KP_4:
 			KeyScrollState &= ~ScrollLeft;
 			break;
 		case SDLK_RIGHT:
-		case SDLK_KP6:
+		case SDLK_KP_6:
 			KeyScrollState &= ~ScrollRight;
 			break;
 		default:
@@ -1156,12 +1237,12 @@ void HandleKeyRepeat(unsigned, unsigned keychar)
 */
 bool HandleMouseScrollArea(const PixelPos &mousePos)
 {
-	if (mousePos.x < SCROLL_LEFT) {
-		if (mousePos.y < SCROLL_UP) {
+	if (mousePos.x < ScrollMarginLeft) {
+		if (mousePos.y < ScrollMarginTop) {
 			CursorOn = CursorOnScrollLeftUp;
 			MouseScrollState = ScrollLeftUp;
 			GameCursor = UI.ArrowNW.Cursor;
-		} else if (mousePos.y > SCROLL_DOWN) {
+		} else if (mousePos.y > (Video.Height - ScrollMarginBottom)) {
 			CursorOn = CursorOnScrollLeftDown;
 			MouseScrollState = ScrollLeftDown;
 			GameCursor = UI.ArrowSW.Cursor;
@@ -1170,12 +1251,12 @@ bool HandleMouseScrollArea(const PixelPos &mousePos)
 			MouseScrollState = ScrollLeft;
 			GameCursor = UI.ArrowW.Cursor;
 		}
-	} else if (mousePos.x > SCROLL_RIGHT) {
-		if (mousePos.y < SCROLL_UP) {
+	} else if (mousePos.x > (Video.Width - ScrollMarginRight)) {
+		if (mousePos.y < ScrollMarginTop) {
 			CursorOn = CursorOnScrollRightUp;
 			MouseScrollState = ScrollRightUp;
 			GameCursor = UI.ArrowNE.Cursor;
-		} else if (mousePos.y > SCROLL_DOWN) {
+		} else if (mousePos.y > (Video.Height - ScrollMarginBottom)) {
 			CursorOn = CursorOnScrollRightDown;
 			MouseScrollState = ScrollRightDown;
 			GameCursor = UI.ArrowSE.Cursor;
@@ -1185,11 +1266,11 @@ bool HandleMouseScrollArea(const PixelPos &mousePos)
 			GameCursor = UI.ArrowE.Cursor;
 		}
 	} else {
-		if (mousePos.y < SCROLL_UP) {
+		if (mousePos.y < ScrollMarginTop) {
 			CursorOn = CursorOnScrollUp;
 			MouseScrollState = ScrollUp;
 			GameCursor = UI.ArrowN.Cursor;
-		} else if (mousePos.y > SCROLL_DOWN) {
+		} else if (mousePos.y > (Video.Height - ScrollMarginBottom)) {
 			CursorOn = CursorOnScrollDown;
 			MouseScrollState = ScrollDown;
 			GameCursor = UI.ArrowS.Cursor;
@@ -1254,13 +1335,8 @@ void HandleButtonUp(unsigned button)
 --  Lowlevel input functions
 ----------------------------------------------------------------------------*/
 
-#ifdef USE_TOUCHSCREEN
-int DoubleClickDelay = 1000;      /// Time to detect double clicks.
-int HoldClickDelay = 2000;        /// Time to detect hold clicks.
-#else
 int DoubleClickDelay = 300;       /// Time to detect double clicks.
 int HoldClickDelay = 1000;        /// Time to detect hold clicks.
-#endif
 
 static enum {
 	InitialMouseState,            /// start state
@@ -1302,7 +1378,9 @@ void InputMouseButtonPress(const EventCallback &callbacks,
 	}
 	LastMouseTicks = ticks;
 
-	callbacks.ButtonPressed(button);
+	if (callbacks.ButtonPressed) {
+		callbacks.ButtonPressed(button);
+	}
 }
 
 /**
@@ -1336,7 +1414,9 @@ void InputMouseButtonRelease(const EventCallback &callbacks,
 	}
 	MouseButtons &= ~(0x01010101 << button);
 
-	callbacks.ButtonReleased(button | mask);
+	if (callbacks.ButtonReleased) {
+		callbacks.ButtonReleased(button | mask);
+	}
 }
 
 /**
@@ -1352,29 +1432,14 @@ void InputMouseMove(const EventCallback &callbacks,
 {
 	PixelPos mousePos(x, y);
 	// Don't reset the mouse state unless we really moved
-#ifdef USE_TOUCHSCREEN
-	const int buff = 32;
-	const PixelDiff diff = LastMousePos - mousePos;
-
-	if (abs(diff.x) > buff || abs(diff.y) > buff) {
-		MouseState = InitialMouseState;
-		LastMouseTicks = ticks;
-		// Reset rectangle select cursor state if we moved by a lot
-		// - rectangle select should be a drag, not a tap
-		if (CursorState == CursorStateRectangle
-			&& (abs(diff.x) > 2 * buff || abs(diff.y) > 2 * buff)) {
-			CursorState = CursorStatePoint;
-		}
-	}
-	LastMousePos = mousePos;
-#else
 	if (LastMousePos != mousePos) {
 		MouseState = InitialMouseState;
 		LastMouseTicks = ticks;
 		LastMousePos = mousePos;
 	}
-#endif
-	callbacks.MouseMoved(mousePos);
+	if (callbacks.MouseMoved) {
+		callbacks.MouseMoved(mousePos);
+	}
 }
 
 /**
@@ -1387,7 +1452,9 @@ void InputMouseExit(const EventCallback &callbacks, unsigned /* ticks */)
 {
 	// FIXME: should we do anything here with ticks? don't know, but conform others
 	// JOHNS: called by callback HandleMouseExit();
-	callbacks.MouseExit();
+	if (callbacks.MouseExit) {
+		callbacks.MouseExit();
+	}
 }
 
 /**
@@ -1405,7 +1472,9 @@ void InputMouseTimeout(const EventCallback &callbacks, unsigned ticks)
 		if (ticks > LastMouseTicks + HoldClickDelay) {
 			LastMouseTicks = ticks;
 			MouseButtons |= (1 << LastMouseButton) << MouseHoldShift;
-			callbacks.ButtonPressed(LastMouseButton | (LastMouseButton << MouseHoldShift));
+			if (callbacks.ButtonPressed) {
+				callbacks.ButtonPressed(LastMouseButton | (LastMouseButton << MouseHoldShift));
+			}
 		}
 	}
 }
@@ -1437,7 +1506,9 @@ void InputKeyButtonPress(const EventCallback &callbacks,
 	LastIKey = ikey;
 	LastIKeyChar = ikeychar;
 	LastKeyTicks = ticks;
-	callbacks.KeyPressed(ikey, ikeychar);
+	if (callbacks.KeyPressed) {
+		callbacks.KeyPressed(ikey, ikeychar);
+	}
 	KeyModifiers &= ~ModifierDoublePress;
 }
 
@@ -1456,7 +1527,9 @@ void InputKeyButtonRelease(const EventCallback &callbacks,
 		LastIKey = 0;
 	}
 	LastKeyTicks = ticks;
-	callbacks.KeyReleased(ikey, ikeychar);
+	if (callbacks.KeyReleased) {
+		callbacks.KeyReleased(ikey, ikeychar);
+	}
 }
 
 /**
@@ -1469,7 +1542,9 @@ void InputKeyTimeout(const EventCallback &callbacks, unsigned ticks)
 {
 	if (LastIKey && ticks > LastKeyTicks + HoldKeyDelay) {
 		LastKeyTicks = ticks - (HoldKeyDelay - HoldKeyAdditionalDelay);
-		callbacks.KeyRepeated(LastIKey, LastIKeyChar);
+		if (callbacks.KeyRepeated) {
+			callbacks.KeyRepeated(LastIKey, LastIKeyChar);
+		}
 	}
 }
 
@@ -1509,4 +1584,15 @@ void SetHoldClickDelay(int delay)
 	HoldClickDelay = delay;
 }
 
+/**
+ * Configure margins (in pixels) on screen where map scrolling starts.
+ */
+void SetScrollMargins(unsigned int top, unsigned int right, unsigned int bottom, unsigned int left)
+{
+	ScrollMarginTop = top;
+	ScrollMarginBottom = bottom + 1;
+	ScrollMarginLeft = left;
+	ScrollMarginRight = right + 1;
+}
+
 //@}
