@@ -207,6 +207,7 @@ static bool DoRightButton_Harvest_Unit(CUnit &unit, CUnit &dest, int flush, int
 		&& dest.Type->CanStore[unit.CurrentResource]
 		&& (dest.Player == unit.Player
 			|| (dest.Player->IsAllied(*unit.Player) && unit.Player->IsAllied(*dest.Player)))) {
+		const ResourceInfo &resinfo = *unit.Type->ResInfo[unit.CurrentResource];
 		dest.Blink = 4;
 		if (!acknowledged) {
 			PlayUnitSound(unit, VoiceAcknowledging);
@@ -220,7 +221,7 @@ static bool DoRightButton_Harvest_Unit(CUnit &unit, CUnit &dest, int flush, int
 	const CUnitType &type = *unit.Type;
 	if (res && type.ResInfo[res] && dest.Type->BoolFlag[CANHARVEST_INDEX].value
 		&& (dest.Player == unit.Player || dest.Player->Index == PlayerNumNeutral)) {
-			if (unit.ResourcesHeld < type.ResInfo[res]->ResourceCapacity) {
+			if (unit.CurrentResource != res || unit.ResourcesHeld < type.ResInfo[res]->ResourceCapacity) {
 				dest.Blink = 4;
 				SendCommandResource(unit, dest, flush);
 				if (!acknowledged) {
@@ -254,15 +255,25 @@ static bool DoRightButton_Harvest_Pos(CUnit &unit, const Vec2i &pos, int flush,
 	for (int res = 0; res < MaxCosts; ++res) {
 		if (type.ResInfo[res]
 			&& type.ResInfo[res]->TerrainHarvester
-			&& Map.Field(pos)->IsTerrainResourceOnMap(res)
-			&& ((unit.CurrentResource != res)
-				|| (unit.ResourcesHeld < type.ResInfo[res]->ResourceCapacity))) {
-			SendCommandResourceLoc(unit, pos, flush);
-			if (!acknowledged) {
-				PlayUnitSound(unit, VoiceHarvesting);
-				acknowledged = 1;
+			&& Map.Field(pos)->IsTerrainResourceOnMap(res)) {
+			if (unit.CurrentResource != res || unit.ResourcesHeld < type.ResInfo[res]->ResourceCapacity) {
+				SendCommandResourceLoc(unit, pos, flush);
+				if (!acknowledged) {
+					PlayUnitSound(unit, VoiceHarvesting);
+					acknowledged = 1;
+				}
+				return true;
+			} else {
+				CUnit *depot = FindDeposit(unit, 1000, unit.CurrentResource);
+				if (depot) {
+					if (!acknowledged) {
+						PlayUnitSound(unit, VoiceAcknowledging);
+						acknowledged = 1;
+					}
+					SendCommandReturnGoods(unit, depot, flush);
+					return true;
+				}
 			}
-			return true;
 		}
 	}
 	return false;
@@ -372,11 +383,12 @@ static void DoRightButton_Attack(CUnit &unit, CUnit *dest, const Vec2i &pos, int
 		}
 	}
 	if (Map.WallOnMap(pos)) {
-		if (unit.Player->Race == PlayerRaceHuman && Map.OrcWallOnMap(pos)) {
+		// XXX: hardcoded for humans and orcs
+		if (PlayerRaces.Name[unit.Player->Race] == "human" && Map.OrcWallOnMap(pos)) {
 			SendCommandAttack(unit, pos, NoUnitP, flush);
 			return;
 		}
-		if (unit.Player->Race == PlayerRaceOrc && Map.HumanWallOnMap(pos)) {
+		if (PlayerRaces.Name[unit.Player->Race] == "orc" && Map.HumanWallOnMap(pos)) {
 			SendCommandAttack(unit, pos, NoUnitP, flush);
 			return;
 		}
@@ -659,6 +671,11 @@ static void HandleMouseOn(const PixelPos screenPos)
 	ButtonAreaUnderCursor = -1;
 	ButtonUnderCursor = -1;
 
+	if (IsDemoMode()) {
+		// If we are in "demo mode", do nothing.
+		return;
+	}
+
 	// BigMapMode is the mode which show only the map (without panel, minimap)
 	if (BigMapMode) {
 		CursorOn = CursorOnMap;
@@ -695,6 +712,21 @@ static void HandleMouseOn(const PixelPos screenPos)
 			}
 		}
 	}
+	if (UI.Resources[FreeWorkersCount].TextX != -1) {
+		int x, y;
+		CGraphic* g = UI.Resources[FreeWorkersCount].G;
+		if (g && (x = UI.Resources[FreeWorkersCount].IconX) && (y = UI.Resources[FreeWorkersCount].IconY)) {
+			int textX = UI.Resources[FreeWorkersCount].TextX;
+			if (textX > 0 || ThisPlayer->GetFreeWorkersCount() > 0) {
+				if (screenPos > PixelPos(x, y) && screenPos < PixelPos(x + g->getWidth(), y + g->getHeight())) {
+					ButtonAreaUnderCursor = ButtonAreaMenu;
+					ButtonUnderCursor = ButtonUnderFreeWorkers;
+					CursorOn = CursorOnButton;
+					return;
+				}
+			}
+		}
+	}
 	for (size_t i = 0; i < UI.UserButtons.size(); ++i) {
 		const CUIUserButton &button = UI.UserButtons[i];
 
@@ -901,6 +933,13 @@ void UIHandleMouseMove(const PixelPos &cursorPos)
 {
 	enum _cursor_on_ OldCursorOn;
 
+	if (IsDemoMode()) {
+		// If we are in "demo mode", exit now.
+		void ActionDraw();
+		ActionDraw();
+		return;
+	}
+
 	OldCursorOn = CursorOn;
 	//  Selecting units.
 	if (CursorState == CursorStateRectangle) {
@@ -1003,7 +1042,7 @@ void UIHandleMouseMove(const PixelPos &cursorPos)
 			CMapField &mf = *Map.Field(tilePos);
 			for (int i = 0; i < PlayerMax; ++i) {
 				if (mf.playerInfo.IsExplored(Players[i])
-					&& (i == ThisPlayer->Index || Players[i].IsBothSharedVision(*ThisPlayer))) {
+					&& (i == ThisPlayer->Index || Players[i].HasSharedVisionWith(*ThisPlayer))) {
 					show = true;
 					break;
 				}
@@ -1290,7 +1329,8 @@ static int SendResource(const Vec2i &pos)
 					if (unit.Type->ResInfo[res]
 						&& unit.Type->ResInfo[res]->TerrainHarvester
 						&& mf.playerInfo.IsExplored(*unit.Player)
-						&& mf.IsTerrainResourceOnMap(res)
+						/// By disabling this, we allow the harvester to find the nearest tile with a resource by itself, in case mf is empty.
+						/*&& mf.IsTerrainResourceOnMap(res)*/ 
 						&& unit.ResourcesHeld < unit.Type->ResInfo[res]->ResourceCapacity
 						&& (unit.CurrentResource != res || unit.ResourcesHeld < unit.Type->ResInfo[res]->ResourceCapacity)) {
 						SendCommandResourceLoc(unit, pos, flush);
@@ -1369,17 +1409,16 @@ static int SendSpellCast(const Vec2i &tilePos)
 			// this unit cannot cast spell
 			continue;
 		}
-		if (dest && &unit == dest) {
-			// no unit can cast spell on himself
-			// n0b0dy: why not?
-			continue;
-		}
 		// CursorValue here holds the spell type id
 		const SpellType *spell = SpellTypeTable[CursorValue];
 		if (!spell) {
 			fprintf(stderr, "unknown spell-id: %d\n", CursorValue);
 			ExitFatal(1);
 		}
+		if (dest && dest == &unit && (!spell->Condition || spell->Condition->TargetSelf == CONDITION_FALSE)) {
+			// Only spells with explicit 'self: true' allows self targetting
+			continue;
+		}
 		SendCommandSpellCast(unit, tilePos, spell->Target == TargetPosition ? NULL : dest , CursorValue, flush);
 		ret = 1;
 	}
@@ -1563,6 +1602,25 @@ static void UISelectStateButtonDown(unsigned)
 
 	if (CursorOn == CursorOnButton) {
 		// FIXME: other buttons?
+		// 74145: Spell-cast on unit portrait
+		if (Selected.size() > 1 && ButtonAreaUnderCursor == ButtonAreaSelected
+			&& CursorAction == ButtonSpellCast) {
+			if (GameObserve || GamePaused || GameEstablishing) {
+				return;
+			}
+			int num = ButtonUnderCursor;
+
+			if (static_cast<size_t>(num) >= Selected.size() || !(MouseButtons & LeftButton)) {
+				return;
+			}
+
+			CUnit &unit = *Selected[num];
+
+			const Vec2i tilePos = unit.tilePos;
+			UnitUnderCursor = &unit;
+			SendSpellCast(tilePos);
+			UnitUnderCursor = NULL;
+		}
 		if (ButtonAreaUnderCursor == ButtonAreaButton) {
 			OldButtonUnderCursor = ButtonUnderCursor;
 			return;
@@ -1585,10 +1643,6 @@ static void UISelectStateButtonDown(unsigned)
 static void UIHandleButtonDown_OnMap(unsigned button)
 {
 	Assert(UI.MouseViewport);
-#ifdef USE_TOUCHSCREEN
-	// Detect double left click
-	const bool doubleLeftButton = MouseButtons & (LeftButton << MouseDoubleShift);
-#endif
 	if ((MouseButtons & LeftButton) && UI.SelectedViewport != UI.MouseViewport) {
 		UI.SelectedViewport = UI.MouseViewport;
 		DebugPrint("selected viewport changed to %ld.\n" _C_
@@ -1597,12 +1651,6 @@ static void UIHandleButtonDown_OnMap(unsigned button)
 
 	// to redraw the cursor immediately (and avoid up to 1 sec delay
 	if (CursorBuilding) {
-#ifdef USE_TOUCHSCREEN
-		// On touch screen is building started with double left click
-		if (!doubleLeftButton) {
-			return;
-		}
-#endif
 		// Possible Selected[0] was removed from map
 		// need to make sure there is a unit to build
 		if (Selected[0] && (MouseButtons & LeftButton)
@@ -1637,11 +1685,7 @@ static void UIHandleButtonDown_OnMap(unsigned button)
 		if (!Selected.empty() && Selected[0]->Player == ThisPlayer && CursorState == CursorStatePoint) {
 			CursorState = CursorStatePieMenu;
 		}
-#ifdef USE_TOUCHSCREEN
-	} else if (doubleLeftButton) {
-#else
 	} else if (MouseButtons & RightButton) {
-#endif
 		if (!GameObserve && !GamePaused && !GameEstablishing && UI.MouseViewport->IsInsideMapArea(CursorScreenPos)) {
 			CUnit *unit;
 			// FIXME: Rethink the complete chaos of coordinates here
@@ -1693,6 +1737,7 @@ static void UIHandleButtonDown_OnButton(unsigned button)
 {
 	// clicked on info panel - selection shown
 	if (Selected.size() > 1 && ButtonAreaUnderCursor == ButtonAreaSelected) {
+		PlayGameSound(GameSounds.Click.Sound, MaxSampleVolume);
 		DoSelectionButtons(ButtonUnderCursor, button);
 	} else if ((MouseButtons & LeftButton)) {
 		//  clicked on menu button
@@ -1704,6 +1749,8 @@ static void UIHandleButtonDown_OnButton(unsigned button)
 			} else if (ButtonUnderCursor == ButtonUnderNetworkDiplomacy && !GameDiplomacyButtonClicked) {
 				PlayGameSound(GameSounds.Click.Sound, MaxSampleVolume);
 				GameDiplomacyButtonClicked = true;
+			} else if (ButtonUnderCursor == ButtonUnderFreeWorkers) {
+				UiFindIdleWorker();
 			}
 			//  clicked on user buttons
 		} else if (ButtonAreaUnderCursor == ButtonAreaUser) {
@@ -1730,6 +1777,7 @@ static void UIHandleButtonDown_OnButton(unsigned button)
 					const COrder_Train &order = *static_cast<COrder_Train *>(Selected[0]->Orders[ButtonUnderCursor]);
 
 					DebugPrint("Cancel slot %d %s\n" _C_ ButtonUnderCursor _C_ order.GetUnitType().Ident.c_str());
+					PlayGameSound(GameSounds.Click.Sound, MaxSampleVolume);
 					SendCommandCancelTraining(*Selected[0], ButtonUnderCursor, &order.GetUnitType());
 				}
 			}
@@ -1738,6 +1786,7 @@ static void UIHandleButtonDown_OnButton(unsigned button)
 			if (!GameObserve && !GamePaused && !GameEstablishing && ThisPlayer->IsTeamed(*Selected[0])) {
 				if (ButtonUnderCursor == 0 && Selected.size() == 1) {
 					DebugPrint("Cancel upgrade %s\n" _C_ Selected[0]->Type->Ident.c_str());
+					PlayGameSound(GameSounds.Click.Sound, MaxSampleVolume);
 					SendCommandCancelUpgradeTo(*Selected[0]);
 				}
 			}
@@ -1746,6 +1795,7 @@ static void UIHandleButtonDown_OnButton(unsigned button)
 			if (!GameObserve && !GamePaused && !GameEstablishing && ThisPlayer->IsTeamed(*Selected[0])) {
 				if (ButtonUnderCursor == 0 && Selected.size() == 1) {
 					DebugPrint("Cancel research %s\n" _C_ Selected[0]->Type->Ident.c_str());
+					PlayGameSound(GameSounds.Click.Sound, MaxSampleVolume);
 					SendCommandCancelResearch(*Selected[0]);
 				}
 			}
@@ -1761,20 +1811,26 @@ static void UIHandleButtonDown_OnButton(unsigned button)
 						if (!uins->Boarded || j >= UI.TransportingButtons.size() || (Selected[0]->Player != ThisPlayer && uins->Player != ThisPlayer)) {
 							continue;
 						}
-					if (ButtonAreaUnderCursor == ButtonAreaTransporting
-						&& static_cast<size_t>(ButtonUnderCursor) == j) {
-							Assert(uins->Boarded);
-							const int flush = !(KeyModifiers & ModifierShift);
-							if (ThisPlayer->IsTeamed(*Selected[0]) || uins->Player == ThisPlayer) {
-								SendCommandUnload(*Selected[0], Selected[0]->tilePos, uins, flush);
+						int lastOccupiedButton = j + uins->Type->BoardSize - 1;
+						if (ButtonAreaUnderCursor == ButtonAreaTransporting) {
+							for (int sub_j = j; sub_j <= lastOccupiedButton; sub_j++) {
+								if (static_cast<size_t>(ButtonUnderCursor) == sub_j) {
+									Assert(uins->Boarded);
+									const int flush = !(KeyModifiers & ModifierShift);
+									if (ThisPlayer->IsTeamed(*Selected[0]) || uins->Player == ThisPlayer) {
+										PlayGameSound(GameSounds.Click.Sound, MaxSampleVolume);
+										SendCommandUnload(*Selected[0], Selected[0]->tilePos, uins, flush);
+									}
+								}
 							}
 						}
-						++j;
+						j = lastOccupiedButton + 1;
 					}
 				}
 			}
 		} else if (ButtonAreaUnderCursor == ButtonAreaButton) {
-			if (!GameObserve && !GamePaused && !GameEstablishing && ThisPlayer->IsTeamed(*Selected[0])) {
+			if (!GameObserve && !GamePaused && !GameEstablishing && (ThisPlayer->IsTeamed(*Selected[0]) || Selected[0]->Player->Index == PlayerMax - 1)) {
+				PlayGameSound(GameSounds.Click.Sound, MaxSampleVolume);
 				OldButtonUnderCursor = ButtonUnderCursor;
 			}
 		}
@@ -1802,14 +1858,12 @@ void UIHandleButtonDown(unsigned button)
 	// Detect long left selection click
 	const bool longLeftButton = (MouseButtons & ((LeftButton << MouseHoldShift))) != 0;
 
-#ifdef USE_TOUCHSCREEN
-	// If we are moving with stylus/finger,
-	// left button on touch screen devices is still clicked
-	// Ignore handle if left button is long cliked
-	if (longLeftButton) {
+	if (IsDemoMode()) {
+		// If we are in "demo mode", exit no matter what we click on.
+		void ActionDraw();
+		ActionDraw();
 		return;
 	}
-#endif
 
 	static bool OldShowSightRange;
 	static bool OldShowReactionRange;
@@ -2013,13 +2067,7 @@ void UIHandleButtonUp(unsigned button)
 					num = SelectUnitsInRectangle(pos0, pos1);
 				}
 			}
-#ifdef USE_TOUCHSCREEN
-			// On touch screen select single unit only when long click is detected
-			// This fix problem with emulating right mouse button as long left click on touch screens
-		} else if (button == 0x1000001) {
-#else
 		} else {
-#endif
 			//
 			// Select single unit
 			//
@@ -2087,7 +2135,7 @@ void UIHandleButtonUp(unsigned button)
 					// FIXME: use GameSounds.Burning
 					PlayGameSound(SoundForName("burning"), MaxSampleVolume);
 				} else if (Selected[0]->Player == ThisPlayer || ThisPlayer->IsTeamed(*Selected[0])
-						   || Selected[0]->Player->Type == PlayerNeutral) {
+						   || Selected[0]->Player->Type == PlayerTypes::PlayerNeutral) {
 					PlayUnitSound(*Selected[0], VoiceSelected);
 				} else {
 					PlayGameSound(GameSounds.Click.Sound, MaxSampleVolume);
