@@ -70,9 +70,15 @@ CPreference Preference;
 ----------------------------------------------------------------------------*/
 
 /**
+** <b>Description</b>
+**
 **  Set speed of key scroll
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>SetKeyScrollSpeed</strong>(4)</code></div>
 */
 static int CclSetKeyScrollSpeed(lua_State *l)
 {
@@ -82,9 +88,16 @@ static int CclSetKeyScrollSpeed(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Get speed of key scroll
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code>scroll_speed = <strong>GetKeyScrollSpeed</strong>()
+**		print(scroll_speed)</code></div>
 */
 static int CclGetKeyScrollSpeed(lua_State *l)
 {
@@ -94,9 +107,15 @@ static int CclGetKeyScrollSpeed(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Set speed of mouse scroll
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>SetMouseScrollSpeed</strong>(2)</code></div>
 */
 static int CclSetMouseScrollSpeed(lua_State *l)
 {
@@ -106,9 +125,16 @@ static int CclSetMouseScrollSpeed(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Get speed of mouse scroll
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code>scroll_speed = <strong>GetMouseScrollSpeed</strong>()
+**		print(scroll_speed)</code></div>
 */
 static int CclGetMouseScrollSpeed(lua_State *l)
 {
@@ -202,74 +228,16 @@ static int CclSetDamageMissile(lua_State *l)
 	return 0;
 }
 
-static int CclSetMaxOpenGLTexture(lua_State *l)
-{
-	LuaCheckArgs(l, 1);
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (CclInConfigFile) {
-		GLMaxTextureSizeOverride = LuaToNumber(l, 1);
-	}
-#endif
-	return 0;
-}
-
-static int CclSetUseTextureCompression(lua_State *l)
-{
-	LuaCheckArgs(l, 1);
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (CclInConfigFile) {
-		UseGLTextureCompression = LuaToBoolean(l, 1);
-	}
-#endif
-	return 0;
-}
-
-static int CclSetUseOpenGL(lua_State *l)
-{
-	LuaCheckArgs(l, 1);
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (CclInConfigFile) {
-		// May have been set from the command line
-		if (!ForceUseOpenGL) {
-			UseOpenGL = LuaToBoolean(l, 1);
-		}
-	}
-#endif
-	return 0;
-}
-
-static int CclGetUseOpenGL(lua_State *l)
-{
-	LuaCheckArgs(l, 0);
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	lua_pushboolean(l, UseOpenGL);
-#else
-	lua_pushboolean(l, 0);
-#endif
-	return 0;
-}
-
-static int CclSetZoomNoResize(lua_State *l)
-{
-	LuaCheckArgs(l, 1);
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (CclInConfigFile) {
-		// May have been set from the command line
-		if (!ForceUseOpenGL) {
-			ZoomNoResize = LuaToBoolean(l, 1);
-			if (ZoomNoResize) {
-				UseOpenGL = true;
-			}
-		}
-	}
-#endif
-	return 0;
-}
-
 /**
+** <b>Description</b>
+**
 **  Set the video resolution.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>SetVideoResolution</strong>(640,480)</code></div>
 */
 static int CclSetVideoResolution(lua_State *l)
 {
@@ -285,9 +253,16 @@ static int CclSetVideoResolution(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Get the video resolution.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code>width,height = <strong>GetVideoResolution</strong>()
+**		print("Resolution  is " .. width .. "x" .. height)</code></div>
 */
 static int CclGetVideoResolution(lua_State *l)
 {
@@ -298,9 +273,18 @@ static int CclGetVideoResolution(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Set the video fullscreen mode.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code>-- Full Screen mode enabled
+**		<strong>SetVideoFullScreen</strong>(true)
+**		-- Full Screen mode disabled
+**		<strong>SetVideoFullScreen</strong>(false)</code></div>
 */
 static int CclSetVideoFullScreen(lua_State *l)
 {
@@ -315,9 +299,16 @@ static int CclSetVideoFullScreen(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Get the video fullscreen mode.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code>fullscreenmode = <strong>GetVideoFullScreen</strong>()
+**		print(fullscreenmode)</code></div>
 */
 static int CclGetVideoFullScreen(lua_State *l)
 {
@@ -326,6 +317,44 @@ static int CclGetVideoFullScreen(lua_State *l)
 	return 1;
 }
 
+/**
+** Request a specific initial window size
+*/
+static int CclSetWindowSize(lua_State *l)
+{
+	LuaCheckArgs(l, 2);
+	if (CclInConfigFile) {
+		// May have been set from the command line
+		if (!Video.WindowWidth || !Video.WindowHeight) {
+			Video.WindowWidth = LuaToNumber(l, 1);
+			Video.WindowHeight = LuaToNumber(l, 2);
+		}
+	}
+	return 0;
+}
+
+/**
+** For games with non-square pixels, this sets the scale of vertical pixels versus horizontal pixels.
+** e.g., if your assets are 320x200, but you render at 320x240, this is 1.2.
+*/
+static int CclSetVerticalPixelSize(lua_State *l)
+{
+	LuaCheckArgs(l, 1);
+	if (CclInConfigFile) {
+		luaL_checktype(l, 1, LUA_TNUMBER);
+		Video.VerticalPixelSize = static_cast<double>(lua_tonumber(l, 1));
+	}
+	return 0;
+}
+
+static int CclShowTitleScreens(lua_State *l)
+{
+	LuaCheckArgs(l, 0);
+	ShowTitleScreens();
+	lua_pushboolean(l, 1);
+	return 1;
+}
+
 /**
 **  Default title screens.
 **
@@ -521,6 +550,8 @@ static CContentType *CclParseContent(lua_State *l)
 				content = new CContentTypeFormattedText2;
 			} else if (!strcmp(key, "Icon")) {
 				content = new CContentTypeIcon;
+			} else if (!strcmp(key, "Graphic")) {
+				content = new CContentTypeGraphic;
 			} else if (!strcmp(key, "LifeBar")) {
 				content = new CContentTypeLifeBar;
 			} else if (!strcmp(key, "CompleteBar")) {
@@ -674,7 +705,7 @@ static int CclDefineViewports(lua_State *l)
 			UI.Viewports[i].MapPos.y = LuaToNumber(l, j + 1, 2);
 			const int slot = LuaToNumber(l, j + 1, 3);
 			if (slot != -1) {
-				UI.Viewports[i].Unit = &UnitManager.GetSlotUnit(slot);
+				UI.Viewports[i].Unit = &UnitManager->GetSlotUnit(slot);
 			}
 			++i;
 		} else {
@@ -710,9 +741,18 @@ static int CclRightButtonMoves(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Enable/disable the fancy buildings.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code>-- Enable fancy buildings
+**		  <strong>SetFancyBuildings</strong>(true)
+**		  -- Disable fancy buildings
+**		  <strong>SetFancyBuildings</strong>(false)</code></div>
 */
 static int CclSetFancyBuildings(lua_State *l)
 {
@@ -917,6 +957,18 @@ void CallHandler(unsigned int handle, int value)
 	lua_pop(Lua, 1);
 }
 
+/**
+**  Clear all buttons
+**
+**  @param l  Lua state.
+*/
+static int CclClearButtons(lua_State *l)
+{
+	LuaCheckArgs(l, 0);
+	CleanButtons();
+	return 0;
+}
+
 /**
 **  Define a button.
 **
@@ -964,6 +1016,8 @@ static int CclDefineButton(lua_State *l)
 				ba.Action = ButtonTrain;
 			} else if (!strcmp(value, "patrol")) {
 				ba.Action = ButtonPatrol;
+			} else if (!strcmp(value, "explore")) {
+				ba.Action = ButtonExplore;
 			} else if (!strcmp(value, "stand-ground")) {
 				ba.Action = ButtonStandGround;
 			} else if (!strcmp(value, "attack-ground")) {
@@ -1028,6 +1082,8 @@ static int CclDefineButton(lua_State *l)
 				ba.Allowed = ButtonCheckUnitsAnd;
 			} else if (!strcmp(value, "check-units-not")) {
 				ba.Allowed = ButtonCheckUnitsNot;
+			} else if (!strcmp(value, "check-units-nor")) {
+				ba.Allowed = ButtonCheckUnitsNor;
 			} else if (!strcmp(value, "check-network")) {
 				ba.Allowed = ButtonCheckNetwork;
 			} else if (!strcmp(value, "check-no-network")) {
@@ -1044,6 +1100,8 @@ static int CclDefineButton(lua_State *l)
 				ba.Allowed = ButtonCheckResearch;
 			} else if (!strcmp(value, "check-single-research")) {
 				ba.Allowed = ButtonCheckSingleResearch;
+			} else if (!strcmp(value, "check-debug")) {
+				ba.Allowed = ButtonCheckDebug;
 			} else {
 				LuaError(l, "Unsupported action: %s" _C_ value);
 			}
@@ -1102,6 +1160,32 @@ static int CclDefineButton(lua_State *l)
 	return 0;
 }
 
+static int CclCopyButtonsForUnitType(lua_State *l)
+{
+	LuaCheckArgs(l, 2);
+
+	// Slot identifier
+	const char* fromName = LuaToString(l, 1);
+	CUnitType *from = UnitTypeByIdent(fromName);
+	const char* toName = LuaToString(l, 2);
+	CUnitType *to = UnitTypeByIdent(toName);
+	if (!to) {
+		LuaError(l, "Unknown unit-type '%s'\n" _C_ toName);
+	}
+	if (!from) {
+		LuaError(l, "Unknown unit-type '%s'\n" _C_ fromName);
+	}
+
+	for (auto btn : UnitButtonTable) {
+		if (btn->UnitMask.find(fromName) != std::string::npos) {
+			btn->UnitMask += toName;
+			btn->UnitMask += ",";
+		}
+	}
+
+	return 0;
+}
+
 /**
 **  Run the set-selection-changed-hook.
 */
@@ -1135,19 +1219,30 @@ void SelectedUnitChanged()
 */
 static int CclSetSelectionStyle(lua_State *l)
 {
-	LuaCheckArgs(l, 1);
+	if (lua_gettop(l) < 1) {
+		LuaError(l, "incorrect argument");
+	}
 
 	const char *style = LuaToString(l, 1);
 	if (!strcmp(style, "rectangle")) {
+		LuaCheckArgs(l, 1);
 		DrawSelection = DrawSelectionRectangle;
 	} else if (!strcmp(style, "alpha-rectangle")) {
+		LuaCheckArgs(l, 1);
 		DrawSelection = DrawSelectionRectangleWithTrans;
 	} else if (!strcmp(style, "circle")) {
+		LuaCheckArgs(l, 1);
 		DrawSelection = DrawSelectionCircle;
 	} else if (!strcmp(style, "alpha-circle")) {
+		LuaCheckArgs(l, 1);
 		DrawSelection = DrawSelectionCircleWithTrans;
 	} else if (!strcmp(style, "corners")) {
+		LuaCheckArgs(l, 1);
 		DrawSelection = DrawSelectionCorners;
+	} else if (!strcmp(style, "ellipse")) {
+		LuaCheckArgs(l, 2);
+		float factor = LuaToFloat(l, 2);
+		DrawSelection = DrawSelectionEllipse(factor);
 	} else {
 		LuaError(l, "Unsupported selection style");
 	}
@@ -1155,8 +1250,14 @@ static int CclSetSelectionStyle(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Add a new message.
 **
+** Example:
+**
+** <div class="example"><code><strong>AddMessage</strong>("Hello World!")</code></div>
+**
 **  @param l  Lua state.
 */
 static int CclAddMessage(lua_State *l)
@@ -1179,9 +1280,15 @@ static int CclSetGroupKeys(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 ** Set basic map caracteristics.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>PresentMap</strong>("Map description", 1, 128, 128, 17)</code></div>
 */
 static int CclPresentMap(lua_State *l)
 {
@@ -1197,9 +1304,16 @@ static int CclPresentMap(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 ** Define the lua file that will build the map
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code>-- Load map setup from file
+**		<strong>DefineMapSetup</strong>("Setup.sms")</code></div>
 */
 static int CclDefineMapSetup(lua_State *l)
 {
@@ -1208,6 +1322,27 @@ static int CclDefineMapSetup(lua_State *l)
 
 	return 0;
 }
+/**
+** <b>Description</b>
+**
+** Declare which codepage the font files are in. Text is handled internally
+** as UTF-8 everywhere, but the font rendering system uses graphics with 256
+** symbols. Commonly, DOS and early Windows games used codepage 437 or 1252 for
+** western European languages, or 866 for Russian and some other cyrillic
+** writing systems. These are the only ones that are currently supported, but
+** more can easily be added. All text is mapped into the codepage that is set
+** for the font files. If the codepage is not one of the supported ones, or if
+** something doesn't map (for example, some accented characters with codepage
+** 866, or cyrillic letters with codepage 437), a simple "visual" mapping to
+** 7-bit ASCII is used to at least print something that may be recognizable.
+*/
+static int CclSetFontCodePage(lua_State *l)
+{
+	LuaCheckArgs(l, 1);
+	FontCodePage = LuaToNumber(l, 1);
+
+	return 0;
+}
 
 /**
 **  Register CCL features for UI.
@@ -1229,17 +1364,17 @@ void UserInterfaceCclRegister()
 	lua_register(Lua, "SetClickMissile", CclSetClickMissile);
 	lua_register(Lua, "SetDamageMissile", CclSetDamageMissile);
 
-	lua_register(Lua, "SetMaxOpenGLTexture", CclSetMaxOpenGLTexture);
-	lua_register(Lua, "SetUseTextureCompression", CclSetUseTextureCompression);
-	lua_register(Lua, "SetUseOpenGL", CclSetUseOpenGL);
-	lua_register(Lua, "GetUseOpenGL", CclGetUseOpenGL);
-	lua_register(Lua, "SetZoomNoResize", CclSetZoomNoResize);
 	lua_register(Lua, "SetVideoResolution", CclSetVideoResolution);
 	lua_register(Lua, "GetVideoResolution", CclGetVideoResolution);
 	lua_register(Lua, "SetVideoFullScreen", CclSetVideoFullScreen);
 	lua_register(Lua, "GetVideoFullScreen", CclGetVideoFullScreen);
+	lua_register(Lua, "SetWindowSize", CclSetWindowSize);
+	lua_register(Lua, "SetVerticalPixelSize", CclSetVerticalPixelSize);
+
+	lua_register(Lua, "SetFontCodePage", CclSetFontCodePage);
 
 	lua_register(Lua, "SetTitleScreens", CclSetTitleScreens);
+	lua_register(Lua, "ShowTitleScreens", CclShowTitleScreens);
 
 	lua_register(Lua, "DefinePanelContents", CclDefinePanelContents);
 	lua_register(Lua, "DefinePopup", CclDefinePopup);
@@ -1250,6 +1385,9 @@ void UserInterfaceCclRegister()
 	lua_register(Lua, "SetFancyBuildings", CclSetFancyBuildings);
 
 	lua_register(Lua, "DefineButton", CclDefineButton);
+	lua_register(Lua, "ClearButtons", CclClearButtons);
+
+	lua_register(Lua, "CopyButtonsForUnitType", CclCopyButtonsForUnitType);
 
 	lua_register(Lua, "DefineButtonStyle", CclDefineButtonStyle);
 
