@@ -79,6 +79,12 @@ void ShowLoadProgress(const char *fmt, ...)
 {
 	va_list va;
 	char temp[4096];
+	static unsigned int lastProgressUpdate = SDL_GetTicks();
+	if (SDL_GetTicks() < lastProgressUpdate + 500) {
+		// Only show progress updates every half second, otherwise we're waiting for the screen too much
+		return;
+	}
+	lastProgressUpdate = SDL_GetTicks();
 
 	va_start(va, fmt);
 	vsnprintf(temp, sizeof(temp) - 1, fmt, va);
@@ -101,6 +107,16 @@ void ShowLoadProgress(const char *fmt, ...)
 	}
 }
 
+/**
+**  Check if the game is running in Demo mode.
+**
+**  @return  True if the game is in demo mode, false otherwise
+*/
+bool IsDemoMode()
+{
+	return GameRunning && ThisPlayer->Type == PlayerTypes::PlayerNobody;
+}
+
 CUnitInfoPanel::~CUnitInfoPanel()
 {
 	for (std::vector<CContentType *>::iterator content = Contents.begin();
@@ -127,7 +143,7 @@ CUserInterface::CUserInterface() :
 	ViewportCursorColor(0), Offset640X(0), Offset480Y(0),
 	VictoryBackgroundG(NULL), DefeatBackgroundG(NULL)
 {
-	MouseWarpPos.x = MouseWarpPos.y = 0;
+	MouseWarpPos.x = MouseWarpPos.y = -1;
 
 	Point.Name = "cursor-point";
 	Glass.Name = "cursor-glass";
@@ -148,6 +164,20 @@ CUserInterface::CUserInterface() :
 
 	NormalFontColor = "light-blue";
 	ReverseFontColor = "yellow";
+
+	LifeBarColorNames.push_back("dark-green");
+	LifeBarColorNames.push_back("yellow");
+	LifeBarColorNames.push_back("orange");
+	LifeBarColorNames.push_back("red");
+
+    LifeBarPercents.push_back(75);
+	LifeBarPercents.push_back(50);
+	LifeBarPercents.push_back(25);
+	LifeBarPercents.push_back(0);
+
+	LifeBarBorder = true;
+	LifeBarYOffset = 0;
+	LifeBarPadding = 0;
 }
 
 /**
@@ -175,12 +205,34 @@ void InitUserInterface()
 	UI.Offset640X = (Video.Width - 640) / 2;
 	UI.Offset480Y = (Video.Height - 480) / 2;
 
+	UI.LifeBarColorsInt.clear();
+	for(std::vector<std::string>::iterator it = UI.LifeBarColorNames.begin(); it != UI.LifeBarColorNames.end(); ++it) {
+		UI.LifeBarColorsInt.push_back(IndexToColor(GetColorIndexByName((*it).c_str())));
+	}
+
 	//
 	// Calculations
 	//
 	if (Map.Info.MapWidth) {
-		UI.MapArea.EndX = std::min<int>(UI.MapArea.EndX, UI.MapArea.X + Map.Info.MapWidth * PixelTileSize.x - 1);
-		UI.MapArea.EndY = std::min<int>(UI.MapArea.EndY, UI.MapArea.Y + Map.Info.MapHeight * PixelTileSize.y - 1);
+		/// Align UI map area if at least one of the map dimensions fits completely into it.
+		const uint16_t mapWidthInPixels = Map.Info.MapWidth * PixelTileSize.x;
+		const uint16_t mapHeightInPixels = Map.Info.MapHeight * PixelTileSize.y;
+		const uint16_t uiMapAreaWidth = UI.MapArea.EndX - UI.MapArea.X;
+		const uint16_t uiMapAreaHeight = UI.MapArea.EndY - UI.MapArea.Y;
+		uint16_t xAlign = 0;
+		uint16_t yAlign = 0;
+		
+		if (mapWidthInPixels < uiMapAreaWidth) {
+			xAlign = (uiMapAreaWidth - mapWidthInPixels) / 2;
+			UI.MapArea.X += xAlign;
+		}	
+		if (mapHeightInPixels < uiMapAreaHeight) {
+			yAlign = (uiMapAreaHeight - mapHeightInPixels) / 2;
+			UI.MapArea.Y += yAlign;
+		}
+
+		UI.MapArea.EndX = std::min<int>(UI.MapArea.EndX - xAlign, UI.MapArea.X + Map.Info.MapWidth * PixelTileSize.x - 1);
+		UI.MapArea.EndY = std::min<int>(UI.MapArea.EndY - yAlign, UI.MapArea.Y + Map.Info.MapHeight * PixelTileSize.y - 1);
 	}
 
 	UI.SelectedViewport = UI.Viewports;
@@ -218,30 +270,24 @@ void CUserInterface::Load()
 	for (int i = 0; i <= FreeWorkersCount; ++i) {
 		if (Resources[i].G) {
 			Resources[i].G->Load();
-			Resources[i].G->UseDisplayFormat();
 		}
 	}
 
 	if (InfoPanel.G) {
 		InfoPanel.G->Load();
-		InfoPanel.G->UseDisplayFormat();
 	}
 	if (ButtonPanel.G) {
 		ButtonPanel.G->Load();
-		ButtonPanel.G->UseDisplayFormat();
 	}
 	if (PieMenu.G) {
 		PieMenu.G->Load();
-		PieMenu.G->UseDisplayFormat();
 	}
 
 	if (Preference.IconFrameG) {
 		Preference.IconFrameG->Load();
-		Preference.IconFrameG->UseDisplayFormat();
 	}
 	if (Preference.PressedIconFrameG) {
 		Preference.PressedIconFrameG->Load();
-		Preference.PressedIconFrameG->UseDisplayFormat();
 	}
 
 	//  Resolve cursors
@@ -318,7 +364,9 @@ void CleanUserInterface()
 
 	// Resource Icons
 	for (int i = 0; i <= FreeWorkersCount; ++i) {
-		CGraphic::Free(UI.Resources[i].G);
+		if (UI.Resources[i].G) {
+			CGraphic::Free(UI.Resources[i].G);
+		}
 	}
 
 	// Info Panel
