@@ -94,19 +94,12 @@ void initGuichan()
 {
 	gcn::Graphics *graphics;
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (UseOpenGL) {
-		graphics = new MyOpenGLGraphics();
-	} else
-#endif
-	{
-		graphics = new gcn::SDLGraphics();
+	graphics = new gcn::SDLGraphics();
 
-		// Set the target for the graphics object to be the screen.
-		// In other words, we will draw to the screen.
-		// Note, any surface will do, it doesn't have to be the screen.
-		((gcn::SDLGraphics *)graphics)->setTarget(TheScreen);
-	}
+	// Set the target for the graphics object to be the screen.
+	// In other words, we will draw to the screen.
+	// Note, any surface will do, it doesn't have to be the screen.
+	((gcn::SDLGraphics *)graphics)->setTarget(&TheScreen);
 
 	Input = new gcn::SDLInput();
 
@@ -115,11 +108,7 @@ void initGuichan()
 	Gui->setInput(Input);
 	Gui->setTop(NULL);
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	Gui->setUseDirtyDrawing(!UseOpenGL);
-#else
 	Gui->setUseDirtyDrawing(1);
-#endif
 
 	GuichanCallbacks.ButtonPressed = &MenuHandleButtonDown;
 	GuichanCallbacks.ButtonReleased = &MenuHandleButtonUp;
@@ -171,11 +160,7 @@ void handleInput(const SDL_Event *event)
 void DrawGuichanWidgets()
 {
 	if (Gui) {
-#if defined(USE_OPENGL) || defined(USE_GLES)
-		Gui->setUseDirtyDrawing(!UseOpenGL && !GameRunning && !Editor.Running);
-#else
 		Gui->setUseDirtyDrawing(!GameRunning && !Editor.Running);
-#endif
 		Gui->draw();
 	}
 }
@@ -211,6 +196,85 @@ void LuaActionListener::action(const std::string &eventId)
 	callback.run();
 }
 
+bool LuaActionListener::keyPress(const gcn::Key& key) {
+	callback.pushPreamble();
+	callback.pushString("keyPress");
+	callback.pushString(key.toString());
+	callback.run(1);
+	return callback.popBoolean();
+}
+
+bool LuaActionListener::keyRelease(const gcn::Key& key) {
+	callback.pushPreamble();
+	callback.pushString("keyRelease");
+	callback.pushString(key.toString());
+	callback.run(1);
+	return callback.popBoolean();
+}
+
+void LuaActionListener::hotKeyPress(const gcn::Key& key) {
+	callback.pushPreamble();
+	callback.pushString("hotKeyPress");
+	callback.pushString(key.toString());
+	callback.run();
+}
+
+void LuaActionListener::hotKeyRelease(const gcn::Key& key) {
+	callback.pushPreamble();
+	callback.pushString("hotKeyRelease");
+	callback.pushString(key.toString());
+	callback.run();
+}
+
+void LuaActionListener::mouseIn() {
+	callback.pushPreamble();
+	callback.pushString("mouseIn");
+	callback.run();
+}
+
+void LuaActionListener::mouseOut() {
+	callback.pushPreamble();
+	callback.pushString("mouseOut");
+	callback.run();
+}
+
+void LuaActionListener::mousePress(int x, int y, int btn) {
+	callback.pushPreamble();
+	callback.pushString("mousePress");
+	callback.pushInteger(btn);
+	callback.run();
+}
+
+void LuaActionListener::mouseRelease(int x, int y, int btn) {
+	callback.pushPreamble();
+	callback.pushString("mouseRelease");
+	callback.pushInteger(btn);
+	callback.run();
+}
+
+void LuaActionListener::mouseClick(int x, int y, int btn, int cnt) {
+	callback.pushPreamble();
+	callback.pushString("mouseClick");
+	callback.pushInteger(btn);
+	callback.pushInteger(cnt);
+	callback.run();
+}
+
+void LuaActionListener::mouseWheelUp(int x, int y) {
+	callback.pushPreamble();
+	callback.pushString("mouseWheelUp");
+	callback.run();
+}
+
+void LuaActionListener::mouseWheelDown(int x, int y) {
+	callback.pushPreamble();
+	callback.pushString("mouseWheelDown");
+	callback.run();
+}
+
+void LuaActionListener::mouseMotion(int x, int y) {}
+
+
 /**
 **  LuaActionListener destructor
 */
@@ -218,110 +282,6 @@ LuaActionListener::~LuaActionListener()
 {
 }
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-
-/*----------------------------------------------------------------------------
---  MyOpenGLGraphics
-----------------------------------------------------------------------------*/
-
-void MyOpenGLGraphics::_beginDraw()
-{
-	gcn::Rectangle area(0, 0, Video.Width, Video.Height);
-	pushClipArea(area);
-}
-
-void MyOpenGLGraphics::_endDraw()
-{
-	popClipArea();
-}
-
-void MyOpenGLGraphics::drawImage(const gcn::Image *image, int srcX, int srcY,
-								 int dstX, int dstY, int width, int height)
-{
-	const gcn::ClipRectangle &r = this->getCurrentClipArea();
-	int right = std::min<int>(r.x + r.width - 1, Video.Width - 1);
-	int bottom = std::min<int>(r.y + r.height - 1, Video.Height - 1);
-
-	if (r.x > right || r.y > bottom) {
-		return;
-	}
-
-	PushClipping();
-	SetClipping(r.x, r.y, right, bottom);
-	((CGraphic *)image)->DrawSubClip(srcX, srcY, width, height,
-									 dstX + mClipStack.top().xOffset, dstY + mClipStack.top().yOffset);
-	PopClipping();
-}
-
-void MyOpenGLGraphics::drawPoint(int x, int y)
-{
-	gcn::Color c = this->getColor();
-	Video.DrawPixelClip(Video.MapRGBA(0, c.r, c.g, c.b, c.a),
-						x + mClipStack.top().xOffset, y + mClipStack.top().yOffset);
-}
-
-void MyOpenGLGraphics::drawLine(int x1, int y1, int x2, int y2)
-{
-	gcn::Color c = this->getColor();
-	const PixelPos pos1(x1 + mClipStack.top().xOffset, y1 + mClipStack.top().yOffset);
-	const PixelPos pos2(x2 + mClipStack.top().xOffset, y2 + mClipStack.top().yOffset);
-
-	Video.DrawLineClip(Video.MapRGBA(0, c.r, c.g, c.b, c.a), pos1, pos2);
-}
-
-void MyOpenGLGraphics::drawRectangle(const gcn::Rectangle &rectangle)
-{
-	gcn::Color c = this->getColor();
-	if (c.a == 0) {
-		return;
-	}
-
-	const gcn::ClipRectangle top = mClipStack.top();
-	gcn::Rectangle area = gcn::Rectangle(rectangle.x + top.xOffset,
-										 rectangle.y + top.yOffset,
-										 rectangle.width, rectangle.height);
-
-	if (!area.intersect(top)) {
-		return;
-	}
-
-	int x1 = std::max<int>(area.x, top.x);
-	int y1 = std::max<int>(area.y, top.y);
-	int x2 = std::min<int>(area.x + area.width, top.x + top.width);
-	int y2 = std::min<int>(area.y + area.height, top.y + top.height);
-
-	Video.DrawTransRectangle(Video.MapRGB(0, c.r, c.g, c.b),
-							 x1, y1, x2 - x1, y2 - y1, mColor.a);
-}
-
-void MyOpenGLGraphics::fillRectangle(const gcn::Rectangle &rectangle)
-{
-	const gcn::Color c = this->getColor();
-
-	if (c.a == 0) {
-		return;
-	}
-
-	const gcn::ClipRectangle top = mClipStack.top();
-	gcn::Rectangle area = gcn::Rectangle(rectangle.x + top.xOffset,
-										 rectangle.y + top.yOffset,
-										 rectangle.width, rectangle.height);
-
-	if (!area.intersect(top)) {
-		return;
-	}
-
-	int x1 = std::max<int>(area.x, top.x);
-	int y1 = std::max<int>(area.y, top.y);
-	int x2 = std::min<int>(area.x + area.width, top.x + top.width);
-	int y2 = std::min<int>(area.y + area.height, top.y + top.height);
-
-	Video.FillTransRectangle(Video.MapRGB(0, c.r, c.g, c.b),
-							 x1, y1, x2 - x1, y2 - y1, c.a);
-}
-
-#endif
-
 /*----------------------------------------------------------------------------
 --  ImageButton
 ----------------------------------------------------------------------------*/
@@ -952,7 +912,8 @@ void MultiLineLabel::draw(gcn::Graphics *graphics)
 	}
 
 	for (int i = 0; i < (int)this->mTextRows.size(); ++i) {
-		graphics->drawText(this->mTextRows[i], textX, textY + i * this->getFont()->getHeight(),
+		if (this->mTextRows[i].length() != 0)
+			graphics->drawText(this->mTextRows[i], textX, textY + i * this->getFont()->getHeight(),
 						   this->getAlignment());
 	}
 }
@@ -1273,6 +1234,12 @@ void ImageTextField::draw(gcn::Graphics *graphics)
 {
 	gcn::Font *font;
 	int x, y;
+	std::string drawText;
+	if (isPassword) {
+		drawText = std::string(mText.size(), '*');
+	} else {
+		drawText = mText;
+	}
 	CGraphic *img = this->itemImage;
 	if (!img) {
 		fprintf(stderr, "Not all graphics for ImageTextField were set\n");
@@ -1283,7 +1250,7 @@ void ImageTextField::draw(gcn::Graphics *graphics)
 
 	if (hasFocus())
 	{
-		drawCaret(graphics, getFont()->getWidth(mText.substr(0, mCaretPosition)) - mXScroll);
+		drawCaret(graphics, getFont()->getWidth(drawText.substr(0, mCaretPosition)) - mXScroll);
 	}
 
 	graphics->setColor(getForegroundColor());
@@ -1303,17 +1270,17 @@ void ImageTextField::draw(gcn::Graphics *graphics)
 
 		getTextSelectionPositions(&first, &len);
 
-		tmpStr = std::string(mText.substr(0, first));
+		tmpStr = std::string(drawText.substr(0, first));
 		selX = font->getWidth(tmpStr);
 
-		tmpStr = std::string(mText.substr(first, len));
+		tmpStr = std::string(drawText.substr(first, len));
 		selW = font->getWidth(tmpStr);
 
 		graphics->setColor(gcn::Color(127, 127, 127));
 		graphics->fillRectangle(gcn::Rectangle(x + selX, y, selW, font->getHeight()));
 	}
 
-	graphics->drawText(mText, x, y);
+	graphics->drawText(drawText, x, y);
 }
 
 void ImageTextField::drawBorder(gcn::Graphics *graphics)
@@ -1442,6 +1409,11 @@ void ImageListBox::mousePress(int, int y, int button)
 		setSelected(y / (itemImage ? std::max<int>(getFont()->getHeight(), itemImage->getHeight()) : getFont()->getHeight()));
 		generateAction();
 	}
+	else if (button == gcn::MouseInput::RIGHT && hasMouse())
+	{
+		setSelected(-1);
+		generateAction();
+	}
 }
 
 void ImageListBox::setSelected(int selected)
@@ -1465,13 +1437,13 @@ void ImageListBox::setSelected(int selected)
 			mSelected = selected;
 		}
 
-		Widget *par = getParent();
-		if (par == NULL)
-		{
-			return;
+		Widget *par = this;
+		while (par != NULL) {
+			par->setDirty(true);
+			par = par->getParent();
 		}
 
-		gcn::ScrollArea* scrollArea = dynamic_cast<gcn::ScrollArea *>(par);
+		gcn::ScrollArea* scrollArea = dynamic_cast<gcn::ScrollArea *>(getParent());
 		if (scrollArea != NULL)
 		{
 			gcn::Rectangle scroll;
@@ -1767,7 +1739,6 @@ void ImageListBoxWidget::draw(gcn::Graphics *graphics)
 
 		if (mContent->getBorderSize() > 0)
 		{
-			img = this->itemImage;
 			gcn::Rectangle rec = mContent->getDimension();
 			rec.x -= mContent->getBorderSize();
 			rec.y -= mContent->getBorderSize();
@@ -2445,15 +2416,12 @@ int MenuScreen::run(bool loop)
 	GameCursor = UI.Point.Cursor;
 	CursorOn = CursorOnUnknown;
 
-	CallbackMusicOn();
-
 	if (loop) {
 		const EventCallback *old_callbacks = GetCallbacks();
 		SetCallbacks(&GuichanCallbacks);
 		while (runLoop) {
 			UpdateDisplay();
 			RealizeVideoMemory();
-			CheckMusicFinished();
 			WaitEventsOneFrame();
 		}
 		SetCallbacks(old_callbacks);
@@ -2473,8 +2441,11 @@ void MenuScreen::stop(int result, bool stopAll)
 {
 	if (!this->runLoop) {
 		Gui->setTop(this->oldtop);
-		Assert(MenuStack.top() == this);
-		MenuStack.pop();
+		if (MenuStack.size() > 0 && MenuStack.top() == this) {
+			MenuStack.pop();
+		} else {
+			DebugPrint("ALARM! A menu we're trying to stop is NOT at the top of the menu stack!!");
+		}
 		if (stopAll) {
 			while (!MenuStack.empty()) {
 				MenuStack.pop();
