@@ -42,6 +42,8 @@
 #include "construct.h"
 #include "interface.h"
 #include "map.h"
+#include "netconnect.h"
+#include "network.h"
 #include "pathfinder.h"
 #include "player.h"
 #include "script.h"
@@ -64,11 +66,20 @@
 extern unsigned CclGetResourceByName(lua_State *l);
 
 /**
+** <b>Description</b>
+**
 **  Set training queue
 **
 **  @param l  Lua state.
 **
 **  @return  The old state of the training queue
+**
+** Example:
+**
+** <div class="example"><code>-- Training queue available. Train multiple units.
+**		<strong>SetTrainingQueue</strong>(true)
+**		-- Train one unit at a time.
+**		<strong>SetTrainingQueue</strong>(false)</code></div>
 */
 static int CclSetTrainingQueue(lua_State *l)
 {
@@ -83,6 +94,11 @@ static int CclSetTrainingQueue(lua_State *l)
 **  @param l  Lua state.
 **
 **  @return   The old state of the flag
+**
+** Example:
+**
+** <div class="example"><code><strong>SetBuildingCapture</strong>(true)
+**	<strong>SetBuildingCapture</strong>(false)</code></div>
 */
 static int CclSetBuildingCapture(lua_State *l)
 {
@@ -97,6 +113,11 @@ static int CclSetBuildingCapture(lua_State *l)
 **  @param l  Lua state.
 **
 **  @return   The old state of the flag
+**
+** Example:
+**
+** <div class="example"><code><strong>SetRevealAttacker</strong>(true)
+**	<strong>SetRevealAttacker</strong>(false)</code></div>
 */
 static int CclSetRevealAttacker(lua_State *l)
 {
@@ -109,6 +130,15 @@ static int CclSetRevealAttacker(lua_State *l)
 **  Set cost multiplier to RepairCost for buildings additional workers helping (0 = no additional cost)
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code>-- No cost
+**	<strong>ResourcesMultiBuildersMultiplier</strong>(0)
+**	-- Each builder helping will cost 1 resource
+**	<strong>ResourcesMultiBuildersMultiplier</strong>(1)
+**	-- Each builder helping will cost 10 resource
+**	<strong>ResourcesMultiBuildersMultiplier</strong>(10)</code></div>
 */
 static int CclResourcesMultiBuildersMultiplier(lua_State *l)
 {
@@ -135,7 +165,7 @@ static CUnit *CclGetUnit(lua_State *l)
 		}
 		return NULL;
 	}
-	return &UnitManager.GetSlotUnit(num);
+	return &UnitManager->GetSlotUnit(num);
 }
 
 /**
@@ -149,8 +179,8 @@ CUnit *CclGetUnitFromRef(lua_State *l)
 {
 	const char *const value = LuaToString(l, -1);
 	unsigned int slot = strtol(value + 1, NULL, 16);
-	Assert(slot < UnitManager.GetUsedSlotCount());
-	return &UnitManager.GetSlotUnit(slot);
+	Assert(slot < UnitManager->GetUsedSlotCount());
+	return &UnitManager->GetSlotUnit(slot);
 }
 
 
@@ -271,6 +301,26 @@ static void CclParseOrders(lua_State *l, CUnit &unit)
 **
 **  @todo  Verify that vision table is always correct (transporter)
 **  @todo (PlaceUnit() and host-info).
+**
+** Example:
+**
+** <div class="example"><code>footman = CreateUnit("unit-footman", 0, {0, 1})
+**	-- The unit will appear selected
+**	<strong>Unit</strong>(footman,{"selected"})
+**	-- The unit will be considered destroyed
+**	<strong>Unit</strong>(footman,{"destroyed"})
+**	-- The unit will be considered removed
+**	<strong>Unit</strong>(footman,{"removed"})
+**	-- The unit will be considered as a summoned unit
+**	<strong>Unit</strong>(footman,{"summoned",500})
+**	-- The unit will face on south
+**	<strong>Unit</strong>(footman,{"direction",0})
+**	-- The unit will be displayed with his 3rd frame
+**	<strong>Unit</strong>(footman,{"frame", 3})
+**	-- The footman will have a high sight
+**	<strong>Unit</strong>(footman,{"current-sight-range",9})
+**	-- Change the unit color to be the ones from player 1
+**	<strong>Unit</strong>(footman,{"rescued-from",1})</code></div>
 */
 static int CclUnit(lua_State *l)
 {
@@ -280,7 +330,8 @@ static int CclUnit(lua_State *l)
 		LuaError(l, "incorrect argument");
 	}
 
-	CUnit *unit = NULL;
+	CUnit *unit = &UnitManager->GetSlotUnit(slot);
+	bool hadType = unit->Type != NULL;
 	CUnitType *type = NULL;
 	CUnitType *seentype = NULL;
 	CPlayer *player = NULL;
@@ -309,7 +360,6 @@ static int CclUnit(lua_State *l)
 			// unit->CurrentAction()==UnitActionDie so we have to wait
 			// until we parsed at least Unit::Orders[].
 			Assert(type);
-			unit = &UnitManager.GetSlotUnit(slot);
 			unit->Init(*type);
 			unit->Seen.Type = seentype;
 			unit->Active = 0;
@@ -332,10 +382,10 @@ static int CclUnit(lua_State *l)
 			pos.y = LuaToNumber(l, -1, 2);
 			w = LuaToNumber(l, -1, 3);
 			h = LuaToNumber(l, -1, 4);
-			MapSight(*player, pos, w, h, unit->CurrentSightRange, MapMarkTileSight);
+			MapSight(*player, *unit, pos, w, h, unit->CurrentSightRange, MapMarkTileSight);
 			// Detectcloak works in container
 			if (unit->Type->BoolFlag[DETECTCLOAK_INDEX].value) {
-				MapSight(*player, pos, w, h, unit->CurrentSightRange, MapMarkTileDetectCloak);
+				MapSight(*player, *unit, pos, w, h, unit->CurrentSightRange, MapMarkTileDetectCloak);
 			}
 			// Radar(Jammer) not.
 			lua_pop(l, 1);
@@ -388,8 +438,8 @@ static int CclUnit(lua_State *l)
 			unit->Selected = 1;
 			--j;
 		} else if (!strcmp(value, "summoned")) {
-			unit->Summoned = 1;
-			--j;
+			// FIXME : unsigned long should be better handled
+			unit->Summoned = LuaToNumber(l, 2, j + 1);
 		} else if (!strcmp(value, "waiting")) {
 			unit->Waiting = 1;
 			--j;
@@ -546,7 +596,7 @@ static int CclUnit(lua_State *l)
 			CclParseOrder(l, *unit, &unit->NewOrder);
 			lua_pop(l, 1);
 		} else if (!strcmp(value, "goal")) {
-			unit->Goal = &UnitManager.GetSlotUnit(LuaToNumber(l, 2, j + 1));
+			unit->Goal = &UnitManager->GetSlotUnit(LuaToNumber(l, 2, j + 1));
 		} else if (!strcmp(value, "auto-cast")) {
 			const char *s = LuaToString(l, 2, j + 1);
 			Assert(SpellTypeByIdent(s));
@@ -596,9 +646,13 @@ static int CclUnit(lua_State *l)
 		MapMarkUnitSight(*unit);
 	}
 
-	// Fix Colors for rescued units.
-	if (unit->RescuedFrom) {
-		unit->Colors = &unit->RescuedFrom->UnitColors;
+	if (!hadType && unit->Container) {
+		// this unit was assigned to a container before it had a type, so we
+		// need to actually add it now, since only with a type do we know the
+		// BoardSize it takes up in the container
+		CUnit *host = unit->Container;
+		unit->Container = NULL;
+		unit->AddInContainer(*host);
 	}
 
 	return 0;
@@ -610,6 +664,13 @@ static int CclUnit(lua_State *l)
 **  @param l  Lua state.
 **
 **  @return   Returns the slot number of the made placed.
+**
+** Example:
+**
+** <div class="example"><code>-- Create the unit
+**	footman = CreateUnit("unit-footman", 0, {7, 4})
+**	-- Move the unit to position 20 (x) and 10 (y)
+**	<strong>MoveUnit</strong>(footman,{20,10})</code></div>
 */
 static int CclMoveUnit(lua_State *l)
 {
@@ -622,6 +683,10 @@ static int CclMoveUnit(lua_State *l)
 	Vec2i ipos;
 	CclGetPos(l, &ipos.x, &ipos.y, 2);
 
+	if (!unit->Removed) {
+		unit->Remove(unit->Container);
+	}
+
 	if (UnitCanBeAt(*unit, ipos)) {
 		unit->Place(ipos);
 	} else {
@@ -635,11 +700,24 @@ static int CclMoveUnit(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Remove unit from the map.
 **
 **  @param l  Lua state.
 **
 **  @return   Returns 1.
+**
+** Example:
+**
+** <div class="example"><code>ogre = CreateUnit("unit-ogre", 0, {24, 89})
+**
+**		AddTrigger(
+**  		function() return (GameCycle > 150) end,
+**  		function()
+**    			<strong>RemoveUnit</strong>(ogre)
+**    			return false end -- end of function
+**		)</code></div>
 */
 static int CclRemoveUnit(lua_State *l)
 {
@@ -647,18 +725,26 @@ static int CclRemoveUnit(lua_State *l)
 	lua_pushvalue(l, 1);
 	CUnit *unit = CclGetUnit(l);
 	lua_pop(l, 1);
-	unit->Remove(NULL);
-	LetUnitDie(*unit);
+	if (unit) {
+		unit->Remove(NULL);
+		LetUnitDie(*unit);
+	}
 	lua_pushvalue(l, 1);
 	return 1;
 }
 
 /**
+** <b>Description</b>
+**
 **  Create a unit and place it on the map
 **
 **  @param l  Lua state.
 **
 **  @return   Returns the slot number of the made unit.
+**
+** Example:
+**
+** <div class="example"><code><strong>CreateUnit</strong>("unit-human-transport", 1, {94, 0})</code></div>
 */
 static int CclCreateUnit(lua_State *l)
 {
@@ -681,7 +767,7 @@ static int CclCreateUnit(lua_State *l)
 		LuaError(l, "bad player");
 		return 0;
 	}
-	if (Players[playerno].Type == PlayerNobody) {
+	if (Players[playerno].Type == PlayerTypes::PlayerNobody) {
 		printf("CreateUnit: player %d does not exist\n", playerno);
 		LuaError(l, "bad player");
 		return 0;
@@ -708,11 +794,20 @@ static int CclCreateUnit(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  'Upgrade' a unit in place to a unit of different type.
 **
 **  @param l  Lua state.
 **
 **  @return   Returns success.
+**
+** Example:
+**
+** <div class="example"><code>-- Make a peon for player 5
+**		peon = CreateUnit("unit-peon", 5, {58, 9})
+**		-- The peon will be trasformed into a Grunt
+**		<strong>TransformUnit</strong>(peon,"unit-grunt")</code></div>
 */
 static int CclTransformUnit(lua_State *l)
 {
@@ -722,17 +817,28 @@ static int CclTransformUnit(lua_State *l)
 	lua_pushvalue(l, 2);
 	const CUnitType *unittype = TriggerGetUnitType(l);
 	lua_pop(l, 1);
-	CommandUpgradeTo(*targetUnit, *(CUnitType*)unittype, 1);
+	if (unittype && targetUnit) {
+		CommandUpgradeTo(*targetUnit, *(CUnitType*)unittype, 1, true);
+	}
 	lua_pushvalue(l, 1);
 	return 1;
 }
 
 /**
+** <b>Description</b>
+**
 **  Damages unit, additionally using another unit as first's attacker
 **
 **  @param l  Lua state.
 **
 **  @return   Returns the slot number of the made unit.
+**
+** Example:
+**
+** <div class="example"><code>-- Make a grunt for player 5
+**		grunt = CreateUnit("unit-grunt", 5, {58, 8})
+**		-- Damage the grunt with 15 points
+**		<strong>DamageUnit</strong>(-1,grunt,15)</code></div>
 */
 static int CclDamageUnit(lua_State *l)
 {
@@ -741,7 +847,7 @@ static int CclDamageUnit(lua_State *l)
 	const int attacker = LuaToNumber(l, 1);
 	CUnit *attackerUnit = NULL;
 	if (attacker != -1) {
-		attackerUnit = &UnitManager.GetSlotUnit(attacker);
+		attackerUnit = &UnitManager->GetSlotUnit(attacker);
 	}
 	lua_pushvalue(l, 2);
 	CUnit *targetUnit = CclGetUnit(l);
@@ -803,11 +909,18 @@ static int CclSetTeleportDestination(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Order a unit
 **
 **  @param l  Lua state.
 **
-**  OrderUnit(player, unit-type, sloc, dloc, order)
+**  OrderUnit(player, unit-type, start_loc, dest_loc, order)
+**
+** Example:
+**
+** <div class="example"><code>-- Move transport from position x=94,y=0 to x=80,y=9
+**		<strong>OrderUnit</strong>(1,"unit-human-transport",{94,0},{80,9},"move")</code></div>
 */
 static int CclOrderUnit(lua_State *l)
 {
@@ -859,10 +972,15 @@ static int CclOrderUnit(lua_State *l)
 			if (plynr == -1 || plynr == unit.Player->Index) {
 				if (!strcmp(order, "move")) {
 					CommandMove(unit, (dpos1 + dpos2) / 2, 1);
+				} else if (!strcmp(order, "stop")) {
+					CommandStopUnit(unit); //Stop the unit
+				} else if (!strcmp(order, "stand-ground")) {
+					CommandStandGround(unit,0); //Stand and flush every order
 				} else if (!strcmp(order, "attack")) {
 					CUnit *attack = TargetOnMap(unit, dpos1, dpos2);
-
 					CommandAttack(unit, (dpos1 + dpos2) / 2, attack, 1);
+				} else if (!strcmp(order, "explore")) {
+					CommandExplore(unit, 1);
 				} else if (!strcmp(order, "patrol")) {
 					CommandPatrolUnit(unit, (dpos1 + dpos2) / 2, 1);
 				} else {
@@ -890,11 +1008,18 @@ private:
 
 
 /**
+** <b>Description</b>
+**
 **  Kill a unit
 **
 **  @param l  Lua state.
 **
 **  @return   Returns true if a unit was killed.
+**
+** Example:
+**
+** <div class="example"><code>-- Kills an ogre controlled by player 3
+**		<strong>KillUnit</strong>("unit-ogre", 3)</code></div>
 */
 static int CclKillUnit(lua_State *l)
 {
@@ -905,9 +1030,9 @@ static int CclKillUnit(lua_State *l)
 	lua_pop(l, 1);
 	const int plynr = TriggerGetPlayer(l);
 	if (plynr == -1) {
-		CUnitManager::Iterator it = std::find_if(UnitManager.begin(), UnitManager.end(), HasSameUnitTypeAs(unittype));
+		CUnitManager::Iterator it = std::find_if(UnitManager->begin(), UnitManager->end(), HasSameUnitTypeAs(unittype));
 
-		if (it != UnitManager.end()) {
+		if (it != UnitManager->end()) {
 			LetUnitDie(**it);
 			lua_pushboolean(l, 1);
 			return 1;
@@ -927,11 +1052,18 @@ static int CclKillUnit(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Kill a unit at a location
 **
 **  @param l  Lua state.
 **
 **  @return   Returns the number of units killed.
+**
+** Example:
+**
+** <div class="example"><code>-- Kill 8 peasants controlled by player 7 from position {27,1} to {34,5}
+**		<strong>KillUnitAt</strong>("unit-peasant",7,8,{27,1},{34,5})</code></div>
 */
 static int CclKillUnitAt(lua_State *l)
 {
@@ -952,6 +1084,12 @@ static int CclKillUnitAt(lua_State *l)
 	Vec2i pos2;
 	CclGetPos(l, &pos1.x, &pos1.y, 4);
 	CclGetPos(l, &pos2.x, &pos2.y, 5);
+	if (pos1.x > pos2.x) {
+		std::swap(pos1.x, pos2.x);
+	}
+	if (pos1.y > pos2.y) {
+		std::swap(pos1.y, pos2.y);
+	}
 
 	std::vector<CUnit *> table;
 
@@ -976,11 +1114,21 @@ static int CclKillUnitAt(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Get a player's units
 **
 **  @param l  Lua state.
 **
 **  @return   Array of units.
+**
+** Example:
+**
+** <div class="example"><code>-- Get units from player 0
+**	units = <strong>GetUnits</strong>(0)
+**	for i, id_unit in ipairs(units) do
+**	    print(id_unit)
+**	end</code></div>
 */
 static int CclGetUnits(lua_State *l)
 {
@@ -991,7 +1139,7 @@ static int CclGetUnits(lua_State *l)
 	lua_newtable(l);
 	if (plynr == -1) {
 		int i = 0;
-		for (CUnitManager::Iterator it = UnitManager.begin(); it != UnitManager.end(); ++it, ++i) {
+		for (CUnitManager::Iterator it = UnitManager->begin(); it != UnitManager->end(); ++it, ++i) {
 			const CUnit &unit = **it;
 			lua_pushnumber(l, UnitNumber(unit));
 			lua_rawseti(l, -2, i + 1);
@@ -1006,11 +1154,19 @@ static int CclGetUnits(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Get a player's units in rectangle box specified with 2 coordinates
 **
 **  @param l  Lua state.
 **
 **  @return   Array of units.
+**
+** Example:
+**
+** <div class="example"><code>circlePower = CreateUnit("unit-circle-of-power", 15, {59, 4})
+**		-- Get the units near the circle of power.
+**      unitsOnCircle = <strong>GetUnitsAroundUnit</strong>(circle,1,true)</code></div>
 */
 static int CclGetUnitsAroundUnit(lua_State *l)
 {
@@ -1020,7 +1176,7 @@ static int CclGetUnitsAroundUnit(lua_State *l)
 	}
 
 	const int slot = LuaToNumber(l, 1);
-	const CUnit &unit = UnitManager.GetSlotUnit(slot);
+	const CUnit &unit = UnitManager->GetSlotUnit(slot);
 	const int range = LuaToNumber(l, 2);
 	bool allUnits = false;
 	if (nargs == 3) {
@@ -1069,11 +1225,26 @@ static int CclGetUnitBoolFlag(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Get the value of the unit variable.
 **
 **  @param l  Lua state.
 **
 **  @return   The value of the variable of the unit.
+**
+** Example:
+**
+** <div class="example"><code>-- Make a grunt for player 5
+**		grunt = CreateUnit("unit-grunt", 5, {58, 8})
+**		-- Take the name of the unit
+**		unit_name = <strong>GetUnitVariable</strong>(grunt,"Name")
+**		-- Take the player number based on the unit
+**		player_type = <strong>GetUnitVariable</strong>(grunt,"PlayerType")
+**		-- Take the value of the armor
+**		armor_value = <strong>GetUnitVariable</strong>(grunt,"Armor")
+**		-- Show the message in the game.
+**		AddMessage(unit_name .. " " .. player_type .. " " .. armor_value)</code></div>
 */
 static int CclGetUnitVariable(lua_State *l)
 {
@@ -1091,6 +1262,8 @@ static int CclGetUnitVariable(lua_State *l)
 	const char *const value = LuaToString(l, 2);
 	if (!strcmp(value, "RegenerationRate")) {
 		lua_pushnumber(l, unit->Variable[HP_INDEX].Increase);
+	} else if (!strcmp(value, "RegenerationFrequency")) {
+		lua_pushnumber(l, std::max((int)unit->Variable[HP_INDEX].IncreaseFrequency, 1));
 	} else if (!strcmp(value, "Ident")) {
 		lua_pushstring(l, unit->Type->Ident.c_str());
 	} else if (!strcmp(value, "ResourcesHeld")) {
@@ -1102,7 +1275,19 @@ static int CclGetUnitVariable(lua_State *l)
 	} else if (!strcmp(value, "Name")) {
 		lua_pushstring(l, unit->Type->Name.c_str());
 	} else if (!strcmp(value, "PlayerType")) {
-		lua_pushinteger(l, unit->Player->Type);
+		lua_pushstring(l, PlayerTypeNames[static_cast<int>(unit->Player->Type)].c_str());
+	} else if (!strcmp(value, "TTLPercent")) {
+		if (unit->Summoned && unit->TTL) {
+			unsigned long time_lived = GameCycle - unit->Summoned;
+			Assert(time_lived >= 0);
+			unsigned long time_to_live = unit->TTL - unit->Summoned;
+			Assert(time_to_live > 0);
+			double pcnt = time_lived * 100.0 / time_to_live;
+			int pcnt_i = (int)round(pcnt);
+			lua_pushinteger(l, pcnt_i);
+		} else {
+			lua_pushinteger(l, -1);
+		}
 	} else if (!strcmp(value, "IndividualUpgrade")) {
 		LuaCheckArgs(l, 3);
 		std::string upgrade_ident = LuaToString(l, 3);
@@ -1118,8 +1303,25 @@ static int CclGetUnitVariable(lua_State *l)
 	} else if (!strcmp(value, "Idle")) {
 		lua_pushboolean(l, unit->IsIdle());
 		return 1;
+	} else if (!strcmp(value, "PixelPos")) {
+		PixelPos pos = unit->GetMapPixelPosCenter();
+		lua_newtable(l);
+		lua_pushnumber(l, pos.x);
+		lua_setfield(l, -2, "x");
+		lua_pushnumber(l, pos.y);
+		lua_setfield(l, -2, "y");
+		return 1;
 	} else {
 		int index = UnitTypeVar.VariableNameLookup[value];// User variables
+		if (index == -1) {
+			if (nargs == 2) {
+				index = UnitTypeVar.BoolFlagNameLookup[value];
+				if (index != -1) {
+					lua_pushboolean(l, unit->Type->BoolFlag[index].value);
+					return 1;
+				}
+			}
+		}
 		if (index == -1) {
 			LuaError(l, "Bad variable name '%s'\n" _C_ value);
 		}
@@ -1133,6 +1335,8 @@ static int CclGetUnitVariable(lua_State *l)
 				lua_pushnumber(l, unit->Variable[index].Max);
 			} else if (!strcmp(type, "Increase")) {
 				lua_pushnumber(l, unit->Variable[index].Increase);
+			} else if (!strcmp(type, "IncreaseFrequency")) {
+				lua_pushnumber(l, std::max((int)unit->Variable[index].IncreaseFrequency, 1));
 			} else if (!strcmp(type, "Enable")) {
 				lua_pushnumber(l, unit->Variable[index].Enable);
 			} else {
@@ -1144,11 +1348,23 @@ static int CclGetUnitVariable(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Set the value of the unit variable.
 **
 **  @param l  Lua state.
 **
 **  @return The new value of the unit.
+**
+** Example:
+**
+** <div class="example"><code>-- Create a blacksmith for player 2
+** blacksmith = CreateUnit("unit-human-blacksmith", 2, {66, 71})
+** -- Specify the amount of hit points to assign to the blacksmith
+** <strong>SetUnitVariable</strong>(blacksmith,"HitPoints",344)
+** -- Set the blacksmiths color to the color of player 4
+** <strong>SetUnitVariable</strong>(blacksmith,"Color",4)
+** </code></div>
 */
 static int CclSetUnitVariable(lua_State *l)
 {
@@ -1159,13 +1375,40 @@ static int CclSetUnitVariable(lua_State *l)
 	CUnit *unit = CclGetUnit(l);
 	lua_pop(l, 1);
 	const char *const name = LuaToString(l, 2);
-	int value;
+	int value = 0;
 	if (!strcmp(name, "Player")) {
 		value = LuaToNumber(l, 3);
-		unit->AssignToPlayer(Players[value]);
+		unit->ChangeOwner(Players[value]);
+	} else if (!strcmp(name, "Color")) {
+		if (lua_isstring(l, 3)) {
+			const char *colorName = LuaToString(l, 3);
+			for (size_t i = 0; i < PlayerColorNames.size(); i++) {
+				if (PlayerColorNames[i] == colorName) {
+					unit->Colors = i;
+					break;
+				}
+			}
+		} else if (lua_isnil(l, 3)) {
+			unit->Colors = -1;
+		} else {
+			value = LuaToNumber(l, 3);
+			unit->Colors = value;
+		}
+	} else if (!strcmp(name, "TTL")) {
+		value = LuaToNumber(l, 3);
+		unit->TTL = GameCycle + value;
+	} else if (!strcmp(name, "Summoned")) {
+		value = LuaToNumber(l, 3);
+		unit->Summoned = value;
 	} else if (!strcmp(name, "RegenerationRate")) {
 		value = LuaToNumber(l, 3);
 		unit->Variable[HP_INDEX].Increase = std::min(unit->Variable[HP_INDEX].Max, value);
+	} else if (!strcmp(name, "RegenerationFrequency")) {
+		value = LuaToNumber(l, 3);
+		unit->Variable[HP_INDEX].IncreaseFrequency = value;
+		if (unit->Variable[HP_INDEX].IncreaseFrequency != value) {
+			LuaError(l, "RegenerationFrequency out of range!");
+		}
 	} else if (!strcmp(name, "IndividualUpgrade")) {
 		LuaCheckArgs(l, 4);
 		std::string upgrade_ident = LuaToString(l, 3);
@@ -1210,6 +1453,11 @@ static int CclSetUnitVariable(lua_State *l)
 				unit->Stats->Variables[index].Max = value;
 			} else if (!strcmp(type, "Increase")) {
 				unit->Stats->Variables[index].Increase = value;
+			} else if (!strcmp(type, "IncreaseFrequency")) {
+				unit->Stats->Variables[index].IncreaseFrequency = value;
+				if (unit->Stats->Variables[index].IncreaseFrequency != value) {
+					LuaError(l, "%s.IncreaseFrequency out of range!" _C_ type);
+				}
 			} else if (!strcmp(type, "Enable")) {
 				unit->Stats->Variables[index].Enable = value;
 			} else {
@@ -1225,6 +1473,11 @@ static int CclSetUnitVariable(lua_State *l)
 				unit->Variable[index].Max = value;
 			} else if (!strcmp(type, "Increase")) {
 				unit->Variable[index].Increase = value;
+			} else if (!strcmp(type, "IncreaseFrequency")) {
+				unit->Variable[index].IncreaseFrequency = value;
+				if (unit->Variable[index].IncreaseFrequency != value) {
+					LuaError(l, "%s.IncreaseFrequency out of range!" _C_ type);
+				}
 			} else if (!strcmp(type, "Enable")) {
 				unit->Variable[index].Enable = value;
 			} else {
@@ -1243,10 +1496,26 @@ static int CclSetUnitVariable(lua_State *l)
 */
 static int CclSlotUsage(lua_State *l)
 {
-	UnitManager.Load(l);
+	UnitManager->Load(l);
 	return 0;
 }
 
+/**
+** <b>Description</b>
+**
+**  Select a single unit 
+**
+**  @param l  Lua state.
+**
+**  @return 0, meaning the unit is selected.
+**
+** Example:
+**
+** <div class="example"><code>-- Make the hero unit Grom Hellscream for player 5
+**		grom = CreateUnit("unit-beast-cry", 5, {58, 8})
+**		-- Select only the unit Grom Hellscream
+**		<strong>SelectSingleUnit</strong>(grom)</code></div>
+*/
 static int CclSelectSingleUnit(lua_State *l)
 {
 	const int nargs = lua_gettop(l);
@@ -1259,6 +1528,64 @@ static int CclSelectSingleUnit(lua_State *l)
 	return 0;
 }
 
+/**
+**  Find the next reachable resource unit that gives resource starting from a worker.
+**  Optional third argument is the range to search.
+**
+**  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code>
+**		peon = CreateUnit("unit-peon", 5, {58, 8})
+**      goldmine = <strong>FindNextResource(peon, 0)</strong></code></div>
+*/
+static int CclFindNextResource(lua_State *l)
+{
+	const int nargs = lua_gettop(l);
+	if (nargs < 2 || nargs > 3) {
+		LuaError(l, "incorrect argument count");
+	}
+
+	lua_pushvalue(l, 1);
+	CUnit *unit = CclGetUnit(l);
+	lua_pop(l, 1);
+
+	lua_pushvalue(l, 2);
+	const int resource = CclGetResourceByName(l);
+	lua_pop(l, 1);
+
+	const int range = nargs == 3 ? LuaToNumber(l, 3) : 1000;
+
+	CUnit *resourceUnit = UnitFindResource(*unit, *unit, range, resource);
+	if (resourceUnit) {
+		lua_pushnumber(l, UnitNumber(*unit));
+	} else {
+		lua_pushnil(l);
+	}
+	return 1;
+}
+
+/**
+**  Enable/disable simplified auto targeting 
+**
+**  @param l  Lua state.
+**
+**  @return   0 for success, 1 for wrong type;
+*/
+static int CclEnableSimplifiedAutoTargeting(lua_State *l)
+{
+	LuaCheckArgs(l, 1);
+	const bool isSimplified = LuaToBoolean(l, 1);
+	if (!IsNetworkGame()) {
+		GameSettings.SimplifiedAutoTargeting = isSimplified;
+	} else {
+		NetworkSendExtendedCommand(ExtendedMessageAutoTargetingDB, 
+								   int(isSimplified), 0, 0, 0, 0);
+	}
+	return 0;
+}
+
 /**
 **  Register CCL features for unit.
 */
@@ -1281,6 +1608,7 @@ void UnitCclRegister()
 	lua_register(Lua, "OrderUnit", CclOrderUnit);
 	lua_register(Lua, "KillUnit", CclKillUnit);
 	lua_register(Lua, "KillUnitAt", CclKillUnitAt);
+	lua_register(Lua, "FindNextResource", CclFindNextResource);
 
 	lua_register(Lua, "GetUnits", CclGetUnits);
 	lua_register(Lua, "GetUnitsAroundUnit", CclGetUnitsAroundUnit);
@@ -1293,6 +1621,7 @@ void UnitCclRegister()
 	lua_register(Lua, "SlotUsage", CclSlotUsage);
 
 	lua_register(Lua, "SelectSingleUnit", CclSelectSingleUnit);
+	lua_register(Lua, "EnableSimplifiedAutoTargeting", CclEnableSimplifiedAutoTargeting);
 }
 
 //@}
