@@ -93,7 +93,11 @@ static const char WALL_KEY[] = "Wall";
 static const char NORANDOMPLACING_KEY[] = "NoRandomPlacing";
 static const char ORGANIC_KEY[] = "organic";
 static const char SIDEATTACK_KEY[] = "SideAttack";
+static const char SKIRMISHER_KEY[] = "Skirmisher";
+static const char ALWAYSTHREAT_KEY[] = "AlwaysThreat";
+static const char ELEVATED_KEY[] = "Elevated";
 static const char NOFRIENDLYFIRE_KEY[] = "NoFriendlyFire";
+static const char MAINFACILITY_KEY[] = "MainFacility";
 
 // names of the variable.
 static const char HITPOINTS_KEY[] = "HitPoints";
@@ -116,6 +120,8 @@ static const char PIERCINGDAMAGE_KEY[] = "PiercingDamage";
 static const char BASICDAMAGE_KEY[] = "BasicDamage";
 static const char POSX_KEY[] = "PosX";
 static const char POSY_KEY[] = "PosY";
+static const char POS_RIGHT_KEY[] = "PosRight";
+static const char POS_BOTTOM_KEY[] = "PosBottom";
 static const char TARGETPOSX_KEY[] = "TargetPosX";
 static const char TARGETPOSY_KEY[] = "TargetPosY";
 static const char RADARRANGE_KEY[] = "RadarRange";
@@ -151,7 +157,8 @@ CUnitTypeVar::CBoolKeys::CBoolKeys()
 							   SHOREBUILDING_KEY, CANATTACK_KEY, BUILDEROUTSIDE_KEY,
 							   BUILDERLOST_KEY, CANHARVEST_KEY, HARVESTER_KEY, SELECTABLEBYRECTANGLE_KEY,
 							   ISNOTSELECTABLE_KEY, DECORATION_KEY, INDESTRUCTIBLE_KEY, TELEPORTER_KEY, SHIELDPIERCE_KEY,
-							   SAVECARGO_KEY, NONSOLID_KEY, WALL_KEY, NORANDOMPLACING_KEY, ORGANIC_KEY, SIDEATTACK_KEY, NOFRIENDLYFIRE_KEY
+							   SAVECARGO_KEY, NONSOLID_KEY, WALL_KEY, NORANDOMPLACING_KEY, ORGANIC_KEY, SIDEATTACK_KEY, SKIRMISHER_KEY,
+							   ALWAYSTHREAT_KEY, ELEVATED_KEY, NOFRIENDLYFIRE_KEY, MAINFACILITY_KEY
 							  };
 
 	for (int i = 0; i < NBARALREADYDEFINED; ++i) {
@@ -169,7 +176,7 @@ CUnitTypeVar::CVariableKeys::CVariableKeys()
 							   RESEARCH_KEY, TRAINING_KEY, UPGRADETO_KEY, GIVERESOURCE_KEY,
 							   CARRYRESOURCE_KEY, XP_KEY, KILL_KEY,	SUPPLY_KEY, DEMAND_KEY, ARMOR_KEY,
 							   SIGHTRANGE_KEY, ATTACKRANGE_KEY, PIERCINGDAMAGE_KEY,
-							   BASICDAMAGE_KEY, POSX_KEY, POSY_KEY, TARGETPOSX_KEY, TARGETPOSY_KEY, RADARRANGE_KEY,
+							   BASICDAMAGE_KEY, POSX_KEY, POSY_KEY, POS_RIGHT_KEY, POS_BOTTOM_KEY, TARGETPOSX_KEY, TARGETPOSY_KEY, RADARRANGE_KEY,
 							   RADARJAMMERRANGE_KEY, AUTOREPAIRRANGE_KEY, BLOODLUST_KEY, HASTE_KEY,
 							   SLOW_KEY, INVISIBLE_KEY, UNHOLYARMOR_KEY, SLOT_KEY, SHIELD_KEY, POINTS_KEY,
 							   MAXHARVESTERS_KEY, POISON_KEY, SHIELDPERMEABILITY_KEY, SHIELDPIERCING_KEY, ISALIVE_KEY, PLAYER_KEY,
@@ -429,10 +436,44 @@ static void UpdateDefaultBoolFlags(CUnitType &type)
 	type.BoolFlag[CANATTACK_INDEX].value             = type.CanAttack;
 }
 
+static const std::string shadowMarker = std::string("MARKER");
 /**
+** <b>Description</b>
+**
 **  Parse unit-type.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>DefineUnitType</strong>("unit-silvermoon-archer", { Name = _("Silvermoon Archer"),
+**			Image = {"file", "human/units/elven_archer.png", "size", {72, 72}},
+**			Animations = "animations-archer", Icon = "icon-archer",
+**			Costs = {"time", 70, "gold", 500, "wood", 50},
+**			Speed = 10,
+**			HitPoints = 45,
+**			DrawLevel = 40,
+**			TileSize = {1, 1}, BoxSize = {33, 33},
+**			SightRange = 6, ComputerReactionRange = 7, PersonReactionRange = 6,
+**			BasicDamage = 4, PiercingDamage = 6, Missile = "missile-arrow",
+**			MaxAttackRange = 4,
+**			Priority = 75,
+**			Points = 60,
+**			Demand = 1,
+**			Corpse = "unit-human-dead-body",
+**			Type = "land",
+**			RightMouseAction = "attack",
+**			CanAttack = true,
+**			CanTargetLand = true, CanTargetSea = true, CanTargetAir = true,
+**			LandUnit = true,
+**			organic = true,
+**			SelectableByRectangle = true,
+**			Sounds = {
+**				"selected", "archer-selected",
+**				"acknowledge", "archer-acknowledge",
+**				"ready", "archer-ready",
+**				"help", "basic human voices help 1",
+**				"dead", "basic human voices dead"} } )</code></div>
 */
 static int CclDefineUnitType(lua_State *l)
 {
@@ -444,17 +485,20 @@ static int CclDefineUnitType(lua_State *l)
 	// Slot identifier
 	const char *str = LuaToString(l, 1);
 	CUnitType *type = UnitTypeByIdent(str);
+
+	constexpr int redefineSprite = 2;
+
 	int redefine;
 	if (type) {
 		redefine = 1;
+		DebugPrint("Redefining unit-type '%s'\n" _C_ str);
 	} else {
 		type = NewUnitTypeSlot(str);
 		redefine = 0;
+		type->NumDirections = 0;
+		type->Flip = 1;
 	}
 
-	type->NumDirections = 0;
-	type->Flip = 1;
-
 	//  Parse the list: (still everything could be changed!)
 	for (lua_pushnil(l); lua_next(l, 2); lua_pop(l, 1)) {
 		const char *value = LuaToString(l, -2);
@@ -471,6 +515,8 @@ static int CclDefineUnitType(lua_State *l)
 
 				if (!strcmp(value, "file")) {
 					type->File = LuaToString(l, -1, k + 1);
+				} else if (!strcmp(value, "alt-file")) {
+					type->AltFile = LuaToString(l, -1, k + 1);
 				} else if (!strcmp(value, "size")) {
 					lua_rawgeti(l, -1, k + 1);
 					CclGetPos(l, &type->Width, &type->Height);
@@ -479,11 +525,39 @@ static int CclDefineUnitType(lua_State *l)
 					LuaError(l, "Unsupported image tag: %s" _C_ value);
 				}
 			}
-			if (redefine && type->Sprite) {
-				CGraphic::Free(type->Sprite);
-				type->Sprite = NULL;
+			if (redefine) {
+				if (type->Sprite && type->Sprite->File != type->File) {
+					redefine |= redefineSprite;
+					CGraphic::Free(type->Sprite);
+					type->Sprite = NULL;
+				}
+				if (type->AltSprite && type->AltSprite->File != type->AltFile) {
+					redefine |= redefineSprite;
+					CGraphic::Free(type->AltSprite);
+					type->AltSprite = NULL;
+				}
+				if (redefine && type->ShadowSprite) {
+					redefine |= redefineSprite;
+					CGraphic::Free(type->ShadowSprite);
+					type->ShadowSprite = NULL;
+				}
+			}
+			if (type->ShadowFile == shadowMarker) {
+				type->ShadowFile = type->File;
+				if (type->ShadowWidth == 0 && type->ShadowHeight == 0) {
+					type->ShadowWidth = type->Width;
+					type->ShadowHeight = type->Height;
+				}
 			}
 		} else if (!strcmp(value, "Shadow")) {
+			// default to same spritemap as unit
+			if (type->File.length() > 0) {
+				type->ShadowFile = type->File;
+				type->ShadowWidth = type->Width;
+				type->ShadowHeight = type->Height;
+			} else {
+				type->ShadowFile = shadowMarker;
+			}
 			if (!lua_istable(l, -1)) {
 				LuaError(l, "incorrect argument");
 			}
@@ -502,11 +576,16 @@ static int CclDefineUnitType(lua_State *l)
 					lua_rawgeti(l, -1, k + 1);
 					CclGetPos(l, &type->ShadowOffsetX, &type->ShadowOffsetY);
 					lua_pop(l, 1);
+				} else if (!strcmp(value, "sprite-frame")) {
+					type->ShadowSpriteFrame = LuaToNumber(l, -1, k + 1);
+				} else if (!strcmp(value, "scale")) {
+					type->ShadowScale = LuaToNumber(l, -1, k + 1);
 				} else {
 					LuaError(l, "Unsupported shadow tag: %s" _C_ value);
 				}
 			}
 			if (redefine && type->ShadowSprite) {
+				redefine |= redefineSprite;
 				CGraphic::Free(type->ShadowSprite);
 				type->ShadowSprite = NULL;
 			}
@@ -522,18 +601,31 @@ static int CclDefineUnitType(lua_State *l)
 		} else if (!strcmp(value, "Icon")) {
 			type->Icon.Name = LuaToString(l, -1);
 			type->Icon.Icon = NULL;
-#ifdef USE_MNG
 		} else if (!strcmp(value, "Portrait")) {
+#ifdef USE_MNG
 			if (!lua_istable(l, -1)) {
 				LuaError(l, "incorrect argument");
 			}
 			const int subargs = lua_rawlen(l, -1);
-			type->Portrait.Num = subargs;
+			int number = 0;
+			for (int k = 0; k < subargs; ++k) {
+				const char *s = LuaToString(l, -1, k + 1);
+				if (strcmp("talking", s)) {
+					number++;
+				}
+			}
+			type->Portrait.Num = number;
+			type->Portrait.Talking = 0;
 			type->Portrait.Files = new std::string[type->Portrait.Num];
 			type->Portrait.Mngs = new Mng *[type->Portrait.Num];
 			memset(type->Portrait.Mngs, 0, type->Portrait.Num * sizeof(Mng *));
 			for (int k = 0; k < subargs; ++k) {
-				type->Portrait.Files[k] = LuaToString(l, -1, k + 1);
+				const char *s = LuaToString(l, -1, k + 1);
+				if (!strcmp("talking", s)) {
+					type->Portrait.Talking = k;
+				} else {
+					type->Portrait.Files[k - (type->Portrait.Talking ? 1 : 0)] = s;
+				}
 			}
 #endif
 		} else if (!strcmp(value, "Costs")) {
@@ -587,6 +679,12 @@ static int CclDefineUnitType(lua_State *l)
 			type->StartingResources = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "RegenerationRate")) {
 			type->DefaultStat.Variables[HP_INDEX].Increase = LuaToNumber(l, -1);
+		} else if (!strcmp(value, "RegenerationFrequency")) {
+			int value = LuaToNumber(l, -1);
+			type->DefaultStat.Variables[HP_INDEX].IncreaseFrequency = value;
+			if (type->DefaultStat.Variables[HP_INDEX].IncreaseFrequency != value) {
+				LuaError(l, "RegenerationFrequency out of range!");
+			}
 		} else if (!strcmp(value, "BurnPercent")) {
 			type->BurnPercent = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "BurnDamageRate")) {
@@ -606,6 +704,8 @@ static int CclDefineUnitType(lua_State *l)
 			CclGetPos(l, &type->TileWidth, &type->TileHeight);
 		} else if (!strcmp(value, "NeutralMinimapColor")) {
 			type->NeutralMinimapColorRGB.Parse(l);
+		} else if (!strcmp(value, "Neutral")) {
+			type->Neutral = LuaToBoolean(l, -1);
 		} else if (!strcmp(value, "BoxSize")) {
 			CclGetPos(l, &type->BoxWidth, &type->BoxHeight);
 		} else if (!strcmp(value, "BoxOffset")) {
@@ -654,8 +754,8 @@ static int CclDefineUnitType(lua_State *l)
 			type->TeleportEffectIn = new LuaCallback(l, -1);
 		} else if (!strcmp(value, "TeleportEffectOut")) {
 			type->TeleportEffectOut = new LuaCallback(l, -1);
-		} else if (!strcmp(value, "DeathExplosion")) {
-			type->DeathExplosion = new LuaCallback(l, -1);
+		} else if (!strcmp(value, "OnDeath")) {
+			type->OnDeath = new LuaCallback(l, -1);
 		} else if (!strcmp(value, "OnHit")) {
 			type->OnHit = new LuaCallback(l, -1);
 		} else if (!strcmp(value, "OnEachCycle")) {
@@ -664,6 +764,8 @@ static int CclDefineUnitType(lua_State *l)
 			type->OnEachSecond = new LuaCallback(l, -1);
 		} else if (!strcmp(value, "OnInit")) {
 			type->OnInit = new LuaCallback(l, -1);
+		} else if (!strcmp(value, "OnReady")) {
+			type->OnReady = new LuaCallback(l, -1);
 		} else if (!strcmp(value, "Type")) {
 			value = LuaToString(l, -1);
 			if (!strcmp(value, "land")) {
@@ -1055,6 +1157,8 @@ static int CclDefineUnitType(lua_State *l)
 					type->Sound.Harvest[resId].Name = LuaToString(l, -1, k + 1);
 				} else if (!strcmp(value, "help")) {
 					type->Sound.Help.Name = LuaToString(l, -1, k + 1);
+				} else if (!strcmp(value, "work-complete")) {
+					type->Sound.WorkComplete.Name = LuaToString(l, -1, k + 1);
 				} else if (!strcmp(value, "dead")) {
 					int death;
 
@@ -1123,15 +1227,237 @@ static int CclDefineUnitType(lua_State *l)
 	}
 	UpdateDefaultBoolFlags(*type);
 	if (!CclInConfigFile) {
+		if (redefine & redefineSprite) {
+			LoadUnitTypeSprite(*type);
+		}
 		UpdateUnitStats(*type, 1);
 	}
 	return 0;
 }
 
 /**
+** <b>Description</b>
+**
+**  Copy a unit type.
+**
+**  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>CopyUnitType</strong>("unit-peasant", "unit-peasant-copy")</code></div>
+*/
+static int CclCopyUnitType(lua_State *l)
+{
+	LuaCheckArgs(l, 2);
+
+	// Slot identifier
+	const char* fromName = LuaToString(l, 1);
+	CUnitType *from = UnitTypeByIdent(fromName);
+	const char* toName = LuaToString(l, 2);
+	CUnitType *to = UnitTypeByIdent(toName);
+	if (to) {
+		DebugPrint("Redefining unit-type '%s'\n" _C_ toName);
+	} else {
+		to = NewUnitTypeSlot(toName);
+	}
+	if (!from) {
+		LuaError(l, "Unknown unit-type '%s'\n" _C_ fromName);
+	}
+
+	to->Flip = from->Flip;
+	to->Name = toName;
+	to->File = from->File;
+	to->AltFile = from->AltFile;
+	to->Width = from->Width;
+	to->Height = from->Height;
+	if (to->Sprite) {
+		CGraphic::Free(to->Sprite);
+		to->Sprite = NULL;
+	}
+	if (to->AltSprite) {
+		CGraphic::Free(to->AltSprite);
+		to->AltSprite = NULL;
+	}
+	to->ShadowFile = from->ShadowFile;
+	to->ShadowWidth = from->ShadowWidth;
+	to->ShadowHeight = from->ShadowHeight;
+	to->ShadowOffsetX = from->ShadowOffsetX;
+	to->ShadowOffsetY = from->ShadowOffsetY;
+	to->ShadowSpriteFrame = from->ShadowSpriteFrame;
+	to->ShadowScale = from->ShadowScale;
+	if (to->ShadowSprite) {
+		CGraphic::Free(to->ShadowSprite);
+		to->ShadowSprite = NULL;
+	}
+	to->OffsetX = from->OffsetX;
+	to->OffsetY = from->OffsetY;
+	to->Animations = from->Animations;
+	to->Icon.Name = from->Icon.Name;
+	to->Icon.Icon = NULL;
+#ifdef USE_MNG
+	to->Portrait.Num = from->Portrait.Num;
+	to->Portrait.Talking = from->Portrait.Talking;
+	to->Portrait.Files = new std::string[to->Portrait.Num];
+	for (int i = 0; i < to->Portrait.Num; i++) {
+		to->Portrait.Files[i] = from->Portrait.Files[i];
+	}
+	to->Portrait.Mngs = new Mng *[to->Portrait.Num];
+	memset(to->Portrait.Mngs, 0, to->Portrait.Num * sizeof(Mng *));
+#endif
+	memcpy(to->DefaultStat.Costs, from->DefaultStat.Costs, sizeof(from->DefaultStat.Costs));
+	memcpy(to->DefaultStat.Storing, from->DefaultStat.Storing, sizeof(from->DefaultStat.Storing));
+	memcpy(to->DefaultStat.ImproveIncomes, from->DefaultStat.ImproveIncomes, sizeof(from->DefaultStat.ImproveIncomes));
+	to->Construction = from->Construction;
+	to->DrawLevel = from->DrawLevel;
+	to->MaxOnBoard = from->MaxOnBoard;
+	to->BoardSize = from->BoardSize;
+	to->ButtonLevelForTransporter = from->ButtonLevelForTransporter;
+	to->StartingResources = from->StartingResources;
+	to->DefaultStat.Variables[HP_INDEX].Increase = from->DefaultStat.Variables[HP_INDEX].Increase;
+	to->DefaultStat.Variables[HP_INDEX].IncreaseFrequency = from->DefaultStat.Variables[HP_INDEX].IncreaseFrequency;
+	to->BurnPercent = from->BurnPercent;
+	to->BurnDamageRate = from->BurnDamageRate;
+	to->PoisonDrain = from->PoisonDrain;
+	to->DefaultStat.Variables[SHIELD_INDEX].Max = from->DefaultStat.Variables[SHIELD_INDEX].Max;
+	to->DefaultStat.Variables[SHIELD_INDEX].Value = from->DefaultStat.Variables[SHIELD_INDEX].Value;
+	to->DefaultStat.Variables[SHIELD_INDEX].Increase = from->DefaultStat.Variables[SHIELD_INDEX].Increase;
+	to->DefaultStat.Variables[SHIELD_INDEX].Enable = from->DefaultStat.Variables[SHIELD_INDEX].Enable;
+	to->TileWidth = from->TileWidth;
+	to->TileHeight = from->TileHeight;
+	to->NeutralMinimapColorRGB = from->NeutralMinimapColorRGB;
+	to->Neutral = from->Neutral;
+	to->BoxWidth = from->BoxWidth;
+	to->BoxHeight = from->BoxHeight;
+	to->BoxOffsetX = from->BoxOffsetX;
+	to->BoxOffsetY = from->BoxOffsetY;
+	to->NumDirections = from->NumDirections;
+	to->ReactRangeComputer = from->ReactRangeComputer;
+	to->ReactRangePerson = from->ReactRangePerson;
+	to->Missile.Name = from->Missile.Name;
+	to->Missile.Missile = NULL; // filled in later
+	to->MinAttackRange = from->MinAttackRange;
+	to->DefaultStat.Variables[ATTACKRANGE_INDEX].Value = from->DefaultStat.Variables[ATTACKRANGE_INDEX].Value;
+	to->DefaultStat.Variables[ATTACKRANGE_INDEX].Max = from->DefaultStat.Variables[ATTACKRANGE_INDEX].Max;
+	to->DefaultStat.Variables[ATTACKRANGE_INDEX].Enable = from->DefaultStat.Variables[ATTACKRANGE_INDEX].Enable;
+	to->DefaultStat.Variables[MAXHARVESTERS_INDEX].Value = from->DefaultStat.Variables[MAXHARVESTERS_INDEX].Value;
+	to->DefaultStat.Variables[MAXHARVESTERS_INDEX].Max = from->DefaultStat.Variables[MAXHARVESTERS_INDEX].Max;
+	to->DefaultStat.Variables[PRIORITY_INDEX].Value = from->DefaultStat.Variables[PRIORITY_INDEX].Value;
+	to->DefaultStat.Variables[PRIORITY_INDEX].Max = from->DefaultStat.Variables[PRIORITY_INDEX].Max;
+	to->AnnoyComputerFactor = from->AnnoyComputerFactor;
+	to->AiAdjacentRange = from->AiAdjacentRange;
+	to->DecayRate = from->DecayRate;
+	to->CorpseName = from->CorpseName;
+	to->CorpseType = from->CorpseType;
+	to->DamageType = from->DamageType;
+	to->ExplodeWhenKilled = from->ExplodeWhenKilled;
+	to->Explosion.Name = from->Explosion.Name;
+	to->Explosion.Missile = NULL; // filled later
+	to->TeleportCost = from->TeleportCost;
+	to->TeleportEffectIn = from->TeleportEffectIn;
+	to->TeleportEffectOut = from->TeleportEffectOut;
+	to->OnDeath = from->OnDeath;
+	to->OnHit = from->OnHit;
+	to->OnEachCycle = from->OnEachCycle;
+	to->OnEachSecond = from->OnEachSecond;
+	to->OnInit = from->OnInit;
+	to->OnReady = from->OnReady;
+	to->UnitType = from->UnitType;
+	for (int k = 0; k < MaxAttackPos; ++k) {
+		for (int m = 0; m < UnitSides; ++m) {
+			to->MissileOffsets[m][k].x = from->MissileOffsets[m][k].x;
+			to->MissileOffsets[m][k].y = from->MissileOffsets[m][k].y;
+		}
+	}
+	for (int i = 0; i < ANIMATIONS_DEATHTYPES + 2; i++) {
+		to->Impact[i].Name = from->Impact[i].Name;
+		to->Impact[i].Missile = from->Impact[i].Missile;
+	}
+	to->MouseAction = from->MouseAction;
+	to->CanAttack = from->CanAttack;
+	to->RepairRange = from->RepairRange;
+	to->RepairHP = from->RepairHP;
+	memcpy(to->RepairCosts, from->RepairCosts, sizeof(from->RepairCosts));
+	to->CanTarget = from->CanTarget;
+	to->Building = from->Building;
+	to->BuildingRules.clear();
+	if (!from->BuildingRules.empty()) {
+		printf("WARNING: unit type copy %s of %s does not inherit BuildingRules\n", fromName, toName);
+	}
+	// XXX: should copy, not share, this will crash
+	// for (auto rule : from->BuildingRules) {
+	// 	to->BuildingRules.push_back(rule);
+	// }
+	to->AiBuildingRules.clear();
+	if (!from->AiBuildingRules.empty()) {
+		printf("WARNING: unit type copy %s of %s does not inherit AiBuildingRules\n", fromName, toName);
+	}
+	// XXX: should copy, not share, this would crash
+	// for (auto rule : from->AiBuildingRules) {
+	// 	to->AiBuildingRules.push_back(rule);
+	// }
+	to->AutoBuildRate = from->AutoBuildRate;
+	to->LandUnit = from->LandUnit;
+	to->AirUnit = from->AirUnit;
+	to->SeaUnit = from->SeaUnit;
+	to->RandomMovementProbability = from->RandomMovementProbability;
+	to->RandomMovementDistance = from->RandomMovementDistance;
+	to->ClicksToExplode = from->ClicksToExplode;
+	to->MaxOnBoard = from->MaxOnBoard;
+	for (unsigned int i = 0; i < from->BoolFlag.size(); i++) {
+		to->BoolFlag[i].value = from->BoolFlag[i].value;
+		to->BoolFlag[i].CanTransport = from->BoolFlag[i].CanTransport;
+		to->BoolFlag[i].CanTargetFlag = from->BoolFlag[i].CanTargetFlag;
+		to->BoolFlag[i].AiPriorityTarget = from->BoolFlag[i].AiPriorityTarget;
+	}
+	memcpy(to->ResInfo, from->ResInfo, sizeof(from->ResInfo));
+	to->GivesResource = from->GivesResource;
+	memcpy(to->CanStore, from->CanStore, sizeof(from->CanStore));
+	to->CanCastSpell = from->CanCastSpell;
+	to->AutoCastActive = from->AutoCastActive;
+	to->Sound.Selected.Name = from->Sound.Selected.Name;
+	to->Sound.Acknowledgement.Name = from->Sound.Acknowledgement.Name;
+	to->Sound.Attack.Name = from->Sound.Attack.Name;
+	to->Sound.Build.Name = from->Sound.Build.Name;
+	to->Sound.Ready.Name = from->Sound.Ready.Name;
+	to->Sound.Repair.Name = from->Sound.Repair.Name;
+	for (int i = 0; i < MaxCosts; i++) {
+		to->Sound.Harvest[i].Name = from->Sound.Harvest[i].Name;
+	}
+	to->Sound.Help.Name = from->Sound.Help.Name;
+	to->Sound.WorkComplete.Name = from->Sound.WorkComplete.Name;
+	for (unsigned int i = 0; i < ANIMATIONS_DEATHTYPES + 1; i++) {
+		to->Sound.Dead[i].Name = from->Sound.Dead[i].Name;
+	}
+	for (unsigned int i = 0; i < UnitTypeVar.GetNumberVariable(); i++) {
+		to->DefaultStat.Variables[i].Enable = from->DefaultStat.Variables[i].Enable;
+		to->DefaultStat.Variables[i].Value = from->DefaultStat.Variables[i].Value;
+		to->DefaultStat.Variables[i].Max = from->DefaultStat.Variables[i].Max;
+		to->DefaultStat.Variables[i].Increase = from->DefaultStat.Variables[i].Increase;
+		to->DefaultStat.Variables[i].IncreaseFrequency = from->DefaultStat.Variables[i].IncreaseFrequency;
+	}
+
+	UpdateDefaultBoolFlags(*to);
+	if (!CclInConfigFile) {
+		UpdateUnitStats(*to, 1);
+	}
+	LoadUnitTypes();
+	return 0;
+}
+
+/**
+** <b>Description</b>
+**
 **  Parse unit-stats.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code><strong>DefineUnitStats</strong>("unit-berserker", 2, {
+**    		"HitPoints", {Value = 55, Max = 55, Increase = 0, Enable = true},
+**    		"AttackRange", {Value = 5, Max = 6, Increase = 0, Enable = true},
+**    		"SightRange", {Value = 7, Max = 7, Increase = 0, Enable = true},
+**  		})</code></div>
 */
 static int CclDefineUnitStats(lua_State *l)
 {
@@ -1304,11 +1630,18 @@ static int CclGetUnitTypeIdent(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Get the name of the unit-type structure.
 **
 **  @param l  Lua state.
 **
 **  @return   The name of the unit-type.
+**
+** Example:
+**
+** <div class="example"><code>name = <strong>GetUnitTypeName</strong>("unit-knight")
+**		  print(name)</code></div>
 */
 static int CclGetUnitTypeName(lua_State *l)
 {
@@ -1320,11 +1653,17 @@ static int CclGetUnitTypeName(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Set the name of the unit-type structure.
 **
 **  @param l  Lua state.
 **
 **  @return   The name of the unit-type.
+**
+** Example:
+**
+** <div class="example"><code><strong>SetUnitTypeName</strong>("unit-beast-cry","Doomhammer")</code></div>
 */
 static int CclSetUnitTypeName(lua_State *l)
 {
@@ -1340,9 +1679,17 @@ static int CclSetUnitTypeName(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Get unit type data.
 **
 **  @param l  Lua state.
+**
+** Example:
+**
+** <div class="example"><code>-- Get the amount of supply from Human Farms
+**		  supply = <strong>GetUnitTypeData</strong>("unit-farm","Supply")
+**		  print(supply)</code></div>
 */
 static int CclGetUnitTypeData(lua_State *l)
 {
@@ -1581,6 +1928,12 @@ void DefineVariableField(lua_State *l, CVariable *var, int lua_index)
 			var->Max = LuaToNumber(l, -1);
 		} else if (!strcmp(key, "Increase")) {
 			var->Increase = LuaToNumber(l, -1);
+		} else if (!strcmp(key, "IncreaseFrequency")) {
+			int value = LuaToNumber(l, -1);
+			var->IncreaseFrequency = value;
+			if (var->IncreaseFrequency != value) {
+				LuaError(l, "IncreaseFrequency out of range!");
+			}
 		} else if (!strcmp(key, "Enable")) {
 			var->Enable = LuaToBoolean(l, -1);
 		} else { // Error.
@@ -1670,6 +2023,8 @@ static int CclDefineDecorations(lua_State *l)
 		bool HideNeutral;
 		bool HideAllied;
 		bool ShowOpponent;
+		bool BoolFlagInvert;
+		int BoolFlag;
 	} tmp;
 
 	const int nargs = lua_gettop(l);
@@ -1723,6 +2078,12 @@ static int CclDefineDecorations(lua_State *l)
 							decovarbar->Height = LuaToNumber(l, -1);
 						} else if (!strcmp(key, "Width")) {
 							decovarbar->Width = LuaToNumber(l, -1);
+						} else if (!strcmp(key, "MinValue")) {
+							decovarbar->MinValue = LuaToNumber(l, -1);
+						} else if (!strcmp(key, "MaxValue")) {
+							decovarbar->MaxValue = LuaToNumber(l, -1);
+						} else if (!strcmp(key, "Invert")) {
+							decovarbar->Invert = LuaToBoolean(l, -1);
 						} else if (!strcmp(key, "Orientation")) {
 							key = LuaToString(l, -1);
 							if (!strcmp(key, "horizontal")) {
@@ -1750,6 +2111,23 @@ static int CclDefineDecorations(lua_State *l)
 						lua_pop(l, 1); // Pop value
 					}
 					decovar = decovarbar;
+				} else if (!strcmp(key, "frame")) {
+					CDecoVarFrame *frame = new CDecoVarFrame;
+					if (!lua_istable(l, -1)) {
+						LuaError(l, "incorrect argument, need table with Thickness= and Color=");
+					}
+					for (lua_pushnil(l); lua_next(l, -2); lua_pop(l, 1)) {
+						const char* innerkey = LuaToString(l, -2);
+						if (!strcmp(innerkey, "Thickness")) {
+							frame->Thickness = LuaToNumber(l, -1);
+						} else if (!strcmp(innerkey, "ColorName")) {
+							const char *const colorName = LuaToString(l, -1);
+							frame->ColorIndex = GetColorIndexByName(colorName);
+						} else {
+							LuaError(l, "'%s' invalid for Method frame" _C_ innerkey);
+						}
+					}
+					decovar = frame;
 				} else if (!strcmp(key, "text")) {
 					CDecoVarText *decovartext = new CDecoVarText;
 
@@ -1775,12 +2153,29 @@ static int CclDefineDecorations(lua_State *l)
 						decovarstaticsprite->FadeValue = LuaToNumber(l, -1, 3);
 					}
 					decovar = decovarstaticsprite;
+				} else if (!strcmp(key, "animated-sprite")) {
+					CDecoVarAnimatedSprite *decovarspritebar = new CDecoVarAnimatedSprite;
+					decovarspritebar->NSprite = GetSpriteIndex(LuaToString(l, -1, 1));
+					if (decovarspritebar->NSprite == -1) {
+						LuaError(l, "invalid sprite-name '%s' for Method in DefineDecorations" _C_ LuaToString(l, -1, 1));
+					}
+					decovarspritebar->WaitFrames = LuaToNumber(l, -1, 2);
+					if (decovarspritebar->WaitFrames <= 0) {
+						LuaError(l, "invalid wait-frames, must be > 0");
+					}
+					decovar = decovarspritebar;
 				} else { // Error
 					LuaError(l, "invalid method '%s' for Method in DefineDecorations" _C_ key);
 				}
 				lua_pop(l, 2); // MethodName and data
-			} else { // Error
-				LuaError(l, "invalid key '%s' for DefineDecorations" _C_ key);
+			} else {
+				tmp.BoolFlag = UnitTypeVar.BoolFlagNameLookup[key];
+				if (tmp.BoolFlag != -1) {
+					tmp.BoolFlagInvert = LuaToBoolean(l, -1);
+				} else {
+					// Error
+					LuaError(l, "invalid key '%s' for DefineDecorations" _C_ key);
+				}
 			}
 			lua_pop(l, 1); // Pop the value
 		}
@@ -1799,6 +2194,8 @@ static int CclDefineDecorations(lua_State *l)
 		decovar->HideNeutral = tmp.HideNeutral;
 		decovar->HideAllied = tmp.HideAllied;
 		decovar->ShowOpponent = tmp.ShowOpponent;
+		decovar->BoolFlag = tmp.BoolFlag;
+		decovar->BoolFlagInvert = tmp.BoolFlagInvert;
 		UnitTypeVar.DecoVar.push_back(decovar);
 	}
 	Assert(lua_gettop(l));
@@ -1823,6 +2220,83 @@ static int CclDefineExtraDeathTypes(lua_State *l)
 	}
 	return 0;
 }
+
+static int CclDefinePaletteSwap(lua_State *l)
+{
+	LuaCheckArgs(l, 2);
+	const char *iconName = LuaToString(l, 1);
+	CIcon *icon = CIcon::Get(iconName);
+	if (!icon) {
+		LuaError(l, "icon %s not found" _C_ iconName);
+	}
+
+	if (!lua_istable(l, 2)) {
+		LuaError(l, "incorrect argument");
+	}
+	const int subargs = lua_rawlen(l, 2);
+	std::vector<PaletteSwap> newSwaps;
+	for (int k = 0; k < subargs; k += 2) {
+		const char *value = LuaToString(l, 2, k + 1);
+		int index = UnitTypeVar.VariableNameLookup[value];
+		if (index == -1) {
+			LuaError(l, "unknown variable name %s" _C_ value);
+		}
+
+		lua_rawgeti(l, 2, k + 2); // swap table
+		if (!lua_istable(l, -1) || lua_rawlen(l, -1) != 2) {
+			LuaError(l, "incorrect argument, need length 2 table with {startColorIndex, { ... color steps ... }");
+		}
+		int startColorIndex = LuaToNumber(l, -1, 1);
+
+		lua_rawgeti(l, -1, 2); // swap table, steps table
+		if (!lua_istable(l, -1)) {
+			LuaError(l, "incorrect argument, need table with color steps");
+		}
+
+		int steps = lua_rawlen(l, -1);
+		std::vector<CColor> colors;
+		int colorCount = 0;
+		int alternativesCount = 0;
+		for (int step = 0; step < steps; step++) {
+			lua_rawgeti(l, -1, step + 1); // swap table, steps table, alternatives table
+			if (alternativesCount) {
+				if (lua_rawlen(l, -1) != alternativesCount) {
+					LuaError(l, "incorrect argument, need table with %d alternatives, got %zu" _C_ alternativesCount _C_ lua_rawlen(l, -1));
+				}
+			} else {
+				alternativesCount = lua_rawlen(l, -1);
+			}
+			for (int alt = 0; alt < alternativesCount; alt++) {
+				lua_rawgeti(l, -1, alt + 1); // swap table, steps table, alternatives table, color table
+				if (!lua_istable(l, -1)) {
+					LuaError(l, "incorrect argument, need table with colors");
+				}
+				if (colorCount) {
+					if (lua_rawlen(l, -1) != colorCount) {
+						LuaError(l, "incorrect argument, need table with %d colors, got %zu" _C_ colorCount _C_ lua_rawlen(l, -1));
+					}
+				} else {
+					colorCount = lua_rawlen(l, -1);
+				}
+				for (int color = 0; color < colorCount; color++) {
+					lua_rawgeti(l, -1, color + 1);
+					CColor c;
+					c.Parse(l);
+					colors.push_back(c);
+					lua_pop(l, 1);
+				}
+				lua_pop(l, 1); // swap table, steps table, alternatives table
+			}
+			lua_pop(l, 1);  // swap table, steps table
+		}
+		lua_pop(l, 1); // swap table
+		lua_pop(l, 1); // <emtpy>
+		newSwaps.emplace_back(index, startColorIndex, colorCount, steps, alternativesCount, colors);
+	}
+	icon->SetPaletteSwaps(newSwaps);
+	return 0;
+}
+
 // ----------------------------------------------------------------------------
 
 /**
@@ -1884,6 +2358,10 @@ void UpdateUnitVariables(CUnit &unit)
 	unit.Variable[POSX_INDEX].Max = Map.Info.MapWidth;
 	unit.Variable[POSY_INDEX].Value = unit.tilePos.y;
 	unit.Variable[POSY_INDEX].Max = Map.Info.MapHeight;
+	unit.Variable[POS_RIGHT_INDEX].Value = unit.tilePos.x + unit.Type->TileWidth;
+	unit.Variable[POS_RIGHT_INDEX].Max = Map.Info.MapWidth;
+	unit.Variable[POS_BOTTOM_INDEX].Value = unit.tilePos.y + unit.Type->TileHeight;
+	unit.Variable[POS_BOTTOM_INDEX].Max = Map.Info.MapHeight;
 
 	// Target Position
 	const Vec2i goalPos = unit.CurrentOrder()->GetGoalPos();
@@ -1902,7 +2380,7 @@ void UpdateUnitVariables(CUnit &unit)
 
 	// SlotNumber
 	unit.Variable[SLOT_INDEX].Value = UnitNumber(unit);
-	unit.Variable[SLOT_INDEX].Max = UnitManager.GetUsedSlotCount();
+	unit.Variable[SLOT_INDEX].Max = UnitManager->GetUsedSlotCount();
 
 	// Is Alive
 	unit.Variable[ISALIVE_INDEX].Value = unit.IsAlive() ? 1 : 0;
@@ -1966,6 +2444,15 @@ void SetMapStat(std::string ident, std::string variable_key, int value, std::str
 				for (int player = 0; player < PlayerMax; ++player) {
 					type->Stats[player].Variables[variable_index].Increase = type->MapDefaultStat.Variables[variable_index].Increase;
 				}
+			} else if (variable_type == "IncreaseFrequency") {
+				type->MapDefaultStat.Variables[variable_index].IncreaseFrequency = value;
+				// TODO: error
+				// if (type->MapDefaultStat.Variables[variable_index].IncreaseFrequency != value) {
+				// 	LuaError(l, "%s.IncreaseFrequency out of range!" _C_ variable_key.c_str());
+				// }
+				for (int player = 0; player < PlayerMax; ++player) {
+					type->Stats[player].Variables[variable_index].IncreaseFrequency = type->MapDefaultStat.Variables[variable_index].IncreaseFrequency;
+				}
 			} else if (variable_type == "Enable") {
 				type->MapDefaultStat.Variables[variable_index].Enable = value;
 				for (int player = 0; player < PlayerMax; ++player) {
@@ -2035,10 +2522,12 @@ void SetMapSound(std::string ident, std::string sound, std::string sound_type, s
 void UnitTypeCclRegister()
 {
 	lua_register(Lua, "DefineUnitType", CclDefineUnitType);
+	lua_register(Lua, "CopyUnitType", CclCopyUnitType);
 	lua_register(Lua, "DefineUnitStats", CclDefineUnitStats);
 	lua_register(Lua, "DefineBoolFlags", CclDefineBoolFlags);
 	lua_register(Lua, "DefineVariables", CclDefineVariables);
 	lua_register(Lua, "DefineDecorations", CclDefineDecorations);
+	lua_register(Lua, "DefinePaletteSwap", CclDefinePaletteSwap);
 
 	lua_register(Lua, "DefineExtraDeathTypes", CclDefineExtraDeathTypes);
 
