@@ -178,7 +178,7 @@ bool FindTerrainType(int movemask, int resmask, int range,
 template <const bool NEARLOCATION>
 class BestDepotFinder
 {
-	inline void operator()(CUnit *const dest)
+	void operator()(CUnit *const dest)
 	{
 		/* Only resource depots */
 		if (dest->Type->CanStore[resource]
@@ -186,64 +186,55 @@ class BestDepotFinder
 			&& dest->CurrentAction() != UnitActionBuilt) {
 			// Unit in range?
 
-			if (NEARLOCATION) {
-				int d = dest->MapDistanceTo(u_near.loc);
+			if constexpr (NEARLOCATION) {
+				const int d = dest->MapDistanceTo(u_near.loc);
 
 				//
 				// Take this depot?
 				//
-				if (d <= range && d < best_dist) {
-					best_depot = dest;
-					best_dist = d;
+				if (d <= this->range && d < this->best_dist) {
+					this->best_depot = dest;
+					this->best_dist = d;
 				}
 			} else {
-				int d;
-				const CUnit *worker = u_near.worker;
-				if (!worker->Container) {
-					d = worker->MapDistanceTo(*dest);
-				} else {
-					d = worker->Container->MapDistanceTo(*dest);
-				}
+				const CUnit *worker = this->u_near.worker;
+				const CUnit *first_container = GetFirstContainer(*worker);
+
+				//simple distance
+				const int distance = first_container->MapDistanceTo(*dest);
 
 				// Use Circle, not square :)
-				if (d > range) {
+				if (distance > this->range) {
 					return;
 				}
 
-				if (best_dist == INT_MAX) {
-					best_depot = dest;
+				if (distance >= this->best_dist) {
+					//if the depot's simple distance is greater or equal to the real travel distance of the currently-chosen depot, then it can never be closer than it, and we have no reason to actually calculate its real travel distance 
+					return;
 				}
 
-				// calck real travel distance
-				if (worker->Container) {
-					UnmarkUnitFieldFlags(*worker->Container);
-				}
-				d = UnitReachable(*worker, *dest, 1);
-				if (worker->Container) {
-					MarkUnitFieldFlags(*worker->Container);
-				}			
+				// calc real travel distance
+				const int travel_distance = UnitReachable(*worker, *dest, 1, worker->Container != nullptr);
 				//
 				// Take this depot?
 				//
-				if (d && d < best_dist) {
-					best_depot = dest;
-					best_dist = d;
+				if (travel_distance && travel_distance < this->best_dist) {
+					this->best_depot = dest;
+					this->best_dist = travel_distance;
 				}
 			}
 		}
 	}
 
 public:
-	BestDepotFinder(const CUnit &w, int res, int ran) :
-		resource(res), range(ran),
-		best_dist(INT_MAX), best_depot(0)
+	explicit BestDepotFinder(const CUnit &w, const int res, const int ran)
+		: resource(res), range(ran)
 	{
 		u_near.worker = &w;
 	}
 
-	BestDepotFinder(const Vec2i &pos, int res, int ran) :
-		resource(res), range(ran),
-		best_dist(INT_MAX), best_depot(0)
+	explicit BestDepotFinder(const Vec2i &pos, const int res, const int ran) :
+		resource(res), range(ran)
 	{
 		u_near.loc = pos;
 	}
@@ -269,9 +260,9 @@ private:
 	} u_near;
 	const int resource;
 	const int range;
-	int best_dist;
+	int best_dist = INT_MAX;
 public:
-	CUnit *best_depot;
+	CUnit *best_depot = nullptr;
 };
 
 CUnit *FindDepositNearLoc(CPlayer &p, const Vec2i &pos, int range, int resource)
@@ -495,7 +486,7 @@ CUnit *FindIdleWorker(const CPlayer &player, const CUnit *last)
 
 	for (int i = 0; i < nunits; ++i) {
 		CUnit &unit = player.GetUnit(i);
-		if (unit.Type->BoolFlag[HARVESTER_INDEX].value && unit.Type->ResInfo && !unit.Removed) {
+		if (unit.Type->BoolFlag[HARVESTER_INDEX].value && !unit.Removed) {
 			if (unit.CurrentAction() == UnitActionStill) {
 				if (SelectNextUnit && !IsOnlySelected(unit)) {
 					return &unit;
@@ -524,7 +515,7 @@ CUnit *FindIdleWorker(const CPlayer &player, const CUnit *last)
 */
 void FindUnitsByType(const CUnitType &type, std::vector<CUnit *> &units, bool everybody)
 {
-	for (CUnitManager::Iterator it = UnitManager.begin(); it != UnitManager.end(); ++it) {
+	for (CUnitManager::Iterator it = UnitManager->begin(); it != UnitManager->end(); ++it) {
 		CUnit &unit = **it;
 
 		if (unit.Type == &type && !unit.IsUnusable(everybody)) {
@@ -548,11 +539,11 @@ void FindPlayerUnitsByType(const CPlayer &player, const CUnitType &type, std::ve
 	if (ai_active) {
 		typecount = player.UnitTypesAiActiveCount[type.Slot];
 	}
-	
+
 	if (typecount < 0) { // if unit type count is negative, something wrong happened
 		fprintf(stderr, "Player %d has a negative %s unit type count of %d.\n", player.Index, type.Ident.c_str(), typecount);
 	}
-	
+
 	if (typecount == 0) {
 		return;
 	}
@@ -701,12 +692,12 @@ private:
 	CUnit *Find(Iterator begin, Iterator end) const
 	{
 		CUnit *enemy = NULL;
-		int best_cost = INT_MAX;
+		int best_cost = GameSettings.SimplifiedAutoTargeting ? INT_MIN : INT_MAX;
 
 		for (Iterator it = begin; it != end; ++it) {
-			const int cost = ComputeCost(*it);
+			int cost = GameSettings.SimplifiedAutoTargeting ? TargetPriorityCalculate(attacker, *it) : ComputeCost(*it);
 
-			if (cost < best_cost) {
+			if (GameSettings.SimplifiedAutoTargeting ? (cost > best_cost) : (cost < best_cost)) {
 				enemy = *it;
 				best_cost = cost;
 			}
@@ -733,7 +724,7 @@ private:
 		// Unit in range ?
 		const int d = attacker->MapDistanceTo(*dest);
 
-		if (d > attackrange && !UnitReachable(*attacker, *dest, attackrange)) {
+		if (d > attackrange && !UnitReachable(*attacker, *dest, attackrange, false)) {
 			return INT_MAX;
 		}
 
@@ -937,7 +928,7 @@ public:
 
 				int attackrange = attacker->Stats->Variables[ATTACKRANGE_INDEX].Max;
 				if (d <= attackrange ||
-					(d <= range && UnitReachable(*attacker, *dest, attackrange))) {
+					(d <= range && UnitReachable(*attacker, *dest, attackrange, false))) {
 					++enemy_count;
 				} else {
 					dest->CacheLock = 1;
@@ -951,17 +942,16 @@ public:
 			// cost map is relative to attacker position
 			const int x = dest->tilePos.x - attacker->tilePos.x + (size / 2);
 			const int y = dest->tilePos.y - attacker->tilePos.y + (size / 2);
-			Assert(x >= 0 && y >= 0);
 
 			// Mark the good/bad array...
 			for (int yy = 0; yy < dtype.TileHeight; ++yy) {
 				for (int xx = 0; xx < dtype.TileWidth; ++xx) {
 					int pos = (y + yy) * (size / 2) + (x + xx);
-					if (pos >= good->size()) {
-						printf("BUG: RangeTargetFinder::FillBadGood.Compute out of range. "\
-						       "size: %d, pos: %d, " \
-						       "x: %d, xx: %d, y: %d, yy: %d",
-						       size, pos, x, xx, y, yy);
+					if (pos < 0 || static_cast<unsigned int>(pos) >= good->size()) {
+						DebugPrint("BUG: RangeTargetFinder::FillBadGood.Compute out of range. "\
+								   "size: %d, pos: %d, "				\
+								   "x: %d, xx: %d, y: %d, yy: %d\n" _C_
+								   size _C_ pos _C_ x _C_ xx _C_ y _C_ yy);
 						break;
 					}
 					if (cost < 0) {
@@ -985,7 +975,9 @@ public:
 
 	CUnit *Find(std::vector<CUnit *> &table)
 	{
-		FillBadGood(*attacker, range, good, bad, size).Fill(table.begin(), table.end());
+		if (!GameSettings.SimplifiedAutoTargeting) {
+			FillBadGood(*attacker, range, good, bad, size).Fill(table.begin(), table.end());
+		}
 		return Find(table.begin(), table.end());
 
 	}
@@ -1012,9 +1004,18 @@ private:
 			dest->CacheLock = 0;
 			return;
 		}
+
+		if (GameSettings.SimplifiedAutoTargeting) {
+			const int cost = TargetPriorityCalculate(attacker, dest);
+			if (cost > best_cost) {
+				best_unit = dest;
+				best_cost = cost;
+			}
+			return;
+		}
+
 		const CUnitType &type = *attacker->Type;
 		const CUnitType &dtype = *dest->Type;
-		const int missile_range = type.Missile.Missile->Range + range - 1;
 		int x = attacker->tilePos.x;
 		int y = attacker->tilePos.y;
 
@@ -1024,12 +1025,11 @@ private:
 		clamp<int>(&y, dest->tilePos.y, dest->tilePos.y + dtype.TileHeight - 1);
 
 		int sbad = 0;
-		int sgood = 0;		
+		int sgood = 0;
 
 		// cost map is relative to attacker position
 		x = dest->tilePos.x - attacker->tilePos.x + (size / 2);
 		y = dest->tilePos.y - attacker->tilePos.y + (size / 2);
-		Assert(x >= 0 && y >= 0);
 
 		// calculate the costs:
 		// costs are the full costs at the target and the splash-factor
@@ -1039,11 +1039,11 @@ private:
 			for (int xx = -1; xx <= 1; ++xx) {
 				int pos = (y + yy) * (size / 2) + (x + xx);
 				int localFactor = (!xx && !yy) ? 1 : splashFactor;
-				if (pos >= good->size()) {
-					printf("BUG: RangeTargetFinder.Compute out of range. " \
-					       "size: %d, pos: %d, "	\
-					       "x: %d, xx: %d, y: %d, yy: %d",
-					       size, pos, x, xx, y, yy);
+				if (pos < 0 || static_cast<unsigned int>(pos) >= good->size()) {
+					DebugPrint("BUG: RangeTargetFinder.Compute out of range. " \
+							   "size: %d, pos: %d, "	\
+							   "x: %d, xx: %d, y: %d, yy: %d \n" _C_
+							   size _C_ pos _C_ x _C_ xx _C_ y _C_ yy);
 					break;
 				}
 				sbad += bad->at(pos) / localFactor;
@@ -1098,7 +1098,7 @@ struct CompareUnitDistance {
 **  @param goal     Second tile
 **  @param flags    Terrain type to check
 **
-**  @return         true, if an obstacle was found, false otherwise
+**  @return         false, if an obstacle was found, true otherwise
 */
 bool CheckObstaclesBetweenTiles(const Vec2i &unitPos, const Vec2i &goalPos, unsigned short flags, int *distance)
 {
@@ -1160,7 +1160,7 @@ CUnit *AttackUnitsInDistance(const CUnit &unit, int range, CUnitFilter pred)
 		const CUnit *firstContainer = unit.Container ? unit.Container : &unit;
 		std::vector<CUnit *> table;
 		SelectAroundUnit(*firstContainer, missile_range, table,
-			MakeAndPredicate(HasNotSamePlayerAs(Players[PlayerNumNeutral]), pred));
+						 MakeAndPredicate(HasNotSamePlayerAs(Players[PlayerNumNeutral]), pred));
 
 		if (table.empty() == false) {
 			return BestRangeTargetFinder(unit, range).Find(table);
@@ -1172,7 +1172,7 @@ CUnit *AttackUnitsInDistance(const CUnit &unit, int range, CUnitFilter pred)
 		std::vector<CUnit *> table;
 
 		SelectAroundUnit(*firstContainer, range, table,
-			MakeAndPredicate(HasNotSamePlayerAs(Players[PlayerNumNeutral]), pred));
+						 MakeAndPredicate(HasNotSamePlayerAs(Players[PlayerNumNeutral]), pred));
 
 		const int n = static_cast<int>(table.size());
 		if (range > 25 && table.size() > 9) {
@@ -1217,7 +1217,7 @@ CUnit *AttackUnitsInRange(const CUnit &unit)
 CUnit *AttackUnitsInReactRange(const CUnit &unit, CUnitFilter pred)
 {
 	Assert(unit.Type->CanAttack);
-	const int range = unit.Player->Type == PlayerPerson ? unit.Type->ReactRangePerson : unit.Type->ReactRangeComputer;
+	const int range = unit.Player->Type == PlayerTypes::PlayerPerson ? unit.Type->ReactRangePerson : unit.Type->ReactRangeComputer;
 	return AttackUnitsInDistance(unit, range, pred);
 }
 
