@@ -375,11 +375,6 @@
 **
 **    Shadow sprite images
 **
-**  CUnitType::PlayerColorSprite
-**
-**    Sprite images of the player colors.  This image is drawn
-**    over CUnitType::Sprite.  Used with OpenGL only.
-**
 **
 */
 /**
@@ -525,8 +520,8 @@ CUnitType::CUnitType() :
 	Slot(0), Width(0), Height(0), OffsetX(0), OffsetY(0), DrawLevel(0),
 	ShadowWidth(0), ShadowHeight(0), ShadowOffsetX(0), ShadowOffsetY(0),
 	Animations(NULL), StillFrame(0),
-	DeathExplosion(NULL), OnHit(NULL), OnEachCycle(NULL), OnEachSecond(NULL), OnInit(NULL),
-	TeleportCost(0), TeleportEffectIn(NULL), TeleportEffectOut(NULL),
+	OnDeath(NULL), OnHit(NULL), OnEachCycle(NULL), OnEachSecond(NULL), OnInit(NULL),
+	OnReady(NULL), TeleportCost(0), TeleportEffectIn(NULL), TeleportEffectOut(NULL),
 	CorpseType(NULL), Construction(NULL), RepairHP(0), TileWidth(0), TileHeight(0),
 	BoxWidth(0), BoxHeight(0), BoxOffsetX(0), BoxOffsetY(0), NumDirections(0),
 	MinAttackRange(0), ReactRangeComputer(0), ReactRangePerson(0),
@@ -541,7 +536,7 @@ CUnitType::CUnitType() :
 	CanAttack(0),
 	Neutral(0),
 	GivesResource(0), PoisonDrain(0), FieldFlags(0), MovementMask(0),
-	Sprite(NULL), ShadowSprite(NULL)
+	Sprite(NULL), AltSprite(NULL), ShadowSprite(NULL), ShadowSpriteFrame(0), ShadowScale(1)
 {
 #ifdef USE_MNG
 	memset(&Portrait, 0, sizeof(Portrait));
@@ -554,11 +549,12 @@ CUnitType::CUnitType() :
 
 CUnitType::~CUnitType()
 {
-	delete DeathExplosion;
+	delete OnDeath;
 	delete OnHit;
 	delete OnEachCycle;
 	delete OnEachSecond;
 	delete OnInit;
+	delete OnReady;
 	delete TeleportEffectIn;
 	delete TeleportEffectOut;
 
@@ -592,12 +588,14 @@ CUnitType::~CUnitType()
 	}
 
 	CGraphic::Free(Sprite);
+	CGraphic::Free(AltSprite);
 	CGraphic::Free(ShadowSprite);
 #ifdef USE_MNG
 	if (this->Portrait.Num) {
-		for (int j = 0; j < this->Portrait.Num; ++j) {
-			delete this->Portrait.Mngs[j];
-			// delete[] this->Portrait.Files[j];
+		if (this->Portrait.Mngs[0]) {
+			for (int j = 0; j < this->Portrait.Num; ++j) {
+				Mng::Free(this->Portrait.Mngs[j]);
+			}
 		}
 		delete[] this->Portrait.Mngs;
 		delete[] this->Portrait.Files;
@@ -654,17 +652,18 @@ void UpdateUnitStats(CUnitType &type, int reset)
 	// Non-solid units can always be entered and they don't block anything
 	if (type.BoolFlag[NONSOLID_INDEX].value) {
 		if (type.Building) {
-			type.MovementMask = MapFieldLandUnit |
-								MapFieldSeaUnit |
-								MapFieldBuilding |
-								MapFieldCoastAllowed |
-								MapFieldWaterAllowed |
-								MapFieldNoBuilding |
-								MapFieldUnpassable;
 			if (type.BoolFlag[DECORATION_INDEX].value && type.MapDefaultStat.Variables[HP_INDEX].Max == 0) {
 				// special case, a decoration with no HP can always be built over
+				type.MovementMask = MapFieldNoBuilding;
 				type.FieldFlags = 0;
 			} else {
+				type.MovementMask = MapFieldLandUnit |
+					MapFieldSeaUnit |
+					MapFieldBuilding |
+					MapFieldCoastAllowed |
+					MapFieldWaterAllowed |
+					MapFieldNoBuilding |
+					MapFieldUnpassable;
 				type.FieldFlags = MapFieldNoBuilding;
 			}
 		} else {
@@ -827,7 +826,7 @@ void SaveUnitTypes(CFile &file)
 		bool somethingSaved = false;
 
 		for (int j = 0; j < PlayerMax; ++j) {
-			if (Players[j].Type != PlayerNobody) {
+			if (Players[j].Type != PlayerTypes::PlayerNobody) {
 				somethingSaved |= SaveUnitStats(type.Stats[j], type, j, file);
 			}
 		}
@@ -894,7 +893,7 @@ CUnitType *NewUnitTypeSlot(const std::string &ident)
 **  @todo  Do screen position caculation in high level.
 **         Better way to handle in x mirrored sprites.
 */
-void DrawUnitType(const CUnitType &type, CPlayerColorGraphic *sprite, int player, int frame, const PixelPos &screenPos)
+void DrawUnitType(const CUnitType &type, CPlayerColorGraphic *sprite, int colorIndex, int frame, const PixelPos &screenPos)
 {
 	PixelPos pos = screenPos;
 	// FIXME: move this calculation to high level.
@@ -905,9 +904,9 @@ void DrawUnitType(const CUnitType &type, CPlayerColorGraphic *sprite, int player
 
 	if (type.Flip) {
 		if (frame < 0) {
-			sprite->DrawPlayerColorFrameClipX(player, -frame - 1, pos.x, pos.y);
+			sprite->DrawPlayerColorFrameClipX(colorIndex, -frame - 1, pos.x, pos.y);
 		} else {
-			sprite->DrawPlayerColorFrameClip(player, frame, pos.x, pos.y);
+			sprite->DrawPlayerColorFrameClip(colorIndex, frame, pos.x, pos.y);
 		}
 	} else {
 		const int row = type.NumDirections / 2 + 1;
@@ -917,7 +916,7 @@ void DrawUnitType(const CUnitType &type, CPlayerColorGraphic *sprite, int player
 		} else {
 			frame = (frame / row) * type.NumDirections + frame % row;
 		}
-		sprite->DrawPlayerColorFrameClip(player, frame, pos.x, pos.y);
+		sprite->DrawPlayerColorFrameClip(colorIndex, frame, pos.x, pos.y);
 	}
 }
 
@@ -989,11 +988,14 @@ void LoadUnitTypeSprite(CUnitType &type)
 	if (!type.ShadowFile.empty()) {
 		type.ShadowSprite = CGraphic::ForceNew(type.ShadowFile, type.ShadowWidth, type.ShadowHeight);
 		type.ShadowSprite->Load();
-		if (type.Flip) {
-			type.ShadowSprite->Flip();
+		if (type.ShadowScale != 1) {
+			type.ShadowSprite->Resize(type.ShadowSprite->GraphicWidth / type.ShadowScale, type.ShadowSprite->GraphicHeight / type.ShadowScale);
 		}
-		if (type.ShadowSprite->Surface->format->BytesPerPixel == 1) {
-			type.ShadowSprite->MakeShadow();
+		if (!type.ShadowSpriteFrame) {
+			if (type.Flip) {
+				type.ShadowSprite->Flip();
+			}
+			type.ShadowSprite->MakeShadow(type.ShadowOffsetX, type.ShadowOffsetY);
 		}
 	}
 
@@ -1030,15 +1032,23 @@ void LoadUnitTypeSprite(CUnitType &type)
 		}
 	}
 
+	if (!type.AltFile.empty()) {
+		type.AltSprite = CPlayerColorGraphic::New(type.AltFile, type.Width, type.Height);
+		type.AltSprite->Load();
+		if (type.Flip) {
+			type.AltSprite->Flip();
+		}
+	}
+
 #ifdef USE_MNG
 	if (type.Portrait.Num) {
 		for (int i = 0; i < type.Portrait.Num; ++i) {
-			type.Portrait.Mngs[i] = new Mng;
-			type.Portrait.Mngs[i]->Load(type.Portrait.Files[i]);
+			type.Portrait.Mngs[i] = Mng::New(type.Portrait.Files[i]);
+			type.Portrait.Mngs[i]->Load();
 		}
 		// FIXME: should be configurable
 		type.Portrait.CurrMng = 0;
-		type.Portrait.NumIterations = SyncRand() % 16 + 1;
+		type.Portrait.NumIterations = MyRand() % 16 + 1;
 	}
 #endif
 }
