@@ -285,6 +285,8 @@ static int CclDefineModifier(lua_State *l)
 #endif
 		if (!strcmp(key, "regeneration-rate")) {
 			um->Modifier.Variables[HP_INDEX].Increase = LuaToNumber(l, j + 1, 2);
+		} else if (!strcmp(key, "regeneration-frequency")) {
+			um->Modifier.Variables[HP_INDEX].IncreaseFrequency = LuaToNumber(l, j + 1, 2);
 		} else if (!strcmp(key, "cost")) {
 			if (!lua_istable(l, j + 1) || lua_rawlen(l, j + 1) != 2) {
 				LuaError(l, "incorrect argument");
@@ -382,7 +384,16 @@ static int CclDefineUnitAllow(lua_State *l)
 }
 
 /**
+** <b>Description</b>
+**
 **  Define which units/upgrades are allowed.
+**
+** Example:
+**
+** <div class="example"><code><strong>DefineAllow</strong>("unit-town-hall","AAAAAAAAAAAAAAAA") -- Available for everybody
+**		<strong>DefineAllow</strong>("unit-stables","FFFFFFFFFFFFFFFF") -- Not available
+**		<strong>DefineAllow</strong>("upgrade-sword1","RRRRRRRRRRRRRRRR") -- Upgrade already researched.</code></div>
+**
 */
 static int CclDefineAllow(lua_State *l)
 {
@@ -452,7 +463,6 @@ int UnitTypeIdByIdent(const std::string &ident)
 		return type->Slot;
 	}
 	DebugPrint(" fix this %s\n" _C_ ident.c_str());
-	Assert(0);
 	return -1;
 }
 
@@ -622,6 +632,7 @@ static void ApplyUpgradeModifier(CPlayer &player, const CUpgradeModifier *um)
 				varModified |= um->Modifier.Variables[j].Value
 							   | um->Modifier.Variables[j].Max
 							   | um->Modifier.Variables[j].Increase
+							   | um->Modifier.Variables[j].IncreaseFrequency
 							   | um->Modifier.Variables[j].Enable
 							   | um->ModifyPercent[j];
 				stat.Variables[j].Enable |= um->Modifier.Variables[j].Enable;
@@ -632,6 +643,7 @@ static void ApplyUpgradeModifier(CPlayer &player, const CUpgradeModifier *um)
 					stat.Variables[j].Value += um->Modifier.Variables[j].Value;
 					stat.Variables[j].Max += um->Modifier.Variables[j].Max;
 					stat.Variables[j].Increase += um->Modifier.Variables[j].Increase;
+					stat.Variables[j].IncreaseFrequency += um->Modifier.Variables[j].IncreaseFrequency;
 				}
 
 				stat.Variables[j].Max = std::max(stat.Variables[j].Max, 0);
@@ -657,6 +669,7 @@ static void ApplyUpgradeModifier(CPlayer &player, const CUpgradeModifier *um)
 						} else {
 							unit.Variable[j].Value += um->Modifier.Variables[j].Value;
 							unit.Variable[j].Increase += um->Modifier.Variables[j].Increase;
+							unit.Variable[j].IncreaseFrequency += um->Modifier.Variables[j].IncreaseFrequency;
 						}
 
 						unit.Variable[j].Max += um->Modifier.Variables[j].Max;
@@ -868,6 +881,7 @@ void ApplyIndividualUpgradeModifier(CUnit &unit, const CUpgradeModifier *um)
 		} else {
 			unit.Variable[j].Value += um->Modifier.Variables[j].Value;
 			unit.Variable[j].Increase += um->Modifier.Variables[j].Increase;
+			unit.Variable[j].IncreaseFrequency += um->Modifier.Variables[j].IncreaseFrequency;
 		}
 		unit.Variable[j].Max += um->Modifier.Variables[j].Max;
 		unit.Variable[j].Max = std::max(unit.Variable[j].Max, 0);
