@@ -72,8 +72,7 @@ static std::map<const CFont *, FontColorGraphicMap> FontColorGraphics;
 static CFont *SmallFont;  /// Small font used in stats
 static CFont *GameFont;   /// Normal font used in game
 
-static int FormatNumber(int number, char *buf);
-
+static std::string FormatNumber(const int number);
 
 CFont &GetSmallFont()
 {
@@ -138,18 +137,10 @@ CFont &GetGameFont()
 static void VideoDrawChar(const CGraphic &g,
 						  int gx, int gy, int w, int h, int x, int y, const CFontColor &fc)
 {
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (UseOpenGL) {
-		g.DrawSub(gx, gy, w, h, x, y);
-	} else
-#endif
-	{
-		SDL_Rect srect = {Sint16(gx), Sint16(gy), Uint16(w), Uint16(h)};
-		SDL_Rect drect = {Sint16(x), Sint16(y), 0, 0};
-		std::vector<SDL_Color> sdlColors(fc.Colors, fc.Colors + MaxFontColors);
-		SDL_SetColors(g.Surface, &sdlColors[0], 0, MaxFontColors);
-		SDL_BlitSurface(g.Surface, &srect, TheScreen, &drect);
-	}
+	SDL_Rect srect = {Sint16(gx), Sint16(gy), Uint16(w), Uint16(h)};
+	SDL_Rect drect = {Sint16(x), Sint16(y), 0, 0};
+	SDL_SetPaletteColors(g.Surface->format->palette, fc.Colors, 0, MaxFontColors);
+	SDL_BlitSurface(g.Surface, &srect, TheScreen, &drect);
 }
 
 /**
@@ -178,108 +169,219 @@ void GetDefaultTextColors(std::string &normalp, std::string &reversep)
 	reversep = DefaultReverseColorIndex;
 }
 
+static const unsigned char codepoint_to_cp437[] = {
+    0xff, 0xad, 0x9b, 0x9c, 0x00, 0x9d, 0x00, 0x00, 0x00, 0x00, 0xa6, 0xae, 0xaa, 0x00, 0x00, 0x00, // 0xa0
+    0xf8, 0xf1, 0xfd, 0x00, 0x00, 0xe6, 0x00, 0xfa, 0x00, 0x00, 0xa7, 0xaf, 0xac, 0xab, 0x00, 0xa8, // 0xb0
+    0x00, 0x00, 0x00, 0x00, 0x8e, 0x8f, 0x92, 0x80, 0x00, 0x90, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 0xc0
+    0x00, 0xa5, 0x00, 0x00, 0x00, 0x00, 0x99, 0x00, 0x00, 0x00, 0x00, 0x00, 0x9a, 0x00, 0x00, 0xe1, // 0xd0
+    0x85, 0xa0, 0x83, 0x00, 0x84, 0x86, 0x91, 0x87, 0x8a, 0x82, 0x88, 0x89, 0x8d, 0xa1, 0x8c, 0x8b, // 0xe0
+    0x00, 0xa4, 0x95, 0xa2, 0x93, 0x00, 0x94, 0xf6, 0x00, 0x97, 0xa3, 0x96, 0x81, 0x00, 0x00, 0x98  // 0xf0
+};
+
+static const unsigned char codepoint_to_cp1252[] = {
+    0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9, 0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf, // 0xa0
+    0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba, 0xbb, 0xbc, 0xbd, 0xbe, 0xbf, // 0xb0
+    0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xcb, 0xcc, 0xcd, 0xce, 0xcf, // 0xc0
+    0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda, 0xdb, 0xdc, 0xdd, 0xde, 0xdf, // 0xd0
+    0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed, 0xee, 0xef, // 0xe0
+    0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff  // 0xf0
+};
+
+static const unsigned char codepoint_to_cp866[] = {
+    0x00, 0xf0, 0x00, 0x00, 0xf2, 0x00, 0x00, 0xf4, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf6, 0x00, // 0x400
+    0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f, // 0x410
+    0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f, // 0x420
+    0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9, 0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf, // 0x430
+    0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed, 0xee, 0xef, // 0x440
+    0x00, 0xf1, 0x00, 0x00, 0xf3, 0x00, 0x00, 0xf5, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf7, 0x00  // 0x450
+};
+
+static const char * codepoint_from_0xa0_to_ascii[] = {
+         " ",      "!",     "C/",     "PS",     "$?",     "Y=",      "|",     "SS",     "\"",    "(c)",      "a",     "<<",      "!",      "?",    "(r)",      "-", // 0xa0
+       "deg",     "+-",      "2",      "3",      "'",      "u",      "P",      "*",      ",",      "1",      "o",     ">>",  " 1/4 ",  " 1/2 ",  " 3/4 ",      "?", // 0xb0
+         "A",      "A",      "A",      "A",      "A",      "A",     "AE",      "C",      "E",      "E",      "E",      "E",      "I",      "I",      "I",      "I", // 0xc0
+         "D",      "N",      "O",      "O",      "O",      "O",      "O",      "x",      "O",      "U",      "U",      "U",      "U",      "Y",     "Th",     "ss", // 0xd0
+         "a",      "a",      "a",      "a",      "a",      "a",     "ae",      "c",      "e",      "e",      "e",      "e",      "i",      "i",      "i",      "i", // 0xe0
+         "d",      "n",      "o",      "o",      "o",      "o",      "o",      "/",      "o",      "u",      "u",      "u",      "u",      "y",     "th",      "y", // 0xf0
+};
+
+static const char * codepoint_from_0x400_to_ascii[] = {
+        "Ie",     "Io",     "Dj",     "Gj",     "Ie",     "Dz",      "I",     "Yi",      "J",     "Lj",     "Nj",    "Tsh",     "Kj",      "I",      "U",    "Dzh", // 0x400
+         "A",      "B",      "V",      "G",      "D",      "E",     "Zh",      "Z",      "I",      "I",      "K",      "L",      "M",      "N",      "O",      "P", // 0x410
+         "R",      "S",      "T",      "U",      "F",     "Kh",     "Ts",     "Ch",     "Sh",   "Shch",      "'",      "Y",      "'",      "E",     "Iu",     "Ia", // 0x420
+         "a",      "b",      "v",      "g",      "d",      "e",     "zh",      "z",      "i",      "i",      "k",      "l",      "m",      "n",      "o",      "p", // 0x430
+         "r",      "s",      "t",      "u",      "f",     "kh",     "ts",     "ch",     "sh",   "shch",      "'",      "y",      "'",      "e",     "iu",     "ia", // 0x440
+        "ie",     "io",     "dj",     "gj",     "ie",     "dz",      "i",     "yi",      "j",     "lj",     "nj",    "tsh",     "kj",      "i",      "u",    "dzh", // 0x450
+};
+
+// default to codepage 437
+int FontCodePage = 437;
+
 /**
-**  Get the next utf8 character from a string
+** Convert a Unicode codepoint to a codepage index, based on the
+** convert table to work with Stratagus char. This maps Unicode
+** codepoints into codepages commonly used for DOS and early Windows
+** games; currently CP437, CP1251, and CP866. More can be added.
+** The games need to ensure that the font graphic represents the
+** appropriate codepage.
+**
+** @param codepoint the char to convert
+** @param subst out variable for a char pointer with a substitution string if the codepage doesn't support the char
+** @return the equivalent codepage index; 0 if a substitution string is set; or '?' if codepoint is unsupported.
 */
-static bool GetUTF8(const std::string &text, size_t &pos, int &utf8)
+static unsigned char codepoint_to_codepage_index(int codepoint, const char **subst)
 {
-	// end of string
-	if (pos >= text.size()) {
-		return false;
+	Assert(codepoint >= 0x80);
+
+	unsigned char cpChar = 0;
+
+	switch (codepoint) {
+		case 0xa1: // special case: print inverted exclamation mark as normal one
+			cpChar = '!';
+			break;
+		case 0x192: // index for this letter in cp437 is 0x9f
+			if (FontCodePage == 437) {
+				cpChar = 0x9f;
+			} else {
+				cpChar = 'f';
+			}
+			break;
+		case 0x2502:
+			cpChar = 0xb3;
+		case 0x2591:
+			cpChar = 0xb0;
+			break;
+		case 0x2592:
+			cpChar = 0xb1;
+			break;
+		case 0x2593:
+			cpChar = 0xb2;
+			break;
+		default:
+			codepoint -= 0xa0;
+			if (codepoint >= 0 && codepoint < sizeof(codepoint_to_cp437)) {
+				// western european
+				switch (FontCodePage) {
+					case 437:
+						cpChar = codepoint_to_cp437[codepoint];
+						break;
+					case 1252:
+						cpChar = codepoint_to_cp1252[codepoint];
+						break;
+					default:
+						*subst = codepoint_from_0xa0_to_ascii[codepoint];
+						return 0;
+				}
+			} else {
+				codepoint -= (0x400 - 0xa0);
+				if (codepoint >= 0 && codepoint < sizeof(codepoint_to_cp866)) {
+					// cyrillic
+					switch (FontCodePage) {
+						case 866:
+							cpChar = codepoint_to_cp866[codepoint];
+							break;
+						default:
+							*subst = codepoint_from_0x400_to_ascii[codepoint];
+							return 0;
+					}
+				}
+			}
+			break;
 	}
 
-	int count;
-	char c = text[pos++];
-
-	// ascii
-	if (!(c & 0x80)) {
-		utf8 = c;
-		return true;
+	if (cpChar < 0x32) {
+		fprintf(stderr, "Can't convert codepoint to codepage %d: '%c' d=%d (0x%04x)\r\n", FontCodePage, codepoint, codepoint, codepoint);
+		cpChar = '?';
 	}
 
-	if ((c & 0xE0) == 0xC0) {
-		utf8 = (c & 0x1F);
-		count = 1;
-	} else if ((c & 0xF0) == 0xE0) {
-		utf8 = (c & 0x0F);
-		count = 2;
-	} else if ((c & 0xF8) == 0xF0) {
-		utf8 = (c & 0x07);
-		count = 3;
-	} else if ((c & 0xFC) == 0xF8) {
-		utf8 = (c & 0x03);
-		count = 4;
-	} else if ((c & 0xFE) == 0xFC) {
-		utf8 = (c & 0x01);
-		count = 5;
-	} else {
-		DebugPrint("Invalid utf8\n");
-		return false;
-	}
-
-	while (count--) {
-		c = text[pos++];
-		if ((c & 0xC0) != 0x80) {
-			DebugPrint("Invalid utf8\n");
-			return false;
-		}
-		utf8 <<= 6;
-		utf8 |= (c & 0x3F);
-	}
-	return true;
+	return cpChar;
 }
 
 /**
-**  Get the next utf8 character from an array of chars
+**  Get the next codepage index to render from a utf8 encoded string. This maps utf8 encoded
+**  unicode codepoints into the codepage index of the active FontCodePage. If no mapping into
+**  the codepage exists, this may offer a single- or multi-char replacement string. The subpos
+**  out-parameter is used to communicate at which position in a multi-char replacement string
+**  we are. Callers should pass in the same pos and subpos arguments while iterating over a loop.
+**  When the subpos is 0 after a call, the utf8 character was completely processed, otherwise
+**  more codepoint indices are needed to substitute the utf8 char.
+**
+**  @return 0 on failure, else the codepage index to print next for the current utf8 character
 */
-static bool GetUTF8(const char text[], const size_t len, size_t &pos, int &utf8)
+static int CodepageIndexFromUTF8(const char text[], const size_t len, size_t &pos, size_t &subpos)
 {
 	// end of string
 	if (pos >= len) {
-		return false;
+		return 0;
 	}
 
+	int codepoint;
 	int count;
-	char c = text[pos++];
+	int postUtf8CharPos = pos;
+	char c = text[postUtf8CharPos++];
 
 	// ascii
 	if (!(c & 0x80)) {
-		utf8 = c;
-		return true;
-	}
-
-	if ((c & 0xE0) == 0xC0) {
-		utf8 = (c & 0x1F);
+		pos = postUtf8CharPos;
+		if (c < 32) {
+			return '?';
+		}
+		return c;
+	} else if ((c & 0xE0) == 0xC0) {
+		codepoint = (c & 0x1F);
 		count = 1;
 	} else if ((c & 0xF0) == 0xE0) {
-		utf8 = (c & 0x0F);
+		codepoint = (c & 0x0F);
 		count = 2;
 	} else if ((c & 0xF8) == 0xF0) {
-		utf8 = (c & 0x07);
+		codepoint = (c & 0x07);
 		count = 3;
 	} else if ((c & 0xFC) == 0xF8) {
-		utf8 = (c & 0x03);
+		codepoint = (c & 0x03);
 		count = 4;
 	} else if ((c & 0xFE) == 0xFC) {
-		utf8 = (c & 0x01);
+		codepoint = (c & 0x01);
 		count = 5;
 	} else {
-		DebugPrint("Invalid utf8 I %c <%s> [%lu]\n" _C_ c _C_ text _C_(long) pos);
-		return false;
+		DebugPrint("Invalid utf8\n");
+		pos = postUtf8CharPos;
+		return 0;
 	}
 
 	while (count--) {
-		c = text[pos++];
+		c = text[postUtf8CharPos++];
 		if ((c & 0xC0) != 0x80) {
-			DebugPrint("Invalid utf8 II\n");
-			return false;
+			DebugPrint("Invalid utf8\n");
+			pos = postUtf8CharPos;
+			return 0;
 		}
-		utf8 <<= 6;
-		utf8 |= (c & 0x3F);
+		codepoint <<= 6;
+		codepoint |= (c & 0x3F);
+	}
+
+	const char *subst;
+	const unsigned char cpIdx = codepoint_to_codepage_index(codepoint, &subst);
+	if (cpIdx) {
+		pos = postUtf8CharPos;
+		return cpIdx;
+	} else {
+		const unsigned char idx = subst[subpos++];
+		if (subpos >= strlen(subst)) {
+			// replacement processed, advance past this utf8 codepoint
+			pos = postUtf8CharPos;
+			subpos = 0;
+		}
+		return idx;
 	}
-	return true;
 }
 
+/**
+**  Get the next codepage index to render from a utf8 encoded string
+*/
+static int CodepageIndexFromUTF8(const std::string &text, size_t &pos, size_t &subpos)
+{
+	return CodepageIndexFromUTF8(text.c_str(), text.size(), pos, subpos);
+}
 
 int CFont::Height() const
 {
@@ -293,52 +395,26 @@ bool CFont::IsLoaded() const
 }
 
 /**
-**  Returns the pixel width of text.
+**  Returns the pixel width of text. This is intimately
+**  tied to FormatNumber
 **
-**  @param text  Text to calculate the width of.
+**  @param number  number to calculate the width of.
 **
 **  @return      The width in pixels of the text.
 */
 int CFont::Width(const int number) const
 {
 	int width = 0;
-#if 0
-	bool isformat = false;
-#endif
-	int utf8;
-	size_t pos = 0;
-	char text[ sizeof(int) * 10 + 2];
-	const int len = FormatNumber(number, text);
+	static_assert(sizeof(int) <= 8); // 64bit numbers need at most 20 decimal places
+	char buf[20] = {'\0'};
+	int len = snprintf(buf, sizeof(buf), "%d", number);
+	int commas = (len - (number < 0 ? 1 : 0) - 1) / 3;
 
 	DynamicLoad();
-	while (GetUTF8(text, len, pos, utf8)) {
-#if 0
-		if (utf8 == '~') {
-			//if (pos >= text.size()) {  // bad formatted string
-			if (pos >= size) {  // bad formatted string
-				break;
-			}
-			if (text[pos] == '<' || text[pos] == '>') {
-				isformat = false;
-				++pos;
-				continue;
-			}
-			if (text[pos] == '!') {
-				++pos;
-				continue;
-			}
-			if (text[pos] != '~') { // ~~ -> ~
-				isformat = !isformat;
-				continue;
-			}
-		}
-		if (!isformat) {
-			width += this->CharWidth[utf8 - 32] + 1;
-		}
-#else
-		width += this->CharWidth[utf8 - 32] + 1;
-#endif
+	for (int i = 0; i < len; i++) {
+		width += this->CharWidth[buf[i] - 32] + 1;
 	}
+	width += (this->CharWidth[',' - 32] + 1) * commas;
 	return width;
 }
 
@@ -355,10 +431,11 @@ int CFont::Width(const std::string &text) const
 	bool isformat = false;
 	int utf8;
 	size_t pos = 0;
+	size_t subpos = 0;
 
 	DynamicLoad();
-	while (GetUTF8(text, pos, utf8)) {
-		if (utf8 == '~') {
+	while ((utf8 = CodepageIndexFromUTF8(text, pos, subpos))) {
+		if (utf8 == '~' && !subpos) {
 			if (text[pos] == '|') {
 				++pos;
 				continue;
@@ -400,7 +477,7 @@ int GetHotKey(const std::string &text)
 	if (text.length() > 1) {
 		hotkey = convertKey(text.c_str());
 	} else if (text.length() == 1) {
-		GetUTF8(text, pos, hotkey);
+		hotkey = CodepageIndexFromUTF8(text, pos, pos);
 	}
 
 	return hotkey;
@@ -461,21 +538,7 @@ unsigned int CFont::DrawChar(CGraphic &g, int utf8, int x, int y, const CFontCol
 
 CGraphic *CFont::GetFontColorGraphic(const CFontColor &fontColor) const
 {
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (UseOpenGL) {
-		CGraphic* fontColorG = FontColorGraphics[this][&fontColor];
-		if (!fontColorG) {
-#ifdef DEBUG
-			fprintf(stderr, "Could not load font color %s for font %s\n", fontColor.Ident.c_str(), this->Ident.c_str());
-#endif
-			return this->G;
-		}
-		return fontColorG;
-	} else
-#endif
-	{
-		return this->G;
-	}
+	return this->G;
 }
 
 /**
@@ -504,16 +567,17 @@ int CLabel::DoDrawText(int x, int y,
 	bool tab;
 	const int tabSize = 4; // FIXME: will be removed when text system will be rewritten
 	size_t pos = 0;
+	size_t subpos = 0;
 	const CFontColor *backup = fc;
 	bool isColor = false;
 	font->DynamicLoad();
 	CGraphic *g = font->GetFontColorGraphic(*FontColor);
 
-	while (GetUTF8(text, len, pos, utf8)) {
+	while ((utf8 = CodepageIndexFromUTF8(text, len, pos, subpos))) {
 		tab = false;
 		if (utf8 == '\t') {
 			tab = true;
-		} else if (utf8 == '~') {
+		} else if (utf8 == '~' && !subpos) {
 			switch (text[pos]) {
 				case '\0':  // wrong formatted string.
 					DebugPrint("oops, format your ~\n");
@@ -610,9 +674,9 @@ int CLabel::Draw(int x, int y, const std::string &text) const
 
 int CLabel::Draw(int x, int y, int number) const
 {
-	char buf[sizeof(int) * 10 + 2];
-	size_t len = FormatNumber(number, buf);
-	return DoDrawText<false>(x, y, buf, len, normal);
+	std::string text = FormatNumber(number);
+	size_t len = text.size();
+	return DoDrawText<false>(x, y, text.c_str(), len, normal);
 }
 
 /// Draw text/number clipped
@@ -635,9 +699,9 @@ int CLabel::DrawClip(int x, int y, const std::string &text, bool is_normal) cons
 
 int CLabel::DrawClip(int x, int y, int number) const
 {
-	char buf[sizeof(int) * 10 + 2];
-	size_t len = FormatNumber(number, buf);
-	return DoDrawText<true>(x, y, buf, len, normal);
+	std::string text = FormatNumber(number);
+	size_t len = text.size();
+	return DoDrawText<true>(x, y, text.c_str(), len, normal);
 }
 
 
@@ -654,9 +718,9 @@ int CLabel::DrawReverse(int x, int y, const std::string &text) const
 
 int CLabel::DrawReverse(int x, int y, int number) const
 {
-	char buf[sizeof(int) * 10 + 2];
-	size_t len = FormatNumber(number, buf);
-	return DoDrawText<false>(x, y, buf, len, reverse);
+	std::string text = FormatNumber(number);
+	size_t len = text.size();
+	return DoDrawText<false>(x, y, text.c_str(), len, reverse);
 }
 
 /// Draw reverse text/number clipped
@@ -672,9 +736,9 @@ int CLabel::DrawReverseClip(int x, int y, const std::string &text) const
 
 int CLabel::DrawReverseClip(int x, int y, int number) const
 {
-	char buf[sizeof(int) * 10 + 2];
-	size_t len = FormatNumber(number, buf);
-	return DoDrawText<true>(x, y, buf, len, reverse);
+	std::string text = FormatNumber(number);
+	size_t len = text.size();
+	return DoDrawText<true>(x, y, text.c_str(), len, reverse);
 }
 
 int CLabel::DrawCentered(int x, int y, const std::string &text) const
@@ -691,31 +755,37 @@ int CLabel::DrawReverseCentered(int x, int y, const std::string &text) const
 	return dx / 2;
 }
 
-
 /**
-**  Format a number using commas
+**	@brief	Format a number using commas
 **
-**  @param number  Number to be formatted
-**  @param buf     Buffer to save the formatted number to
+**  This is closely tied to CFont::Width(int number).
 **
-**  @return      The real length of the Formated Number.
+**	@param	number	Number to be formatted
+**
+**	@return	The formatted number as a string
 */
-static int FormatNumber(int number, char *buf)
+static std::string FormatNumber(const int number)
 {
+	std::string str;
 	const char sep = ',';
-	char bufs[sizeof(int) * 10 + 2];
-	int s = 0;
-	int d = number < 0 ? 1 : 0;
-	const int sl = snprintf(bufs, sizeof(bufs), "%d", abs(number));
+	int n = abs(number);
 
-	while (s <= sl) {
-		if (s > 0 && s < sl && (s - (sl % 3)) % 3 == 0) {
-			buf[d++] = sep;
+	int loop = 0;
+	while (n > 0 || loop == 0) {
+		if (loop > 0 && loop % 3 == 0) {
+			str.insert(0, 1, sep);
 		}
-		buf[d++] = bufs[s++];
+		const char c = n % 10 + 48;
+		str.insert(0, 1, c);
+		n /= 10;
+		loop++;
 	}
-	buf[0] = number < 0 ? '-' : buf[0];
-	return d - 1;
+
+	if (number < 0) {
+		str.insert(0, 1, '-');
+	}
+	
+	return str;
 }
 
 /**
@@ -802,10 +872,11 @@ void CFont::MeasureWidths()
 	CharWidth = new char[maxy];
 	memset(CharWidth, 0, maxy);
 	CharWidth[0] = G->Width / 2;  // a reasonable value for SPACE
-	const Uint32 ckey = G->Surface->format->colorkey;
+	Uint32 ckey = 0;
 	const int ipr = G->Surface->w / G->Width; // images per row
 
 	SDL_LockSurface(G->Surface);
+	SDL_GetColorKey(G->Surface, &ckey);
 	for (int y = 1; y < maxy; ++y) {
 		const unsigned char *sp = (const unsigned char *)G->Surface->pixels +
 								  (y / ipr) * G->Surface->pitch * G->Height +
@@ -817,6 +888,10 @@ void CFont::MeasureWidths()
 			break;
 		}
 		while (sp < gp) {
+			// Some accented glyphes are not perfectly aligned on the glyph grid
+			// (like ï or î ).  So we could do -1 to not compute width on the
+			// next glyph, but that breaks other fonts like the one used in
+			// war1gus. TODO: figure out what to do
 			const unsigned char *lp = sp + G->Width;
 
 			for (; sp < lp; --lp) {
@@ -830,40 +905,6 @@ void CFont::MeasureWidths()
 	SDL_UnlockSurface(G->Surface);
 }
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-/**
-**  Make font bitmap.
-*/
-void CFont::MakeFontColorTextures() const
-{
-	if (!FontColorGraphics[this].empty()) {
-		// already loaded
-		return;
-	}
-	const CGraphic &g = *this->G;
-	SDL_Surface *s = g.Surface;
-
-	for (FontColorMap::iterator it = FontColors.begin(); it != FontColors.end(); ++it) {
-		CFontColor *fc = it->second;
-		CGraphic *newg = FontColorGraphics[this][fc] = new CGraphic;
-
-		newg->Width = g.Width;
-		newg->Height = g.Height;
-		newg->NumFrames = g.NumFrames;
-		newg->GraphicWidth = g.GraphicWidth;
-		newg->GraphicHeight = g.GraphicHeight;
-		newg->Surface = g.Surface;
-
-		SDL_LockSurface(s);
-		for (int j = 0; j < MaxFontColors; ++j) {
-			s->format->palette->colors[j] = fc->Colors[j];
-		}
-		SDL_UnlockSurface(s);
-		MakeTexture(newg);
-	}
-}
-#endif
-
 void CFont::Load()
 {
 	if (this->IsLoaded()) {
@@ -871,15 +912,8 @@ void CFont::Load()
 	}
 
 	if (this->G) {
-		//ShowLoadProgress("Fonts %s", this->G->File.c_str());
 		this->G->Load();
 		this->MeasureWidths();
-
-#if defined(USE_OPENGL) || defined(USE_GLES)
-		if (UseOpenGL) {
-			this->MakeFontColorTextures();
-		}
-#endif
 	}
 }
 
@@ -907,31 +941,6 @@ void LoadFonts()
 	GameFont = CFont::Get("game");
 }
 
-#if defined(USE_OPENGL) || defined(USE_GLES)
-void CFont::FreeOpenGL()
-{
-	if (this->G) {
-		for (FontColorGraphicMap::iterator it = FontColorGraphics[this].begin();
-			 it != FontColorGraphics[this].end(); ++it) {
-			CGraphic &g = *it->second;
-			glDeleteTextures(g.NumTextures, g.Textures);
-		}
-	}
-}
-
-/**
-**  Free OpenGL fonts
-*/
-void FreeOpenGLFonts()
-{
-	for (FontMap::iterator it = Fonts.begin(); it != Fonts.end(); ++it) {
-		CFont &font = *it->second;
-
-		font.FreeOpenGL();
-	}
-}
-#endif
-
 void CFont::Reload() const
 {
 	if (this->G) {
@@ -939,23 +948,15 @@ void CFont::Reload() const
 		for (FontColorGraphicMap::iterator it = fontColorGraphicMap.begin();
 			 it != fontColorGraphicMap.end(); ++it) {
 			CGraphic *g = it->second;
-#if defined(USE_OPENGL) || defined(USE_GLES)
-			delete[] g->Textures;
-#endif
 			delete g;
 		}
 		fontColorGraphicMap.clear();
-#if defined(USE_OPENGL) || defined(USE_GLES)
-		if (UseOpenGL) {
-			this->MakeFontColorTextures();
-		}
-#endif
 	}
 }
 
 
 /**
-**  Reload OpenGL fonts
+**  Reload fonts
 */
 void ReloadFonts()
 {
@@ -1013,10 +1014,13 @@ void ReloadFonts()
 CFontColor::CFontColor(const std::string &ident)
 {
 	Ident = ident;
+	Colors = (SDL_Color*)calloc(sizeof(SDL_Color), MaxFontColors);
+	Assert(Colors);
 }
 
 CFontColor::~CFontColor()
 {
+	free(Colors);
 }
 
 /**
@@ -1054,23 +1058,6 @@ CFontColor::~CFontColor()
 
 void CFont::Clean()
 {
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	CFont *font = this;
-
-	if (UseOpenGL) {
-		FontColorGraphicMap &fontColorGraphicMap = FontColorGraphics[font];
-		if (!fontColorGraphicMap.empty()) {
-			for (FontColorGraphicMap::iterator it = fontColorGraphicMap.begin();
-				 it != fontColorGraphicMap.end(); ++it) {
-				CGraphic *g = it->second;
-				glDeleteTextures(g->NumTextures, g->Textures);
-				delete[] g->Textures;
-				delete g;
-			}
-			fontColorGraphicMap.clear();
-		}
-	}
-#endif
 }
 
 /**
@@ -1084,11 +1071,6 @@ void CleanFonts()
 		font->Clean();
 		delete font;
 	}
-#if defined(USE_OPENGL) || defined(USE_GLES)
-	if (UseOpenGL) {
-		FontColorGraphics.clear();
-	}
-#endif
 	Fonts.clear();
 
 	for (FontColorMap::iterator it = FontColors.begin(); it != FontColors.end(); ++it) {
